<?php

if(!class_exists('DPC_Admin_Settings')){
	class DPC_Admin_Settings {
		
	
		public $settings_id = '';	
	
		public $tabs = array( 'general' => 'General' );
	
		protected $settings = array();
	
		protected $fields = array();
	
		protected $posted_data = array();
	
		public function init_settings() {
			$this->settings = (array) get_option( $this->settings_id );
			foreach ( $this->fields as $tab_key => $tab ) {
				
				foreach ( $tab as $name => $field ) {
					
					if( isset( $this->settings[ $name ] ) ) {
						$this->fields[ $tab_key ][ $name ]['default'] = $this->settings[ $name ];
					}	
				
				}
			}
		}
	
		public function save_settings(){
			
		 	$this->posted_data = $_POST;
		 	if( empty( $this->settings ) ) {
		 		$this->init_settings();
		 	}
		 	foreach ($this->fields as $tab => $tab_data ) {
		 		foreach ($tab_data as $name => $field) {
		 			
		 			$this->settings[ $name ] = $this->{ 'validate_' . $field['type'] }( $name );
		 	
		 		}
		 	}
		 	update_option( $this->settings_id, $this->settings );	
		}
		
		public function get_option( $key, $empty_value = null ) {
			if ( empty( $this->settings ) ) {
				$this->init_settings();
			}
			// Get option default if unset.
			if ( ! isset( $this->settings[ $key ] ) ) {
				$form_fields = $this->fields;
				foreach ( $this->tabs as $tab_key => $tab_title ) {
					if( isset( $form_fields[ $tab_key ][ $key ] ) ) {
						$this->settings[ $key ] = isset( $form_fields[ $tab_key ][ $key ]['default'] ) ? $form_fields[ $tab_key ][ $key ]['default'] : '';
					
					}
				}
				
			}
			if ( ! is_null( $empty_value ) && empty( $this->settings[ $key ] ) && '' === $this->settings[ $key ] ) {
				$this->settings[ $key ] = $empty_value;
			}
			return $this->settings[ $key ];
		}
	  	
		/**
		 * Validate divider
		 * @param  string $key name of the field
		 * @return string     
		 */
		public function validate_divider( $key ){
			$divider  = $this->get_option( $key );
			return $divider;
		}
		
	  	/**
		 * Validate text field
		 * @param  string $key name of the field
		 * @return string     
		 */
		public function validate_text( $key ){
			$text  = $this->get_option( $key );
			if ( isset( $this->posted_data[ $key ] ) ) {
				$text = wp_kses_post( trim( stripslashes( $this->posted_data[ $key ] ) ) );
			}
			return $text;
		}
		
		/**
		 * Validate date field
		 * @param  string $key name of the field
		 * @return string     
		 */
		public function validate_date( $key ){
			$date  = $this->get_option( $key );
			if ( isset( $this->posted_data[ $key ] ) ) {
				$date = wp_kses_post( trim( stripslashes( $this->posted_data[ $key ] ) ) );
			}
			return strtotime($date);
		}
		
		/**
		 * Validate textarea field
		 * @param  string $key name of the field
		 * @return string      
		 */
		public function validate_textarea( $key ){
			$text  = $this->get_option( $key );
			 
			if ( isset( $this->posted_data[ $key ] ) ) {
				$text = wp_kses( trim( stripslashes( $this->posted_data[ $key ] ) ),
					array_merge(
						array(
							'iframe' => array( 'src' => true, 'style' => true, 'id' => true, 'class' => true )
						),
						wp_kses_allowed_html( 'post' )
					)
				);
			}
			return $text;
		}
		/**
		 * Validate WPEditor field
		 * @param  string $key name of the field
		 * @return string      
		 */
		public function validate_wpeditor( $key ){
			$text  = $this->get_option( $key );
			 
			if ( isset( $this->posted_data[ $key ] ) ) {
				$text = wp_kses( trim( stripslashes( $this->posted_data[ $key ] ) ),
					array_merge(
						array(
							'iframe' => array( 'src' => true, 'style' => true, 'id' => true, 'class' => true )
						),
						wp_kses_allowed_html( 'post' )
					)
				);
			}
			return $text;
		}
		/**
		 * Validate select field
		 * @param  string $key name of the field
		 * @return string      
		 */
		public function validate_select( $key ) {
			$value = $this->get_option( $key );
			if ( isset( $this->posted_data[ $key ] ) ) {
				$value = stripslashes( $this->posted_data[ $key ] );
			}
			return $value;
		}
		
		/**
		 * Validate radio
		 * @param  string $key name of the field
		 * @return string      
		 */
		public function validate_radio( $key ) {
			$value = $this->get_option( $key );
			if ( isset( $this->posted_data[ $key ] ) ) {
				$value = stripslashes( $this->posted_data[ $key ] );
			}
			return $value;
		}
		
		/**
		 * Validate multi_checkbox
		 * @param  string $key name of the field
		 * @return string      
		 */
		public function validate_multi_checkbox( $key ) {
			$value = $this->get_option( $key );

			$value = wp_json_encode( $this->posted_data[ $key ] );
			
			return $value;
		}
		
		
		/**
		 * Validate checkbox field
		 * @param  string $key name of the field
		 * @return string      
		 */
		public function validate_checkbox( $key ) {
			$status = '';
			if ( isset( $this->posted_data[ $key ] ) && ( 1 == $this->posted_data[ $key ] ) ) {
				$status = '1';
			}
			return $status;
		}
	
		/**
		 * Adding fields 
		 * @param array $array options for the field to add
		 * @param string $tab tab for which the field is
		 */
		public function add_field( $array, $tab = 'general' ) {
			$allowed_field_types = array(
				'divider',
				'text',
				'textarea',
				'wpeditor',
				'select',
				'date',
				'radio',
				'multi_checkbox',
				'checkbox' );
			// If a type is set that is now allowed, don't add the field
			if( isset( $array['type'] ) &&$array['type'] != '' && ! in_array( $array['type'], $allowed_field_types ) ){
				return;
			}
			$defaults = array(
				'name' => '',
				'title' => '',
				'default' => '',
				'placeholder' => '',
				'type' => 'text',
				'options' => array(),
				'default' => '',
				'desc' => '',
				);
			$array = array_merge( $defaults, $array );
			if( $array['name'] == '' ) {
				return;
			}
			foreach ( $this->fields as $tabs ) {
				if( isset( $tabs[ $array['name'] ] ) ) {
					trigger_error( 'There is alreay a field with name ' . esc_html($array['name']) );
					return;
				}
			}
			// If there are options set, then use the first option as a default value
			if( ! empty( $array['options'] ) && $array['default'] == '' ) {
				$array_keys = array_keys( $array['options'] );
				$array['default'] = $array_keys[0];
			}
			if( ! isset( $this->fields[ $tab ] ) ) {
				$this->fields[ $tab ] = array();
			}
			$this->fields[ $tab ][ $array['name'] ] = $array;
		}
		
		/**
		 * Adding tab
		 * @param array $array options
		 */
		public function add_tab( $array ) {
			$defaults = array(
				'slug' => '',
				'title' => '' );
			$array = array_merge( $defaults, $array );
			if( $array['slug'] == '' || $array['title'] == '' ){
				return;
			}
			$this->tabs[ $array['slug'] ] = $array['title'];
		}
	
		/**
		 * Rendering fields 
		 * @param  string $tab slug of tab
		 * @return void  
		 */
		public function render_fields( $tab ) {
			if( ! isset( $this->fields[ $tab ] ) ) {
				echo '<p>' . esc_html(__( 'There are no settings on these page.', 'webseide' )) . '</p>';
				return;
			}
			foreach ( $this->fields[ $tab ] as $name => $field ) {
				$this->{ 'render_' . $field['type'] }( $field );
			}
		}
	
		
		/**
		 * Render divider
		 * @param  string $field options
		 * @return void     
		 */
		public function render_divider( $field ){
			extract( $field );
			?>
			<tr>
				<td colspan="2">
					<hr/>
				</td>
			</tr>
	
			<?php
		}
		
		/**
		 * Render text field
		 * @param  string $field options
		 * @return void     
		 */
		public function render_text( $field ){
			extract( $field );
			?>
			<tr>
				<th>
					<label for="<?php echo esc_attr($name); ?>"><?php echo esc_html($title); ?></label>
				</th>
				<td>
					<input type="<?php echo esc_attr($type); ?>" name="<?php echo esc_attr($name); ?>" id="<?php echo esc_attr($name); ?>" value="<?php echo esc_attr($default); ?>" placeholder="<?php echo esc_attr($placeholder); ?>" style="width:234px"/>	
					<?php if( $desc != '' ) {
						echo '<p class="description" style="font-size: 80%; opacity: .7;">' . esc_html($desc) . '</p>';
					}?>
				</td>
			</tr>
	
			<?php
		}

		/**
		 * Render text field
		 * @param  string $field options
		 * @return void     
		 */
		public function render_date( $field ){
			extract( $field );
			?>
			<tr>
				<th>
					<label for="<?php echo esc_attr($name); ?>"><?php echo esc_html($title); ?></label>
				</th>
				<td>
					<input type="<?php echo esc_attr($type); ?>" name="<?php echo esc_attr($name); ?>" id="<?php echo esc_attr($name); ?>" value="<?php echo esc_attr((empty($default) ? $default : gmdate('Y-m-d',$default))); ?>" placeholder="<?php echo esc_attr($placeholder); ?>" />	
					<?php if( $desc != '' ) {
						echo '<p class="description" style="font-size: 80%; opacity: .7;">' . esc_html($desc) . '</p>';
					}?>
				</td>
			</tr>
	
			<?php
		}
		
		/**
		 * Render textarea field
		 * @param  string $field options
		 * @return void      
		 */
		public function render_textarea( $field ){
			extract( $field );
			?>
	
			<tr>
				<th>
					<label for="<?php echo esc_attr($name); ?>"><?php echo esc_html($title); ?></label>
				</th>
			</tr>
			<tr>
				<td>
					<textarea name="<?php echo esc_attr($name); ?>" id="<?php echo esc_attr($name); ?>" placeholder="<?php echo esc_attr($placeholder); ?>" style="width:200px"><?php echo esc_html($default); ?></textarea>	
					<?php if( $desc != '' ) {
						echo '<p class="description" style="font-size: 80%; opacity: .7;">' . esc_html($desc) . '</p>';
					}?>
				</td>
			</tr>
	
			<?php
		}
		/**
		 * Render WPEditor field
		 * @param  string $field  options
		 * @return void      
		 */
		public function render_wpeditor( $field ){
			
			extract( $field );
			?>
	
			<tr>
				<th>
					<label for="<?php echo esc_attr($name); ?>"><?php echo esc_html($title); ?></label>
				</th>
				<td>
					<?php wp_editor( $default, $name, array('wpautop' => false) ); ?>
					<?php if( $desc != '' ) {
						echo '<p class="description" style="font-size: 80%; opacity: .7;">' . esc_html($desc) . '</p>';
					}?>
				</td>
			</tr>
	
			<?php
		}
		/**
		 * Render select field
		 * @param  string $field options
		 * @return void      
		 */
		public function render_select( $field ) {
			extract( $field );
			?>
	
			<tr>
				<th>
					<label for="<?php echo esc_attr($name); ?>"><?php echo esc_html($title); ?></label>
				</th>
				<td>
					<select name="<?php echo esc_attr($name); ?>" id="<?php echo esc_attr($name); ?>" style="width:200px">
						<?php 
							foreach ($options as $value => $text) {
								echo '<option ' . selected( $default, $value, false ) . ' value="' . esc_attr($value) . '">' . esc_html($text) . '</option>';
							}
						?>
					</select>
					<?php if( $desc != '' ) {
						echo '<p class="description" style="font-size: 80%; opacity: .7;">' . esc_html($desc) . '</p>';
					}?>
				</td>
			</tr>
	
			<?php
		}
		/**
		 * Render radio
		 * @param  string $field options
		 * @return void      
		 */
		public function render_radio( $field ) {
			extract( $field );
			?>
	
			<tr>
				<th>
					<label for="<?php echo esc_attr($name); ?>"><?php echo esc_html($title); ?></label>
				</th>
				<td>
					<?php 
						foreach ($options as $value => $text) {
							echo '<input name="' . esc_attr($name) . '" id="' . esc_attr($name) . '" type="'.  esc_attr($type) . '" ' . checked( $default, $value, false ) . ' value="' . esc_attr($value) . '">' . esc_html($text) . '<br/>';
						}
					?>
					<?php if( $desc != '' ) {
						echo '<p class="description" style="font-size: 80%; opacity: .7;">' . esc_html($desc) . '</p>';
					}?>
				</td>
			</tr>
	
			<?php
		}
		/**
		 * Render checkbox field
		 * @param  string $field options
		 * @return void      
		 */
		public function render_checkbox( $field ) {
			extract( $field );
			?>
	
			<tr>
				<th>
					<label for="<?php echo esc_attr($name); ?>"> 
						<input <?php checked( $default, '1', true ); ?> type="<?php echo esc_attr($type); ?>" name="<?php echo esc_attr($name); ?>" id="<?php echo esc_attr($name); ?>" value="1" />
						<?php echo esc_html($title); ?>
					</label>
				</th>
			</tr>
	
			<?php
		}
		
		/**
		 * Render multi_checkbox field
		 * @param  string $field options
		 * @return void      
		 */
		public function render_multi_checkbox( $field ) {
			extract( $field );
			
			?>
			
			<tr>
				<th>
					<label for="<?php echo esc_attr($name); ?>"><?php echo esc_html($title); ?></label>
				</th>
				<td>
					<?php 
						foreach ($options as $value => $text) {
							echo '<input name="' . esc_attr($name) . '[]" id="' . esc_attr($name) . '" type="checkbox" ' . (is_array($default) && in_array($value, $default) ? 'checked="checked"' : '') . ' value="' . esc_attr($value) . '">' . esc_html($text) . '<br/>';
						}
					?>
					<?php if( $desc != '' ) {
						echo '<p class="description" style="font-size: 80%; opacity: .7;">' . esc_html($desc) . '</p>';
					}?>
				</td>
			</tr>
			
	
			<?php
		}
		
	  
		
	}
}