<?php

#echo "<pre>";
#print_r(get_option('dpc_license_status'));
#echo "</pre>";


class DPC_Helper_License {

	var $dpc 			= null;
	var $licenseServer 	= 'http://delucks.com';
	var $license 		= '';
	var $oldLicense 	= '';
	var $products		= array('professional' => array('name' => 'DELUCKS SEO Plugin for WordPress Premium'), 'basic' => array('name' => 'DELUCKS SEO Plugin for WordPress FREE'));
	var $details		= '';
	var $versionData	= null;
	var $licenseUpdated = false;

	function __construct(&$dpc){
		if(isset($_REQUEST['action']) && $_REQUEST['action'] == 'heartbeat'){
			return false;
		}

		if(is_admin()){
			$this->dpc = $dpc;
			$this->updateLicense();

			if(isset($_GET['dpc-cancel-demo'])){
				$this->cancelDemo();
			} elseif(isset($_GET['dpc-start-demo'])){
				$this->startDemo();
			}

			if($this->licenseUpdated || $this->licenseExpired() || isset($_POST['dpc-submit-license'])){
				$this->checkLicense();
				header("Location: " . $_SERVER['REQUEST_URI']);
			}

			$this->checkLicenseStatus();

			if($this->getDemoDaysLeft() && $this->getDemoDaysLeft() != 'expired'){
				$this->products['basic']['status'] = true;
				$this->products['professional']['status'] = true;
				$this->updateProductStatus($this->products);
			}

			add_action('admin_init', array(&$this, 'hook_admin_init'));
		}
	}

	function licenseExpired(){
		$expired = false;
		$lastCheck = get_option('dpc_license_last_check_ts', 0);
		if(($lastCheck + (60*2)) < time()){
			$data = get_option('dpc_license_status', array());
			if(isset($data['status']) && $data['status'] == 'expired'){
				$expired = true;
			}
			if(isset($data['expires']) && new DateTime($data['expires']) < new DateTime("now")){
				$expired = true;
			}
			update_option('dpc_license_last_check_ts', time());
		}
		return $expired;
	}

	function hook_admin_init(){
		wp_enqueue_style('dpc-general', plugins_url( 'assets/dpc/css/general.css', DPC_FILE ));
		wp_enqueue_script('dpc-general', plugins_url( 'assets/dpc/js/general.js', DPC_FILE ), array('jquery'));
	}

	function updateLicense(){
		if($this->isLicenseKeySubmitted()){
			if(isset($_POST['dpcLicenseKey']) && $this->getLicense() !== $_POST['dpcLicenseKey']){
			    update_option('dpc_license_key', sanitize_text_field($_POST['dpcLicenseKey']));
				$this->licenseUpdated = true;

				if($this->checkLicense() && $this->checkProduct('professional')){
				    $this->dpc->addAdminMessage('success', wp_kses(__('Your premium functions have been activated.', 'delucks-seo'), $this->dpc->text_allowed_html));
				    
				} else {
				    $this->dpc->addAdminMessage('error', wp_kses(__('No valid license key detected. You are using the free version of DELUCKS SEO.', 'delucks-seo'), $this->dpc->text_allowed_html));
				    
				}
				return true;
			} else {
				return false;
			}
		}
	}

	function updateProductStatus($data = array()){
		update_option('dpc_product_status', $data);
	}

	function isLicenseKeySubmitted(){
		if(isset($_POST['dpcLicenseKey'])){
			return true;
		}
		return false;
	}

	function getLicense(){
		return get_option('dpc_license_key');
	}

	function checkLicense() {
		if(!strlen($this->getLicense())){
			update_option('dpc_license_status', array());
			$this->products['basic']['status'] = true;
			$this->products['professional']['status'] = false;
			$this->updateProductStatus($this->products);
			return false;
		}

		$url = $this->licenseServer . '/?dpc_spll_action=check_license&domain='. $_SERVER['HTTP_HOST'] . '&license='. $this->getLicense() . '&version='.$this->dpc->version.'&ts=' . time();
		// https://delucks.com/?dpc_spll_action=check_license&domain=dpc.delucks.com&license=dpc-dev&version=2.4.6&ts=1630759573


		if(is_wp_error($response = wp_remote_get($url, array('timeout' => 5, 'sslverify' => false)))){
		    array_push($this->dpc->messages, array('type' => 'error', 'text' => esc_html__('Cant connect to the license server. Please try again later', 'delucks-seo').'! (Code <b>'. __LINE__ . '</b>)'));
			return false;
		}

		if(!is_array($data = json_decode(wp_remote_retrieve_body($response), true))){
		    array_push($this->dpc->messages, array('type' => 'error', 'text' => esc_html__('Cant proceed license data', 'delucks-seo').'! (Code <b>'. __LINE__ . '</b>)'));
			return false;
		}

		if(isset($data['status']) && $data['status'] == 'active'){
			$this->products['basic']['status'] = true;
			$this->products['professional']['status'] = true;
		} else {
			$this->products['basic']['status'] = true;
			$this->products['professional']['status'] = false;
		}

		$this->updateProductStatus($this->products);
		update_option('dpc_license_status', $data);

		return true;
	}

	function checkLicenseStatus(){
		$data = $this->getLicenseData();
		if(isset($data['status']) && $data['status'] == 'active'){
			$this->products['basic']['status'] = true;
			$this->products['professional']['status'] = true;
		} else {
			$this->products['basic']['status'] = true;
			$this->products['professional']['status'] = false;
		}
	}

	function getLicenseData(){
		return get_option('dpc_license_status', array());
	}

	function checkProduct($key){
		$product_status = get_option('dpc_product_status', array());
		if(isset($product_status[$key]) && isset($product_status[$key]['status'])){
			return $product_status[$key]['status'];
		}
		return false;
	}

	function checkModule($moduleName){
		$parts = explode('_', $moduleName);
		if(in_array($parts[0], array('basic', 'dpc'))){
			return true;
		}
		if(is_array($parts)){
			return $this->checkProduct($parts[0]);
		}
		return false;
	}

	function printStatus(){
		echo "<pre>Product:\t";
		if($this->versionData->type == 'demo'){
			echo "DEMO License";
		} elseif($this->versionData->type == 'envato'){
			echo "Envato License (Premium only)";
		} elseif($this->versionData->type == 'delucks'){
			echo "Delucks License (Suite)";
		}
		echo "<br/>License:\t";
		print_r($this->license);
		echo "<br/>License Status:\t";
		print_r($this->status);
		echo "<br/><br/><br/>";
		print_r($this->versionData);

		if($this->isDelucks){
			echo "<br/><br/>Details:";
			print_r($this->details);
			print_r($this->products);
		}
		echo "</pre>";
	}

	function cancelDemo(){
		update_option('dpc_demo_start_ts', 'expired');
		$this->checkLicense();
	}

	function startDemo(){
		$demoStartTimestamp = get_option('dpc_demo_start_ts', false);
		if(!$demoStartTimestamp && $demoStartTimestamp !== 'expired'){
			update_option('dpc_demo_start_ts', time());
		}
	}

	function getDemoDaysLeft(){
		$demoStartTimestamp = get_option('dpc_demo_start_ts', false);
		if($demoStartTimestamp && $demoStartTimestamp != 'expired'){
			if(time() < ($demoStartTimestamp + (60*60*24*30))){
				$secondsToExpire = ($demoStartTimestamp + (60*60*24*30)) - time();
				return intval(ceil($secondsToExpire / (60*60*24)));
			} else {
				update_option('dpc_demo_start_ts', 'expired');
				$this->checkLicense();
				return 'expired';
			}
		}
		return $demoStartTimestamp;
	}


	function getVersionData(){
		return $this->versionData;
	}

}