<?php

class DPC_Helper_Messages {

	var $dpc 		= null;
	var $currentId	= null;
	var $type		= 'post';

	function __construct(&$dpc){
		$this->currentId 	= (isset($_GET['id']) && $_GET['id']) ? $_GET['id'] : false;
		$this->dpc 			= $dpc;
		$this->dpc->registerAjaxAction('dpc_post_messages_mark_all_as_read', array($this, 'markAllPostMessagesAsRead_ajax'));
		$this->dpc->registerAjaxAction('dpc_post_messages_clear', array($this, 'clearPostMessages_ajax'));
	}

	function addPostMessage($id, $message){
		if(!isset($message['user'])){
			$current_user 			= wp_get_current_user();
			$message['user'] 		= $current_user->ID;
		}
		$message['timestamp'] 	= current_time('timestamp');
		$message['read'] 		= 0;
		update_post_meta($id, '_dpc-post-messages', array_merge($this->getPostMessages($id, false), array(0 => $message)));

		//prepare for ajax response
		$date_format = get_option('date_format');
		$time_format = get_option('time_format');
		$message['timestamp'] = gmdate($date_format.' '.$time_format, $message['timestamp']);
		$user = get_user_by('ID', $message['user']);
		if(is_object($user) && property_exists($user, 'ID')){
			$message['user'] = $user->user_nicename;
		}
		return $message;
	}

	function markAllPostMessagesAsRead_ajax(){
		if(current_user_can('edit_posts') && isset($_GET['id']) && $_GET['id']){
			$this->markAllPostMessagesAsRead($_GET['id']);
		}
	}

	function clearPostMessages_ajax(){
		if(current_user_can('edit_posts') && isset($_GET['id']) && $_GET['id']){
			$this->clearPostMessages($_GET['id']);
		}
	}

	function markAllPostMessagesAsRead($id){
		$messages = $this->getPostMessages($id, false);
		if(count($messages)){
			foreach($messages as $k => $message){
				$messages[$k]['read'] = 1;
			}
		}
		update_post_meta($id, '_dpc-post-messages', $messages);
	}

	function getUnreadPostMessageCount($id){
		$count = 0;
		$messages = $this->getPostMessages($id);
		if(count($messages)){
			foreach($messages as $message){
				if(!$message['read']){
					$count++;
				}
			}
		}
		return $count;
	}

	function clearPostMessages($id){
		update_post_meta($id, '_dpc-post-messages', array());
	}

	function getPostMessages($id, $formatted = true){
		$messages = get_post_meta($id, '_dpc-post-messages', true);
		if(!is_array($messages) || !count($messages)){
			$messages = array();
		}

		if($formatted){
			if(count($messages)){
				$date_format = get_option('date_format');
				$time_format = get_option('time_format');
				foreach($messages as $k =>$message){
					if(is_array($message)){
						$messages[$k]['timestamp'] = gmdate($date_format.' '.$time_format, $message['timestamp']);
						$user = get_user_by('ID', $message['user']);
						if(is_object($user) && property_exists($user, 'ID')){
							$messages[$k]['user'] = $user->user_nicename;
						}
					}
				}
			}
		}

		return $messages;
	}

}