<?php

#use Elementor\Controls_Manager;
#use Elementor\Core\DocumentTypes\PageBase;

class DPC_Helper_Modal {

	var $dpc 		            = null;
	var $currentId	            = null;
	var $meta		            = array();
	var $oTasks		            = null;
	var $oMessages	            = null;
	var $type		            = 'post';
	var $incompatiblePlugins    = array('js_composer/js_composer.php');

	function __construct(&$dpc){
	    
		$this->dpc 			= $dpc;
		$this->currentId 	= (isset($_GET['id']) && $_GET['id']) ? $_GET['id'] : false;

		if(is_admin()){
			add_action('admin_init', array($this, 'addPostListColumns'));
			if(current_user_can('edit_posts')){
			    add_action('admin_enqueue_scripts', array($this, 'adminEnqueueScripts'));
			}
		}

		add_action('wp_before_admin_bar_render', array($this, 'addAdminTopMenu'));
		$this->dpc->registerAjaxAction('dpc_modal_show', array($this, 'modalShow'));
		$this->dpc->registerAjaxAction('dpc_modal_save', array($this, 'modalSave'));

		#require_once(DPC_PLUGIN_HELPER_DIR . 'tasks.class.php');
		#$this->oTasks = new DPC_Helper_Tasks($this->dpc);

		#require_once(DPC_PLUGIN_HELPER_DIR . 'messages.class.php');
		#$this->oMessages = new DPC_Helper_Messages($this->dpc);

		if(isset($_GET['action']) && $_GET['action'] == 'dpc_modal_show'){
			remove_action('wp_head', 'print_emoji_detection_script', 7);
			remove_action('admin_print_scripts', 'print_emoji_detection_script');
			remove_action('wp_print_styles', 'print_emoji_styles');
			remove_action('admin_print_styles', 'print_emoji_styles');
		}
		

		add_action( 'init', [ $this, 'register_elementor_hooks' ], PHP_INT_MAX ); /* Elementor add Tab & Section */ /* ELEMENTOR TAB TESTING */
	}
	
	/* ELEMENTOR TAB TESTING ----------------------------------------------------------------------------- */
	
	
	function register_elementor_hooks(){ /* ELEMENTOR TAB TESTING */
	
		add_action( 'elementor/editor/before_enqueue_scripts', [ $this, 'adminEnqueueScripts' ] );
		
	    // We are too late for elementor/init. We should see if we can be on time, or else this workaround works (we do always get the "else" though).
	    if ( !did_action( 'elementor/init' ) ) {
	        add_action( 'elementor/init', [ $this, 'elementor_add_panel_tab' ] );
	    }
	    else {
	        $this->elementor_add_panel_tab();
	    }

	    add_action( 'elementor/documents/register_controls', [ $this, 'register_document_controls' ] );
	}
	
	
	function elementor_add_panel_tab() { /* ELEMENTOR TAB TESTING */
	    \Elementor\Controls_Manager::add_tab(
	        'delucks-seo-tab',
	        esc_html__( 'DELUCKS SEO', 'delucks-seo' )
	        );
	}
	
	
	public function register_document_controls( $document ) { /* ELEMENTOR TAB TESTING */
	    // PageBase is the base class for documents like `post` `page` and etc.
	    if ( ! $document instanceof PageBase || ! $document::get_property( 'has_elements' ) ) {
	        #return;
	    }

	    // This is needed to get the tab to appear, but will be overwritten in the JavaScript.
	    $document->start_controls_section(
	        'delucks_temporary_section',
	        [
	            'label' => 'DELUCKS SEO',
	            'tab'   => 'delucks-seo-tab',
	        ]
	    );
	    
	    $document->end_controls_section();
	}
	
	/* ELEMENTOR TAB TESTING ----------------------------------------------------------------------------- */


	function adminEnqueueScripts(){
		
		$post_id = get_queried_object_id();
	    wp_enqueue_script('dpc-modal-trigger', plugins_url( 'assets/dpc/js/modal-trigger.js', DPC_FILE ), array('jquery'), NULL, true);
	    
	    wp_enqueue_style('dpc-edit-style', plugins_url( 'assets/dpc/css/edit.min.css', DPC_FILE ));
	    wp_enqueue_style('dpc-modal-style', plugins_url( 'assets/dpc/css/modal.css', DPC_FILE ));
	    wp_enqueue_style('dpc-modal-add', plugins_url( 'assets/dpc/css/modalforms.min.css', DPC_FILE ));
	    
	    wp_localize_script('dpc-modal-trigger', 'dpc_seo_status_text', $this->getTranslationJs());

	    $str = '	var dpc_plugin_url = \'' . plugins_url('delucks-seo') . "';\n";
	    $str .= '	var dpc_permalink = \'' . get_permalink($post_id) . "';\n";
	    $str .= '	var dpc_edit_link = \'' . get_edit_post_link($post_id) . "';\n";
	    $str .= '	var dpc_modal_trigger_link = \'' . admin_url( 'admin-ajax.php?action=dpc_modal_show&id=' . $post_id ) . "';\n";
		
	    wp_add_inline_script('dpc-modal-trigger', $str, 'before');
	}

	function getTranslationJs(){
	    return array(
	        'status_-1'                    => esc_html__('Indexation set to no_follow', 'delucks-seo'),
	        'status_0'                     => esc_html__('Module inactive', 'delucks-seo'),
	        'status_1'                     => esc_html__('Automatically optimized', 'delucks-seo'),
	        'status_2'                     => esc_html__('Manually optimized', 'delucks-seo'),
	        'check_in_google_link_label'   => esc_html__('Check in Google', 'delucks-seo'),
	        'sistrix_analysis_link_label'  => esc_html__('SI Check', 'delucks-seo'),
	        'sistrix_sync_post_link_label' => esc_html__('Update SI Rank', 'delucks-seo'),
	        'validate_microdata'           => esc_html__('Validate Microdata', 'delucks-seo'),
	        'facebook_debugger'            => esc_html__('Facebook debugger', 'delucks-seo'),
	        'linkedin_debugger'            => esc_html__('Linkedin debugger', 'delucks-seo'),
	        'content_questions'            => esc_html__('Content questions', 'delucks-seo'),
	        'duplicate_content'            => esc_html__('Duplicate Content', 'delucks-seo'),
	        'ai_content_detector'          => esc_html__('AI Content Detector', 'delucks-seo'),
			'competitors'                  => esc_html__('Competitors', 'delucks-seo'),
	        'optimization'                 => esc_html__('Optimization', 'delucks-seo'),
	        'keywords'                     => esc_html__('Keywords', 'delucks-seo'),
	        'description'                  => esc_html__('Description', 'delucks-seo'),
	        'title'                        => esc_html__('Title', 'delucks-seo'),
	        'ai'                           => esc_html__('AI', 'delucks-seo'),
	        '...'                          => esc_html__('...', 'delucks-seo'),
	        'chatbot'                      => esc_html__('AI chat', 'delucks-seo'),
	        'save'                         => esc_html__('Save', 'delucks-seo'),
	        'saved'                        => esc_html__('Saved', 'delucks-seo'),
	        'error'                        => esc_html__('Error', 'delucks-seo'),
	        'close'                        => esc_html__('Close', 'delucks-seo'),
	    );
	}

	/**
	 * addAdminTopMenu() registers the modal trigger button in the admin top menu
	 *
	 * @return void
	 */
	function addAdminTopMenu(){
	    if( !is_admin() || (function_exists('get_post_type') && in_array(get_post_type(), array('shop_order'))) ){
			return;
		}

		foreach($this->incompatiblePlugins as $plugin){
		    if(is_plugin_active($plugin)){
		        return;
		    }
		}

		if(($this->dpc->isEditPage() && current_user_can('edit_posts')) || is_singular()){
		    $id = get_the_id();
			global $wp_admin_bar;

			// add status links to the head menu
			$statusText     = $this->getTranslationJs();
	        $seoStatus      = get_post_meta($id, '_delucks-seo-status', true);
	        $keysToCheck	= array('meta' => 'Meta', 'micro' => 'Micro', 'social' => 'Social', 'text' => 'Text');
	        $statusClasses	= array();
	        $label			= '';

	        foreach($keysToCheck as $key => $name){
	            $status = (isset($seoStatus[$key]) ? $seoStatus[$key] : 1);

	            switch($key){
	                case 'meta':
	                    $moduleName = 'basic_metadata';
	                    break;
	                case 'micro':
	                    $moduleName = 'professional_microdata';
	                    break;
	                case 'social':
	                    $moduleName = 'professional_sharing';
	                    break;
	                case 'text':
	                    $moduleName = 'professional_textopt';
	                    break;
	            }

	            if(!$this->dpc->moduleIsActive($moduleName)){
	                $status = 0;
	            }

	            if(in_array($key, array('micro', 'social', 'text'))){
	                if(!isset($this->dpc->oLicense->products['professional']) || !isset($this->dpc->oLicense->products['professional']['status']) || $this->dpc->oLicense->products['professional']['status'] != 1){
	                    $status = 0;
	                }
	            }

	            $wp_admin_bar->add_menu(array(
	                'parent' => false,
	                'id' => 'delucks-seo-'.$key,
	                'title' => '<span class="delucks-seo-status delucks-seo-status-'.esc_attr($key).'" title="'. esc_attr($statusText['status_' . $status]) .'" status="'.esc_attr($status).'">' . esc_html($name) . '</span>',
	                'href' => $status == 0 ? '' : admin_url( 'admin-ajax.php?action=dpc_modal_show&id=' . $id . '&tab=' . $key),
	                'meta' => array('class' => 'dpcModalTrigger delucks-seo-modal-topmenu-sublinks')
	            ));
	        }
		}
	}


    function removeHeadActions(){
        $this->dpc->oFunctions->removeActions('wp_head', array('wp_enqueue_scripts', 'wp_print_styles', 'wp_print_head_scripts'));
    }


	/**
	 * modalShow() loads the modal environment
	 *
	 * @return void
	 */
	function modalShow(){
		if (!isset($_GET['id']) || FALSE === get_post_status( $_GET['id'] ) ) {
		    _doing_it_wrong( __FUNCTION__, esc_html__( 'ID missing' ), '1.0.0');
			return false;
		}

		add_action('wp_print_scripts', array($this, 'modalRemoveAllScripts'), 900000);
		add_action('wp_print_footer_scripts', array($this, 'modalRemoveAllScripts'), 900000);
		add_action('wp_print_styles', array($this, 'modalRemoveAllStyles'), 900000);
		add_action('wp_head', array($this, 'removeHeadActions'), 890000);

		$this->currentId		= (int)$_GET['id'];
		$this->dpc->currentId	= $this->currentId;
		#$unreadMessageCount		= $this->oMessages->getUnreadPostMessageCount($this->currentId);
		#$undoneTaskCount		= $this->oTasks->getUndonePostTaskCount($this->currentId);

		$meta = get_post_meta($this->currentId);
		foreach($meta as $k => $v){
			$meta[$k] = get_post_meta($this->currentId, $k, true);
		}
		$this->meta = $meta;

		echo '<html>';
		wp_head();
		add_thickbox();
		echo '<body>';
		echo '<form id="dpc_modal_data" name="dpc_modal_data" method="post" action="'.esc_url(admin_url( 'admin-ajax.php?action=dpc_modal_save')).'">';
		echo '	<div class="dpc_"></div>';
		echo '	<div class="dpc_modal_message"></div>';
		#echo '	<input type="submit" value="Speichern" />';
		echo '	<input type="hidden" name="dpc_action" value="dpc_modal_save" />';
		echo '	<input type="hidden" name="dpc_js_error" value="true" />';
		echo '	<input type="hidden" name="dpc_current_id" value="'.esc_attr($this->currentId).'" />';
		echo '	<input type="hidden" name="dpc_permalink" value="'.esc_attr(get_permalink($this->currentId)).'" />';

		/* TABS: start */
		echo '<div class="dpc-modal-action-bar">';
		#echo '	<span class="dpc-modal-tab dpc-modal-tab-trigger dpc-modal-tab-trigger-icon" id="tab-dpc-overview"><span class="dashicons dashicons-dashboard"></span></span>';
		echo '	<div class="dpc-modal-tabs">';
			foreach($this->dpc->moduleInstances as $moduleName => $module){
				if($this->dpc->moduleIsActive($moduleName)){
					$this->dpc->triggerModuleMethod($module, 'modalTab');
				}
			}
		echo '	</div>';
		#echo '	<span dpc-tooltip="' . esc_attr(__('Remote connection', 'delucks-seo')).'" dpc-tooltip-position="bottom" class="dpc-modal-tab dpc-modal-tab-trigger dpc-modal-tab-trigger-icon" id="tab-dpc-connect"><span class="dashicons dashicons-admin-site"></span></span>';
		#echo '	<span dpc-tooltip="' . esc_attr(__('Tasks', 'delucks-seo')).'" dpc-tooltip-position="bottom" class="dpc-modal-tab dpc-modal-tab-trigger dpc-modal-tab-trigger-icon '.($undoneTaskCount ? 'has-indications' : '').'" id="tab-dpc-tasks"><span class="dashicons dashicons-list-view"></span><span class="dpc-indicator-count">'.esc_html($undoneTaskCount).'</span></span>';
		#echo '	<span dpc-tooltip="' . esc_attr(__('Notifications', 'delucks-seo')).'" dpc-tooltip-position="bottom" class="dpc-modal-tab dpc-modal-tab-trigger dpc-modal-tab-trigger-icon '.($unreadMessageCount ? 'has-indications' : '').'" id="tab-dpc-notices"><span class="dashicons dashicons-email"></span><span class="dpc-indicator-count">'.esc_html($unreadMessageCount).'</span></span>';
		echo '	<span id="toggle_modal_position" dpc-tooltip="' . esc_attr(__('Toggle position', 'delucks-seo')).'" dpc-tooltip-position="left" class="dpc-modal-action-item"><span class="dashicons dashicons-image-flip-vertical"></span></span>';
		
		echo '</div>';
		/* TABS: end */
		/* CONTENT: start */
		echo '<div class="dpc-modal-contents form-wrap">';
		#$this->showTabContentOverview();
		foreach($this->dpc->moduleInstances as $moduleName => $module){
			if($this->dpc->moduleIsActive($moduleName)){
				$this->dpc->triggerModuleMethod($module, 'modalBody');
			}
		}
		#$this->showTabContentConnect();
		#$this->showTabContentTasks();
		#$this->showTabContentNotices();
		echo '</div>';
		/* CONTENT: end */

		echo '</form>';
		wp_footer();
		echo '</body></html>';
		die();
	}

	function showTabContentOverview(){
		echo '<div class="dpc-modal-content" id="content-dpc-overview">';
		echo 'Hier Preview-Snippet';
		echo '</div>';
	}

	function showTabContentTasks(){
		#$tasks = $this->oTasks->getPostTasks($this->currentId);
		echo '<div class="dpc-modal-content" id="content-dpc-tasks">';

		echo '<div id="dpc-task-actions">';
		echo '	<input type="text" name="task" />';
		echo '	<span class="addTask button button-secondary button-large">'.esc_html__('Add task', 'delucks-seo').'</span>';
		echo '	<span class="clearTasks button button-secondary button-large">'.esc_html__('Clear all tasks', 'delucks-seo').'</span>';
		echo '</div>';

		echo '	<div class="tasks">';
		if(count($tasks)){
			foreach($tasks as $task){
				if($task['status'] == 'undone'){
					$nextState = 'done';
				} else if($task['status'] == 'done'){
					$nextState = 'hold';
				} else {
					$nextState = 'undone';
				}
				echo '	<div data-taskid="'.esc_attr($task['taskid']).'" data-status="'.esc_attr($task['status']).'" data-next-status="'.esc_attr($nextState).'" class="notice notice-info dpc-tasks-entry dpc-notice-list-entry">
							<div class="dpc-notice-list-content dpc-task-content" dpc-tooltip="Mark task as '.esc_attr($nextState).'">
								<div class="dpc-notice-list-title dpc-task-title">
									<span class="time">'.esc_html($task['timestamp']).' - </span>
									<span class="user">'.esc_html($task['user']).'</span>
								</div>
								<div class="dpc-notice-list-status">
									<strong>Status:</strong> <span class="status">'.esc_html($task['status']).'</span>
								</div>
								<span class="message">'.esc_html($task['task']).'</span>
							</div>
							<div class="dpc-notice-list-actions dpc-task-actions">
								<span class="dashicons dashicons-no-alt remove-task"></span>
							</div>
						</div>';
			}
		}
		echo '	</div>';

		echo '</div>';
	}

	function showTabContentNotices(){
		$messages = $this->oMessages->getPostMessages($this->currentId);
		echo '<div class="dpc-modal-content" id="content-dpc-notices">';
		echo '	<span class="clearMessages button button-secondary button-large">'.esc_html__('Clear all notices', 'delucks-seo').'</span>';
		echo '	<div class="messages">';
		if(count($messages)){
			foreach($messages as $message){
			    echo '		<div class="notice notice-'.esc_attr($message['type']).' dpc-notice-entry '.esc_attr($message['type']).(!$message['read'] ? ' unread' : '').'">'.$message['icon'].' <span class="time">'.esc_html($message['timestamp']).'</span> <span class="user">'.esc_html($message['user']).'</span> <span class="message">'.esc_html($message['message']).'</span> </div>';
			}
		}
		echo '	</div>';
		echo '</div>';
	}

	function showTabContentConnect(){
		echo '<div class="dpc-modal-content" id="content-dpc-connect">';
		echo 'Remote connection';
		echo '</div>';
	}


	/**
	 * modalSave() triggers the method "modalSave" of all modules
	 *
	 * @return void
	 */
	function modalSave(){
		if(isset($_POST['dpc_js_error'])){
		    echo esc_html__('A JavaScript error prevents to save your SEO data. Please debug your JavaScript code first.', 'delucks-seo');
			die();
		}

		$response = array();
		$status = array();
		$message = array();
		$recheck = array();

		if((isset($_POST['dpc_action']) && $_POST['dpc_action'] == 'dpc_modal_save') && (isset($_POST['dpc_current_id']) && !empty($_POST['dpc_current_id']))){
			$this->currentId		= $_POST['dpc_current_id'];
			$this->dpc->currentId	= $this->currentId;
			foreach($this->dpc->moduleInstances as $moduleName => $module){
				if($this->dpc->moduleIsActive($moduleName)){
					$this->dpc->triggerModuleMethod($module, 'modalSave');
				}
			}

			$recheck = time() + 31536000;
			update_post_meta($this->currentId, '_delucks-seo-recheck', $recheck);
			$status = get_post_meta($this->currentId, '_delucks-seo-status', true);

			foreach($status as $key => $current_status){
			    switch($key){
			        case 'meta':
			            $moduleName = 'basic_metadata';
			            break;
			        case 'micro':
			            $moduleName = 'professional_microdata';
			            break;
			        case 'social':
			            $moduleName = 'professional_sharing';
			            break;
			        case 'text':
			            $moduleName = 'professional_textopt';
			            break;
			    }

			    if(!$this->dpc->moduleIsActive($moduleName)){
			        $status[$key] = 0;
			    }

			    if(in_array($key, array('micro', 'social', 'text'))){
			        if(!isset($this->dpc->oLicense->products['professional']) || !isset($this->dpc->oLicense->products['professional']['status']) || $this->dpc->oLicense->products['professional']['status'] != 1){
			            $status[$key] = 0;
			        }
			    }
			}

			$message['category']	= 'form';
			$message['type']		= 'success';
			$message['icon']		= '<span class="dashicons dashicons-yes"></span>';
			$message['message']		= 'Metas have been saved.';
		} else {
			$message['category']	= 'form';
			$message['type']		= 'error';
			$message['icon']		= '<span class="dashicons dashicons-no-alt"></span>';
			$message['message']		= 'No data id found. Please reload page.';
		}

		#$message = $this->oMessages->addPostMessage($this->currentId, $message);
		$response['messages'][] = $message;
		$response['status'] = $status;
		$response['recheck'] = $recheck;
		echo wp_json_encode($response);
		die();
	}

	/**
	 * modalRemoveAllScripts() removes all script we dont need in our modal and enqueues all we need by triggering the method "modalEnqueueScripts" in all modules
	 *
	 * @return void
	 */
	function modalRemoveAllScripts() {
	    global $wp_scripts;
	    $wp_scripts->queue = array();
	    $post = (array)get_post(intval($_GET['id']));

	    wp_enqueue_script('jquery.form', plugins_url( 'assets/dpc/js/jquery.form.min.js', DPC_FILE ), array('jquery'), DPC_VERSION, true);
	    wp_enqueue_script('dpc-modal', plugins_url( 'assets/dpc/js/modal.js', DPC_FILE ), array('jquery', 'jquery.form'), DPC_VERSION, true);
	    wp_localize_script('dpc-modal', 'dpc_post_data', $post);

	    foreach($this->dpc->moduleInstances as $moduleName => $module){
	        if($this->dpc->moduleIsActive($moduleName)){
	            $this->dpc->triggerModuleMethod($module, 'modalEnqueueScripts');
	        }
	    }

	    /* remove wrong enqueued scripts: start */
	    $pluginUrl    = plugins_url() . '/delucks-seo/';
	    $includePath  = $wp_scripts->default_dirs[1];
	    $adminPath    = $wp_scripts->default_dirs[0];
	    foreach($wp_scripts->queue as $index => $handle){
	        if(isset($wp_scripts->registered[$handle])){
	            if( substr($wp_scripts->registered[$handle]->src, 0, strlen($pluginUrl)) === $pluginUrl ||
	                substr($wp_scripts->registered[$handle]->src, 0, strlen($includePath)) === $includePath ||
	                substr($wp_scripts->registered[$handle]->src, 0, strlen($adminPath)) === $adminPath){
	                    continue;
	            } else {
	                unset($wp_scripts->queue[$index]);
	            }

	        }
	    }
	    /* remove wrong enqueued scripts: end */

	    $this->modalRemoveAllStyles();

	    wp_enqueue_script('dpc', plugins_url( 'assets/dpc/js/functions.js', DPC_FILE ), array('jquery'), DPC_VERSION, true);
	}

	/**
	 * modalRemoveAllStyles() removes all style we dont need in our modal and enqueues all we need by triggering the method "modalEnqueueStyles" in all modules
	 *
	 * @return void
	 */
	function modalRemoveAllStyles() {
	    foreach($this->dpc->moduleInstances as $moduleName => $module){
	        if($this->dpc->moduleIsActive($moduleName)){
	            $this->dpc->triggerModuleMethod($module, 'modalEnqueueStyles');
	        }
	    }

	    /* remove wrong enqueued styles: start */
	    global $wp_styles;
	    $pluginUrl     = plugins_url() . '/delucks-seo/';
	    $includePath   = '/' . WPINC . '/css/';
	    $adminPath     = '/wp-admin/css/';
	    foreach($wp_styles->queue as $index => $handle){
	        if(isset($wp_styles->registered[$handle])){
	            if( substr($wp_styles->registered[$handle]->src, 0, strlen($pluginUrl)) === $pluginUrl ||
	                substr($wp_styles->registered[$handle]->src, 0, strlen($includePath)) === $includePath ||
	                substr($wp_styles->registered[$handle]->src, 0, strlen($adminPath)) === $adminPath){
	                    continue;
	            } else {
	                unset($wp_styles->queue[$index]);
	            }

	        }
	    }
	    /* remove wrong enqueued styles: end */
	}

	/**
	 * addPostListColumns() register columns for post listings
	 *
	 * @return void
	 */
	function addPostListColumns() {
		add_filter('manage_posts_columns', array($this, 'addPostListColumnsSeoLink'));
		add_action('manage_posts_custom_column', array($this, 'valuePostListColumnsSeoLink'), 10, 2);
		add_filter('manage_edit-post_sortable_columns', array($this, 'sortablePostListColumns'), 1, 2);
		add_filter('manage_pages_columns', array($this, 'addPostListColumnsSeoLink'));
		add_action('manage_pages_custom_column', array($this, 'valuePostListColumnsSeoLink'), 10, 2);
		add_filter('manage_edit-page_sortable_columns', array($this, 'sortablePostListColumns'), 1, 2);
		add_filter('request', array($this, 'orderbyPostListColumns'));
	}

	function orderbyPostListColumns( $vars ) {
	    if ( isset( $vars['orderby'] ) && 'dpc-seo-modal' == $vars['orderby'] ) {
	        $vars = array_merge($vars, array(
	            'orderby' => 'meta_value',
	            'meta_query' => array(
	                'relation' => 'OR',
	                array(
	                    'key'=>'_delucks-seo-recheck',
	                    'compare' => 'EXISTS'
	                ),
	                array(
	                    'key'=>'_delucks-seo-recheck',
	                    'compare' => 'NOT EXISTS'
	                )
	            ),
	            'posts_per_page' => -1
	        ));
	    }
	    return $vars;
	}

	/**
	 * addPostListColumnsSeoLink() adds the seo-modal-link column to post listings
	 *
	 * @return void
	 */
	function addPostListColumnsSeoLink( $cols ) {
	    $cols["dpc-seo-modal"] = esc_html__('SEO', 'delucks-seo');
		return $cols;
	}

	/**
	 * valuePostListColumnsSeoLink() gets the content for the seo-modal-link column in post listings
	 *
	 * @return void
	 */
	function valuePostListColumnsSeoLink( $column_name, $id ) {
	    $statusText = $this->getTranslationJs();
		switch($column_name){
			case 'dpc-seo-modal':
			    $recheck        = get_post_meta($id, '_delucks-seo-recheck', true);
				$seoStatus		= get_post_meta($id, '_delucks-seo-status', true);
				$keysToCheck	= array('meta' => 'Meta', 'micro' => 'Micro', 'social' => 'Social', 'text' => 'Text');
				$statusClasses	= array();
				$label			= '<input type="hidden" class="recheckTimestamp" value="'. esc_attr($recheck) .'" />';

				foreach($keysToCheck as $key => $name){
				    $status = (isset($seoStatus[$key]) ? $seoStatus[$key] : 1);

				    switch($key){
				        case 'meta':
				            $moduleName = 'basic_metadata';
				            break;
				        case 'micro':
				            $moduleName = 'professional_microdata';
				            break;
				        case 'social':
				            $moduleName = 'professional_sharing';
				            break;
				        case 'text':
				            $moduleName = 'professional_textopt';
				            break;
				    }

				    if(!$this->dpc->moduleIsActive($moduleName)){
				        $status = 0;
				    }

				    if(in_array($key, array('micro', 'social', 'text'))){
				        if(!isset($this->dpc->oLicense->products['professional']) || !isset($this->dpc->oLicense->products['professional']['status']) || $this->dpc->oLicense->products['professional']['status'] != 1){
				            $status = 0;
				        }
				    }

				    if($status == 0){
				        $label .= '<span class="delucks-seo-status delucks-seo-status-' . esc_attr($key) . '" status="'. esc_attr($status) .'" title="'. esc_attr($statusText['status_' . $status]) .'">'.esc_html($name).'</span>';
				    } else {
				        $label .= '<a class="dpcModalTriggerButton delucks-seo-status delucks-seo-status-' . esc_attr($key) . '" status="'. esc_attr($status) .'" href="' . esc_url(admin_url( 'admin-ajax.php?action=dpc_modal_show&id=' . $id .'&tab=' . $key)) . '" title="'. esc_attr($statusText['status_' . $status]) .'">'.esc_html($name).'</a>';
				    }

				}

				$label .= '<span class="delucks-seo-recheck-icon'. (!empty($recheck) && ($recheck - time()) < 0 ? ' active' : '') .'" title="'. esc_attr(__('This content has not been updated for over a year. Please check, update and save!', 'delucks-seo')) .'">!</span>';

				echo $label;
				break;
		}
	}

	/**
	 * sortablePostListColumns() make the seo-modal-link column in post listings sortable
	 *
	 * @return array $cols
	 */
	function sortablePostListColumns( $cols ) {
	    $cols["dpc-seo-modal"] = "dpc-seo-modal";
	    return $cols;
	}

	/**
	 * initXMLText() help poedit to parse text
	 *
	 * @return void
	 */
	function initXMLText(){
	    esc_html__('Metas have been saved', 'delucks-seo');
	    esc_html__('No data id found', 'delucks-seo');
	}


}