<?php

class DPC_Helper_Tasks {

	var $dpc 		= null;
	var $currentId	= null;
	var $type		= 'post';

	function __construct(&$dpc){
		$this->currentId 	= (isset($_GET['id']) && $_GET['id']) ? $_GET['id'] : false;
		$this->dpc			= $dpc;
		$this->dpc->registerAjaxAction('dpc_tasks_add_task',			array($this, 'addTask_ajax'));
		$this->dpc->registerAjaxAction('dpc_tasks_remove_task',			array($this, 'removeTask_ajax'));
		$this->dpc->registerAjaxAction('dpc_tasks_clear_tasks',			array($this, 'clearTasks_ajax'));
		$this->dpc->registerAjaxAction('dpc_tasks_change_task_status',	array($this, 'changeTaskStatus_ajax'));
	}

	function addTask_ajax(){
		if(current_user_can('edit_posts') && $this->currentId !== false){
			if(isset($_GET['task']) && strlen(trim($_GET['task']))){
				$task = $this->addTask($this->currentId, array('task' => $_GET['task']));
				echo wp_json_encode($task);
				die();
			}
		}
	}

	function addTask($id, $task){
		if(!isset($task['user'])){
			$current_user 		= wp_get_current_user();
			$task['user'] 		= $current_user->ID;
		}

		$task['taskid']		= md5(wp_rand(0,999).current_time('timestamp').wp_rand(0,999));
		$task['timestamp'] 	= current_time('timestamp');
		$task['status'] 	= 'undone';
		update_post_meta($id, '_dpc-post-tasks', array_merge($this->getPostTasks($id, false), array(0 => $task)));

		//prepare for ajax response
		$date_format = get_option('date_format');
		$time_format = get_option('time_format');
		$task['timestamp'] = gmdate($date_format.' '.$time_format, $task['timestamp']);
		$user = get_user_by('ID', $task['user']);
		if(is_object($user) && property_exists($user, 'ID')){
			$task['user'] = $user->user_nicename;
		}
		return $task;
	}

	function removeTask_ajax(){
		if(current_user_can('edit_posts') && $this->currentId !== false){
			if(isset($_GET['taskid']) && strlen(trim($_GET['taskid']))){
				$this->removeTask($this->currentId, trim($_GET['taskid']));
			}
		}
	}

	function removeTask($id, $taskid){
		$tasks = $this->getPostTasks($id, false);
		if(count($tasks)){
			foreach($tasks as $k => $task){
				if(isset($task['taskid']) && $task['taskid'] == $taskid){
					unset($tasks[$k]);
					update_post_meta($id, '_dpc-post-tasks', $tasks);
					return true;

				}
			}
		}
		return false;
	}

	function getPostTasks($id, $formatted = true){
		$tasks = get_post_meta($id, '_dpc-post-tasks', true);
		if(!is_array($tasks) || !count($tasks)){
			$tasks = array();
		}

		if($formatted){
			if(count($tasks)){
				$date_format = get_option('date_format');
				$time_format = get_option('time_format');
				foreach($tasks as $k =>$task){
					if(is_array($task)){
						$tasks[$k]['timestamp'] = gmdate($date_format.' '.$time_format, $task['timestamp']);
						$user = get_user_by('ID', $task['user']);
						if(is_object($user) && property_exists($user, 'ID')){
							$tasks[$k]['user'] = $user->user_nicename;
						}
					}
				}
			}
		}
		return $tasks;
	}

	function changeTaskStatus_ajax(){
		if(current_user_can('edit_posts') && $this->currentId !== false){
			if(isset($_GET['taskid']) && strlen(trim($_GET['taskid'])) && strlen(trim($_GET['status']))){
				$task = $this->changeTaskStatus($this->currentId, trim($_GET['taskid']), trim($_GET['status']));
				echo wp_json_encode($task);
				die();
			}
		}
	}

	function changeTaskStatus($id, $taskid, $status){
		$tasks = $this->getPostTasks($id, false);
		if(count($tasks)){
			foreach($tasks as $k => $task){
				if(isset($task['taskid']) && $task['taskid'] == $taskid){
					$tasks[$k]['status'] = $status;
					update_post_meta($id, '_dpc-post-tasks', $tasks);
					return $tasks[$k];
				}
			}
		}
		return false;
	}

	function getUndonePostTaskCount($id){
		$count = 0;
		$tasks = $this->getPostTasks($id);
		if(count($tasks)){
			foreach($tasks as $task){
				if($task['status'] == 'undone'){
					$count++;
				}
			}
		}
		return $count;
	}

	function clearTasks_ajax(){
		if(current_user_can('edit_posts') && $this->currentId !== false){
			$this->clearTasks($this->currentId);
		}
	}

	function clearTasks($id){
		update_post_meta($id, '_dpc-post-tasks', array());
	}

	/**
	 * initXMLText() help poedit to parse text
	 *
	 * @return void
	 */
	function initXMLText(){
	}


}