<?php
if (!class_exists('DPC_Uninstall_Feedback')){

    /**
     * Uninstall feedback class
     */
    class DPC_Uninstall_Feedback {

		function __construct(&$dpc){
			add_action("wp_ajax_dpc_send_uninstall_reason", array($this, "send_uninstall_reason"));

			add_action('admin_footer', array($this, 'deactivate_scripts'));
        }
		

		
        public function deactivate_scripts() {
            global $pagenow;
			
            if ('plugins.php' != $pagenow) {
                return true;
            }
			
            $reasons = $this->get_uninstall_reasons();
            ?>
            <div class="pipe-modal" id="pipe-pipe-modal">
                <div class="pipe-modal-wrap">
                    <div class="pipe-modal-header">
                        <h3><?php esc_html_e('If you have a moment, please let us know why you are uninstalling:', 'delucks-seo'); ?></h3>
                    </div>
                    <div class="pipe-modal-body">
                            <?php wp_nonce_field( 'dpc_send_uninstall_reason', 'dpc_uninstall_reason_nonce' ); ?>
                    		<div class="form-row">
                    			<label for="reply">
                    				<input type="checkbox" name="reply" id="reply" value="" />
                    				<?php esc_html_e('A reply is requested', 'delucks-seo'); ?>
                    			</label>
                    		</div>
                    		<div class="form-row reply" style="display: none;">
                    			<label for="email"><?php esc_html_e('Your name', 'delucks-seo'); ?></label>
                    			<input type="text" name="name" id="name" value=""/>
                    		</div>
                    		<div class="form-row reply" style="display: none;">
                    			<label for="email"><?php esc_html_e('Your email address', 'delucks-seo'); ?></label>
                    			<input type="text" name="email" id="email" value="" />
                    		</div>
                    		 
                        <ul class="reasons">
                            <?php foreach ($reasons as $reason) { ?>
                                <li data-type="<?php echo esc_attr($reason['type']); ?>" data-placeholder="<?php echo esc_attr($reason['placeholder']); ?>">
                                    <label><input type="radio" name="selected-reason" value="<?php echo esc_attr($reason['id']); ?>"> <?php echo esc_html($reason['text']); ?></label>
                                </li>
                            <?php } ?>
                        </ul>
                       
                    </div>
                    <div class="pipe-modal-footer">
                        <a href="#" class="dont-bother-me"><?php esc_html_e('I rather wouldn\'t say', 'delucks-seo'); ?></a>
                        <a class="button-primary" href="https://wordpress.org/support/plugin/delucks-seo/" target="_blank">
                        		<span class="dashicons dashicons-external" style="margin-top:3px;"></span>
                        		<?php esc_html_e('Get support', 'delucks-seo'); ?>
                        	</a>
                        <button class="button-primary pipe-model-submit"><?php esc_html_e('Submit & Deactivate', 'delucks-seo'); ?></button>
                        <button class="button-secondary pipe-model-cancel"><?php esc_html_e('Cancel', 'delucks-seo'); ?></button>
                    </div>
                </div>
            </div>

            <style type="text/css">
                .pipe-modal {
                    position: fixed;
                    z-index: 99999;
                    top: 0;
                    right: 0;
                    bottom: 0;
                    left: 0;
                    background: rgba(0,0,0,0.5);
                    display: none;
                }
                .pipe-modal.modal-active {display: block;}
                .pipe-modal-wrap {
                    width: 50%;
                    position: relative;
                    margin: 10% auto;
                    background: #fff;
                }
                .pipe-modal-header {
                    border-bottom: 1px solid #eee;
                    padding: 8px 20px;
                }
                .pipe-modal-header h3 {
                    line-height: 150%;
                    margin: 0;
                }
                .pipe-modal-body {padding: 5px 20px 20px 20px;}
                .pipe-modal-body .input-text,.pipe-modal-body textarea {width:75%;}
                .pipe-modal-body .reason-input {
                    margin-top: 5px;
                    margin-left: 20px;
                }
                .pipe-modal-footer {
                    border-top: 1px solid #eee;
                    padding: 12px 20px;
                    text-align: right;
                }
                 .pipe-modal label {
                 	display: block;
                 }
                .reviewlink{
                    padding:10px 0px 0px 35px !important;
                    font-size: 15px;
                }
                .review-and-deactivate{
                    padding:5px;
                }
                .wt-uninstall-feedback-privacy-policy {
                    text-align: left;
                    font-size: 12px;
                    color: #aaa;
                    line-height: 14px;
                    margin-top: 20px;
                    font-style: italic;
                }

                .wt-uninstall-feedback-privacy-policy a {
                    font-size: 11px;
                    color: #4b9cc3;
                    text-decoration-color: #99c3d7;
                }
            </style>
            <script type="text/javascript">
                (function ($) {
                    $(function () {
                        var modal = $('#pipe-pipe-modal');
                        var uninstallLink = '';

                        $('#the-list').on('click', 'a#deactivate-delucks-seo', function (e) {
                        		e = e || window.event;
                            	e.preventDefault();
                            	e.stopPropagation();

                            	modal.addClass('modal-active');
                            	uninstallLink = $(this).attr('href');
                            	modal.find('a.dont-bother-me').attr('href', uninstallLink).css('float', 'left');
                        });
						
						modal.on('change', 'input[id="reply"]', function () {
							$('.form-row.reply').toggle('show')
						});
						
                        $('#pipe-pipe-modal').on('click', 'a.review-and-deactivate', function (e) {
                            e.preventDefault();
                            window.open("https://de.wordpress.org/plugins/delucks-seo/reviews/#new-post");
                            window.location.href = uninstallLink;
                        });
                        modal.on('click', 'button.pipe-model-cancel', function (e) {
                            e.preventDefault();
                            modal.removeClass('modal-active');
                        });
                        modal.on('click', 'input[type="radio"]', function () {
                            var parent = $(this).parents('li:first');
                            modal.find('.reason-input').remove();
                            var inputType = parent.data('type'),
                                    inputPlaceholder = parent.data('placeholder');
                            if ('reviewhtml' === inputType) {
                                var reasonInputHtml = '<div class="reviewlink"><a href="#" target="_blank" class="review-and-deactivate"><?php esc_html_e('Uninstall and leave a review', 'delucks-seo'); ?> <span class="xa-pipe-rating-link"> &#9733;&#9733;&#9733;&#9733;&#9733; </span></a></div>';
                            } else {
                                var reasonInputHtml = '<div class="reason-input">' + (('text' === inputType) ? '<input type="text" class="input-text" size="40" />' : '<textarea rows="5" cols="45"></textarea>') + '</div>';
                            }
                            if (inputType !== '') {
                                parent.append($(reasonInputHtml));
                                parent.find('input, textarea').attr('placeholder', inputPlaceholder).focus();
                            }
                        });

                        modal.on('click', 'button.pipe-model-submit', function (e) {
                            e.preventDefault();
                            var button = $(this);
                            if (button.hasClass('disabled')) {
                                return;
                            }
                            var $radio = $('input[type="radio"]:checked', modal);
                            var $selected_reason = $radio.parents('li:first'),
                                    $input = $selected_reason.find('textarea, input[type="text"]');

                            $.ajax({
                                url: '<?php echo esc_url(admin_url('admin-ajax.php')); ?>',
                                type: 'POST',
                                data: {
                                    action: 'dpc_send_uninstall_reason',
                                    reason_id: (0 === $radio.length) ? 'none' : $radio.val(),
                                    reason_info: (0 !== $input.length) ? $input.val().trim() : '',
                                    name: $('input#name').val(),
                                    email: $('input#email').val(),
                                    reply: $('input#reply').val(),
                                    nonce: $('input#dpc_uninstall_reason_nonce').val()
                                },
                                beforeSend: function () {
                                    button.addClass('disabled');
                                    button.text('Processing...');
                                },
                                complete: function () {
                                    window.location.href = uninstallLink;
                                }
                            });
                        });
                    });
                }(jQuery));
            </script>
            <?php
        }

        function send_uninstall_reason() {

            if ( ! isset( $_POST['nonce'] ) || ! wp_verify_nonce( $_POST['nonce'], 'dpc_send_uninstall_reason' ) || !current_user_can( 'activate_plugins' ) ) {
                wp_send_json_error( __('You are not allowed to execute this action', 'delucks-seo') );
                wp_die();
            }

			$site_title 		= get_bloginfo( 'name' );
			$site_url 			= network_site_url( '/' );
			$theme				= wp_get_theme();

			$to 		= 'uninstall@delucks.com';
			$subject 	= 'DPC Uninstall Notice';
			$headers 	= array( 'Content-Type: text/html; charset=UTF-8' );
			
			$body 		 = 'Das Plugin wurde auf '.$site_title.' ('.$site_url.') deaktiviert. Folgende Infos wurden &uuml;bermittelt:<br/>';
			$body 		.= '<strong>Name:</strong> ' . (isset($_POST['name']) ? $_POST['name'] : '<i>kein Name angegeben</i>') . '<br/>';
			$body 		.= '<strong>E-Mail:</strong> ' . (isset($_POST['email']) ? $_POST['email'] : '<i>keine E-Mail angegeben</i>') . '<br/>';
			$body 		.= '<strong>R&uuml;ckmeldung:</strong> ' . (isset($_POST['reply']) && $_POST['reply'] == 1 ? '<b>Um R&uuml;ckmeldung wird gebeten</b>' : '<i>keine R&uuml;ckmeldung gew&uuml;nscht</i>') . '<br/>';
			$body 		.= '<strong>Grund:</strong> ' . (isset($_REQUEST['reason_id']) ? trim(stripslashes($_REQUEST['reason_id'])) : '<i>kein Grund angegeben</i>') . '<br/><br/>';

			$body 		.= 'Weitere Infos: ' . (isset($_REQUEST['reason_info']) ? trim(stripslashes($_REQUEST['reason_info'])) : '<i>keine weiteren Infos angegeben</i>') . '<br/><br/>';

			$body 		.= '<strong>WP-Version:</strong> ' . get_bloginfo('version') . '<br/>';
			$body 		.= '<strong>Theme:</strong> ' . $theme->get( 'Name' ) . ' (' . $theme->get( 'Version' ) . ') <br/>';
			$body 		.= '<strong>Plugins (1):</strong> ' . implode(', ', get_option('active_plugins')) . ' <br/>';
			$body 		.= '<strong>Plugins (2):</strong> ' . implode(', ', $this->get_installed_plugins()) . ' <br/>';
			$body 		.= '<strong>PHP-Version:</strong> ' . phpversion() . '<br/>';

			
			
			wp_mail( $to, $subject, $body, $headers, array( '' ) );
           	
           	wp_send_json_success('');
			wp_die();
        }

        private function get_uninstall_reasons() {
            $reasons = array(
                array(
                    'id' => 'used-it',
                    'text' => __('Used it successfully. Don\'t need it anymore.', 'delucks-seo'),
                    'type' => '',
                    'placeholder' => __('Have used it successfully and aint in need of it anymore', 'delucks-seo')
                ),
                array(
                    'id' => 'could-not-understand',
                    'text' => __('I couldn\'t understand how to make it work', 'delucks-seo'),
                    'type' => 'textarea',
                    'placeholder' => __('Would you like us to assist you?', 'delucks-seo')
                ),
                array(
                    'id' => 'found-better-plugin',
                    'text' => __('I found a better plugin', 'delucks-seo'),
                    'type' => 'text',
                    'placeholder' => __('Which plugin?', 'delucks-seo')
                ),
                array(
                    'id' => 'not-have-that-feature',
                    'text' => __('The plugin is great, but I need specific feature that you don\'t support', 'delucks-seo'),
                    'type' => 'textarea',
                    'placeholder' => __('Could you tell us more about that feature?', 'delucks-seo')
                ),
                array(
                    'id' => 'is-not-working',
                    'text' => __('The plugin is not working', 'delucks-seo'),
                    'type' => 'textarea',
                    'placeholder' => __('Could you tell us a bit more whats not working?', 'delucks-seo')
                ),
                array(
                    'id' => 'looking-for-other',
                    'text' => __('It\'s not what I was looking for', 'delucks-seo'),
                    'type' => 'textarea',
                    'placeholder' => __('Could you tell us a bit more?', 'delucks-seo')
                ),
                array(
                    'id' => 'did-not-work-as-expected',
                    'text' => __('The plugin didn\'t work as expected', 'delucks-seo'),
                    'type' => 'textarea',
                    'placeholder' => __('What did you expect?', 'delucks-seo')
                ),
                array(
                    'id' => 'other',
                    'text' => __('Other', 'delucks-seo'),
                    'type' => 'textarea',
                    'placeholder' => __('Could you tell us a bit more?', 'delucks-seo')
                ),
            );

            return $reasons;
        }
		
		private function get_installed_plugins(){
			$plugins 		= get_plugins();
			$plugin_list 	= array();
			
			foreach($plugins as $plugin){
				array_push($plugin_list, $plugin['Name'] . '(V. '. $plugin['Version'] .')');
			}
			
			return $plugin_list;
		}
    }

    new DPC_Uninstall_Feedback($this);

}

