<?php
/**
 * Plugin Name: DELUCKS SEO
 * Description: Easy WordPress SEO plugin made in Germany. Powerful functions, fast loading times & optimization process for WordPress posts, pages, WooCommerce & more.
 * Plugin URI: https://delucks.com/en/wordpress-seo-plugin/
 * Version: 2.5.5
 * Author: DELUCKS GmbH
 * Author URI: http://delucks.com
 * Text Domain: delucks-seo
 *
 * WC requires at least: 2.0
 * WC tested up to: 4.0.1
 */

if ( ! defined( 'ABSPATH' ) ) {
	die();
}

defined('DPC_VERSION') 					or define('DPC_VERSION', '2.5.5');
defined('DPC_FILE') 						or define('DPC_FILE', __FILE__);
defined('DPC_HOOK') 						or define('DPC_HOOK', 'DPC');
defined('DPC_PLUGIN_FILE') 				or define('DPC_PLUGIN_FILE', __FILE__);
defined('DPC_PLUGIN_DIR') 				or define('DPC_PLUGIN_DIR', plugin_dir_path( __FILE__ ));
defined('DPC_PLUGIN_URL') 				or define('DPC_PLUGIN_URL', plugin_dir_url( __FILE__ ));
defined('DPC_PLUGIN_MODULES_DIR') 		or define('DPC_PLUGIN_MODULES_DIR', DPC_PLUGIN_DIR . 'modules/');
defined('DPC_PLUGIN_HELPER_DIR') 		or define('DPC_PLUGIN_HELPER_DIR', DPC_PLUGIN_DIR . 'helper/');
defined('DPC_PLUGIN_UPLOAD_DIRNAME') 	or define('DPC_PLUGIN_UPLOAD_DIRNAME', '/dpc-data/');

class DPC {

	var $version				= '2.5.5';
	var $license				= '';
	var $author					= 'DELUCKS GmbH';
	var $moduleList 			= array();
	var $moduleInstances 		= array();
	var $moduleSettings			= array();
	var $modal					= null;
	var $guiHelper				= null;
	var $guiSettings			= false;
	var $uploadDir				= null;
	var $uploadUrl				= null;
	var $oFunctions				= null;
	var $oLicense				= null;
	var $oPlaceholder           = null;
	var $log					= null;
	var $objectType				= null;
	var $objectId				= null;
	var $messages				= array();
	var $translatorJs			= array();
	var $cc						= false;
	var $load_scripts_in_footer = false;
	var $show_expert_settings   = false;
	var $currentId				= null;
	var $text_allowed_html      = array('a' => array('href' => array(), 'title' => array(), 'class' => array(), 'target' => array()), 'img' => array('src' => array(), 'title' => array(), 'class' => array(), 'alt' => array()), 'br' => array(), 'em' => array(), 'strong' => array('class' => array()), 'span' => array('class' => array()));
	
    /**
     * The constructor automatically kicks of the initialization of all the modules, registering in the navigation and
     * triggering FrontendActions if called from the WP frontend. As long as the dpc plugin is activated this happens
     * automatically thanks to the instantiation of this class after the class implementation.
     */
    function __construct(){
        #set_error_handler(array($this, 'error_handler'));
		#$this->hook_init();
		$this->load_scripts_in_footer 	= (bool) get_option('dpc_load_scripts_in_footer', false) ? true : false;
		$this->show_expert_settings 	= (bool) get_option('dpc_expert_settings', false) ? true : false;
		$wpUploadDir					= wp_upload_dir();
		$this->uploadDir				= $wpUploadDir['basedir'] . DPC_PLUGIN_UPLOAD_DIRNAME;
		$this->uploadUrl				= $wpUploadDir['baseurl'] . DPC_PLUGIN_UPLOAD_DIRNAME;
		// phpcs:ignore WordPress.Security.NonceVerification.Recommended
		$this->objectType				= (isset($_REQUEST['object_type']) ? sanitize_text_field($_REQUEST['object_type']) : null); //@todo: this seems to be not in use?
		// phpcs:ignore WordPress.Security.NonceVerification.Recommended
		$this->objectId					= (isset($_REQUEST['object_id']) ? intval($_REQUEST['object_id']) : null);

		
		require_once(DPC_PLUGIN_HELPER_DIR . 'functions.php');
		$this->oFunctions = new DPC_Functions($this);
		add_action('wp_enqueue_scripts', array($this, 'printScripts'));
		require_once(DPC_PLUGIN_HELPER_DIR . 'license.class.php');
		
		$this->oLicense = new DPC_Helper_License($this);
		$this->initModules();

	    add_action('init', array($this, 'initModal'));
	    add_action('init', array($this, 'saveAction'));
		
		add_filter( 'admin_body_class', array($this, 'dpc_admin_body_classes') );

		$this->initModules();

		if(is_admin()){
			global $pagenow;
			
			add_action('wp_ajax_dpc_remove_admin_message', array($this, 'ajaxRemoveAdminMessage'));
			$this->indexationCheck();
			$this->showAdminMessages();
			
			add_filter('plugin_action_links_' . plugin_basename( __FILE__ ), array($this, 'dpc_plugin_action_links') );
			require_once(DPC_PLUGIN_HELPER_DIR . 'uninstall.php');
			
			add_action('save_post', array(&$this, 'check'));
			add_action('admin_notices', array($this, 'adminNotices'), 20);

			$this->translatorJs = array(
			    'cancel' 							=> esc_html__('Cancel', 'delucks-seo'),
			    'install_defaults' 					=> esc_html__('Install Defaults', 'delucks-seo'),
			    'are_you_sure'						=> esc_html__('Are you sure', 'delucks-seo'),
			    'install_defaults_modal_content'		=> esc_html__('If the default settings are being installed all your settings and changes within this module will be deleted', 'delucks-seo'),
			);

			add_action('admin_init', array(&$this, 'hook_admin_init'));
			add_action('admin_head', array(&$this, 'adminHead'));

			foreach($this->moduleInstances as $moduleName => $module){
				// phpcs:ignore WordPress.Security.NonceVerification.Recommended
				if(isset($_REQUEST['page']) && is_array($page = explode('-', $_REQUEST['page']))){
					if(isset($page[1]) && key_exists($page[1], $this->moduleList) && preg_match('/^'.$page[1].'/', $moduleName)){
						add_action('admin_init', array($module, 'adminSettingsHead'));
					}
				}
				if($this->moduleIsActive($moduleName) || in_array($moduleName, array('dpc_dashboard', 'dpc_importexport', 'dpc_wizard'))){
					add_action('admin_init', array($module, 'adminHead'));
					$this->triggerModuleMethod($module, 'executeBackendActions');
				}
			}

			add_action('admin_menu', array(&$this, 'adminInit' ), 998);
			$this->executeBackendActions();
			add_filter( 'custom_menu_order', array($this, 'adminMenuOrder') );
		} else {
			foreach($this->moduleInstances as $moduleName => $module){
				if($this->moduleIsActive($moduleName)){
					$this->triggerModuleMethod($module, 'executeFrontendActions');
				}
			}
			$this->executeFrontendActions();
		}
		
		
		
		if(wp_doing_ajax()){
		    foreach($this->moduleInstances as $moduleName => $module){
		        if($this->moduleIsActive($moduleName)){
		            $this->triggerModuleMethod($module, 'executeFrontendActions');
		        }
		    }
		}

		foreach($this->moduleInstances as $moduleName => $module){
			if($this->moduleIsActive($moduleName)){
				$this->triggerModuleMethod($module, 'registerWidgets');
			}
		}

		register_activation_hook(__FILE__, array($this, 'activatePlugin'));
		
	}
	
	function dpc_admin_body_classes($classes){
		$modal_position = get_option('dpc_modal_position', false);
		
		$classes .= 'dpc-modal-position-'.$modal_position;
		
		return $classes;
	}
	
	function error_handler($errNo, $errStr, $errFile, $errLine) {
		ini_set('display_errors', 'on');
    		error_reporting(E_ALL);

		$msg = "DELUCKS SEO - ERROR: $errStr in $errFile on line $errLine";
	    if ($errNo == E_NOTICE || $errNo == E_WARNING) {
			echo '<pre>' . esc_html($errNo) .': ' . esc_html($msg) . '</pre>';
       		//throw new ErrorException($msg, $errNo);
	    } else {
	        echo esc_html($msg);
	    }
	}
	
	function saveAction(){
	    if(is_admin() && is_user_logged_in()){
	        $this->registerAjaxAction('dpc_save_toggle_state', array($this, 'saveToggleState'));
	        $this->saveSettings();
	        $this->installDefaultSettingsByXml();
	    }
	}
	
	function dpc_plugin_action_links($links){
		if ( array_key_exists( 'deactivate', $links ) ) {
			$links[ 'deactivate' ] = str_replace( '<a', '<a class="dpc-deactivate-link"', $links[ 'deactivate' ] );
		}
		return $links;
	}
	
	function hook_init(){
	    $locale = apply_filters( 'plugin_locale', get_locale(), 'delucks-seo' );
	    $loaded = load_textdomain( 'delucks-seo', trailingslashit( WP_LANG_DIR ) . 'plugins/' . 'delucks-seo' . '-' . $locale . '.mo');
	    if($loaded) {
	        return $loaded;
	    } else {
	        load_plugin_textdomain( 'delucks-seo');
	    }
	}

	function initModal(){
	    require_once(ABSPATH . 'wp-includes/pluggable.php');
	    if(current_user_can('edit_posts')){
	        require_once(DPC_PLUGIN_HELPER_DIR . 'modal.class.php');
	        $this->modal = new DPC_Helper_Modal($this);
	        wp_enqueue_script('jquery-ui-core');
	        wp_enqueue_script( 'jquery-ui-resizable' );
	    }
		
		
	}

	function getPlaceholder(){
	    if(!is_object($this->oPlaceholder)){
	        require_once(DPC_PLUGIN_HELPER_DIR . 'placeholder.class.php');
	        $this->oPlaceholder = new DPC_Helper_Placeholder($this);
	    }
	    return $this->oPlaceholder;
	}

	function activatePlugin(){
		if($this->isNewInstallation()){
			$this->getModuleInstance('dpc_importexport')->setupDefault();
			foreach($this->moduleInstances as $module){
				$this->triggerModuleMethod($module, 'newInstallation');
			}
		}

		if(is_admin()){
			foreach($this->moduleInstances as $module){
				$this->triggerModuleMethod($module, 'activatePlugin');
			}
		}
		
		//register deactivation hook
		register_uninstall_hook( __FILE__, 'dpc_uninstall' );
	}

	function isNewInstallation(){
		if($this->getModuleSettings('dpc_dashboard', false) == false && $this->getModuleSettings('basic_metadata', false) == false && $this->getModuleSettings('professional_breadcrumbs', false) == false){
			return true;
		}
		return false;
	}

	function isDpcPage(){
	    foreach(array_keys($this->moduleInstances) as $moduleName){
			// phpcs:ignore WordPress.Security.NonceVerification.Recommended
			if(isset($_REQUEST['page']) && is_array($page = explode('-', $_REQUEST['page']))){
				if(isset($page[1]) && key_exists($page[1], $this->moduleList) && preg_match('/^'.$page[1].'/', $moduleName)){
				    return true;
				}
			}
		}
		return false;
	}

	function ajaxRemoveAdminMessage(){
		$messages = get_option('dpc_admin_messages', array());
		// phpcs:ignore WordPress.Security.NonceVerification.Recommended
		if(isset($_GET['hash']) && is_array($messages) && count($messages)){
			foreach($messages as $index => $message){
				// phpcs:ignore WordPress.Security.NonceVerification.Recommended
			    if($message['hash'] == sanitize_text_field($_GET['hash'])){
					unset($messages[$index]);
				}
			}
		}

		update_option('dpc_admin_messages', $messages);
	}

	function addAdminMessage($type, $text){
		$messages = get_option('dpc_admin_messages', array());
		$messages[] = array('type' => $type, 'text' => $text, 'hash' => md5($type.$text.time()));
		update_option('dpc_admin_messages', $messages);
	}

	function getAdminMessages(){
		$messages = get_option('dpc_admin_messages', array());
		return $messages;
	}

	function showAdminMessages(){
		$messages = $this->getAdminMessages();
		$icons = array('success' => '<i class="fa fa-check"></i>', 'warning' => '<i class="fa fa-exclamation-triangle"></i>', 'error' => '<i class="fa fa-dot-circle-o"></i>');
		
		if(is_array($messages) && count($messages)){
			foreach($messages as $message){
			    array_push($this->messages, array('type' => $message['type'], 'text' => $icons[$message['type']] .' '. esc_html__($message['text'], 'delucks-seo') . '<button type="button" class="close pull-right dpc_remove_message" data-hash="'.$message['hash'].'" aria-label="Close"><span aria-hidden="true">&times;</span></button>'));
			}
		}
	}

	function indexationCheck(){
		if($this->isDpcPage() && get_option('blog_public') == "0"){
		    array_push($this->messages, array('type' => 'error', 'text' => wp_kses(__('You do not allow search engines to index your website. Please deactivate this function in the <a href="/wp-admin/options-reading.php">reading settings</a> of your website, to allow Google & co to index your website.', 'delucks-seo'), $this->text_allowed_html)));
		}
	}

	function adminNotices(){
		if(current_user_can('manage_options')){
			if(version_compare($this->version, get_option('dpc_version', 999)) > 0){
			    echo '<div id="message" class="updated"><p>'.wp_kses(__('<b>DELUCKS SEO Plugin</b> has been updated.', 'delucks-seo'), $this->text_allowed_html).'</p>';
				if($this->oLicense->products['professional']['status'] == true){
					//Message for professional users only
				}
				echo '</div>';
			}

			if(get_option('dpc_version', 999) != 999){
				if(version_compare('1.5.1', get_option('dpc_version', 999)) > 0){
				    echo '<div id="message" class="updated"><p>'.wp_kses(__('<b>DELUCKS SEO Plugin</b> We moved the keyword field from the text optimization back to the meta data module. Please re-activate it there to be able to use it.', 'delucks-seo'), $this->text_allowed_html).'</p></div>';
				}
				
				if(version_compare('1.5.4', get_option('dpc_version', 999)) > 0){
				    $this->oLicense->checkLicense();
				}
				
				if(version_compare('2.0', get_option('dpc_version', 999)) > 0){
				    $this->update2Changes();
				}
				
				if(version_compare('2.0', get_option('dpc_version', 999)) > 0){
				    $this->update2Changes();
				}
				
				if(version_compare('2.1.9', get_option('dpc_version', 999)) > 0){
				    $this->update2_1_9Changes();
				}

				if(version_compare('2.5', get_option('dpc_version', 999)) > 0){
				    $this->update2_5Changes();
				}
			}
		}

		update_option('dpc_version', $this->version);
	}

    /**
     * getModuleList() returns an array with a list of all valid modules including their subModules
     * in the modules directory.
     *
     * @return array
     */
    function getModuleList(){
		$moduleNames = array();
		foreach(scandir(DPC_PLUGIN_MODULES_DIR) as $filename){
			$dirName = DPC_PLUGIN_MODULES_DIR . $filename;
			if(is_dir($dirName) && is_readable($dirName) && file_exists($dirName . '/module.class.php') && !in_array($filename, array('.', '..'))){
				$moduleNames[$filename] = array();
				foreach(scandir($dirName) as $subModuleName){
					if(is_dir($dirName . '/' . $subModuleName) && is_readable($dirName . '/' . $subModuleName) && file_exists($dirName . '/' . $subModuleName . '/module.class.php') && !in_array($subModuleName, array('.', '..'))){
						if(!preg_match('/^_(.*)/', $subModuleName)){
							$moduleNames[$filename][] = $subModuleName;
						}
					}
				}
			}
		}

		$sortorder					= array();
		$sortorder['professional']	= array('externallinks' => 0, 'jobs' => 1, 'microdata' => 2, 'rating' => 3, 'textopt' => 4, 'sharing' => 55, 'author' => 6);
		$sortorder['basic']	        = array('metadata' => 0, 'sitemaps' => 1, 'statistics' => 2, 'urls' => 3, 'breadcrumbs' => 4); 

		//sort order
		foreach($moduleNames as $module => $submodules){
			if(isset($sortorder[$module])){
				$newsubmenu = array();
				foreach($submodules as $v){
				    $newsubmenu[$sortorder[$module][$v]] = $v;
				}
				ksort($newsubmenu);
				$moduleNames[$module] = $newsubmenu;
			}
		}

		return $moduleNames;
	}


	function checkProduct($key){
		return $this->oLicense->checkProduct($key);
	}

	function checkModule($moduleName){
		return $this->oLicense->checkModule($moduleName);
		$parts = explode('_', $moduleName);
		if(in_array($parts[0], array('basic', 'dpc'))){
			return true;
		}
		if(is_array($parts)){
			return $this->checkProduct($parts[0]);
		}
		return false;
	}

    /**
     * getModuleInstance() builds the correct classname from the moduleName and subModuleName, instantiates it,
     * saves the instance into an attribute and returns it.
     *
     * @param $moduleName The name of the module to instantiate
     * @param bool $subModuleName The name of the submodule to instantiate
     * @return bool|object Instance of the module object or false if the module class doesn't exist
     */
    function getModuleInstance($moduleName, $subModuleName = false){
		$moduleName = strtolower($moduleName);
		if($subModuleName !== false){
			$subModuleName = strtolower($subModuleName);
			if(isset($this->moduleInstances[$moduleName.'_'.$subModuleName])){
				return $this->moduleInstances[$moduleName.'_'.$subModuleName];
			}
			$moduleClassName = get_class() . '_Module_' . ucfirst($moduleName) . '_' . ucfirst($subModuleName);
			$moduleClassFile = DPC_PLUGIN_MODULES_DIR . str_replace('_', '/', strtolower($moduleName . '/' . $subModuleName . '/module.class.php'));
			if(file_exists($moduleClassFile) && is_readable($moduleClassFile)){
				include_once(DPC_PLUGIN_MODULES_DIR . 'interface.php');
				require_once($moduleClassFile);
				if(class_exists($moduleClassName)){
					$this->moduleInstances[$moduleName.'_'.$subModuleName] = new $moduleClassName($this);
					return $this->moduleInstances[$moduleName.'_'.$subModuleName];
				}
			}
		} else {
			if(isset($this->moduleInstances[$moduleName])){
				return $this->moduleInstances[$moduleName];
			}
			$moduleClassName = get_class() . '_Module_' . ucfirst($moduleName);
			$moduleClassFile = DPC_PLUGIN_MODULES_DIR . str_replace('_', '/', strtolower($moduleName . '/module.class.php'));
			if(file_exists($moduleClassFile) && is_readable($moduleClassFile)){
				require_once($moduleClassFile);
				if(class_exists($moduleClassName)){
					if($this->checkModule($moduleName)){
						$this->moduleInstances[$moduleName] = new $moduleClassName($this);
						return $this->moduleInstances[$moduleName];
					}
				}
			}
		}
		return false;
	}

    /**
     * triggerModuleMethod() calls the given method of the given module with the given parameters and returns
     * the return value of it.
     *
     * @param $module Modulename or Instance to trigger the method on
     * @param $method Name of the method to trigger
     * @param null $params Optional params to pass to the method
     * @return bool|mixed Returns false if the method doesn't exist or the return value of the method itself
     */
    function triggerModuleMethod($module, $method, $params = null){
   		if(!is_object($module) && is_string($module)){
			$module = $this->getModuleInstance($module);
		}

		if(is_object($module)){
			if(method_exists($module, $method)){
				$params = (is_null($params) ? array() : $params);
				return call_user_func_array(array($module, $method), $params);
			}
		}
		return false;
	}

    /**
     * initModules() initializes all valid modules and submodules saving them in attributes for later use.
     */
    function initModules(){
		$this->moduleList = $this->getModuleList();
		foreach($this->moduleList as $moduleName => $subModuleNames){

			if(is_object($this->getModuleInstance($moduleName))){
				foreach($subModuleNames as $subModuleName){
					if(is_object($subModule = $this->getModuleInstance($moduleName, $subModuleName))){
						if($this->checkModule($moduleName . '_' . $subModuleName)){
							$this->moduleInstances[$moduleName.'_'.$subModuleName] = $subModule;
						}
					}
				}
			}	
		}
	}

    /**
     * setupPluginDirectory() creates a directory with the given chmod rights
     *
     * @param $dir Path of the directory to create
     * @param int $chmod Userrights of the directory to be created
     * @return bool Returns true if the directory already exists or the result of the mkdir function
     */
    function setupPluginDirectory($dir, $chmod = 0755){
		if(!file_exists($dir)){
			return mkdir($dir, $chmod, true);
		}
		return true;
	}

	/**
	 * getFilters() get a filter list for the given hook
	 *
	 * @param $hook hook we are looking for
	 * @return array the filter list
	 */
	function getFilters($hook = '') {
		global $wp_filter;
		if(empty($hook) || !isset($wp_filter[$hook])){
			return;
		}
		return $wp_filter[$hook];
	}


    /**
     * parseSettingsByXml() takes a XML settings file, loads the given section and instantiates an object of the
     * GUI Helper class to render the section in the WP backend. Sections in the XML file describe the backend forms,
     * pages etc
     *
     * @param $file Path to the XML settings file
     * @param $section Name of the section to load from the file (name attribute of a section tag)
     * @param $inEach if true, no new gui object will be created to hold loop variables
     * @return bool Returns true on successful display of the section and false if there was an error
     */
    function parseSettingsByXml($file, $section, $inEach = false, $skipWrapper = false, $checkExists = false, $settings = false){
		$this->lastSettingsFile = $file;
		if(is_readable($file)){
			$options 	= simplexml_load_file($file, 'SimpleXMLElement', LIBXML_NOCDATA);
			$module 	= $options['module'];
			$defaults   = array();

			foreach($options->xpath('//defaults') as $xml_defaults){
				$defaults[] = $xml_defaults;
			}

			foreach($options->xpath('//section') as $xml_section){
				if((string)$xml_section['name'] == (string)$section){
					if($checkExists){ return true; }
					$fields = $xml_section;
					break;
				}
			}

			if(!isset($fields) || !count($fields)){ return false; }


			if($this->guiSettings !== false){
				$settings = $this->guiSettings;
			} elseif($settings !== false){
				$this->guiSettings = $settings;
			} else {
				$settings = $this->getModuleSettings($module);
			}

			require_once(DPC_PLUGIN_HELPER_DIR . 'gui.class.php');
			$gui 					= (!$inEach ? new DPC_Helper_Gui($this) : $this->guiHelper);
			$gui->module 			= $module;
			$gui->defaults			= (isset($defaults) && count($defaults) ? $defaults : false);
			$gui->settings 			= $settings;
			$gui->sectionAttributes = $gui->replaceXmlAttributes($fields);
			$this->guiHelper 		= $gui;
			$gui->showSection($fields, true, $skipWrapper);
			return true;
		}
		return false;
	}

	/**
	 * parseSettingsByXml() takes a XML settings file, loads the given section and instantiates an object of the
	 * GUI Helper class to render the section in the WP backend. Sections in the XML file describe the backend forms,
	 * pages etc
	 *
	 * @param $file Path to the XML settings file
	 * @param $section Name of the section to load from the file (name attribute of a section tag)
	 * @param $inEach if true, no new gui object will be created to hold loop variables
	 * @return bool Returns true on successful display of the section and false if there was an error
	 */
	function new_parseSettingsByXml($file, $section, $inEach = false, $skipWrapper = false, $checkExists = false, $settings = false){
		$this->lastSettingsFile = $file;
		if(is_readable($file)){
			$options 	= simplexml_load_file($file, 'SimpleXMLElement', LIBXML_NOCDATA);
			$module 	= $options['module'];
			$defaults   = array();

			foreach($options->xpath('//defaults') as $xml_defaults){
				$defaults[] = $xml_defaults;
			}

			foreach($options->xpath('//section') as $xml_section){
				if((string)$xml_section['name'] == (string)$section){
					if($checkExists){ return true; }
					$fields = $xml_section;
					break;
				}
			}

			if(!isset($fields) || !count($fields)){ return false; }

			if($this->guiSettings !== false){
				$settings = $this->guiSettings;
			} elseif($settings !== false){
				$this->guiSettings = $settings;
			} else {
				$settings = $this->getModuleSettings($module);
			}

			require_once(DPC_PLUGIN_HELPER_DIR . 'gui.php');
			$gui 					= (!$inEach ? new DPC_Gui($this) : $this->guiHelper);
			$gui->module 			= $module;
			$gui->defaults			= (isset($defaults) && count($defaults) ? $defaults : false);
			$gui->settings 			= $settings;
			$gui->sectionAttributes = $gui->replaceXmlAttributes($fields);
			$this->guiHelper 		= $gui;
			$gui->showSection($fields, true, $skipWrapper);
			return true;
		}
		return false;
	}

	/**
	 * installDefaultSettingsByXml() takes a XML settings file, loads the given default values and saves them to the database
	 *
	 * @return bool Returns true on successful install default module settings and false if there was an error
	 */ 
	function installDefaultSettingsByXml(){

		
		if(!isset($_POST['dpc']['install_defaults'])){
			return false;
		}


		if ( ! wp_verify_nonce( $_POST['dpc']['save_nonce'], 'dpc-save-nonce' ) ) {
			die ( 'You are not allowed to do that.' );
		}

		foreach($_POST['dpc']['install_defaults'] as $moduleName => $file){
		    $moduleName   = sanitize_text_field($moduleName);
		    $file         = sanitize_text_field($file);

		    if(!file_exists($file)){
		        return false;
		    }

			$this->lastSettingsFile = $file;
			if(is_readable($file)){
				$options 	= simplexml_load_file($file, 'SimpleXMLElement', LIBXML_NOCDATA)->xpath('//defaults');
				$options	= $options[0];
				$settings	= array();

				if(!count($options)){ return false; }
				foreach($options as $var){
					if(property_exists($var->attributes(), 'function')){
						switch($var->attributes()->function->__toString()){
							case 'explode':
								$settings[$var->attributes()->name->__toString()] = explode(',', $var->attributes()->value->__toString());
								break;
						}
					} else {
						$settings[(string) $var->attributes()->name] = (string) $var->attributes()->value;
					}
				}

				$module = $this->getModuleInstance($moduleName);
			    if(method_exists($module, 'beforeInstallDefaultSettings')){
			        $settings = $this->triggerModuleMethod($module, 'beforeInstallDefaultSettings', array($settings, $module->settings, $moduleName));
			    }

			    $settings['dpc_status_'.$moduleName] = intval($_POST['dpc'][$moduleName]['dpc_status_'.$moduleName]);
				update_option('dpc_'.$moduleName, $settings);
				update_option('dpc_status_' . $moduleName, intval($_POST['dpc'][$moduleName]['dpc_status_'.$moduleName]));

				if(isset($_POST['dpc'][$moduleName])){
					unset($_POST['dpc'][$moduleName]);
				}
				return true;
			}
		}

		return false;
	}

	/**
	 * isEditPage() check whether we are on a page with post data or not
	 *
	 * @return bool
	 */
	function isEditPage($new_edit = null){
		global $pagenow;
		if (!is_admin()) return false;

		add_action( 'enqueue_block_editor_assets', array($this, 'dpc_gutenberg_toolbar') );

		if(!isset($pagenow) || empty($pagenow)){
			// phpcs:ignore WordPress.Security.NonceVerification.Recommended
			if(!strpos($_SERVER['REQUEST_URI'], 'post.php') === false || !strpos($_SERVER['REQUEST_URI'], 'post-new.php') === false){
				// phpcs:ignore WordPress.Security.NonceVerification.Recommended
				if(isset($_GET['action']) && ($_GET['action'] == 'edit' || $_GET['action'] == 'new')){
					return true;
				}
			}
		}

		if($new_edit == "edit"){
			return in_array($pagenow, array('post.php'));
		} elseif($new_edit == "new"){
			return in_array($pagenow, array('post-new.php'));
		} else {
			return in_array($pagenow, array('post.php', 'post-new.php'));
		}
	}

	/**
	 * Button for Gutenberg Toolbar
	 */
	function dpc_gutenberg_toolbar(){
		$id = get_the_id();

		wp_enqueue_script('dpc-gutenberg-functions', plugins_url( 'assets/dpc/js/gutenberg-functions.js', __FILE__ ), array('dpc-modal-trigger'), NULL, true);
		wp_localize_script('dpc-gutenberg-functions', 'dpc_gutenberg_functions', array('dpc_button_url' => admin_url( 'admin-ajax.php?action=dpc_modal_show&id=' . $id . '&tab=meta')));
	}

	/**
	 * isDpcSettingsPage() check whether we are on a plugins settings page or not
	 *
	 * @return bool
	 */
	function isDpcSettingsPage(){
		$screen = get_current_screen();
		if($screen->id !== 'toplevel_page_dpc' && !preg_match('/^seo_page_dpc/', $screen->id)){
			return false;
		}

		return true;
	}


    /**
     * adminHead() adds style- and script files to the page that will be used in the backend templates later.
     * Gets called on admin_head trigger.
     *
     * @return bool Returns false if this is not the right screen to add the scripts and styles
     */
    function adminHead(){
		if($this->isEditPage()){
			wp_enqueue_style('dpc-font-awesome', plugins_url( 'assets/font-awesome/css/font-awesome.min.css', __FILE__ ));
			wp_enqueue_style('dpc-edit', plugins_url( 'assets/dpc/css/edit.min.css', __FILE__ ));
		} elseif($this->isDpcSettingsPage()){
			if(function_exists('wp_enqueue_media')){
				wp_enqueue_media();
			} else { // older WP < 3.5
				wp_enqueue_script('media-upload');
				wp_enqueue_script('thickbox');
				wp_enqueue_style('thickbox');
			}

			wp_register_script('popper', plugins_url( 'assets/bootstrap/js/popper.min.js', DPC_FILE ), array('jquery'), NULL, true);


			wp_enqueue_style('bootstrap-grid-css', plugins_url( 'assets/bootstrap/css/bootstrap-grid.css', __FILE__ ));
			wp_enqueue_style('bootstrap-css', plugins_url( 'assets/bootstrap/css/bootstrap.css', __FILE__ ));
			wp_enqueue_script('bootstrap', plugins_url( 'assets/bootstrap/js/bootstrap.min.js', __FILE__ ), array('jquery', 'popper'), NULL, true);

			wp_enqueue_style('dpc-css', plugins_url( 'assets/dpc/css/style.min.css', __FILE__ ));
			wp_enqueue_style('dpc-settings', plugins_url( 'assets/dpc/css/settings.css', __FILE__ ));
			wp_enqueue_style('dpc-responsive', plugins_url( 'assets/dpc/css/style-responsive.css', __FILE__ ));

			wp_enqueue_script('dpc', plugins_url( 'assets/dpc/js/functions.js', __FILE__ ), array('jquery'), NULL, true);

			wp_localize_script('dpc', 'dpc_vars', array(
				'ajax_url' 		=> admin_url('admin-ajax.php'),
				'save_nonce' 	=> wp_create_nonce('dpc-save-nonce')
			));

			wp_localize_script('dpc', 'dpc_main_translator', $this->translatorJs);
			wp_enqueue_style('dpc-font-awesome', plugins_url( 'assets/font-awesome/css/font-awesome.min.css', __FILE__ ));
		}
	}

    /**
     * adminInit() adds the DELUCKS navigation point and cycles through every module to trigger the methods
     * for registering the module in the navigation
     */
    function adminInit(){
		add_menu_page(strtolower(DPC_HOOK), 'SEO', 'manage_options', strtolower(DPC_HOOK), array($this, 'adminSettings'), plugins_url( 'assets/img/dllogo.png', __FILE__ ), 98);
		foreach($this->moduleInstances as $moduleName => $module){
			if($this->oLicense->checkModule($moduleName)){
				$this->triggerModuleMethod($module, 'registerInAdminMenu');
			}
		}
	}

    /**
     * saveSettings() takes the data the user entered into the backend forms and saves them to the database.
     * Before and after the save-proce431ooks onSaveActionBefore and onSaveActionAfter will be triggered on each module.
     * If data is configured to have realnames in the settings XML file it will be saved with that name, otherwise the
     * option name includes the moduleName.
     * For every submodule of the active module the status will be set to true or false if an appropriate variable is
     * set in the data array to activate or deactivate that submodule.
     *
     * @return bool Returns false if POST value dpc_save_settings isn't set
     */
    function saveSettings(){
        if(isset($_POST['dpc']['install_defaults']) || !isset($_POST['dpc_save_settings']) || !isset($_POST['dpc_save_settings_nonce']) || !wp_verify_nonce($_POST['dpc_save_settings_nonce'], 'dpc_save_settings') || !current_user_can('manage_options')){
			return false;
		}

		foreach($this->moduleInstances as $moduleName => $module){
		    if($this->moduleIsActive($moduleName) || in_array($moduleName, array('dpc_wizard', 'dpc_importexport'))){
				$this->triggerModuleMethod($module, 'onSaveActionBefore');
			}
		}

		if(isset($_POST['dpc'])){
			if(isset($_POST['dpc']['realnames'])){
				require_once(ABSPATH . 'wp-includes/pluggable.php');
				foreach($_POST['dpc']['realnames'] as $k => $v){
				    update_option($k, sanitize_option($k, $v));
				}

				unset($_POST['dpc']['realnames']);
			}

			foreach($_POST['dpc'] as $moduleName => $moduleSettingFields){
				update_option('dpc_'.$moduleName, $moduleSettingFields);
			}
		}

		if(isset($_REQUEST['page']) && is_array($page = explode('-', $_REQUEST['page']))){
			if(key_exists($page[1], $this->moduleList)){
				foreach($this->moduleList[$page[1]] as $subModuleName){
					$moduleName = $page[1] . '_' . $subModuleName;
					$optionStatusName = 'dpc_status_' . $moduleName;
					if(isset($_POST['dpc'][$moduleName]['dpc_status_'.$moduleName]) && $_POST['dpc'][$moduleName]['dpc_status_'.$moduleName] == '1'){
						update_option($optionStatusName, true);
					} else {
						update_option($optionStatusName, false);
					}
				}
			}
		}

		foreach($this->moduleInstances as $moduleName => $module){
		    if($this->moduleIsActive($moduleName) || in_array($moduleName, array('dpc_wizard', 'dpc_importexport'))){
				$this->triggerModuleMethod($module, 'onSaveActionAfter');
			}
		}

		array_push($this->messages, array('type' => 'success', 'text' => esc_html__('Settings have been updated', 'delucks-seo') . '!'));
	}

    /**
     * getModuleSettings() returns the settings of the given module from the database.
     *
     * @param $moduleName The name of the module to read the settings from
     * @return mixed Returns the settings from the database or false if they aren't found
     */
    function getModuleSettings($moduleName, $default = array()){
		return get_option('dpc_'.$moduleName, $default);
	}

    /**
     * moduleIsActive() checks the option for the active-status of the given module and returns the status
     * or a default of false.
     *
     * @param $moduleName Name of the module to check
     * @return bool Returns active state from the database for the given module. If no status is found, false is returned.
     */
    function moduleIsActive($moduleName){
		if((bool)get_option('dpc_status_'.$moduleName, false) === false){
			return false;
		}
		return true;
	}

	/**
	 * registerAjaxAction() registers an ajax-action which can be called from backend or frontend
	 *
	 * @param $action Name of the action
	 * @param $function callback function
	 * @param $nopriv Defines wether not logged in users hav privileges to execute the registered action
	 * @return void
	 */
	function registerAjaxAction($action, $function, $nopriv = false){
		if($nopriv === true){
			add_action('wp_ajax_nopriv_'.$action, $function);
		} else {
			add_action('wp_ajax_'.$action, $function);
		}
	}

	function registerAjaxFrontendAction($action, $function){
		add_action('wp_ajax_nopriv_' . $action, $function);
	}

	function executeFrontendActions(){
		if(!is_admin()){
			#add_action('wp', array(&$this, 'hook_wp_frontend'));
			add_action('wp_head', array($this, 'printVersionComment') ,1000);
		}
	}

	function saveToggleState(){
		
		if ( ! wp_verify_nonce( $_POST['save_nonce'], 'dpc-save-nonce' ) ) {
			die ( 'You are not allowed to do that.' );
		}

		if(get_current_user_id()){
		    update_user_meta(get_current_user_id(), 'dpc_toggle_state_'.sanitize_text_field($_POST['module']), intval($_POST['status']));
		}

		die();
	}

	function executeBackendActions(){
		add_action('load-edit.php', array(&$this, 'hook_load_edit_php'));
	}

	function check($postId = false){
		$objectId = $postId ? $postId : (!is_null($this->objectId) ? $this->objectId : false);
		if($objectId == false || get_post_type($objectId) == 'revision' || get_post_type($objectId) == 'auto-draft' || (defined( 'DOING_AUTOSAVE' ) && DOING_AUTOSAVE) || !is_object($this->log)){
			return false;
		}

		$this->log->clearPostLog($objectId);
		foreach($this->moduleInstances as $module){
			if(method_exists($module, 'check')){
				$module->check($objectId);
			}
		}
	}

	function replaceTextLinks($matches){
		unset($matches[0]);
		if(count($matches) !== 6){
			return '<b>[LINK SYNTAX ERROR]</b>';
		}

		return '<a target="'.$matches[2].'" href="'.$matches[4].'">'.$matches[6].'</a>';
	}

	function getText($text){
		$text = preg_replace_callback('~\[LINK.*(target):(.*)\|(href):(.*)\|(text):(.*)\]~iU', array($this, 'replaceTextLinks'), $text);

		if (func_num_args() <= 1){
		    return esc_html__($text, 'delucks-seo');
		}

		$args = func_get_args();
		array_shift($args);
		if(is_array($args[0])){
			$args = $args[0];
		} elseif($args[0] === false){
		    return esc_html__($text, 'delucks-seo');
		}

		return vsprintf(esc_html__($text, 'delucks-seo'), $args);
	}

	function printScripts(){
		$lang = $this->oFunctions->getPostLang(get_the_ID(), false);
		$lang = substr($lang, 0, (strpos($lang, '_') ? strpos($lang, '_') : 2));
		echo '<script type="text/javascript">'."\n";
		echo '	var dpc_lang = \'' . esc_js($lang) . "';\n";
		echo '	var dpc_current_language = \'' . esc_js($this->oFunctions->getUserLang()) . "';\n";
		echo '	var dpc_plugin_url = \'' . esc_url(plugins_url('delucks-seo')) . "';\n";
		#echo '	var dpc_post_lang = \'' . esc_js($this->oFunctions->getPostLang()) . "';\n";
		echo '</script>'."\n";
	}

	/*
	function hook_wp_frontend(){
		global $post;
		$newcount = 0;
		if(is_object($post) && property_exists($post, 'ID') && get_post($post->ID)) {
			$count = get_post_meta($post->ID, '_dpc-hits', true);
			$newcount = $count + 1; //TODO: check
			update_post_meta($post->ID, '_dpc-hits', $newcount);
		}
	}
	*/

	function printVersionComment(){
		if ( is_feed() ) return;
		$info  = "\n\n<!-- ";
		$info .= "Optimized by DELUCKS SEO Plugin for WordPress Version: ".DPC_VERSION;
		$info .= " -->\n\n";
		echo $info;
	}

	function adminSettings(){ }

	function hook_admin_init(){
		global $pagenow;
		if($pagenow == 'admin.php'){
			// phpcs:ignore WordPress.Security.NonceVerification.Recommended
			if(isset($_GET['page']) && $_GET['page'] == 'dpc'){
				wp_redirect(admin_url('admin.php?page=dpc-dpc'), 302);
			}
		}
	}

	function hook_load_edit_php(){
		wp_enqueue_style('dpc-outer-style', plugins_url( 'assets/css/outer_dpc.css', __FILE__ ));
	}

	function adminMenuOrder($menu_ord){
		global $submenu;
		$newmenu = array();

		if(isset($submenu['dpc'])){
			foreach($submenu['dpc'] as $v){
				switch($v[2]){
					case 'dpc-dpc':
						$newmenu[0] = $v;
						break;
					case 'dpc-basic':
						$newmenu[1] = $v;
						break;
					case 'dpc-professional':
						$newmenu[2] = $v;
						break;
					case 'dpc-suite':
						$newmenu[3] = $v;
						break;
					case 'dpc-bulkmanager':
						$newmenu[4] = $v;
						break;
					case 'dpc-licenses':
						$newmenu[5] = $v;
						break;
					case 'dpc-seo-plugin-licenses':
						$newmenu[6] = $v;
						break;
				}
			}
			ksort($newmenu);
			$submenu['dpc'] = $newmenu;
		}
		return $menu_ord;
	}

	/* cleanup the settings once for the case that javascript have been injected remotely */
	function update2_1_9Changes(){
	    require_once(DPC_PLUGIN_HELPER_DIR . 'run_once.class.php');
	    $run_once = new run_once();
	    if($run_once->run('delucks-seo-update-2-1-9_2019-09-26')){
            foreach(array('basic_metadata', 'basic_sitemaps', 'basic_urls') as $moduleName){
                $settings = wp_json_encode($this->getModuleSettings($moduleName));
                $settings = preg_replace('/<script\b[^>]*>(.*?)<\/script>/is', '', stripcslashes(str_replace(array("\r\n","\r","\n"),'', str_replace('"">"', '""', $settings))));
                update_option('dpc_'.$moduleName, $this->oFunctions->object2array(json_decode($settings)));
            }
	    }
	}
	
	/* move breadcrumb to free modules */
	function update2_5Changes(){
	    require_once(DPC_PLUGIN_HELPER_DIR . 'run_once.class.php');
	    $run_once = new run_once();
	    if($run_once->run('delucks-seo-update-2-5')){
    	    $newSettings = $this->getModuleSettings('basic_breadcrumbs');
	        $oldSettings = $this->getModuleSettings('professional_breadcrumbs');
	        update_option('dpc_basic_breadcrumbs', $oldSettings);
	        if($this->moduleIsActive('professional_breadcrumbs')){
	            $this->activateModule('basic_breadcrumbs');
	        } else {
	            $this->deactivateModule('basic_breadcrumbs');
	        }
	        //$this->addAdminMessage('warning', wp_kses(__('The breadcrumbs module is now part of the free version.', 'delucks-seo'), $this->text_allowed_html));
	    }
	}

	function update2Changes(){
	    require_once(DPC_PLUGIN_HELPER_DIR . 'run_once.class.php');
	    $run_once = new run_once();

	    if($run_once->run('delucks-seo-update-2-2018-10-22')){
	        $meta_keys_rename = array(
	            array( 'delucks-seo-recheck',               '_delucks-seo-recheck' ),
	            array( 'delucks-seo-status',                '_delucks-seo-status' ),

	            array( 'dpc-textopt-description',           '_dpc-meta-description' ),
	            array( 'dpc-meta-title',                    '_dpc-meta-title' ),
	            array( 'dpc-meta-og-title',                 '_dpc-meta-og-title' ),
	            array( 'dpc-meta-og-description',           '_dpc-meta-og-description' ),
	            array( 'dpc-meta-og-image',                 '_dpc-meta-og-image' ),
	            array( 'dpc-memo',                          '_dpc-memo' ),
	            array( 'dpc_keyword',                       '_dpc-keyword' ),
	            array( 'dpc_keyword_list',                  '_dpc-keyword-list' ),
	            array( 'dpc_hits',                          '_dpc-hits' ),

	            array( 'dpc_sharing_count_buffer',          '_dpc-sharing-count-buffer' ),
	            array( 'dpc_sharing_count_facebook',        '_dpc-sharing-count-facebook' ),
	            array( 'dpc_sharing_count_google',          '_dpc-sharing-count-google' ),
	            array( 'dpc_sharing_count_linkedin',        '_dpc-sharing-count-linkedin' ),
	            array( 'dpc_sharing_count_mail',            '_dpc-sharing-count-mail' ),
	            array( 'dpc_sharing_count_pinterest',       '_dpc-sharing-count-pinterest' ),
	            array( 'dpc_sharing_count_print',           '_dpc-sharing-count-print' ),
	            array( 'dpc_sharing_count_skype',           '_dpc-sharing-count-skype' ),
	            array( 'dpc_sharing_count_total',           '_dpc-sharing-count-total' ),
	            array( 'dpc_sharing_count_tumblr',          '_dpc-sharing-count-tumblr' ),
	            array( 'dpc_sharing_count_twitter',         '_dpc-sharing-count-twitter' ),
	            array( 'dpc_sharing_count_xing',            '_dpc-sharing-count-xing' ),
	            array( 'dpc_sharing_count_week',            '_dpc-sharing-count-week' ),

	            array( 'no_index',                          '_dpc-no-index' ),
	            array( 'no_follow',                         '_dpc-no-follow' ),

	            array( 'dpc-competitor-sites',              '_dpc-competitor-sites' ),
	            array( 'dpc-microdata-video',               '_dpc-microdata-video' ),
	            array( 'dpc-microdata-location',            '_dpc-microdata-location' ),
	            array( 'dpc-microdata-cwork',               '_dpc-microdata-cwork' ),
	            array( 'dpc-microdata-recipe',              '_dpc-microdata-recipe' ),
	            array( 'dpc-microdata-event',               '_dpc-microdata-event' ),
	            array( 'dpc-microdata-person',              '_dpc-microdata-person' ),
	            array( 'dpc-microdata-job',                 '_dpc-microdata-job' ),

	            array( 'dpc_sistrix_last_week',             '_dpc-sistrix-last-week' ),
	            array( 'dpc_sistrix_current_week',          '_dpc-sistrix-current-week' ),
	        );

	        $meta_keys_delete = array(
	            'dpc-metatitle',
	            'dpc-preview-title',
	            'dpc_rating_klicks_1.5',
	            'dpc_rating_klicks_2.5',
	            'dpc_rating_klicks_3.5',
	            'dpc_rating_klicks_4.5',
	            'dpc-dummy-text-field',
	            'dpc-post-messages',
	            'dpc-post-tasks',
	            'dpc-textopt-sites',
	            'dpc_externallinks_title',
	            'dpc_rating_sum',
	            '_dpc-rating-sum',
	            'dpc_rating_score',
	            '_dpc-rating-score',
	            'dpc_rating_klicks_total',
	            '_dpc-rating-clicks-total',
	            'dpc_rating_klicks_1',
	            '_dpc-rating-clicks-1',
	            'dpc_rating_klicks_2',
	            '_dpc-rating-clicks-2',
	            'dpc_rating_klicks_3',
	            '_dpc-rating-clicks-3',
	            'dpc_rating_klicks_4',
	            '_dpc-rating-clicks-4',
	            'dpc_rating_klicks_5',
	            '_dpc-rating-clicks-5'
	        );

	        //rename meta keys
	        foreach ($meta_keys_rename as $k) {
	            $posts = get_posts('numberposts=-1&post_type=any&post_status=any&meta_key='.$k[0]);
	            if(is_array($posts) && count($posts)){
	                foreach($posts as $post){
	                    $meta = get_post_meta( $post->ID, $k[0], true );
	                    if ( $meta ) {
	                        update_post_meta($post->ID, $k[1], $meta );
	                    }
	                    delete_post_meta($post->ID, $k[0] );
	                }
	            }
	            delete_post_meta_by_key($k[0]);
	        }

	        //delete useless meta keys
	        foreach ($meta_keys_delete as $key) {
	            $posts = get_posts('numberposts=-1&post_type=any&post_status=any&meta_key='.$key);
	            if(is_array($posts) && count($posts)){
	                foreach($posts as $post){
	                    delete_post_meta($post->ID, $key);
	                }
	            }
	            delete_post_meta_by_key($key);
	        }

	        //check seo status first time
	        foreach(get_posts(array('fields' => 'ids', 'post_type' => 'any', 'post_status' => 'any', 'posts_per_page' => -1)) as $postId){
	            $metaKeyword     = get_post_meta($postId, '_dpc-keyword', true);
	            $metaDescription = get_post_meta($postId, '_dpc-meta-description', true);
	            $metaTitle       = get_post_meta($postId, '_dpc-meta-title', true);

	            if(strlen($metaKeyword) || strlen($metaDescription) || strlen($metaTitle)){
	                $statusCode = 2;
	            } else {
	                $statusCode = 1;
	            }

	            $status = get_post_meta($postId, '_delucks-seo-status', true);
	            $status = is_array($status) ? $status : array();
	            $status['meta'] = $statusCode;
	            update_post_meta($postId, '_delucks-seo-status', $status);
	        }
	    }
	}

	function activateModule($moduleName){
	    if(current_user_can('manage_options') && is_admin() && is_user_logged_in()){
    	    $settings = $this->getModuleSettings($moduleName);
    	    $settings['dpc_status_'.$moduleName] = 1;
    	    update_option('dpc_'.$moduleName, $settings);
    	    update_option('dpc_status_' . $moduleName, 1);
	    }
	}
	
	function deactivateModule($moduleName){
	    if(current_user_can('manage_options') && is_admin() && is_user_logged_in()){
	        $settings = $this->getModuleSettings($moduleName);
	        $settings['dpc_status_'.$moduleName] = 0;
	        update_option('dpc_'.$moduleName, $settings);
	        update_option('dpc_status_' . $moduleName, 0);
	    }
	}

	function activateAllModules(){
	    if(current_user_can('manage_options') && is_admin() && is_user_logged_in()){
	        if(count($moduleList = $this->getModuleList())){
	            if(isset($moduleList['dpc'])){ unset($moduleList['dpc']); }
	            foreach($moduleList as $groupname => $modules){
	                foreach($modules as $module){
	                    $this->activateModule($groupname."_".$module);
	                }
	            }
	        }
	    }
	}



}

$plugin_dpc = new DPC();