<?php

class DPC_Module_Basic_Breadcrumbs implements DPC_Module_Interface{

	var $dpc					= null;
	var $isSubmodule			= true;
	var $moduleDirectory 		= 'basic/';
	var $moduleTitle 			= '';
	var $optionsHook			= 'dpc-basic-breadcrumbs';
	var $settingsFile			= false;
	var $settings				= array();
	var $defaultSeperator		= ' &raquo; ';
	var $defaultHomePrefix		= '';
	var $defaultArchivePrefix	= '';
	var $defaultSearchPrefix	= '';
	var $default404Prefix		= '';
	var $breacrumbChildIndex	= '';
	var $items                  = array();

	function __construct(&$dpc){
		$this->settingsFile 		= dirname(__FILE__) . '/settings.xml';
		$this->dpc 					= $dpc;
		$this->moduleTitle			= esc_html__('Breadcrumbs', 'delucks-seo');
		$this->settings 			= $this->dpc->getModuleSettings('basic_breadcrumbs');
		$this->defaultHomePrefix	= esc_html__('Frontpage', 'delucks-seo');
		$this->defaultArchivePrefix	= esc_html__('Archive for', 'delucks-seo');
		$this->defaultSearchPrefix	= esc_html__('You searched for', 'delucks-seo').':';
		$this->default404Prefix		= esc_html__('Page not found', 'delucks-seo');


		echo $this->getJsonString();
	}

	function adminSettings(){
		$this->dpc->parseSettingsByXml($this->settingsFile, 'general');
	}

	public function adminHead(){
	    wp_enqueue_style('dpc-basic-breadcrumbs-backend', plugins_url( 'assets/css/backend.css', __FILE__ ));
	    wp_enqueue_style('dpc-basic-breadcrumbs-settings', plugins_url( 'assets/css/settings.css', __FILE__ ));
	}

	public function adminSettingsHead(){}

	public function executeBackendActions(){}

	function executeFrontendActions(){
		add_shortcode( 'dpc_breadcrumbs', array($this, 'dpc_breadcrumbs') );
	}

	function registerWidgets(){
		add_action( 'widgets_init', array(&$this, 'hook_widgets_init'));
	}

	function hook_widgets_init(){
		require_once( 'widgets/breadcrumbs.php' );
		register_widget( 'DelucksBreadcrumbs' );
	}


	function getJsonString(){
	    if(!count($this->items)){
	        return '';
	    }

	    $items[] = array('name' => 'Item 1', 'url' => 'http://www.eins.de');
	    $items[] = array('name' => 'Item 2', 'url' => 'http://www.zwei.de');
	    $items[] = array('name' => 'Item 3', 'url' => 'http://www.drei.de');

	    $position = 0;

	    $str  = '<script type="application/ld+json">';
	    $str .= '{';
	    $str .= '"@context": "https://schema.org",';
	    $str .= '"@type": "BreadcrumbList",';
	    $str .= '"itemListElement": [';
		
		
	    foreach($this->items as $item){
	        $str .= '{';
	        $str .= '"@type": "ListItem",';
	        $str .= '"position": '.++$position.',';
	        $str .= '"name": "'.htmlspecialchars($item['name']).'",';
	        $str .= '"item": "'.esc_url($item['url']).'"';
	        $str .= '},';
	    }

	    $str = trim($str, ',');

	    $str .= ']';
	    $str .= '}';
	    $str .= '</script>';
	    return $str;
	}

	function getCategoryParents($id, $link = false, $separator = '/', $nicename = false, $visited = array()){
		global $wpdb, $wp_locale, $product, $post, $wp_query;

	    $chain 	= '';
	    $parent = get_category($id); //&get_category($id)

	    if(is_wp_error($parent)){
	        return $parent;
	    }

	    if($nicename){
	    	$name = $parent->slug;
	    }else{
	        $name = $parent->name;
	    }

	    if($parent->parent && ($parent->parent != $parent->term_id) && !in_array($parent->parent, $visited)){
	        $visited[] = $parent->parent;
	        $chain .= $this->getCategoryParents( $parent->parent, $link, $separator, $nicename, $visited );
	    }

	    if($link){
	        $this->items[] = array(
	            'name' => esc_attr($name),
	            'url'  => get_category_link($parent->term_id)
	        );
	    	$chain .= '	<div class="dpc-child dpc-category dpc-child-'.$this->breacrumbChildIndex++.'">
	    					<a href="' . get_category_link( $parent->term_id ) . '" title="' . esc_attr( sprintf( esc_html__( "View all posts in %s" ), $parent->name ) ) . '">
	    						<span>'.$name.'</span>
	    					</a>
	    				</div>' . $separator;

	   	}else{
	        $chain .= $name.$separator;
	    }

	    return $chain;
	}

	function the_breadcrumb(){
	    wp_enqueue_style('dpc-basic-breadcrumbs-frontend', plugins_url( 'assets/css/frontend.css', __FILE__ ), array(), DPC_VERSION);
		global $wpdb, $wp_locale, $product, $post, $wp_query, $wp;
		if(is_object($wp_query)){
			$m			= $wp_query->get('m');
	        $year		= $wp_query->get('year');
	        $monthnum	= $wp_query->get('monthnum');
	        $day			= $wp_query->get('day');
	        $search		= $wp_query->get('s');
		}

		$this->breacrumbChildIndex = 0;

		// If no language or no seperator specified, use default seperator
		if(!isset($this->settings[$this->dpc->oFunctions->getUserLang()]) || !isset($this->settings[$this->dpc->oFunctions->getUserLang()]['seperator'])){
			$this->settings[$this->dpc->oFunctions->getUserLang()]['seperator'] = $this->defaultSeperator;
		}

		$this->settings[$this->dpc->oFunctions->getUserLang()]['seperator'] = '<span class="dpc-seperator">&nbsp;'.$this->settings[$this->dpc->oFunctions->getUserLang()]['seperator'].'&nbsp;</span>';

		// If no language or no prefeixBefore specified, leave blank
		if(!isset($this->settings[$this->dpc->oFunctions->getUserLang()]) || !isset($this->settings[$this->dpc->oFunctions->getUserLang()]['prefixBefore'])){
			$this->settings[$this->dpc->oFunctions->getUserLang()]['prefixBefore'] = '';
		}
		// If no language or no home text specified, use default home text
		if(!isset($this->settings[$this->dpc->oFunctions->getUserLang()]) || !isset($this->settings[$this->dpc->oFunctions->getUserLang()]['homeText'])){
			$this->settings[$this->dpc->oFunctions->getUserLang()]['homeText'] = $this->defaultHomePrefix;
		}

		$trail = '<div class="dpc-breadcrumbs">';
		$trail .= '<span class="pre-breadcrumbs">'.(strlen($this->settings[$this->dpc->oFunctions->getUserLang()]['prefixBefore']) ? $this->settings[$this->dpc->oFunctions->getUserLang()]['prefixBefore'].' ' : '').'</span>';

		if(!isset($this->settings['homeicon'])){
			$trail .= '	<div>
							<a class="dpc-breadcrumbs-home dpc-child-'.$this->breacrumbChildIndex++.'" href="'.$this->dpc->oFunctions->getMultilanguageHomeUrl().'">
								<span>'.$this->settings[$this->dpc->oFunctions->getUserLang()]['homeText'].'</span>
							</a>';
		} else{
			$trail .= '	<div>
							<a class="dpc-breadcrumbs-home dpc-child-'.$this->breacrumbChildIndex++.'" href="'.$this->dpc->oFunctions->getMultilanguageHomeUrl().'"><i class="delucks-icon-'.(!empty($this->settings['home_icon_symbol']) ? $this->settings['home_icon_symbol'] : 'home-2').'"></i></a>';
		}

		//first entry
		$this->items[] = array(
		    'name' => $this->settings[$this->dpc->oFunctions->getUserLang()]['homeText'],
		    'url'  => $this->dpc->oFunctions->getMultilanguageHomeUrl()
		);

		if ( !is_front_page() && is_home() ) { /** Check if is BLOG home archive **/
		    $blogCategory = get_the_category();
		    if($blogPageId = get_option( 'page_for_posts' )){
		        $blogPage = get_post($blogPageId);
		        if(is_object($blogPage)){
		            $this->items[] = array(
		                'name' => $blogPage->post_title,
		                'url'  => get_permalink($blogPage->ID)
		            );

		            $trail .= $this->settings[$this->dpc->oFunctions->getUserLang()]['seperator'].'
								<div class="dpc-child dpc-child-'.$this->breacrumbChildIndex++.'">
									<a href="' . get_permalink($blogPage->ID) . '"><span>'.$blogPage->post_title.'</span></a>
								</div>';
		        }
		    }

		} elseif(is_front_page()){ /** Check if is frontpage **/
			/**
			 *
			 * mach nix!
			 *
			 */
		} elseif (is_page()){ /** Check if it is a page **/
		    #$pid		= $post->ID;
		    $pid        = url_to_postid($wp->request);
			$skip 		= array('post','page');
			$bcarray 	= array();
			$pdata 		= get_post($pid);
			$bcarray[] 	= $pdata->post_title;

			while ($pdata->post_parent) {
				$pdata 		= get_post($pdata->post_parent);
				$this->items[] = array(
				    'name' => $pdata->post_title,
				    'url'  => get_permalink($pdata->ID)
				);

				$bcarray[] 	= '<span class="dpc-seperator">'.(strlen($this->settings[$this->dpc->oFunctions->getUserLang()]['seperator']) ? ' '.$this->settings[$this->dpc->oFunctions->getUserLang()]['seperator'].' ' : $this->defaultSeperator).'</span>'.'
								<div class="dpc-child dpc-child-{INDEX}">
									<a href="'.get_permalink($pdata->ID).'">
										<span>'.$pdata->post_title.'</span>
									</a>
								</div>';
			}

			$bcarray = array_reverse($bcarray);
			$numItems = count($bcarray);
			$i = 0;
			foreach($bcarray as $listitem){
				if(++$i === $numItems) {
				    $this->items[] = array(
				        'name' => $listitem,
				        'url'  => get_permalink()
				    );

					$trail .= '<span class="dpc-seperator">'.(strlen($this->settings[$this->dpc->oFunctions->getUserLang()]['seperator']) ? ' '.$this->settings[$this->dpc->oFunctions->getUserLang()]['seperator'].' ' : $this->defaultSeperator).'</span>'.
							  //'<div class="dpc-child dpc-child-'.$this->breacrumbChildIndex++.' dpc-child-item-'.$i.'" itemscope="" itemprop="child" itemtype="http://data-vocabulary.org/Breadcrumb">
							  	'<div class="dpc-child dpc-child-'.$this->breacrumbChildIndex++.'">
							  		<a class="dpc-child-item-link" href="'.get_permalink().'">
										<span class="dpc-child-item-title">'.$listitem.'</span>
									</a>
								</div>';
				}else{
					$trail .= str_replace("{INDEX}", $this->breacrumbChildIndex++, $listitem);
				}
			}

		} elseif(function_exists('is_shop') && is_shop()){ /* shop main page*/
		    $shop_page_id = wc_get_page_id( 'shop' );
		    $shop_page = get_post($shop_page_id);
		    $trail .= $this->settings[$this->dpc->oFunctions->getUserLang()]['seperator'].$shop_page->post_title;
		} elseif (is_single()){ /** Check if it is single view of blog post; CPTs down below!**/
			$pid	= $post->ID;
			$adata	= get_post($pid);

			/**
			 * Posts
			 */
			if(get_post_type() == 'post'){ /** Check if is single view of a blogpost **/

	    			if(isset($this->settings['blogArchivePage'])){
	    				$post_type 		= get_query_var( 'post' );
	    				$post_type_obj 	= get_post_type_object( 'post' );
	    
	    				$title = (strlen($this->settings[$this->dpc->oFunctions->getUserLang()]['blogArchivePageTitle']) ? $this->settings[$this->dpc->oFunctions->getUserLang()]['blogArchivePageTitle'] : $post_type_obj->labels->name );
	    
	    				$this->items[] = array(
	    				    'name' => $title,
	    				    'url'  => get_post_type_archive_link('post')
	    				);
	    
	    				$trail .= $this->settings[$this->dpc->oFunctions->getUserLang()]['seperator'].
	    							'<div class="dpc-child dpc-child-'.$this->breacrumbChildIndex++.'">
	    								<a href="' . get_post_type_archive_link( 'post' ) . '"><span>' . $title . '</span></a>
	    							</div>';
	    			}

				switch ($this->settings['showPost']) {
				  case 'postCategory':
				  	$pdata = get_the_category($pid);
					if(!empty($pdata)){
						$data 	 = $this->getCategoryParents($pdata[0]->cat_ID, TRUE, $this->settings[$this->dpc->oFunctions->getUserLang()]['seperator']);
						$trail	.= $this->settings[$this->dpc->oFunctions->getUserLang()]['seperator'].substr($data, 0, strlen($data) -(strlen($this->settings[$this->dpc->oFunctions->getUserLang()]['seperator'])));
					}
				    break;
				  case 'postTag':
					if($tags = wp_get_object_terms( $adata->ID, 'post_tag')){
						$main_tag = $tags[0];
						$this->items[] = array(
						    'name' => $main_tag->name,
						    'url'  => get_term_link($main_tag->slug, 'post_tag')
						);

						$trail .= $this->settings[$this->dpc->oFunctions->getUserLang()]['seperator'].'<div class="dpc-child dpc-child-'.$this->breacrumbChildIndex++.'"><a href="' . get_term_link($main_tag->slug, 'post_tag') . '"><span> ' . $main_tag->name . '</span></a></div>';
					}
				    break;
				  case 'postFormat':
					$format = get_post_format($adata->ID);
					if(!$format){ break; }
					
					$this->items[] = array(
					    'name' => ucwords($format),
					    'url'  => get_post_format_link($format)
					);

					$trail .= $this->settings[$this->dpc->oFunctions->getUserLang()]['seperator'].sprintf( '<div class="dpc-child dpc-child-'.$this->breacrumbChildIndex++.'"><a itemprop="url" href="%1$s"><span>%2$s</span></a></div>', get_post_format_link($format), ucwords($format));
				    break;
				  default:
				    break;
				}
			} elseif(get_post_type() != 'post'){ /** Check if is single view of any CPT **/
				$skip = array('post','page');
				foreach(get_post_types() as $postType){
					if(get_post_type() == $postType && !in_array($postType, $skip)){
						$type = 'showCpt_'.$postType;
						
						if(isset($this->settings['blogArchivePage'])){
			    				$post_type 		= get_query_var( $postType );
			    				$post_type_obj 	= get_post_type_object( $postType );
								
			    				$title = (strlen($this->settings[$this->dpc->oFunctions->getUserLang()]['blogArchivePageTitle']) ? $this->settings[$this->dpc->oFunctions->getUserLang()]['blogArchivePageTitle'] : $post_type_obj->labels->name );
			    
			    				$this->items[] = array(
			    				    'name' => $title,
			    				    'url'  => get_post_type_archive_link($postType)
			    				);
			    				
			    				$trail .= $this->settings[$this->dpc->oFunctions->getUserLang()]['seperator'].
			    							'<div class="dpc-child dpc-child-'.$this->breacrumbChildIndex++.'">
			    								<a href="' . get_post_type_archive_link( $postType ) . '"><span>' . $title . '</span></a>
			    							</div>';
			    			}

						if(isset($this->settings[$type])){
							
							switch ($this->settings[$type]){
							  	case 'category':
							  		$pdata = get_the_category($pid);
									if(!empty($pdata)){
									
										$data 	 = $this->getCategoryParents($pdata[0]->cat_ID, TRUE, $this->settings[$this->dpc->oFunctions->getUserLang()]['seperator']);
										$trail	.= $this->settings[$this->dpc->oFunctions->getUserLang()]['seperator'].substr($data, 0, strlen($data) -(strlen($this->settings[$this->dpc->oFunctions->getUserLang()]['seperator'])));
									}
							    		break;
							  	case 'tag':
									if($tags = wp_get_object_terms( $adata->ID, 'post_tag')){
										$main_tag = $tags[0];
										$this->items[] = array(
									    		'name' => $main_tag->name,
									    		'url'  => get_term_link($main_tag->slug, 'post_tag')
										);

										$trail .= $this->settings[$this->dpc->oFunctions->getUserLang()]['seperator'].'<div class="dpc-child dpc-child-'.$this->breacrumbChildIndex++.'" href="' . get_term_link($main_tag->slug, 'post_tag') . '"><span> ' . $main_tag->name . '</span></a></div>';
									}
						    		break;
								default:
							    break;
							}
						}
					}
				}
			}

			/**
			 * Woocommerce
			 */
			if(get_post_type() == 'product'){ /** Check if is single view of a woocommerce product page **/
				switch ($this->settings['showWoo']) {
				  case 'wooCategory':
				   	if (is_product()) {
						if($terms = wc_get_product_terms(get_the_ID(), 'product_cat', array( 'orderby' => 'parent', 'order' => 'DESC' ))) {
							$main_term = $terms[0];
							$ancestors = get_ancestors( $main_term->term_id, 'product_cat' );
							$ancestors = array_reverse( $ancestors );

							$trail .= $this->settings[$this->dpc->oFunctions->getUserLang()]['seperator'];
							foreach ($ancestors as $ancestor) {
								$ancestor = get_term($ancestor, 'product_cat');
								if ( ! is_wp_error($ancestor) && $ancestor){
								    $this->items[] = array(
								        'name' => $ancestor->name,
								        'url'  => get_term_link($ancestor->slug, 'product_cat')
								    );

									$trail .= '<div class="dpc-child dpc-child-'.$this->breacrumbChildIndex++.'"><a href="' . get_term_link($ancestor->slug, 'product_cat') . '"><span> ' . $ancestor->name . '</span></a></div>'.$this->settings[$this->dpc->oFunctions->getUserLang()]['seperator'];
								}
							}
							$this->items[] = array(
							    'name' => $main_term->name,
							    'url'  => get_term_link($main_term->slug, 'product_cat')
							);

							$trail .= '<div class="dpc-child dpc-child-'.$this->breacrumbChildIndex++.'"><a href="' . get_term_link($main_term->slug, 'product_cat') . '"><span> ' . $main_term->name . '</span></a></div>';
						}
					}
				    break;
				  case 'wooTag':
					if (is_product()) {
						if($tags = wp_get_object_terms( get_the_ID(), 'product_tag')){
							$main_tag = $tags[0];

							$this->items[] = array(
							    'name' => $main_tag->name,
							    'url'  => get_term_link($main_tag->slug, 'product_tag')
							);

							$trail .= $this->settings[$this->dpc->oFunctions->getUserLang()]['seperator'].'<div class="dpc-child dpc-child-'.$this->breacrumbChildIndex++.'"><a href="' . get_term_link($main_tag->slug, 'product_tag') . '"><span> ' . $main_tag->name . '</span></a></div>';
						}
					}
				    break;
				  case 'wooShipping':
				    if (is_product()) {
				    	if($classes = get_the_terms( get_the_ID(), 'product_shipping_class' )){
							foreach ($classes as $class) {
							    $this->items[] = array(
							        'name' => $class->name,
							        'url'  => get_term_link($class->slug, 'product_shipping_class')
							    );

								$trail .= $this->settings[$this->dpc->oFunctions->getUserLang()]['seperator'].'<div class="dpc-child dpc-child-'.$this->breacrumbChildIndex++.'"><a href="' . get_term_link($class->slug, 'product_shipping_class') . '"><span> ' . $class->name . '</span></a></div>';
							}
						}
					}
				    break;
				  default:
				    break;
				}
			}



			$this->items[] = array(
			    'name' => $adata->post_title,
			    'url'  => get_permalink()
			);

			/**
			 * Show post name
			 */
			$trail .= $this->settings[$this->dpc->oFunctions->getUserLang()]['seperator'].'<div class="dpc-child dpc-child-'.$this->breacrumbChildIndex++.'">
						<a href="' . get_permalink() . '">
							<span>'.$adata->post_title.'</span>
						</a></div>'."\n";
						
		} elseif (is_category() || is_tag()){ /** END OF SINGLE VIEW - Check if is category or tag archive **/
			$cat_id = get_queried_object_id();
			$category = get_category($cat_id);
			$parent_id = $category->category_parent;
			
			// $this->items[] = array(
			    // 'name' => single_cat_title( '', false),
			    // 'url'  =>  get_category_link( $cat_id )
			// );
			
			if($parent_id){
				$ancestors = get_category_parents($parent_id, true, $this->settings[$this->dpc->oFunctions->getUserLang()]['seperator']);
				$trail .= 	$this->settings[$this->dpc->oFunctions->getUserLang()]['seperator'].
							$ancestors.(strlen($this->settings[$this->dpc->oFunctions->getUserLang()]['prefixArchive']) ? $this->settings[$this->dpc->oFunctions->getUserLang()]['prefixArchive'] : $this->defaultArchivePrefix.' ' ).single_cat_title( '', false);
			}else{
				$trail .= $this->settings[$this->dpc->oFunctions->getUserLang()]['seperator'].(strlen($this->settings[$this->dpc->oFunctions->getUserLang()]['prefixArchive']) ? $this->settings[$this->dpc->oFunctions->getUserLang()]['prefixArchive'] : $this->defaultArchivePrefix.' ' ).single_cat_title( '', false);
			}


		} elseif (is_author()){ /** Check if is author view **/
	        $author	 = get_queried_object();
	        $trail	.= $this->settings[$this->dpc->oFunctions->getUserLang()]['seperator'].$author->display_name;
			//$trail	.= $this->settings[$this->dpc->oFunctions->getUserLang()]['seperator'].$author->display_name;
        } elseif (is_tax()){ /** Check if is taxonomy view **/
            $term = get_queried_object();
            if($term){
                $tax	 = get_taxonomy($term->taxonomy);
                $trail	.= $this->settings[$this->dpc->oFunctions->getUserLang()]['seperator'].single_term_title($tax->labels->name . $seperator, false);
            }
		} elseif (is_archive() && !empty($m)){ /** Check if is archive time**/
	        $my_year	 = substr($m, 0, 4);
	        $my_month	 = $wp_locale->get_month(substr($m, 4, 2));
	        $my_day		 = intval(substr($m, 6, 2));
	        $trail		.= $this->settings[$this->dpc->oFunctions->getUserLang()]['seperator'].(strlen($this->settings[$this->dpc->oFunctions->getUserLang()]['prefixArchive']) ? $this->settings[$this->dpc->oFunctions->getUserLang()]['prefixArchive'] : $this->defaultArchivePrefix ).$my_year.( $my_month ? $my_month : '' ).( $my_day ? $my_day : '' );
        } elseif (is_archive() && !empty($year)){ /** Check if is archive time**/
	        $title = $year;
	        if(!empty($monthnum))
	            $trail .= $this->settings[$this->dpc->oFunctions->getUserLang()]['seperator'].(strlen($this->settings[$this->dpc->oFunctions->getUserLang()]['prefixArchive']) ? $this->settings[$this->dpc->oFunctions->getUserLang()]['prefixArchive'] : $this->defaultArchivePrefix ).$wp_locale->get_month($monthnum);
	        if(!empty($day))
	            $trail .= $this->settings[$this->dpc->oFunctions->getUserLang()]['seperator'].(strlen($this->settings[$this->dpc->oFunctions->getUserLang()]['prefixArchive']) ? $this->settings[$this->dpc->oFunctions->getUserLang()]['prefixArchive'] : $this->defaultArchivePrefix ).zeroise($day, 2);
        } elseif (is_archive() ){ /** Check if is archive**/
	            $trail .= $this->settings[$this->dpc->oFunctions->getUserLang()]['seperator'].(strlen($this->settings[$this->dpc->oFunctions->getUserLang()]['prefixArchive']) ? $this->settings[$this->dpc->oFunctions->getUserLang()]['prefixArchive'] : $this->defaultArchivePrefix ).get_the_archive_title();
        } elseif (is_search()) { /** Check if is search **/
            $trail .= $this->settings[$this->dpc->oFunctions->getUserLang()]['seperator'].(strlen($this->settings[$this->dpc->oFunctions->getUserLang()]['prefixSearch']) ? $this->settings[$this->dpc->oFunctions->getUserLang()]['prefixSearch'] : $this->defaultSearchPrefix ).' '.sprintf(esc_html__(' %1$s'), wp_strip_all_tags($search));
        } elseif (is_404()) { /** Check if is 404 **/
            $trail .= $this->settings[$this->dpc->oFunctions->getUserLang()]['seperator'].(strlen($this->settings[$this->dpc->oFunctions->getUserLang()]['prefixNotFound']) ? $this->settings[$this->dpc->oFunctions->getUserLang()]['prefixNotFound'] : $this->default404Prefix );
        } else {
            $trail .= $this->settings[$this->dpc->oFunctions->getUserLang()]['seperator'].get_the_title();
        }
		$trail .= '</div></div>';

		if(isset($this->settings['hideLastElement'])){
		    $trail .= '<style>.dpc-breadcrumbs .dpc-seperator:last-of-type, .dpc-breadcrumbs .dpc-child:last-of-type { display: none !important;}</style>';
		}

		return $this->getJsonString(). $trail;
	}

	function dpc_breadcrumbs( ) {
	    return  $this->the_breadcrumb();
	}

	function initXMLText(){
	    esc_html__('Breadcrumbs', 'delucks-seo');
	    esc_html__("With navigation paths users and search engines can find one's way easier through your site, similar to Hansel and Gretel.", 'delucks-seo');
	    esc_html__('Google knows that breadcrumbs make it easier for your target group to navigate through your website therefore the search engine honors your website, when it uses breadcrumbs.', 'delucks-seo');
		esc_html__('Activate and embed breadcrumbs to help your visitors to find their way back to the home page of your website.', 'delucks-seo');
		esc_html__('Activated', 'delucks-seo');
		esc_html__('Deactivated', 'delucks-seo');
		esc_html__('Shortcode', 'delucks-seo');
		esc_html__('PHP', 'delucks-seo');
		esc_html__('Use it as a widget', 'delucks-seo');
		esc_html__('Frontpage', 'delucks-seo');
		esc_html__('Delimiter', 'delucks-seo');
		esc_html__('Prefixes', 'delucks-seo');
		esc_html__('You are here', 'delucks-seo');
		esc_html__('Before breadcrumbs', 'delucks-seo');
		esc_html__('Archive for', 'delucks-seo');
		esc_html__('Before archive', 'delucks-seo');
		esc_html__('You searched for', 'delucks-seo');
		esc_html__('Search result pages', 'delucks-seo');
		esc_html__('Page not found', 'delucks-seo');
		esc_html__('404 pages', 'delucks-seo');
		esc_html__('Show house icon instead of frontpage text', 'delucks-seo');
		esc_html__('Select home icon', 'delucks-seo');
		esc_html__('Category basis', 'delucks-seo');
		esc_html__('Posts', 'delucks-seo');
		esc_html__('Category', 'delucks-seo');
		esc_html__('Tag', 'delucks-seo');
		esc_html__('Post format', 'delucks-seo');
		esc_html__('Products', 'delucks-seo');
		esc_html__('Product category', 'delucks-seo');
		esc_html__('Product tag', 'delucks-seo');
		esc_html__('Shipping class', 'delucks-seo');
		esc_html__('Insert archive page in breadcrumb', 'delucks-seo');
		esc_html__('Archive page titles', 'delucks-seo');
		esc_html__('Hide Last Element', 'delucks-seo');
	}
}

if(!function_exists('dpc_breadcrumbs')){
	function dpc_breadcrumbs(){
		if(shortcode_exists('dpc_breadcrumbs')){
			return do_shortcode('[dpc_breadcrumbs]');
		}
	}
}