<?php

class DPC_Module_Basic_Metadata implements DPC_Module_Interface {

    var $dpc				= null;
    var $isSubmodule		= true;
    var $moduleDirectory 	= 'basic/';
    var $moduleTitle 		= '';
	var $optionsHook		= 'dpc-basic-metadata';
    var $settingsFile		= false;
    var $settings			= array();
    var $translatorJs       = array();
    var $icon				= '';
    var $metaText			= '';
    var $shortenDescText		= '...';
    var $config             	= array('homeTitle'         => '',
                                    'divider'           => '',
                                    'websiteTitle'      => '',
                                    'maxTitleLength'    => 65,
                                    'maxDescLength'     => 155
                              );


    function __construct(&$dpc){
        $this->dpc 						= $dpc;
        $this->moduleTitle				= esc_html__('Metadata', 'delucks-seo');
        $this->settingsFile				= dirname(__FILE__) . '/settings.xml';
        $this->settings 				= $this->dpc->getModuleSettings('basic_metadata');

        //Product categories
        add_action('edited_product_cat', array($this, 'saveCategoryDescriptionField'));
        add_action('create_product_cat', array($this, 'saveCategoryDescriptionField'));

        //Categories
        add_action('edit_category_form_fields', array($this, 'editCategoryDescriptionField'));
        add_action('category_add_form_fields', array($this, 'addCategoryDescriptionField'));
        add_action('edited_category', array($this, 'saveCategoryDescriptionField'));
        add_action('create_category', array($this, 'saveCategoryDescriptionField'));

        //Tags
        add_action('edit_tag_form_fields', array($this, 'editTagDescriptionField'));
        add_action('add_tag_form_fields', array($this, 'addTagDescriptionField'));
        add_action('edit_term', array($this, 'saveTagDescriptionField'));
        add_action('create_term', array($this, 'saveTagDescriptionField'));

        add_filter("attachment_fields_to_edit", array($this,'filter_attachment_fields_to_edit'), null, 2);
        add_filter("attachment_fields_to_save", array($this,'image_attachment_fields_to_save'), null, 2);
        add_action( 'admin_init', array($this, 'hook_new_media_columns') );
        add_action( 'admin_init', array($this, 'hook_new_post_columns') );
        add_shortcode('dpc_metadescription', array(&$this, 'the_metadescription'));
        add_shortcode('dpc_metadescription_loop', array(&$this, 'the_metadescription_loop'));

        add_action('init', array($this, 'removeThemeSupportTitle'));
        add_action('feed_link', array($this, 'feedburner_url'), 10, 2);
    }

    function executeBackendActions(){
        add_action('admin_enqueue_scripts', array(&$this, 'hook_admin_enqueue_scripts'));
    }

    function newInstallation(){
        $title		= get_option('blogname', '');
        $subtitle	= get_option('blogdescription', '');

        foreach($this->dpc->oFunctions->getLanguages() as $lang){
            $this->settings[$lang['lang_key']]['title'] = array(
                'frontpage' => $subtitle,
                'delimiter' => ' - ',
                'website' => $title
            );
        }

        update_option('dpc_basic_metadata', $this->settings);

        //activate this module on a new installation
        $settings = $this->settings;
        $settings['dpc_status_basic_metadata'] = 1;
        update_option('dpc_basic_metadata', $settings);
        update_option('dpc_status_basic_metadata', 1);
    }

    function feedburner_url($output, $feed) {
        if(strpos( $output, 'comments')){
            return $output;
        }

        if(isset($this->settings['feedburner_url']) && strlen($this->settings['feedburner_url'])){
            return esc_url(trim($this->settings['feedburner_url']));
        }

        return $output;
    }


    function filter_attachment_fields_to_edit( $form_fields, $post ) {
        $value = (bool) get_post_meta($post->ID, '_dpc-no-index', true);
        $form_fields['no_index'] = array(
            'label' => esc_html__('Don�t allow search engines to index this page (�noindex�)', 'delucks-seo'),
            'input' => 'html',
            'html' => '<label for="attachments-'.$post->ID.'-no_index"> '.
            '<input type="checkbox" id="attachments-'.$post->ID.'-no_index" name="attachments['.$post->ID.'][no_index]" value="1"'.($value ? ' checked="checked"' : '').' /></label>  ',
            'value' => $value
        );
        return $form_fields;
    }

    function image_attachment_fields_to_save($post, $attachment) {
        update_post_meta($post['ID'], '_dpc-no-index', (isset($attachment['no_index']) === true ? $attachment['no_index'] : ''));
        return $post;
    }

    function removeThemeSupportTitle() {
        remove_theme_support( 'title-tag' );
    }

    function getTranslationJs(){
        return array(
            'metatitle' 		=> esc_html__('Meta-Title', 'delucks-seo'),
            'or' 				=> esc_html__('or', 'delucks-seo'),
            'chars' 			=> esc_html__('characters', 'delucks-seo'),
            'tooMuchChars' 		=> esc_html__('characters too much', 'delucks-seo'),
            'remainChars' 	 	=> esc_html__('characters left', 'delucks-seo'),
            'warning'			=> esc_html__('Warning', 'delucks-seo'),
            'duplicateTitle' 	=> esc_html__('The title is already used by another post', 'delucks-seo'),
            'duplicateDesc' 	=> esc_html__('The description is already used by another post', 'delucks-seo'),
            'withPost'			=> esc_html__('View post', 'delucks-seo'),
            'setAsPrimary' 		=> esc_html__('Set as primary', 'delucks-seo'),
            'quit'		 		=> esc_html__('Quit', 'delucks-seo'),
            'remove' 			=> esc_html__('Remove', 'delucks-seo'),
            'keywordMissing' 	=> esc_html__('Keyword not yet used in metadata, please mention it.', 'delucks-seo'),
            'keywordFound' 		=> esc_html__('Keyword used in metadata.', 'delucks-seo'),
            'clickToEdit'		=> esc_html__('Place the keywords in the content. Ideal also for the title and content. Click to edit', 'delucks-seo')
        );
    }

    function getMetaText(){
        return htmlspecialchars(__('Some themes are using their own output for the website title. In order for the meta title to work please check the header.php of your theme and search for the <title> tag. Change the code between <title> and </title> to <?php echo wp_title(); ?>. If no header.php exists please contact the provider/developer of your theme and ask them for the position of the meta title.', 'delucks-seo'), ENT_QUOTES);
    }

    function adminSettings(){
        $this->dpc->parseSettingsByXml($this->settingsFile, 'general');
    }

    function adminSettingsHead(){
        wp_enqueue_style('dpc-basic-metadata-settings', plugins_url( 'assets/css/settings.css', __FILE__ ));
        wp_enqueue_script('dpc-basic-metadata-settings', plugins_url( 'assets/js/settings.js', __FILE__ ), array('jquery'), false);

        $str =  '	var dpc_basic_metadata_config = ' . wp_json_encode($this->config) . ";\n";
        $str .= '	var dpc_basic_metadata_settings = ' . wp_json_encode($this->settings) . ";\n";
        wp_add_inline_script('dpc-basic-metadata-settings', $str, 'before');
    }

    function adminHead(){
        #add_filter('attachment_fields_to_edit', array(&$this, 'hook_attachment_fields_to_edit'), null, 2);
        #add_filter('attachment_fields_to_save', array(&$this, 'hook_attachment_fields_to_save'), null , 2);
    }

    function hook_widgets_init(){
        require_once( 'widgets/description.php' );
        register_widget( 'DelucksMetaDescription' );
    }

    function executeFrontendActions(){
        add_filter('robots_txt', 	array($this, 'filterRobots'), 1001);
        add_filter('wp_title',		array($this, 'filterWpTitle'), 1000, 3);
        add_action('wp_head', 		array($this, 'initMetaTagValues'));
        add_action('wp_head', 		array($this, 'action_printOpenGraphMeta'), 10);
        add_action('wp_head', 		array($this, 'action_printWebsiteSchema'), 10);
        add_action('wp_head', 		array($this, 'action_printVerifyMeta'), 10);
        add_action('wp_head', 		array($this, 'action_printCanonicalTag'), 10);
        add_action('wp_head', 		array($this, 'action_printMetaKeywords'), 10);
        add_action('wp_head', 		array($this, 'action_printMetaDescription'), 10);
    }

    /*
     * When saving the settings page */
    function onSaveActionBefore(){
        if(isset($_POST['dpc']['basic_metadata'])){
            foreach(array('bing', 'google', 'yandex', 'baidu', 'alexa', 'pinterest') as $field){
                if(strlen($_POST['dpc']['basic_metadata']['verify'][$field])){
                    $array = array();
                    preg_match( '/content="([^"]*)"/i', $_POST['dpc']['basic_metadata']['verify'][$field], $array );
                    if(isset($array[1])){
                        $_POST['dpc']['basic_metadata']['verify'][$field] = trim($array[1]);
                    }
                }
            }
        }
    }

    function registerWidgets(){
        add_action( 'widgets_init', array(&$this, 'hook_widgets_init'));
    }

    function getWebsiteTitle(){
        $lang = (defined('ICL_LANGUAGE_CODE') && ICL_LANGUAGE_CODE != 'all' ? ICL_LANGUAGE_CODE : $this->getLang(0));
        if(isset($this->settings[$lang]) && isset($this->settings[$lang]['title']) && isset($this->settings[$lang]['title']['website']) && strlen($this->settings[$lang]['title']['website'])){
            return $this->settings[$lang]['title']['website'];
        }

        return get_option('blogname');
    }
    
    function getFrontpageTitle(){
        $lang = (defined('ICL_LANGUAGE_CODE') && ICL_LANGUAGE_CODE != 'all' ? ICL_LANGUAGE_CODE : $this->getLang(0));
        if(isset($this->settings[$lang]) && isset($this->settings[$lang]['title']) && isset($this->settings[$lang]['title']['frontpage']) && strlen($this->settings[$lang]['title']['frontpage'])){
            return $this->settings[$lang]['title']['frontpage'];
        }
        
        return get_option('blogdescription');
    }
    
    function getDelimiter(){
        $lang = (defined('ICL_LANGUAGE_CODE') && ICL_LANGUAGE_CODE != 'all' ? ICL_LANGUAGE_CODE : $this->getLang(0));
        if(isset($this->settings[$lang]) && isset($this->settings[$lang]['title']) && isset($this->settings[$lang]['title']['delimiter']) && strlen($this->settings[$lang]['title']['delimiter'])){
            return $this->settings[$lang]['title']['delimiter'];
        }
        
        return '';
    }
    
    function getTitleFull(){
        $lang = (defined('ICL_LANGUAGE_CODE') && ICL_LANGUAGE_CODE != 'all' ? ICL_LANGUAGE_CODE : $this->getLang(0));
        if(isset($this->settings[$lang]) && isset($this->settings[$lang]['title'])){
            return join('', $this->settings[$lang]['title']);
        }
        
        return '';
    }

    function modalKeyword(){
        $post 				        = get_post($this->dpc->modal->currentId);
        $permalink			        = get_permalink($post->ID);
        $url 				        = str_replace($post->post_name.'/', '', $permalink);
        $lang 				        = (defined('ICL_LANGUAGE_CODE') && ICL_LANGUAGE_CODE != 'all' ? ICL_LANGUAGE_CODE : $this->getLang(0));
        $currentPostType	        = get_post_type($post->ID);
        $customTitle		        = false;
        $title				        = get_post_meta($post->ID, '_dpc-meta-title', true);
        $default_title		        = '';
        $is_frontpage               = ($post->ID == get_option( 'page_on_front' ) ? true : false);
        $index_state                = get_post_meta($post->ID, '_dpc-no-index', true);
        $follow_state               = get_post_meta($post->ID, '_dpc-no-follow', true);
        
        if(isset($this->settings['metaDescriptionAsExcerpt']) && strlen($this->settings['metaDescriptionAsExcerpt'])){
            $metaDescriptionAsExcerpt = $this->settings['metaDescriptionAsExcerpt'];
        } else { $metaDescriptionAsExcerpt = false; }

		$dpc_permalink 		= (get_post_meta($post->ID, '_dpc-canonical', true) != '' ? get_post_meta($post->ID, '_dpc-canonical', true) : $permalink );
		
        /**
         * Title
         */
        if(isset($this->settings[$lang]) && isset($this->settings[$lang]['posttypes']) && isset($this->settings[$lang]['posttypes'][$currentPostType]) && isset($this->settings[$lang]['posttypes'][$currentPostType]['title']) && isset($this->settings[$lang]['posttypes'][$currentPostType]['status']) && $this->settings[$lang]['posttypes'][$currentPostType]['status'] == 'on'){
            $titleParts = $this->settings[$lang]['posttypes'][$currentPostType]['title'];
        } else {
            $titleParts = $this->settings[$lang]['title'];
        }


        /* cut website name and delimiter if title is too long */
        if(is_array($titleParts) && count($titleParts)){
            $titleParts['frontpage'] = get_the_title($post->ID);
            $titleParts = $this->shortenTitle($titleParts, $this->config['maxTitleLength']);
        }

        foreach($titleParts as $k => $v){
            $default_title .= esc_attr($v);
        }

        if(strlen($title)){ /** if has dpc-meta-title **/
            $customTitle = true;
        }

        if(!strlen($default_title)){
            $default_title = esc_html__('Undefined', 'delucks-seo');
        }

        if($customTitle){
            $meta_title = esc_attr($title);
        } else {
            $meta_title = $default_title;
        }


        /**
         * Description
         */
        $default_description 	= $this->settings[$lang]['desc'];
        $meta_fallback			= esc_html__('No meta description provided! Please click to enter a description.', 'delucks-seo');
        
        $excerpt = $post->post_content;
        $excerpt = trim($excerpt, " \t\n\r\0\x0B,");
        $excerpt = trim($excerpt);
        $excerpt = wp_strip_all_tags($excerpt);
        $excerpt = preg_replace('/\s+/', ' ', preg_replace('/\n\r+/', ' ', $excerpt));
        $excerpt = trim(preg_replace('#\[[^\]]+\]#', '',$excerpt));
        $excerpt = wp_trim_words($excerpt, 100, '');
        
        if(get_post_meta( $post->ID, '_dpc-meta-description', true ) != ''){
            $meta_desc = esc_attr( get_post_meta( $post->ID, '_dpc-meta-description', true ) );
        }elseif(has_excerpt($post->ID)){
            $meta_desc = wp_strip_all_tags( get_the_excerpt($post->ID) );
        }elseif(strlen($excerpt)){ //todo get description
            $meta_desc = $excerpt;
        }elseif(strlen($default_description)){
            $meta_desc = $default_description;
        }else{
            $meta_desc = $meta_fallback;
        }
		
		
        
        $meta_desc = $this->dpc->getPlaceholder()->replace(trim(esc_attr(wp_strip_all_tags(strip_shortcodes(preg_replace('#\[[^\]]+\]#', '',$meta_desc))))));

        ?>

		<div id="titlewrap">
			<div id="dpc-metadata-wrapper">
				<div class="dpc-section dpc-group keyword-field-wrapper">
					<div class="dpc-col dpc-col-4-12 no-margin-top">
						<div id="dpc_keywords">
							<?php
								$this->settings_ranking_system 	= $this->dpc->getModuleSettings('basic_statistics');
								$external_ranking_system = $this->settings_ranking_system['dpc_external_ranking_system'];
								if (isset($external_ranking_system)) {
									if ($external_ranking_system === 'dpc_sistrix_system') {
										?>
										<div class="form-group">
											<input type="text" id="dpc-keyword" name="dpc-keyword" value="" size="25" placeholder="<?php echo esc_html__('Keywords', 'delucks-seo'); ?>"/>
											<div class="dpc-inline-input-actions">
												<a id="dpc-add-keyword" class="btn btn-info"><i class="fa fa-plus"></i></a>
											</div>
											<a id="dpc-get-ai-keywords" class="btn btn-ai-alt tooltip-toggle" dpc-tooltip="<?php echo esc_html__('Generate keywords with AI', 'delucks-seo'); ?>">
													<div class="btn-spinner dark"></div>
													<i class="fa fa-magic"></i>
												</a>
										</div>

										<div style="clear: both; height: 1px;"></div>
										<div class="keyword-list-wrapper sistrix-keyword">
											<?php
												$statisticsSettings = $this->dpc->getModuleSettings('basic_statistics');

												$dpcKeywordPrimary = esc_attr( get_post_meta( $post->ID, '_dpc-keyword', true ) );
												if(strlen($dpcKeywordPrimary)){
													$rankingData      = '';
													$rankingDisplay   = '';
													$keyword          = trim($dpcKeywordPrimary);

													if($this->dpc->moduleIsActive('basic_statistics') && isset($statisticsSettings['sistrix_api_key']) && strlen($statisticsSettings['sistrix_api_key'])){
														$statistics        = $this->dpc->getModuleInstance('basic_statistics');
														$rankingData       = $statistics->getRankingTrend($post->ID, $keyword);
														$rankingDisplay    = '	<span class="ranking dpc-ranking" title="'.esc_attr(__('Last Week', 'delucks-seo')).': '.$rankingData['last_rank'].'"><span class="dots-loader"><span class="dot dot-one"></span><span class="dot dot-two"></span><span class="dot dot-three"></span><span class="dot dot-four"></span></span><span class="ranking_count">'. $rankingData['current_rank'] .'</span><span class="ranking_trend '. $rankingData['trend'] .'"><i class="dashicons '. $rankingData['icon'] .'"></i></span></span>';
													}

													echo nl2br('<span class="keyword-list-item primary" data-keyword="'.esc_attr($keyword).'"><span class="keyword">'.esc_html($keyword).'</span> <span class="status-icon icon dashicons dashicons-no" title="'.esc_attr(__('Keyword not yet used in metadata, please mention it.', 'delucks-seo')).'"></span> '. $rankingDisplay .' </span>');
												}

												$dpcKeywordList = get_post_meta( $post->ID, '_dpc-keyword-list', true );
												if(is_array($dpcKeywordList) && count($dpcKeywordList)){
													foreach($dpcKeywordList as $keyword){
														$rankingData      = '';
														$rankingDisplay   = '';
														$keyword          = trim($keyword);

														if($this->dpc->moduleIsActive('basic_statistics') && isset($statisticsSettings['sistrix_api_key']) && strlen($statisticsSettings['sistrix_api_key'])){
															$statistics     = $this->dpc->getModuleInstance('basic_statistics');
															$rankingData    = $statistics->getRankingTrend($post->ID, $keyword);
															$rankingDisplay = '	<span class="ranking dpc-ranking" title="'.esc_attr(__('Last Week', 'delucks-seo')).': '.$rankingData['last_rank'].'"><span class="dots-loader"><span class="dot dot-one"></span><span class="dot dot-two"></span><span class="dot dot-three"></span><span class="dot dot-four"></span></span><span class="ranking_count">'. $rankingData['current_rank'] .'</span><span class="ranking_trend '. $rankingData['trend'] .'"><i class="dashicons '. $rankingData['icon'] .'"></i></span></span>';
														}

														echo nl2br('<span title="'.esc_attr(__('Click to edit', 'delucks-seo')).'" class="keyword-list-item" data-keyword="'.esc_attr($keyword).'"><span class="keyword">'.esc_html($keyword).'</span> <span class="status-icon icon dashicons dashicons-no" title="'.esc_attr(__('Keyword not yet used in metadata, please mention it.', 'delucks-seo')).'"></span> '. $rankingDisplay .' </span>');
													}
												}
											?>
										</div>
										<?php
									} elseif ($external_ranking_system === 'dpc_google_search_console_system') {
									?>
										<!-- GSC Keywords Wrapper  -->
										<div class="form-group">
											<input type="text" id="dpc-gsc-keyword" name="dpc-gsc-keyword" value="" size="25" placeholder="<?php echo esc_html__('Keywords', 'delucks-seo'); ?>"/>
											<div class="dpc-inline-input-actions">
												<a id="dpc-gsc-add-keyword" class="btn btn-info pull-right"><i class="fa fa-plus"></i></a>
											</div>
										</div>
										<div style="clear: both; height: 1px;"></div>
										<div class="gsc-appended-keyword-list-wrapper">
											<?php
												$statisticsSettings = $this->dpc->getModuleSettings('basic_statistics');

												$dpcGscKeywordPrimary = get_post_meta( $post->ID, '_dpc-gsc-keyword', true );
												$dpcGscKeywordPrimaryResult = json_decode($dpcGscKeywordPrimary, true);
												if(!empty($dpcGscKeywordPrimaryResult)){
													$rankingData      = '';
													$rankingDisplay   = '';

													if($this->dpc->moduleIsActive('basic_statistics') && isset($statisticsSettings['dpc_gsc_json_client_secret']) && strlen($statisticsSettings['dpc_gsc_json_client_secret'])){
														$rankingDisplay    = '	<span class="ranking dpc-ranking" title="'.esc_attr(__('Last Week', 'delucks-seo')).'"><span class="dots-loader"><span class="dot dot-one"></span><span class="dot dot-two"></span><span class="dot dot-three"></span><span class="dot dot-four"></span></span><span class="ranking_count">'. $dpcGscKeywordPrimaryResult[0]['ranking'] .'</span></span>';
													}

													echo nl2br('<span class="gsc-keyword-list-item primary" data-keyword="'.esc_attr($dpcGscKeywordPrimaryResult[0]['keyword']).'"><span class="gsc-keyword">'.esc_html($dpcGscKeywordPrimaryResult[0]['keyword']).'</span> <span class="status-icon icon dashicons dashicons-no" title="'.esc_attr(__('Keyword not yet used in metadata, please mention it.', 'delucks-seo')).'"></span> '. $rankingDisplay .' </span>');
												}

												$dpcGscKeywordList = get_post_meta( $post->ID, '_dpc-gsc-keyword-list', true );
												if (!empty($dpcGscKeywordList)) {
													$result = implode(',', $dpcGscKeywordList);
													$result = str_replace('}{', '},{', $result);
													$dpcGscKeywordList = json_decode($result);
													if(!empty($dpcGscKeywordList) && !is_null($dpcGscKeywordList) && isset($dpcGscKeywordList)){
														foreach($dpcGscKeywordList as $item){
															$rankingData      = '';
															$rankingDisplay   = '';
															$keyword          = $item->keyword;
															$ranking          = $item->ranking;
	
															if($this->dpc->moduleIsActive('basic_statistics') && isset($statisticsSettings['dpc_gsc_json_client_secret']) && strlen($statisticsSettings['dpc_gsc_json_client_secret'])){
																$rankingDisplay = '	<span class="ranking dpc-ranking" title="'.esc_attr(__('Last Week', 'delucks-seo')).'"><span class="dots-loader"><span class="dot dot-one"></span><span class="dot dot-two"></span><span class="dot dot-three"></span><span class="dot dot-four"></span></span><span class="ranking_count">'. $ranking .'</span></span>';
															}
	
															echo nl2br('<span title="'.esc_attr(__('Click to edit', 'delucks-seo')).'" class="gsc-keyword-list-item" data-keyword="'.esc_attr($keyword).'"><span class="gsc-keyword">'.esc_html($keyword).'</span> <span class="status-icon icon dashicons dashicons-no" title="'.esc_attr(__('Keyword not yet used in metadata, please mention it.', 'delucks-seo')).'"></span> '. $rankingDisplay .' </span>');
														}
													}
												}
											?>
										</div>
										<div class="gsc-keyword-list-wrapper gsc-keyword">
											<?php
												// Retrieve the JSON data from the wp_options table
												$dpc_gsc_data = json_decode(get_option('dpc_gsc_data'));
												$rows = $dpc_gsc_data->rows;

												// Counter variable to track the number of displayed keywords
												$counter = 0;
												$headingDisplayed = false; // Flag variable to track if the heading is displayed

												// Loop through the rows and display the data for the first five keywords
												foreach ($rows as $row) {
													$url = $permalink;
													if (in_array($url, $row->keys)) {
														$gsc_ranking_display = '<span class="ranking dpc-ranking" title="Keyword' . $row->keys[0] . '"><span class="dots-loader"><span class="dot dot-one"></span><span class="dot dot-two"></span><span class="dot dot-three"></span><span class="dot dot-four"></span></span><span class="ranking_count">' . round($row->position, 1) . '</span></span>';

														// Display the heading if it hasn't been displayed already
														if (!$headingDisplayed) {
															echo '<h3 class="mb-1">Google Search Console Keywords</h3>';
															$headingDisplayed = true;
														}

														echo nl2br('<span class="gsc-keyword-list-item pr-5 dpc-gsc-keyword primary dpc-gsc-keyword-' . esc_attr($row->keys[0]) . '" data-keyword="' . esc_attr($row->keys[0]) . '" data-ranking="'. esc_attr(round($row->position, 1)) .'"><span class="gsc-keyword">' . esc_html($row->keys[0]) . '</span>' . $gsc_ranking_display . ' <span class="fa fa-plus add-keyword-icon" title="Add this keyword to metadata."></span></span>');

														// Increment the counter
														$counter++;

														// Break the loop if the counter reaches five
														if ($counter >= 5) {
															break;
														}
													}
												}
											?>
										</div>
									<?php
									}
								}							
							?>

            				<div class="dpc-section dpc-group indexation-wrapper">
            					<div class="haswrap dpc-col dpc-col-4-12">
            						<div class="form-field dpcbasic_metadatano_index">
            							<label for="dpc[basic_metadata][no_index]">
            							<input name="dpc[basic_metadata][no_index]" id="dpc_metadata_no_index" type="checkbox" <?php echo $index_state ? 'checked="checked"' : ''; ?>><?php echo esc_html__('no index', 'delucks-seo'); ?></label>
            						</div>
            					</div>
            					<div class="haswrap dpc-col dpc-col-4-12">
            						<div class="form-field dpcbasic_metadatano_follow">
            							<label for="dpc[basic_metadata][no_follow]">
            							<input name="dpc[basic_metadata][no_follow]" id="dpc_metadata_no_follow" type="checkbox" <?php echo $follow_state ? 'checked="checked"' : ''; ?>><?php echo esc_html__('no follow', 'delucks-seo'); ?></label>
            						</div>
            					</div>
            					<div class="haswrap dpc-col dpc-col-4-12"></div>
            					<?php if($metaDescriptionAsExcerpt){ ?>
            					<div class="haswrap dpc-col dpc-col-12-12">
            						<div class="form-field dpcbasic_metadatano_metaDescriptionAsExcerpt">
            							<label for="dpc[basic_metadata][metaDescriptionAsExcerpt]">
            							<input name="dpc[basic_metadata][metaDescriptionAsExcerpt]" id="metaDescriptionAsExcerpt" type="checkbox" <?php echo $metaDescriptionAsExcerpt == 'auto' ? 'checked="checked" disabled="disabled"' : ''; ?>><?php echo esc_html__('Save meta description as excerpt', 'delucks-seo'); ?></label>
            						</div>
            					</div>
            					<?php } ?>
            				</div>
						</div>
					</div>

					<div class="dpc-googlesearch-preview-wrapper dpc-col dpc-col-8-12 no-margin-top">
						<div class="dpc-googlesearch-preview-column">
							<div id="dpc_googlesearch_preview" class="">
								<span class="focus-keyword-preview" style="display:none;"><?php echo esc_html(trim($dpcKeywordPrimary)); ?></span>
			                    <div class="serp-item">
			                        <div id="headline_serp" class="headline dpc-editable" >
				                        	<span id="dpc_post_title" style="display:none;" data-applicable="false"></span>
				                        	<span class="dpc-edit tooltip-toggle" data-label="<?php echo esc_attr(__('Edit meta title', 'delucks-seo')); ?>" dpc-tooltip="<?php echo esc_attr__('Edit meta title', 'delucks-seo'); ?>" data-default="<?php echo esc_attr($default_title); ?>" data-input-type="input" data-save="post_title" data-max-length="<?php echo esc_attr($this->config['maxTitleLength']); ?>" data-prevtext="<?php echo esc_attr($title); ?>" data-extra-text=" <?php echo sprintf(esc_html__('characters of max. %s characters', 'delucks-seo'), esc_attr($this->config['maxTitleLength'])); ?>">
				                        		<?php echo esc_html($meta_title); ?>
				                        	</span>
											<a id="dpc-get-ai-headline_serp" class="btn btn-ai-alt tooltip-toggle" dpc-tooltip="<?php echo esc_attr(__('Generate meta title with AI', 'delucks-seo')); ?>">
												<div class="btn-spinner dark"></div>
												<i class="fa fa-magic"></i>
											</a>
			                        </div>
			                        <div id="canonical" class="canonical dpc-editable" style="clear:left;">
			                        		<span id="dpc_post_canonical" style="display:none;" data-applicable="false"></span>
				                        	<span class="dpc-edit url tooltip-toggle" data-label="<?php echo esc_attr(__('Edit canonical URL', 'delucks-seo')); ?>" dpc-tooltip="<?php echo esc_attr(__('Edit canonical URL', 'delucks-seo')); ?>" data-default="<?php echo esc_url($permalink); ?>" data-input-type="input" data-save="post_canonical" data-prevtext="<?php echo esc_url($dpc_permalink); ?>">
				                        		<?php echo esc_url($dpc_permalink); ?>
				                        	</span>
				                        	<div class="urlserp-arrow"><span class="fa fa-caret-down"></span></div>
			                        </div>
			                        <div style="clear:both;">
				                        <span class="description dpc-editable">
				                        	<span id="dpc_post_description" style="display:none;" data-applicable="false"></span>
				                            <span id="description_serp" data-label="<?php echo esc_attr(__('Edit meta description', 'delucks-seo')); ?>" dpc-tooltip="<?php echo esc_attr(__('Edit meta description', 'delucks-seo')); ?>" class="dpc-edit tooltip-toggle" data-default="<?php echo esc_attr(strip_shortcodes($default_description)); ?>" data-fallback="<?php echo esc_attr($meta_fallback); ?>" data-input-type="textarea" data-save="dpc-textopt-description"  data-min-length="<?php echo esc_attr($this->config['maxDescLength']); ?>" data-extra-text=" <?php echo sprintf(esc_html__('characters of min. %s characters', 'delucks-seo'), esc_attr($this->config['maxDescLength'])); ?>">
				                            		<?php echo $this->dpc->getPlaceholder()->replace(trim(esc_attr(wp_strip_all_tags(strip_shortcodes(preg_replace('#\[[^\]]+\]#', '',$meta_desc)))))); ?>
				                            </span>
											<a id="dpc-get-ai-description" class="btn btn-ai-alt tooltip-toggle" dpc-tooltip="<?php echo esc_attr(__('Generate meta description with AI', 'delucks-seo')); ?>">
												<div class="btn-spinner dark"></div>
												<i class="fa fa-magic"></i>
											</a>
				                        </span>
			                        </div>
			                    </div>
			                </div>
							<!-- OpenAI Widget -->
							<div class="chat-interactions-widget-area btnCallsResponseArea"><span class="chat-label">Chat</span><span id="chatResponse"></span></div>
			        	</div>
					</div>
				</div>
	 		</div>
	 	</div>

	 	<?php
	}

	function getTitleCounterText(){
	    return htmlspecialchars(sprintf(esc_html__('characters of max. %s characters', 'delucks-seo'), $this->config['maxTitleLength']));
	}

	function getDescriptionCounterText(){
	    return htmlspecialchars(sprintf(esc_html__('characters of min. %s characters', 'delucks-seo'), $this->config['maxDescLength']));
	}

	function editCategoryDescriptionField( $tag ) {
		$t_id		= $tag->term_id;
		$cat_meta	= get_option( "category_$t_id");
		$lang 		= (defined('ICL_LANGUAGE_CODE') && ICL_LANGUAGE_CODE != 'all' ? ICL_LANGUAGE_CODE : $this->getLang(0));
		?>
			<tr class="form-field admin-title-wrap">
				<th scope="row" valign="top"><label for="dpc-metatitle[<?php echo esc_attr($lang); ?>]"><?php echo esc_html__('Meta-Title', 'delucks-seo'); ?></label></th>
				<td>
					<input type="text" class="dpc-metatitle" name="dpc-metatitle[<?php echo esc_attr($lang); ?>]" id="dpc-metatitle[<?php echo esc_attr($lang); ?>]" value="<?php echo isset($cat_meta['dpc-metatitle_'.$lang]) ? esc_attr($cat_meta['dpc-metatitle_'.$lang]) : ''; ?>" placeholder="<?php echo esc_attr(__('Automatically', 'delucks-seo')); ?>" />
					<div><span class="green title-counter"> <?php echo sprintf(esc_html__('characters of max. %s characters', 'delucks-seo'), esc_html($this->config['maxTitleLength'])); ?></span></div>
				</td>
			</tr>
			<tr class="form-field admin-description-wrap">
				<th scope="row" valign="top"><label for="dpc-description[<?php echo esc_attr($lang); ?>]"><?php echo esc_html__('Meta-Description', 'delucks-seo'); ?></label></th>
				<td>
					<textarea class="dpc-metadesc" name="dpc-description[<?php echo esc_attr($lang); ?>]" id="dpc-description[<?php echo esc_attr($lang); ?>]"><?php echo isset($cat_meta['dpc-description_'.$lang]) ? esc_textarea($cat_meta['dpc-description_'.$lang]) : ''; ?></textarea>
					<div><span class="green desc-counter"> <?php echo sprintf(esc_html__('characters of min. %s characters', 'delucks-seo'), esc_html($this->config['maxDescLength'])); ?></span></div>
				</td>
			</tr>
		<?php
	}

	function addCategoryDescriptionField( $tag ) {
		$t_id		= is_object($tag) ? $tag->term_id : '';
		$cat_meta	= get_option( "category_$t_id");
		$lang 		= (defined('ICL_LANGUAGE_CODE') && ICL_LANGUAGE_CODE != 'all' ? ICL_LANGUAGE_CODE : $this->getLang(0));

		?>
			<div class="form-field admin-title-wrap">
				<label for="dpc-metatitle[<?php echo esc_attr($lang); ?>]"><?php echo esc_html__('Meta-Title', 'delucks-seo'); ?></label>
				<input type="text" class="dpc-metatitle" name="dpc-metatitle[<?php echo esc_attr($lang); ?>]" id="dpc-metatitle[<?php echo esc_attr($lang); ?>]" value="<?php echo isset($cat_meta['dpc-metatitle_'.$lang]) ? esc_attr($cat_meta['dpc-metatitle_'.$lang]) : ''; ?>" placeholder="<?php echo esc_attr(__('Automatically', 'delucks-seo')); ?>" /><br />
				<span class="green title-counter"> <?php echo sprintf(esc_html__('characters of max. %s characters', 'delucks-seo'), esc_html($this->config['maxTitleLength'])); ?></span>
			</div>
			<div class="form-field admin-description-wrap">
				<label for="dpc-description[<?php echo esc_attr($lang); ?>]"><?php echo esc_html__('Meta-Description', 'delucks-seo'); ?></label>
				<textarea class="dpc-metadesc" name="dpc-description[<?php echo esc_attr($lang); ?>]" id="dpc-description[<?php echo esc_attr($lang); ?>]"><?php echo (isset($cat_meta['dpc-description_'.$lang]) ? esc_textarea($cat_meta['dpc-description_'.$lang]) : ''); ?></textarea><br />
				<div><span class="green desc-counter"> <?php echo sprintf(esc_html__('characters of min. %s characters', 'delucks-seo'), esc_html($this->config['maxDescLength'])); ?></span></div>
			</div>
		<?php
	}

	function saveCategoryDescriptionField( $term_id ) {
		if ( isset( $_POST['dpc-description'] ) ) {
			$t_id = $term_id;
			$cat_meta = get_option( "category_$t_id");
			$cat_keys = array_keys($_POST['dpc-description']);
			foreach ($cat_keys as $key){
				if (isset($_POST['dpc-description'][$key])){
				    $cat_meta['dpc-description_'.$key] = sanitize_textarea_field($_POST['dpc-description'][$key]);
				}
				if (isset($_POST['dpc-metatitle'][$key])){
				    $cat_meta['dpc-metatitle_'.$key] = sanitize_textarea_field($_POST['dpc-metatitle'][$key]);
				}
			}
			update_option( "category_$t_id", $cat_meta );
		}
	}

	function editTagDescriptionField( $tag ) {
		$t_id		= $tag->term_id;
		$cat_meta	= get_option( "tag_$t_id");
		$lang 		= (defined('ICL_LANGUAGE_CODE') && ICL_LANGUAGE_CODE != 'all' ? ICL_LANGUAGE_CODE : $this->getLang(0));
		?>
			<tr class="form-field admin-title-wrap">
				<th scope="row" valign="top"><label for="dpc-metatitle[<?php echo esc_attr($lang); ?>]"><?php echo esc_html__('Meta-Title', 'delucks-seo'); ?></label></th>
				<td>
					<input type="text" class="dpc-metatitle" name="dpc-metatitle[<?php echo esc_attr($lang); ?>]" id="dpc-metatitle[<?php echo esc_attr($lang); ?>]" value="<?php echo isset($cat_meta['dpc-metatitle_'.$lang]) ? esc_attr($cat_meta['dpc-metatitle_'.$lang]) : ''; ?>" placeholder="<?php echo esc_attr(__('Automatically', 'delucks-seo')); ?>" />
					<div><span class="green title-counter"> <?php echo sprintf(esc_html__('characters of max. %s characters', 'delucks-seo'), esc_html($this->config['maxTitleLength'])); ?></span></div>
				</td>
			</tr>
			<tr class="form-field admin-description-wrap">
				<th scope="row" valign="top"><label for="dpc-description[<?php echo esc_attr($lang); ?>]"><?php echo esc_html__('Meta-Description', 'delucks-seo'); ?></label></th>
				<td>
					<textarea class="dpc-metadesc" name="dpc-description[<?php echo esc_attr($lang); ?>]" id="dpc-description[<?php echo esc_attr($lang); ?>]"><?php echo isset($cat_meta['dpc-description_'.$lang]) ? esc_textarea($cat_meta['dpc-description_'.$lang]) : ''; ?></textarea>
					<div><span class="green desc-counter"> <?php echo sprintf(esc_html__('characters of min. %s characters', 'delucks-seo'), esc_html($this->config['maxDescLength'])); ?></span></div>
				</td>
			</tr>
		<?php
		}

	function addTagDescriptionField( $tag ) {
		$t_id		= is_object($tag) ? $tag->term_id : '';
		$cat_meta	= get_option( "tag_$t_id");
		$lang 		= (defined('ICL_LANGUAGE_CODE') && ICL_LANGUAGE_CODE != 'all' ? ICL_LANGUAGE_CODE : $this->getLang(0));
		?>
			<div class="form-field admin-title-wrap">
				<label for="dpc-metatitle[<?php echo esc_attr($lang); ?>]"><?php echo esc_html__('Meta-Title', 'delucks-seo'); ?></label>
				<input type="text" class="dpc-metatitle" name="dpc-metatitle[<?php echo esc_attr($lang); ?>]" id="dpc-metatitle[<?php echo esc_attr($lang); ?>]" value="<?php echo isset($cat_meta['dpc-metatitle_'.$lang]) ? esc_attr($cat_meta['dpc-metatitle_'.$lang]) : ''; ?>" placeholder="<?php echo esc_attr(__('Automatically', 'delucks-seo')); ?>" /><br />
				<div><span class="green title-counter"> <?php echo sprintf(esc_html__('characters of max. %s characters', 'delucks-seo'), esc_html($this->config['maxTitleLength'])); ?></span></div>
			</div>
			<div class="form-field admin-description-wrap">
				<label for="dpc-description[<?php echo esc_attr($lang); ?>]"><?php echo esc_html__('Meta-Description', 'delucks-seo'); ?></label>
				<textarea class="dpc-metadesc" name="dpc-description[<?php echo esc_attr($lang); ?>]" id="dpc-description[<?php echo esc_attr($lang); ?>]"><?php echo $cat_meta['dpc-description_'.$lang] ? esc_textarea($cat_meta['dpc-description_'.$lang]) : ''; ?></textarea><br />
				<div><span class="green desc-counter"> <?php echo sprintf(esc_html__('characters of min. %s characters', 'delucks-seo'), esc_html($this->config['maxDescLength'])); ?></span></div>
			</div>
		<?php
	}

	function saveTagDescriptionField( $term_id ) {
		if ( isset( $_POST['dpc-description'] ) ) {
			$t_id = $term_id;
			$cat_meta = get_option( "tag_$t_id");
			$cat_keys = array_keys($_POST['dpc-description']);
			foreach ($cat_keys as $key){
				if (isset($_POST['dpc-description'][$key])){
				    $cat_meta['dpc-description_'.$key] = sanitize_textarea_field($_POST['dpc-description'][$key]);
				}
				if (isset($_POST['dpc-metatitle'][$key])){
				    $cat_meta['dpc-metatitle_'.$key] = sanitize_text_field($_POST['dpc-metatitle'][$key]);
				}
			}
			update_option( "tag_$t_id", $cat_meta );
		}
	}

	function getLang($objectID = 0){
		$lang = $this->dpc->oFunctions->getPostLang($objectID, false);
		$lang = substr($lang, 0, (strpos($lang, '_') ? strpos($lang, '_') : 2));
		return $lang;
	}

	function filterWpTitle($title, $sep,$seplocation){
		global $id, $post;
		if (defined("ICL_LANGUAGE_CODE")){ // notice: not 100%
			$lang = ICL_LANGUAGE_CODE;
		} else {
			$lang = substr(get_locale(), 0, (strpos(get_locale(), '_') == false ? strlen(get_locale()) : strpos(get_locale(), '_')));
		}

		if(isset($this->settings[$lang])){
			$settings = $this->settings[$lang];
		}

		if(is_tag()){
			if(isset($settings['tags']) && isset($settings['tags']['status'])){
				$settings['title'] = $settings['tags']['title'];
				$settings['title']['frontpage'] = $this->getTitle($settings['title']['delimiter']);
			}
		} elseif(is_author()){
			if(isset($settings['profiles']) && isset($settings['profiles']['status'])){
				$settings['title'] = $settings['profiles']['title'];
				$settings['title']['frontpage'] = $this->getTitle($settings['title']['delimiter']);
			}
		} elseif(is_archive()){
			if(isset($settings['archives']) && isset($settings['archives']['status'])){
				$settings['title'] = $settings['archives']['title'];
				$settings['title']['frontpage'] = $this->getTitle($settings['title']['delimiter']);
			}
		} /*elseif(is_home() && !is_front_page()){
			if(isset($settings['blogpage']) && isset($settings['blogpage']['status'])){
				$settings['title'] = $settings['blogpage']['title'];
				$settings['title']['frontpage'] = (isset($settings['blogpage']['title']['frontpage']) && strlen($settings['blogpage']['title']['frontpage']) ? $settings['blogpage']['title']['frontpage'] : $this->getTitle($settings['title']['delimiter'])) ;
			}
		}*/ elseif(is_search()){
			if(isset($settings['searchresults']) && isset($settings['searchresults']['status'])){
				$settings['title'] = $settings['searchresults']['title'];
				$settings['title']['frontpage'] = wp_strip_all_tags(get_query_var('s'));
			}
		#} elseif(is_singular()){
		    #$desc = get_post_meta($id, '_dpc-meta-description', true);
		#    $settings['title']['frontpage'] = $post->post_title;
		} else {
			if((!is_home() && !is_front_page()) && is_object($post) && strlen($post->post_type)){
				if(isset($settings['posttypes'][$post->post_type]) && isset($settings['posttypes'][$post->post_type]['status'])){
					$settings['title'] = $this->settings[$lang]['posttypes'][$post->post_type]['title'];
				}
			}

			if((!is_home() || !is_front_page()) && isset($settings['title']['delimiter']) && strlen($this->getTitle($settings['title']['delimiter']))){
				$settings['title']['frontpage'] = $this->getTitle($settings['title']['delimiter']);
			}
		}

		/* cut website name and delimiter if title is too long */
		if(isset($settings['title']) && is_array($settings['title'])){
		    $settings['title'] = $this->shortenTitle($settings['title'], $this->config['maxTitleLength']);
		}

		if(isset($settings)){
			$title = implode('', $settings['title']);
		}

		return $this->replaceTitleVariables($title);
	}

	function shortenTitle($titleArray, $maxTitleLength = 0){
	    if(!$maxTitleLength){ $maxTitleLength = $this->config['maxTitleLength']; }
	    if(strlen(utf8_decode(implode('', $titleArray))) > intval($maxTitleLength)){
	        unset($titleArray['website'], $titleArray['delimiter']);
	    }

	    return $titleArray;

	}

	function replaceTitleVariables($title = ''){
		global $post;
		$id = get_queried_object_id();
		if(defined("ICL_LANGUAGE_CODE")){ //notice: not 100%
			$lang = ICL_LANGUAGE_CODE;
		} else {
			$lang = substr(get_locale(), 0, strpos(get_locale(), '_'));
		}

		if(isset($this->settings[$lang])){
			$settings = $this->settings[$lang];
		}

		if(is_tag()){
			if(isset($this->settings[$lang]['tags']) && isset($this->settings[$lang]['tags']['status'])){
				$settings = $this->settings[$lang]['tags'];
			}
		} elseif(is_object($post) && strlen($post->post_type)){
			if(isset($this->settings[$lang]['posttypes'][$post->post_type]) && isset($this->settings[$lang]['posttypes'][$post->post_type]['status'])){
				$settings = $this->settings[$lang]['posttypes'][$post->post_type];
			}
		}

		if(!isset($settings['title']['delimiter'])){
			$settings['title']['delimiter'] = '/';
		}

		if(!isset($settings['title']['website'])){
			$settings['title']['website'] = get_bloginfo('name');
		}

		return str_replace(
				array('%%sitename%%','%%title%%','%%page%%', '%%sep%%', '%%keyword%%'),
				array($settings['title']['website'], get_the_title($id), 'PAGENUMBER??', $settings['title']['delimiter'], get_post_meta($id, '_dpc-keyword', true)),
		$title);
	}

	function initMetaTagValues(){
		global $post;
		$globalIndexSettings    = get_option('blog_public');
		$id 		            = get_queried_object_id();
		$keyword 	            = get_post_meta($id, '_dpc-keyword', true);
		$desc 		            = '';
		$thumbnail	            = '';

		if(defined("ICL_LANGUAGE_CODE")){ //notice: not 100%
			$lang = ICL_LANGUAGE_CODE;
		} else {
			$lang = substr(get_locale(), 0, (strpos(get_locale(), '_') == false ? strlen(get_locale()) : strpos(get_locale(), '_')));
		}

		$title	= wp_title('&raquo;', false);
		$type   = 'website';
		$url 	= (isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] == 'on' ? 'https://' : 'http://') . $_SERVER['HTTP_HOST'] . $_SERVER['REQUEST_URI'];

		if(is_singular() || get_option('show_on_front') === 'page' && get_queried_object_id() == get_option('page_for_posts')){
			$id = get_queried_object_id();

			if ($keyword !== false && !empty($keyword)){
			    $keywords = get_post_meta($id, '_dpc-keyword-list', true);
			    if(is_array($keywords)){
			        $keyword.= ','. implode(',', $keywords);
			    }
			}

			$desc	= get_post_meta($id, '_dpc-meta-description', true);

			if(strlen(get_post_meta($id, '_dpc-meta-title', true))){
			    $title	= get_post_meta($id, '_dpc-meta-title', true);
			    $title	= $this->replaceTitleVariables($title);
			}

			if(!$desc){
			    if(is_front_page()){
			        //front page
			        $desc = $this->settings[$lang]['desc'];
			        if(!strlen($desc)){
			            if(has_excerpt($id)){
			                $desc = wp_strip_all_tags(get_the_excerpt($id));
			            } else {
			                $excerpt                  = get_queried_object()->post_content;
			                $excerpt                  = trim($excerpt, " \t\n\r\0\x0B,");
			                $excerpt                  = trim($excerpt);
			                $excerpt                  = wp_strip_all_tags($excerpt);
			                $excerpt                  = $this->remove_inactive_shortcodes($excerpt);
			                $excerpt                  = strip_shortcodes($excerpt);
			                $excerpt                  = preg_replace('/\s+/', ' ', preg_replace('/\n\r+/', ' ', $excerpt));
			                $excerpt                  = wp_trim_words($excerpt, 100, '');

			                if(strlen($excerpt)){
			                    $desc = $excerpt;
			                } else {
			                    $desc = '';
			                }
			            }
			        }
			    } else {
			        //any post/page
    			    if(has_excerpt($id)){
    			        $desc = wp_strip_all_tags(get_the_excerpt($id));
    			    } else {
    			        $excerpt                  = get_queried_object()->post_content;
    			        $excerpt                  = trim($excerpt, " \t\n\r\0\x0B,");
    			        $excerpt                  = trim($excerpt);
    			        $excerpt                  = wp_strip_all_tags($excerpt);
    			        $excerpt                  = $this->remove_inactive_shortcodes($excerpt);
    			        $excerpt                  = strip_shortcodes($excerpt);
    			        $excerpt                  = preg_replace('/\s+/', ' ', preg_replace('/\n\r+/', ' ', $excerpt));
    			        $excerpt                  = wp_trim_words($excerpt, 100, '');

    			        if(strlen($excerpt)){
    			            $desc = $excerpt;
    			        } else {
    			            $desc = '';
    			        }
    			    }
			    }
			}

			if (has_post_thumbnail($id)){
				$thumbnail = get_the_post_thumbnail($id, 'full');
				if (preg_match('|\ssrc="(.*?)"|i', $thumbnail, $matches)){
						$thumbnail = $matches[1];
				}else{
					$thumbnail = false;
				}
			}

			$type = get_queried_object()->post_type == 'post' ? 'article' : 'website';
		} elseif(is_category()){
		    global $cat;
		    $cat = get_option('category_'.$cat);
		    $lang = $this->getLang();
		    if(is_array($cat) && isset($cat['dpc-description_'.$lang])){
		        $desc = $cat['dpc-description_'.$lang];
		    }
		    if(is_array($cat) && isset($cat['dpc-metatitle_'.$lang]) && strlen($cat['dpc-metatitle_'.$lang])){
		        $title = $cat['dpc-metatitle_'.$lang];
		    }
		} elseif(is_tax('product_cat') && $this->dpc->oFunctions->pluginActive('woocommerce')){ //woocommerce categories
		    global $wp_query;
		    $tax_data = get_term_by('slug',get_query_var($wp_query->query_vars['taxonomy']),$wp_query->query_vars['taxonomy']);
		    $tax_meta	= get_option( "category_".$tax_data->term_id);
		    if(isset($tax_meta['dpc-metatitle_'.$lang])){
		        $title = $tax_meta['dpc-metatitle_'.$lang];
		    }
		    
		    if(isset($tax_meta['dpc-description_'.$lang]) && strlen($tax_meta['dpc-description_'.$lang])){
		        $desc = $tax_meta['dpc-description_'.$lang];
		    }
		    
		    if(!$desc){
		        $exclude_codes = ' ';
		        $excerpt = wp_trim_excerpt($tax_data->description);
		        $excerpt = wp_strip_all_tags($excerpt);
		        $excerpt = preg_replace("~(?:\[/?)(?!(?:$exclude_codes))[^/\]]+/?\]~s", '', $excerpt);
		        if(strlen($excerpt)){
		            $maxDescLength = (isset($this->settings['descLimit']) ? $this->settings['descLimit'] : 155);
		            if(strlen($excerpt) > $maxDescLength){
		                $excerpt = substr($excerpt, 0, ($maxDescLength - strlen($this->shortenDescText))) . $this->shortenDescText;
		            }
		            $desc = $excerpt;
		        } elseif(isset($this->settings[$lang]['desc']) && !empty($this->settings[$lang]['desc'])){
		            $desc = $this->settings[$lang]['desc'];
		        }
		    }
		} elseif(is_tag()){
		    $cat = get_option('tag_'.get_queried_object()->term_id);
		    $lang = $this->getLang();
		    if(is_array($cat) && isset($cat['dpc-description_'.$lang])){
		        $desc = $cat['dpc-description_'.$lang];
		    }
		    if(is_array($cat) && isset($cat['dpc-metatitle_'.$lang]) && strlen($cat['dpc-metatitle_'.$lang])){
		        $title = $cat['dpc-metatitle_'.$lang];
		    } else { /*die('not set');*/}
		} elseif (!is_front_page() && is_home()){
		    //blog page
		    if (isset($this->settings[$lang]['blogdesc'])){
		        $desc = $this->settings[$lang]['blogdesc'];
		    }
		} elseif(is_archive()){
		    if(is_object($postType = get_queried_object())){
		        $desc = (isset($this->settings[$lang]) && isset($this->settings[$lang]['posttypes']) && isset($this->settings[$lang]['posttypes'][$postType->name]) && isset($this->settings[$lang]['posttypes'][$postType->name]['desc'])) ? $this->settings[$lang]['posttypes'][$postType->name]['desc'] : '';
		    }
		} else {
		    if (isset($this->settings[$lang]['desc'])){
		        $desc = $this->settings[$lang]['desc'];
		    }
		}


		/* On Frontpage with empty title and subtitle, use values from the wordpress settings */
		//TODO:
		/*
		$title		= get_option('blogname', '');
		$subtitle	= get_option('blogdescription', '');

		foreach($this->dpc->oFunctions->getLanguages() as $lang){
		    $this->settings[$lang['lang_key']]['title'] = array(
		        'frontpage' => $subtitle,
		        'delimiter' => ' - ',
		        'website' => $title
		    );
		}
		*/


		$this->ogSiteName 	   = (isset($this->settings[$lang]['title']['website']) ? $this->settings[$lang]['title']['website'] : '');
		$this->ogTitle 		   = apply_filters('delucks_seo_metadata_og_title', $title);
		$this->ogDescription   = apply_filters('delucks_seo_metadata_og_description', $desc);
		$this->ogImage 		   = apply_filters('delucks_seo_metadata_og_image', $thumbnail);
		$this->ogSiteName      = apply_filters('delucks_seo_metadata_og_site_name', $this->ogSiteName);
		$this->ogType		   = apply_filters('delucks_seo_metadata_og_type', $type);
		$this->url	 		   = apply_filters('delucks_seo_metadata_og_url', $url);
		$this->title	 	   = apply_filters('delucks_seo_metadata_title', $title);
		$this->desc		 	   = apply_filters('delucks_seo_metadata_description', $desc);
		$this->keywords	 	   = apply_filters('delucks_seo_metadata_keywords', $keyword);

		/* follow: start */
		if(is_paged() && isset($this->settings['follow_paginated'])){
			$options = array();
			$options[] = $this->settings['follow_paginated'] ? $this->settings['follow_paginated'] : 'nofollow';
			$options[] = $this->settings['index_paginated'] ? $this->settings['index_paginated'] : 'noindex';
			#echo '<meta name="robots" content="'.implode(',', $options).'">'."\n";
		} elseif(is_tax('product_cat') && $this->dpc->oFunctions->pluginActive('woocommerce')){ //woocommerce categories
			global $wp_query;
			$tax_data = get_term_by('slug',get_query_var($wp_query->query_vars['taxonomy']),$wp_query->query_vars['taxonomy']);
			if(isset($this->settings['woocommerce_categories']) && isset($this->settings['woocommerce_categories'][$tax_data->term_id])){
				$options = array();
				$options[] = $this->settings['woocommerce_categories'][$tax_data->term_id]['follow'] ? $this->settings['woocommerce_categories'][$tax_data->term_id]['follow'] : 'nofollow';
				$options[] = $this->settings['woocommerce_categories'][$tax_data->term_id]['index'] ? $this->settings['woocommerce_categories'][$tax_data->term_id]['index'] : 'noindex';
				#echo '<meta name="robots" content="'.implode(',', $options).'">'."\n";
			}# else {
			#	echo '<meta name="robots" content="'.implode(',', array('nofollow', 'noindex')).'">'."\n";
			#}
		} elseif(is_category()){ //wordpress categories
			$tax_data = get_the_category();
			if(is_array($tax_data)) { $tax_data = $tax_data[0]; }
			if(is_null($tax_data)){
				$tax_data = new stdClass();
				$tax_data->term_id = $id;
			}

			if(isset($this->settings['categories']) && isset($this->settings['categories'][$tax_data->term_id])){
				$options = array();
				$options[] = $this->settings['categories'][$tax_data->term_id]['follow'] ? $this->settings['categories'][$tax_data->term_id]['follow'] : 'nofollow';
				$options[] = $this->settings['categories'][$tax_data->term_id]['index'] ? $this->settings['categories'][$tax_data->term_id]['index'] : 'noindex';
				#echo '<meta name="robots" content="'.implode(',', $options).'">'."\n";
			}# else {
			#	echo '<meta name="robots" content="'.implode(',', array('nofollow', 'noindex')).'">'."\n";
			#}
		} elseif(is_author()){
			$options = array();
			$options[] = (isset($this->settings['profiles']['follow']) && $this->settings['profiles']['follow']) ? $this->settings['profiles']['follow'] : 'nofollow';
			$options[] = (isset($this->settings['profiles']['index']) && $this->settings['profiles']['index']) ? $this->settings['profiles']['index'] : 'noindex';
			#echo '<meta name="robots" content="'.implode(',', $options).'">'."\n";
		} elseif(is_tag()){
		    $options = array();
		    $options[] = (isset($this->settings['tags']['follow']) && $this->settings['tags']['follow']) ? $this->settings['tags']['follow'] : 'nofollow';
		    $options[] = (isset($this->settings['tags']['index']) && $this->settings['tags']['index']) ? $this->settings['tags']['index'] : 'noindex';
		    #echo '<meta name="robots" content="'.implode(',', $options).'">'."\n";
		} elseif((is_tax() || is_archive()) && isset($this->settings['follow_texonomies'])){
			$options = array();
			$options[] = $this->settings['follow_texonomies'] ? $this->settings['follow_texonomies'] : 'nofollow';
			$options[] = $this->settings['index_texonomies'] ? $this->settings['index_texonomies'] : 'noindex';
			#echo '<meta name="robots" content="'.implode(',', $options).'">'."\n";
		} elseif(is_object($post)){
			if(is_attachment()){
			    $attachmentPostIndexSettings = get_post_meta( $post->ID, '_dpc-no-index', true );
				$options = array();
				$attachmentPostIndexSettings ? $options[] = 'nofollow' : $options[] = (isset($this->settings['attachments']['follow']) && $this->settings['attachments']['follow']) ? $this->settings['attachments']['follow'] : 'nofollow';
				$attachmentPostIndexSettings ? $options[] = 'noindex' : $options[] = (isset($this->settings['attachments']['index']) && $this->settings['attachments']['index']) ? $this->settings['attachments']['index'] : 'noindex';
			} elseif($this->isPaginatedCommentPage() && isset($this->settings['follow_paginated'])){
				$options = array();
				$options[] = $this->settings['follow_paginated'] ? $this->settings['follow_paginated'] : 'nofollow';
				$options[] = $this->settings['index_paginated'] ? $this->settings['index_paginated'] : 'noindex';
				#echo '<meta name="robots" content="'.implode(',', $options).'">'."\n";
			} else {
				$options = array();
				$options[] = get_post_meta( $post->ID, '_dpc-no-index', true ) == 'on' ? 'noindex' : 'index';
				$options[] = get_post_meta( $post->ID, '_dpc-no-follow', true ) == 'on' ? 'nofollow' : 'follow';
			}
			echo '<meta name="date" content="'.esc_attr(gmdate('Y-m-d', get_the_modified_time('G', $post))).'" />'."\n";
		}
				
		/* Add fixed meta robots */
		if(!isset($options) || !is_array($options)){ $options = array(); }
		$options[] = 'max-image-preview:large';
		$options[] = 'max-snippet:-1';
		$options[] = 'max-video-preview:-1';

		if($globalIndexSettings && isset($options) && is_array($options) && count($options)){
		    echo '<meta name="robots" content="'.esc_attr(implode(',', $options)).'">'."\n";
		}
		/* follow: end */


		if(!current_theme_supports('title-tag')){
		    echo "<title>".esc_html($this->dpc->getPlaceholder()->replace(esc_attr(trim($title))))."</title>";
		}
	}
	
	
	function filterRobots($output){
	    if(!isset($this->settings['excludeContentFromChatGPT']) || !strlen($this->settings['excludeContentFromChatGPT'])){
	        return $output;
	    }
	    
	    if($this->settings['excludeContentFromChatGPT'] == 'everything'){
	        $output .= "\n\nUser-agent: GPTBot";
	        $output .= "\nDisallow: /";
	        return $output;
	    }
	}
	

	
	function remove_inactive_shortcodes( $content ) {
	    global $shortcode_tags;
	    
	    $keys          = array_keys( $shortcode_tags );
	    $exclude_codes = array();
	    $the_content   = get_the_content();
	    
	    // strip all shortcodes except $exclude_codes and keep all content
	    $the_content = preg_replace( "~(?:\[/?)(?!(?:".join('|', $exclude_codes)."))[^/\]]+/?\]~s", '', $the_content );
	    
	    $content = $the_content;
	    return $content;
	}
	
	
	function action_printCanonicalTag(){
		if(isset($this->settings['setCanonicalTag'])){
			if(is_front_page()){
				$url = str_replace(array('https://', 'http://'), wp_parse_url(get_home_url(), PHP_URL_SCHEME).'://', get_home_url());
				if(strpos($url, '?') === false){
					if(substr($url, -1) != '/'){
						$url .= '/';
					}
				}
				echo '<link rel="canonical" href="'.esc_url($url).'" />'."\n";
			} elseif(is_singular()){
			    
			    if($canonical = get_post_meta(get_the_ID(), '_dpc-canonical', true)){
			        echo '<link rel="canonical" href="'.esc_url(str_replace(array('https://', 'http://'), wp_parse_url(get_option('siteurl'), PHP_URL_SCHEME).'://', $canonical)).'" />'."\n";
			    } else {
			        echo '<link rel="canonical" href="'.esc_url(str_replace(array('https://', 'http://'), wp_parse_url(get_option('siteurl'), PHP_URL_SCHEME).'://', get_permalink())).'" />'."\n";
			    }
			} elseif(is_tax('product_cat') && $this->dpc->oFunctions->pluginActive('woocommerce')){ //woocommerce categories
				echo '<link rel="canonical" href="'.esc_url(str_replace(array('https://', 'http://'), wp_parse_url(get_option('siteurl'), PHP_URL_SCHEME).'://', get_permalink())).'" />'."\n";
			}
		}
	}

	function action_printOpenGraphMeta(){
	    echo strlen($this->ogTitle)        ? '<meta property="og:title" content="'.esc_attr($this->dpc->getPlaceholder()->replace(trim($this->ogTitle))).'" />'."\n" : '';
	    echo strlen($this->ogDescription)  ? '<meta property="og:description" content="'.esc_attr($this->dpc->getPlaceholder()->replace(trim(esc_attr(wp_strip_all_tags(preg_replace("/[\n\r]/", ' ', preg_replace('#\[[^\]]+\]#', '',$this->ogDescription))))))).'" />'."\n" : '';
	    echo strlen($this->ogImage)        ? '<meta property="og:image" content="'.esc_attr(trim($this->ogImage)).'" />'."\n" : '';
	    echo strlen($this->url)            ? '<meta property="og:url" content="'.esc_attr(trim($this->url)).'" />'."\n" : '';
	    echo strlen($this->ogSiteName)     ? '<meta property="og:site_name" content="'.esc_attr(trim($this->ogSiteName)).'" />'."\n" : '';
	    echo strlen($this->ogType)         ? '<meta property="og:type" content="'.esc_attr(trim($this->ogType)).'" />'."\n" : '';
	}
	
	function action_printWebsiteSchema(){
	    if(strlen($this->ogType) && strlen($this->ogSiteName) && is_front_page()){
	        echo '<script type="application/ld+json">{ "@context" : "https://schema.org", "@type" : "' . esc_attr($this->ogType) . '", "name" : "'. esc_attr($this->ogSiteName) .'", "url" : "'. esc_url(get_site_url()) .'" } </script>';
	    } else {

	    
	    }
	}

	function action_printMetaKeywords(){
	    echo strlen($this->keywords)       ? '<meta name="keywords" content="'.esc_attr(trim($this->keywords)).'" />'."\n" : '';
	}

	function action_printMetaDescription(){
	    echo strlen($this->desc)           ? '<meta name="description" content="'.esc_attr($this->dpc->getPlaceholder()->replace(trim(esc_attr(wp_strip_all_tags(preg_replace("/[\n\r]/", ' ', preg_replace('#\[[^\]]+\]#', '',$this->desc))))))).'" />'."\n" : '';
	}

	function action_printVerifyMeta(){
		if(is_front_page()){
			if(isset($this->settings['verify'])){
			    echo (isset($this->settings['verify']['alexa']) && strlen(isset($this->settings['verify']['alexa'])) ? '<meta name="alexaVerifyID" content="'.esc_attr($this->settings['verify']['alexa']).'"/>'."\n" : '');
			    echo (isset($this->settings['verify']['pinterest']) && strlen(isset($this->settings['verify']['pinterest'])) ? '<meta name="p:domain_verify" content="'.esc_attr($this->settings['verify']['pinterest']).'"/>'."\n" : '');

				/**
				 * Bing
				*/
				if(isset($this->settings['verify']['bing']) && strlen(isset($this->settings['verify']['bing']))){
					if(strpos($this->settings['verify']['bing'], '<meta')){
						echo esc_html($this->settings['verify']['bing']);
					}else{
					    echo '<meta name="msvalidate.01" content="'.esc_attr($this->settings['verify']['bing']).'"/>'."\n";
					}
				}

				/**
				 * Google
				 */
				if(isset($this->settings['verify']['google']) && strlen(isset($this->settings['verify']['google']))){
					if(strpos($this->settings['verify']['google'], '<meta')){
						echo esc_html($this->settings['verify']['google']);
					}else{
					    echo '<meta name="google-site-verification" content="'.esc_attr($this->settings['verify']['google']).'"/>'."\n";
					}
				}

			}
		}
	}

	function isPaginatedCommentPage( $post_id = 0 ) {
		if(get_option( 'page_comments' )  && ( $pagi = (int) get_query_var( 'comments_per_page' ) ) && wp_count_comments( $post_id )->total_comments > $pagi){
			$cpage = get_query_var('cpage');
			if(!empty($cpage)){
				return true;
			}
		}
		return false;
	}

	function getTitle($sep = '&raquo;', $seplocation = '') {
		global $wpdb, $wp_locale;

		$m = get_query_var('m');
		$year = get_query_var('year');
		$monthnum = get_query_var('monthnum');
		$day = get_query_var('day');
		$search = get_query_var('s');
		$title = '';

		$t_sep = '%WP_TITILE_SEP%'; // Temporary separator, for accurate flipping, if necessary

		// If there is a post
		if ( is_single() || ( is_home() && !is_front_page() ) || ( is_page() && !is_front_page() ) ) {
			$title = single_post_title( '', false );
		}

		// If there's a post type archive
		if ( is_post_type_archive() ) {
			$post_type = get_query_var( 'post_type' );
			if ( is_array( $post_type ) )
				$post_type = reset( $post_type );
			$post_type_object = get_post_type_object( $post_type );
			if ( ! $post_type_object->has_archive )
				$title = post_type_archive_title( '', false );
		}

		// If there's a category or tag
		if ( is_category() ) {
			$title = single_term_title( '', false );
			$catid = get_cat_ID( $title );
		}

		if ( is_tag() ) {
			$title = single_term_title( '', false );
		}

		// If there's a taxonomy
		if ( is_tax() ) {
			$term = get_queried_object();
			if ( $term ) {
				$tax = get_taxonomy( $term->taxonomy );
				$title = single_term_title( $tax->labels->name . $t_sep, false );
			}
		}

		// If there's an author
		if ( is_author() ) {
			$author = get_queried_object();
			if ( $author )
				$title = $author->display_name;
		}

		// Post type archives with has_archive should override terms.
		if ( is_post_type_archive() && $post_type_object->has_archive )
			$title = post_type_archive_title( '', false );

		// If there's a month
		if ( is_archive() && !empty($m) ) {
			$my_year = substr($m, 0, 4);
			$my_month = $wp_locale->get_month(substr($m, 4, 2));
			$my_day = intval(substr($m, 6, 2));
			$title = $my_year . ( $my_month ? $t_sep . $my_month : '' ) . ( $my_day ? $t_sep . $my_day : '' );
		}

		// If there's a year
		if ( is_archive() && !empty($year) ) {
			$title = $year;
			if ( !empty($monthnum) )
				$title .= $t_sep . $wp_locale->get_month($monthnum);
			if ( !empty($day) )
				$title .= $t_sep . zeroise($day, 2);
		}

		// If it's a search
		if ( is_search() ) {
			/* translators: 1: separator, 2: search phrase */
		    $title = sprintf(esc_html__('Search Results %1$s %2$s'), $t_sep, wp_strip_all_tags($search));
		}

		// If it's a 404 page
		if ( is_404() ) {
		    $title = esc_html__('Page not found');
		}

		$prefix = '';
		if ( !empty($title) )
			$prefix = " $sep ";

		// Determines position of the separator and direction of the breadcrumb
		if ( 'right' == $seplocation ) { // sep on right, so reverse the order
			$title_array = explode( $t_sep, $title );
			$title_array = array_reverse( $title_array );
			$title = implode( " $sep ", $title_array ) ;
		} else {
			$title_array = explode( $t_sep, $title );
			$title = implode( " $sep ", $title_array );
		}

		return $title;
	}

	function the_metadescription($atts = array()){
	    $atts = shortcode_atts(array('shortcodes' => 'false', 'own_shortcodes' => 'false', 'tags' => 'false', 'limit' => 0, 'shorten_text' => ''), $atts);

	    $customMetaDescription = false;
	    $desc = '';
	    $lang = (defined('ICL_LANGUAGE_CODE') && ICL_LANGUAGE_CODE != 'all' ? ICL_LANGUAGE_CODE : $this->getLang(0));
	    if(is_singular()){
	        $id 	= get_queried_object_id();
	        $desc	= get_post_meta($id, '_dpc-meta-description', true);
	        $customMetaDescription = true;
	    }elseif(is_category()){
	        global $cat;
	        if(!is_array($cat)){
	            $cat = get_option('category_'.$cat);
	        }

	        $lang = $this->getLang();
	        if(is_array($cat) && isset($cat['dpc-description_'.$lang])){
	            $desc = $cat['dpc-description_'.$lang];
	            $customMetaDescription = true;
	        }
	    } elseif(is_tag()){
	        $cat = get_option('tag_'.get_queried_object()->term_id);
	        if(is_array($cat) && isset($cat['dpc-description_'.$lang])){
	            $desc = $cat['dpc-description_'.$lang];
	            $customMetaDescription = true;
	        }
	    }

	    if(strlen($desc)){
	        if($customMetaDescription){
	            if($atts['own_shortcodes'] === 'true'){
	                $desc = do_shortcode($desc);
	            } else {
	                $desc = strip_shortcodes($desc);
	            }
	        } else {
	            if($atts['shortcodes'] === 'true'){
	                $desc = do_shortcode($desc);
	            } else {
	                $desc = strip_shortcodes($desc);
	            }
	        }

	        if(!$atts['tags'] === 'true'){
	            $desc = wp_strip_all_tags($desc);
	        }


	        if((int)$atts['limit'] > 0 && strlen($desc) > (int)$atts['limit']){
	            $desc = substr($desc, 0, ((int)$atts['limit'] - strlen($atts['shorten_text']))) . $atts['shorten_text'];
	        }
	    }


	    return $desc;
	}

	function the_metadescription_loop($atts = array()){
	    if(!In_the_loop()){
	        return '';
	    }
	    
	    $atts = shortcode_atts(array('shortcodes' => 'false', 'own_shortcodes' => 'false', 'tags' => 'false', 'limit' => 0, 'shorten_text' => ''), $atts);
	    
	    $id = get_the_ID();
	    $desc = '';
	    $lang = (defined('ICL_LANGUAGE_CODE') && ICL_LANGUAGE_CODE != 'all' ? ICL_LANGUAGE_CODE : $this->getLang(0));
	    $desc	= get_post_meta($id, '_dpc-meta-description', true);
	    
	    
	    if(!$desc){
	        $customMetaDescription = false;
	        $desc = get_the_excerpt();
	        
	        if(!strlen($desc) && isset($this->settings[$lang]['desc']) && !empty($this->settings[$lang]['desc'])){
	            $desc = $this->settings[$lang]['desc'];
	            
	        } else {
	            
	        }
	    } else {
	        $customMetaDescription = true;
	    }

	    if(strlen($desc)){
	        if($customMetaDescription){
	            if($atts['own_shortcodes'] === 'true'){
	                $desc = do_shortcode($desc);
	            } else {
	                $desc = strip_shortcodes($desc);
	            }
	        } else {
	            if($atts['shortcodes'] === 'true'){
	                $desc = do_shortcode($desc);
	            } else {
	                $desc = strip_shortcodes($desc);
	            }
	        }
	        
	        if(!$atts['tags'] === 'true'){
	            $desc = wp_strip_all_tags($desc);
	        }
	        
	        
	        
	        if(!$customMetaDescription){
    	        if((int)$atts['limit'] > 0 && strlen($desc) > (int)$atts['limit']){
    	            $desc = substr($desc, 0, ((int)$atts['limit'] - strlen($atts['shorten_text']))) . $atts['shorten_text'];
    	        }
	        }
	    }
	    
	    return $desc;
	}

	function dpc_metadescription($atts = array()){
	    return  $this->the_metadescription($atts);
	}

	/****************/
	/* POSTS        */
	function dpc_post_column( $cols ) {
	    $cols["dpc-keyword"] = esc_html__('Keyword', 'delucks-seo');
	    $cols["dpc-description"] =  esc_html__('Description', 'delucks-seo');
		return $cols;
	}

	function dpc_post_value( $column_name, $id ) {
		switch($column_name){
			case 'dpc-keyword':
				echo esc_html(get_post_meta($id, '_dpc-keyword', true));
				break;
			case 'dpc-description':
				echo esc_html(get_post_meta($id, '_dpc-meta-description', true));
				break;
		}
	}

	function dpc_post_column_sortable( $cols ) {
		$cols["dpc-keyword"] = "dpc_keyword";
		$cols["dpc-description"] = "dpc_description";
		return $cols;
	}

	function hook_new_post_columns() {
		#add_filter( 'manage_posts_columns', array($this, 'dpc_post_column'), 1, 2  );
		#add_action( 'manage_posts_custom_column', array($this, 'dpc_post_value'), 1, 2 );
		#add_filter( 'manage_edit-post_sortable_columns', array($this, 'dpc_post_column_sortable'), 1, 2 );
		#add_filter( 'manage_pages_columns', array($this, 'dpc_post_column'), 1, 2 );
		#add_action( 'manage_pages_custom_column', array($this, 'dpc_post_value'), 10, 2 );
		#add_filter( 'manage_edit-page_sortable_columns', array($this, 'dpc_post_column_sortable'), 1, 2 );
		add_filter( 'request', array($this, 'dpc_keyword_column_orderby') );
		add_filter( 'request', array($this, 'dpc_description_column_orderby') );
	}

	function dpc_keyword_column_orderby( $vars ) {
		if ( isset( $vars['orderby'] ) && 'dpc_keyword' == $vars['orderby'] ) {
			$vars = array_merge( $vars, array(
					'meta_key' => '_dpc-keyword',
					'orderby' => 'meta_value'
			) );
		}
		return $vars;
	}

	function dpc_description_column_orderby( $vars ) {
		if ( isset( $vars['orderby'] ) && 'dpc_description' == $vars['orderby'] ) {
			$vars = array_merge( $vars, array(
					'meta_key' => '_dpc-meta-description',
					'orderby' => 'meta_value'
			) );
		}
		return $vars;
	}

	/****************/
	/* MEDIA LIBARY */
	function dpc_column( $cols ) {
		$cols["alt"] = "Alt";
		$cols["description"] =  esc_html__('Description', 'delucks-seo');
		return $cols;
	}

	function dpc_value( $column_name, $id ) {
		switch($column_name){
			case 'alt':
				$meta = get_post_meta( $id, '_wp_attachment_image_alt', true);
				echo esc_html($meta);
				break;
			case 'description':
				$post = get_post( $id );
				echo strlen($post->post_content) ? esc_html(wp_strip_all_tags($post->post_content)) : '';
				break;
		}
	}

	function dpc_column_sortable( $cols ) {
		$cols["alt"] = "name";
		$cols["description"] = "name";
		return $cols;
	}

	function hook_new_media_columns() {
		add_filter( 'manage_media_columns', array($this, 'dpc_column') );
		add_action( 'manage_media_custom_column', array($this, 'dpc_value'), 10, 2 );
		add_filter( 'manage_upload_sortable_columns', array($this, 'dpc_column_sortable') );
	}

	/*
	 *  Get description by post id */
	function get_description($id = null){
		if(is_null($id)){
			$id = get_the_id();
		}

		return get_post_meta($id, '_dpc-meta-description', true);
	}

	function hook_admin_enqueue_scripts(){
	    wp_enqueue_script('dpc-basic-metadata-admin', plugins_url( 'assets/js/admin.js', __FILE__ ));
	    $str =  '	var dpc_basic_metadata_config = ' . wp_json_encode($this->config) . ";\n";
	    $str .= '	var dpc_basic_metadata_settings = ' . wp_json_encode($this->settings) . ";\n";
	    wp_add_inline_script('dpc-basic-metadata-admin', $str, 'before');
	}

	function modalEnqueueStyles(){
	    wp_enqueue_style('dpc-basic-metadata-jquery-context', plugins_url( 'assets/contextMenu/jquery.contextMenu.min.css', __FILE__ ));
	    wp_enqueue_style('dpc-basic-metadatat-modalstyles', plugins_url( 'assets/css/modal.css', __FILE__ ));
	}

	function modalEnqueueScripts(){
	    wp_enqueue_script('jquery-ui-autocomplete');
	    wp_enqueue_script('dpc-basic-metadata-contextMenu', plugins_url( 'assets/contextMenu/contextMenu.min.js', __FILE__ ));
	    wp_enqueue_script('dpc-basic-metadata-jquery-ui-pos', plugins_url( 'assets/contextMenu/jquery.ui.position.min.js', __FILE__ ));
	    wp_enqueue_script('dpc-basic-metadata-modal', plugins_url( 'assets/js/modal.js', __FILE__ ), array('jquery'), DPC_VERSION, true);
	    wp_enqueue_script('dpc-inlineedit', plugins_url( 'assets/js/jquery.inline-edit.js', __FILE__ ), array('jquery'), DPC_VERSION, true);

	    echo '<script type="text/javascript">'."\n";
	    echo '	var dpc_basic_metadata_translator = '.wp_json_encode($this->getTranslationJs()).';'."\n";
	    echo '	var dpc_basic_metadata_suggest_callback = "' . (isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] == 'on' ? 'https://' : 'http://') . 'suggestqueries.google.com/complete/search?callback=?";' . "\n";
		echo '	var dpc_basic_metadata_config = ' . wp_json_encode($this->config) . ";\n";
        echo '	var dpc_basic_metadata_settings = ' . wp_json_encode($this->settings) . ";\n";	    
        echo '</script>'."\n";
		
        wp_add_inline_script('dpc-basic-metadata-settings', $str, 'before');
	}

	function modalTab(){
	    echo '<span class="dpc-modal-tab dpc-modal-tab-trigger '.((isset($_GET['tab']) && $_GET['tab'] == 'meta') || !isset($_GET['tab']) ? 'active' : '').'" id="tab-'. esc_attr($this->optionsHook) .'">'. esc_html($this->moduleTitle) .'</span>';
	}

	function modalBody(){
	    echo '<div class="dpc-modal-content '.((isset($_GET['tab']) && $_GET['tab'] == 'meta') || !isset($_GET['tab']) ? 'active' : '').'" id="content-'. esc_attr($this->optionsHook) .'">';
		$this->dpc->new_parseSettingsByXml($this->settingsFile, 'modal', false, true, false, $this->dpc->modal->meta);
		echo '</div>';
	}

	function modalSave(){
	    $lang 				      = (defined('ICL_LANGUAGE_CODE') && ICL_LANGUAGE_CODE != 'all' ? ICL_LANGUAGE_CODE : $this->getLang(0));
	    $currentPostType	      = get_post_type(intval($_POST['dpc_current_id']));
	    $post 				      = get_post(intval($_POST['dpc_current_id']));
	    $defaultMetaDescription   = $this->settings[$lang]['desc'];
	    $defaultMetatitle         = '';
	    $excerpt                  = wp_trim_words($post->post_content, 55, '');
	    $excerpt                  = trim($excerpt, " \t\n\r\0\x0B,");
	    $excerpt                  = wp_strip_all_tags($excerpt);
	    $excerpt                  = strip_shortcodes($excerpt);

	    /* build default title: start */
	    if(isset($this->settings[$lang]) && isset($this->settings[$lang]['posttypes']) && isset($this->settings[$lang]['posttypes'][$currentPostType]) && isset($this->settings[$lang]['posttypes'][$currentPostType]['title']) && isset($this->settings[$lang]['posttypes'][$currentPostType]['status']) && $this->settings[$lang]['posttypes'][$currentPostType]['status'] == 'on'){
	        $titleParts = $this->settings[$lang]['posttypes'][$currentPostType]['title'];
	    } else {
	        $titleParts = $this->settings[$lang]['title'];
	    }
	    foreach($titleParts as $k => $v){
	        if($k == 'frontpage'){
	            $v = get_the_title(intval($_POST['dpc_current_id']));
	        }
	        $defaultMetatitle .= esc_attr($v);
	    }
	    /* build default title: end */


	    if(isset($_POST['dpc-meta-title'])){
	        $defaultMetatitle = get_permalink($_POST['dpc_current_id']);
	        $newMetatitle = ($_POST['dpc-meta-title'] == $defaultMetatitle) ? '' : $_POST['dpc-meta-title'];
	        update_post_meta(intval($_POST['dpc_current_id']), '_dpc-meta-title', sanitize_text_field($newMetatitle));
	    }
	    
	    if(isset($_POST['dpc-canonical'])){
	        $newCanonical = ($_POST['dpc-canonical'] == $defaultMetatitle) ? '' : $_POST['dpc-canonical'];
	        update_post_meta(intval($_POST['dpc_current_id']), '_dpc-canonical', sanitize_text_field($newCanonical));
	    }

	    if(isset($_POST['dpc-textopt-description'])){
	        $newMetaDescription = ($_POST['dpc-textopt-description'] == $defaultMetaDescription || $_POST['dpc-textopt-description'] == esc_html__('No meta description provided! Please click to enter a description.', 'delucks-seo') || $_POST['dpc-textopt-description'] == $excerpt) ? '' : $_POST['dpc-textopt-description'];
	        update_post_meta(intval($_POST['dpc_current_id']), '_dpc-meta-description', sanitize_textarea_field(strip_shortcodes($newMetaDescription)));
	        
	        if(isset($_POST['dpc-metaDescriptionAsExcerpt']) === true){
	            wp_update_post(array(
	                'ID' => intval($_POST['dpc_current_id']),
	                'post_excerpt' => sanitize_textarea_field(strip_shortcodes($newMetaDescription))
	            ));
	        }
	    }

	    if(isset($_POST['dpc-keyword-primary']) && strlen($_POST['dpc-keyword-primary'])){
	        $keyword = $_POST['dpc-keyword-primary'];
	    } elseif(isset($_POST['dpc-keyword']) && strlen($_POST['dpc-keyword'])){
	        $keyword = $_POST['dpc-keyword'];
	    } else {
	        $keyword = '';
	    }
		
		/* GSC keyword check addition */
	    if(isset($_POST['dpc-gsc-keyword-primary']) && strlen($_POST['dpc-gsc-keyword-primary'])){
	        $gsc_keyword = $_POST['dpc-gsc-keyword-primary'];
	    } elseif(isset($_POST['dpc-gsc-keyword']) && strlen($_POST['dpc-gsc-keyword'])){
	        $gsc_keyword = $_POST['dpc-gsc-keyword'];
	    } else {
	        $gsc_keyword = '';
	    }

	    update_post_meta(intval($_POST['dpc_current_id']), '_dpc-no-index', (isset($_POST['dpc-no_index']) === true ? sanitize_text_field($_POST['dpc-no_index']) : ''));
	    update_post_meta(intval($_POST['dpc_current_id']), '_dpc-no-follow', (isset($_POST['dpc-no_follow']) === true ? sanitize_text_field($_POST['dpc-no_follow']) : ''));
		/* Check whick keyword ranking is enabled and update only those keywords. It could be GSC or sistrix. Previously it has been updating both at the same time regardless of which one was enabled*/
		$this->settings_ranking_system 	= $this->dpc->getModuleSettings('basic_statistics');
		$external_ranking_system = $this->settings_ranking_system['dpc_external_ranking_system'];
	    isset($external_ranking_system) && $external_ranking_system === 'dpc_sistrix_system' && update_post_meta(intval($_POST['dpc_current_id']), '_dpc-keyword', sanitize_text_field($keyword));
	    isset($external_ranking_system) && $external_ranking_system === 'dpc_sistrix_system' && update_post_meta(intval($_POST['dpc_current_id']), '_dpc-keyword-list', (isset($_POST['dpc-keyword-list']) && strlen($_POST['dpc-keyword-list']) ? explode(',', sanitize_text_field($_POST['dpc-keyword-list'])) : ''));
		isset($external_ranking_system) && $external_ranking_system === 'dpc_google_search_console_system' && update_post_meta(intval($_POST['dpc_current_id']), '_dpc-gsc-keyword', sanitize_text_field($gsc_keyword));
		isset($external_ranking_system) && $external_ranking_system === 'dpc_google_search_console_system' && update_post_meta(intval($_POST['dpc_current_id']), '_dpc-gsc-keyword-list', (isset($_POST['dpc-gsc-keyword-list']) && strlen($_POST['dpc-gsc-keyword-list']) ? explode(',', sanitize_text_field($_POST['dpc-gsc-keyword-list'])) : ''));


	    #$post                  = get_post(intval($_POST['dpc_current_id']));
	    $postMeta                 = get_post_meta(intval($_POST['dpc_current_id']));
	    $metaTitle                = get_post_meta(intval($_POST['dpc_current_id']), '_dpc-meta-title', true);
	    $metaDescription          = get_post_meta(intval($_POST['dpc_current_id']), '_dpc-meta-description', true);
	    $canonical                = get_post_meta(intval($_POST['dpc_current_id']), '_dpc-canonical', true);

	    //update status
	    $status = get_post_meta(intval($_POST['dpc_current_id']), '_delucks-seo-status', true);
	    $status = is_array($status) ? $status : array();

	    if(isset($_POST['dpc-keyword-primary']) && strlen($_POST['dpc-keyword-primary']) ||
	        (strlen($metaTitle) && $metaTitle != $defaultMetatitle) ||
	        (strlen($metaDescription) && $metaDescription != $defaultMetaDescription)
	    ) {
	        $status['meta'] = 2; } else { $status['meta'] = 1;
	    }

		// Only execute this code if the external ranking system in the settings page is set to the google search console system 
		if(isset($external_ranking_system) && $external_ranking_system === 'dpc_google_search_console_system'){
			if(isset($_POST['dpc-gsc-keyword-primary']) && (strlen($_POST['dpc-gsc-keyword-primary']) && $_POST['dpc-gsc-keyword-primary'] !== '[]') ||
				(strlen($metaTitle) && $metaTitle != $defaultMetatitle) ||
				(strlen($metaDescription) && $metaDescription != $defaultMetaDescription)
			) {
				$status['meta'] = 2; } else { $status['meta'] = 1;
			}
		}



	    if(isset($_POST['dpc-no_index']) && strlen($_POST['dpc-no_index'])){
	        $status['meta'] = -1;
	    }

	    #echo "<pre>";
	    #var_dump();
	    #die();

	    #if(sanitize_text_field($_POST['dpc-no_index']))

	    update_post_meta(intval($_POST['dpc_current_id']), '_delucks-seo-status', $status);
	}

	function beforeInstallDefaultSettings($defaults, $currentSettings, $moduleName){
        // prevent title override
        foreach($this->dpc->oFunctions->getLanguages() as $lang){
            if(isset($currentSettings[$lang['lang_key']]) && isset($currentSettings[$lang['lang_key']]['title'])){
                $defaults[$lang['lang_key'].'][title][frontpage'] = $currentSettings[$lang['lang_key']]['title']['frontpage'];
                $defaults[$lang['lang_key'].'][title][website'] = $currentSettings[$lang['lang_key']]['title']['website'];
            }
        }
        return $defaults;
    }

	function initXMLText(){
	    esc_html__('Metadata', 'delucks-seo');
	    esc_html__('Meta data is implemented in the source code of your site and serves with the title and description as a preview in search engines results pages. Insert here and in all of your contents relevant keywords.', 'delucks-seo');
	    esc_html__('Activated', 'delucks-seo');
	    esc_html__('Deactivated', 'delucks-seo');
	    esc_html__('Shortcode', 'delucks-seo');
	    esc_html__('PHP', 'delucks-seo');
	    esc_html__('Title', 'delucks-seo');
		esc_html__('default', 'delucks-seo');
		esc_html__('Frontpage title (placeholder for content titles)', 'delucks-seo');
		esc_html__('Delimiter', 'delucks-seo');
		esc_html__('Website Title / Company name', 'delucks-seo');
		esc_html__('characters left', 'delucks-seo');
		esc_html__('Active', 'delucks-seo');
		esc_html__('Post title', 'delucks-seo');
		esc_html__('Suffix', 'delucks-seo');
		esc_html__('Prefix', 'delucks-seo');
		esc_html__('Author pages', 'delucks-seo');
		esc_html__('Author name', 'delucks-seo');
		esc_html__('Archives', 'delucks-seo');
		esc_html__('Archive title', 'delucks-seo');
		esc_html__('Blog archives', 'delucks-seo');
		esc_html__('Tags', 'delucks-seo');
		esc_html__('Tag', 'delucks-seo');
		esc_html__('Search results', 'delucks-seo');
		esc_html__('Keyword', 'delucks-seo');
		esc_html__('Settings', 'delucks-seo');
		esc_html__('Enable 55 characters, as a 69-character title is often cut off from Google or exchanged and replaced by machine-generated text. 69 characters is old fashioned and would therefore be advisable only without a focus on Google SEO.', 'delucks-seo');
		esc_html__('Characters', 'delucks-seo');
		esc_html__('Remove website title / company name if title limit is exceeded', 'delucks-seo');
		esc_html__('Description', 'delucks-seo');
		esc_html__('Set a limit of', 'delucks-seo');
		esc_html__('155 characters is the perfect limit, if your website is set up on a western lanugage. 115 characters are only recommended, if your language uses �broader� letters like Chinese, Arabian or Hieroglyphics.', 'delucks-seo');
		esc_html__('Verify website', 'delucks-seo');
		esc_html__('Google', 'delucks-seo');
		esc_html__('Bing', 'delucks-seo');
		esc_html__('Pinterest', 'delucks-seo');
		esc_html__('Alexa', 'delucks-seo');
		esc_html__('Tutorials and help', 'delucks-seo');
		esc_html__('Webmaster tools', 'delucks-seo');
		esc_html__('Google Search Console', 'delucks-seo');
		esc_html__('Process', 'delucks-seo');
		esc_html__('Categories and taxonimies', 'delucks-seo');
		esc_html__('do follow', 'delucks-seo');
		esc_html__('no follow', 'delucks-seo');
		esc_html__('index', 'delucks-seo');
		esc_html__('no index', 'delucks-seo');
		esc_html__('Paginated sites', 'delucks-seo');
		esc_html__('Behavior for archives and pagination', 'delucks-seo');
		esc_html__('Behavior for categories', 'delucks-seo');
		esc_html__('Behavior for author pages', 'delucks-seo');
		esc_html__('Behavior for tags', 'delucks-seo');
		esc_html__('Behavior for WooCommerce categories', 'delucks-seo');
		esc_html__('Set canonical tag', 'delucks-seo');
		esc_html__('Use alternate title', 'delucks-seo');
		esc_html__('Use keyword', 'delucks-seo');
		esc_html__('Show a keyword field in your article editor, to focus on the topic to which you want to be found.', 'delucks-seo');
		esc_html__('Activate autocompletion', 'delucks-seo');
		esc_html__('Keyword check in', 'delucks-seo');
		esc_html__('We recommend you to focus your contents on the information connected to your keyword. The plugin will then give you an advice, if you do not use the keyword in the specific field(s). As sometimes slightly different syntax in the descriptions make better sense, we recommend you to let the plugin check the keyword only in the title field and in the content.', 'delucks-seo');
		esc_html__('preview', 'delucks-seo');
		esc_html__('Blog', 'delucks-seo');
		esc_html__('Behavior for Attachments', 'delucks-seo');
		esc_html__('Author pages', 'delucks-seo');
		esc_html__('Attachments', 'delucks-seo');
		esc_html__('Tags', 'delucks-seo');
		esc_html__('Feedburner URL', 'delucks-seo');
		esc_html__('Exclude content from ChatGPT', 'delucks-seo');
		esc_html__('{@text:Nothing}', 'delucks-seo');
		esc_html__('Everything', 'delucks-seo');
		esc_html__('Certain Post-Types', 'delucks-seo');
	}
}

if(!function_exists('dpc_metadescription')){
    function dpc_metadescription($atts = array()){
        global $plugin_dpc;
        return $plugin_dpc->oFunctions->callShortcode('dpc_metadescription', $atts);
    }
}

if(!function_exists('dpc_metadescription_loop')){
    function dpc_metadescription_loop($atts = array()){
        global $plugin_dpc;
        return $plugin_dpc->oFunctions->callShortcode('dpc_metadescription_loop', $atts);
    }
}

if(!function_exists('dpc_get_description')){
	function dpc_get_description($id = null){
		global $plugin_dpc;
		return $plugin_dpc->triggerModuleMethod('basic_metadata', 'get_description');
	}
}