<?php

class DPC_Module_Basic_Statistics implements DPC_Module_Interface {

    var $dpc				= null;
    var $isSubmodule		= true;
    var $moduleDirectory 	= 'basic/';
    var $moduleTitle 		= '';
    var $optionsHook		= 'dpc-basic-statistics';
    var $settingsFile		= false;
    var $settings			= array();
    var $helpLink			= '';

    function __construct(&$dpc){
        $this->dpc 			= $dpc;
        $this->moduleTitle	= esc_html__('Statistics', 'delucks-seo');
        $this->settingsFile = dirname(__FILE__) . '/settings.xml';
        $this->settings 	= $this->dpc->getModuleSettings('basic_statistics');
        add_shortcode('dpc_analytics_opt_out', array(&$this, 'dpc_analytics_opt_out'));

        // Initialize Google Search Console integration.
        $this->init_gsc_integration();
    }

    function getPostsWithKeyword($postType = 'any'){
        $args = array(
            'post_type' => $postType,
            'posts_per_page' => -1,
            'meta_key' => '_dpc-keyword',
            'meta_value' => array(''),
            'meta_compare' => 'NOT IN'
        );
        return get_posts( $args );
    }

    function sistrixWeeklySync(){
        if(isset($this->settings['sistrix_api_key']) && strlen($this->settings['sistrix_api_key'])){
            require_once(DPC_PLUGIN_HELPER_DIR . 'run_once.class.php');
            $calendarWeek  = gmdate('W', time());
            $year          = gmdate('Y', time());
            $syncKey       = $calendarWeek . '_' . $year;
            $run_once      = new run_once();

            if($run_once->run('dpc_sistrix_sync_weekly_t1_'. $syncKey)){
                $posts = $this->getPostsWithKeyword();
                foreach($posts as $post){
                    $keywords = $this->getPostKeywords($post->ID);
                    if(is_array($keywords) && count($keywords)){
                        foreach($keywords as $keyword){
                            $this->sistrixQueueAdd($post->ID, $keyword);
                        }
                    }
                }
            }
        }
    }

    function getPostKeywords($postId){
        $kw  = get_post_meta($postId, '_dpc-keyword', true);
        $kwl = get_post_meta($postId, '_dpc-keyword-list', true);

        if(!is_string($kw) || !strlen($kw)){ return array(); }

        if(is_array($kwl) && count($kwl)){
            $kw = array_merge(array($kw), $kwl);
        } else {
            $kw = array($kw);
        }
        return $kw;
    }

    function sistrixQueueAdd($postId, $keyword){
        $queue = get_option('dpc_sistrix_queue', array());
        if(!isset($queue[$postId]) || !is_array($queue[$postId])){
            $queue[$postId] = array();
        }

        if(!in_array($keyword, $queue[$postId])){
            array_push($queue[$postId], $keyword);
        }
        update_option('dpc_sistrix_queue', $queue);
    }

    function ajaxSistrixSyncPost(){
        $response  = array();
        $data      = array();
        $queue     = get_option('dpc_sistrix_queue', array());

        if(is_array($queue) && count($queue)){
            $postId = key($queue);

            if(isset($queue[$postId]) && is_array($queue[$postId]) && count($queue[$postId])){
                $keyword = array_shift($queue[$postId]);
                $data = $this->sistrixSyncPost($postId, $keyword);

                if($data !== false){
                    if(isset($queue[$postId]) && !count($queue[$postId])){
                        unset($queue[$postId]);
                    }
                    update_option('dpc_sistrix_queue', $queue);
                }
            }
        }

        $response['data'] = $data;
        $response['queue'] = $queue;
        echo wp_json_encode($response);
        die();
    }

    function sistrixSyncPost($postId, $keyword){
        if($postId && is_object($post = get_post($postId))){
            $timeRange             = array('now', 'last+week');
            $sistrixLastWeek       = get_post_meta($postId, '_dpc-sistrix-last-week', true);
            $sistrixCurrentWeek    = get_post_meta($postId, '_dpc-sistrix-current-week', true);
            $permalink             = get_permalink($postId);

            if(!$sistrixLastWeek){ $sistrixLastWeek = array(); }
            if(!$sistrixCurrentWeek){ $sistrixCurrentWeek = array(); }

            $keyword = trim($keyword);
            foreach($timeRange as $timeUrlKey){
                $rank = 0;
                $url = 'https://api.sistrix.com/keyword.seo?format=json&api_key='. trim($this->settings['sistrix_api_key']) .'&kw='. urldecode(trim($keyword)) .'&date='. $timeUrlKey .'&url=' . $permalink;
                $response = wp_remote_get($url);
                $response = wp_remote_retrieve_body($response);
                $response = json_decode($response);

                if((!is_object($response) || !property_exists($response, 'answer')) || (!is_array($response->answer) || !count($response->answer)) || (!isset($response->answer[0]) || !is_object($response->answer[0]) || !property_exists($response->answer[0], 'result')) || (!is_array($response->answer[0]->result) || !count($response->answer[0]->result)) || (!isset($response->answer[0]->result[0]) || !is_object($response->answer[0]->result[0]) || !property_exists($response->answer[0]->result[0], 'position'))){
                    $rank = 0;
                } else {
                    $rank = $response->answer[0]->result[0]->position;
                }

                switch($timeUrlKey){
                    case 'now':
                        $sistrixCurrentWeek[$keyword] = $rank;
                        #$sistrixCurrentWeek[$keyword] = rand(1,40);
                        break;
                    case 'last+week':
                        $sistrixLastWeek[$keyword] = $rank;
                        #$sistrixLastWeek[$keyword] = rand(1,40);
                        break;
                }
            }

            update_post_meta($postId, '_dpc-sistrix-last-week', $sistrixLastWeek);
            update_post_meta($postId, '_dpc-sistrix-current-week', $sistrixCurrentWeek);
        }

        return $this->getRankingTrend($postId, $keyword);
    }

    function getRankingTrend($postId = false, $keyword = false, $lastRank = false, $currentRank = false){
        $data              = array();
        $data['post_id']   = $postId;
        $data['keyword']   = $keyword;

        if($postId !== false && $keyword !== false){
            $keyword               = trim($keyword);
            $sistrixLastWeek       = get_post_meta($postId, '_dpc-sistrix-last-week', true);
            $sistrixCurrentWeek    = get_post_meta($postId, '_dpc-sistrix-current-week', true);

            $data['last_rank']     = isset($sistrixLastWeek[$keyword]) ? $sistrixLastWeek[$keyword] : 0;
            $data['current_rank']  = isset($sistrixCurrentWeek[$keyword]) ? $sistrixCurrentWeek[$keyword] : 0;
        } else {
            $data['last_rank']     = $lastRank;
            $data['current_rank']  = $currentRank;
        }

        $data['diff'] = ($data['current_rank'] - $data['last_rank']);

        if($data['diff'] == 0){
            $data['trend'] = 'equal';
            $data['icon'] = 'dashicons-arrow-right';
        } elseif($data['diff'] < 0){
            $data['trend'] = 'up';
            $data['icon'] = 'dashicons-arrow-up';
        } else {
            $data['trend'] = 'down';
            $data['icon'] = 'dashicons-arrow-down';
        }

        return $data;
    }

    function ajaxSistrixQueueAdd(){
        if(isset($this->settings['sistrix_api_key']) && strlen($this->settings['sistrix_api_key'])){
            if(isset($_REQUEST['post_id']) && !empty($_REQUEST['post_id'])){
                $keywords = $this->getPostKeywords(intval($_REQUEST['post_id']));
                if(is_array($keywords) && count($keywords)){
                    foreach($keywords as $keyword){
                        $this->sistrixQueueAdd(intval($_REQUEST['post_id']), $keyword);
                    }
                }
            }
        }
        echo wp_json_encode(get_option('dpc_sistrix_queue', array()));
        die();
    }

    function gethelpLink(){
        return htmlspecialchars('https://support.google.com/analytics/answer/1008080?hl=de&utm_medium=et&utm_campaign=en_us&utm_source=SetupChecklist', ENT_QUOTES);
    }

    function adminSettingsHead() {
        wp_enqueue_style('dpc-basic-statistics-settings', plugins_url( 'assets/css/settings.css', __FILE__ ));
    }

    function adminSettings(){
        $this->dpc->parseSettingsByXml($this->settingsFile, 'general');
    }

    function adminHead(){
        add_action('admin_enqueue_scripts', array(&$this, 'hook_admin_enqueue_scripts'));
    }

    function hook_admin_enqueue_scripts(){
        wp_enqueue_script('dpc-basic-statistics-backend', plugins_url( 'assets/js/backend.js', __FILE__ ), array('jquery'));
        $postLang = $this->dpc->oFunctions->getPostLang(get_the_ID(), false);
        $postLang = substr($postLang, 0, (strpos($postLang, '_') ? strpos($postLang, '_') : 2));

        $queue = get_option('dpc_sistrix_queue', array());
        $str = '	var dpc_basic_statistics_post_language = \'' . $postLang . '\'' . ";\n";
        $str .= '	var dpc_basic_statistics_settings = ' . wp_json_encode($this->dpc->getModuleInstance('basic_statistics')->settings) . ";\n";
        $str .= '	var dpc_basic_statistics_sistrix_queue = ' . wp_json_encode($queue) . ";\n";
        wp_add_inline_script('dpc-basic-statistics-backend', $str, 'before');
    }

    function executeBackendActions(){
        $this->dpc->registerAjaxAction('dpc_basic_statistics_diagramm', array($this, 'ajaxSaveStatisticsInterval'));
        $this->dpc->registerAjaxAction('dpc_basic_statistics_sistrix_queue_add', array($this, 'ajaxSistrixQueueAdd'));
        $this->dpc->registerAjaxAction('dpc_basic_statistics_sistrix_sync', array($this, 'ajaxSistrixSyncPost'));
        add_action('load-edit.php', array(&$this, 'hook_load_edit_php'));
        add_action('init', array($this, 'sistrixWeeklySync'));

    }

    function hook_load_edit_php(){
        if((isset($this->settings['sistrix_api_key']) && strlen($this->settings['sistrix_api_key'])) || (isset($this->settings['dpc_external_ranking_system']) && strlen($this->settings['dpc_external_ranking_system']))){
            add_filter( 'manage_posts_columns', 				array($this, 'addColumns'));
            add_filter( 'manage_edit-post_sortable_columns',	array($this, 'sortableColumns'));
            add_action( 'manage_posts_custom_column', 			array($this, 'contentColumns'), 11, 2 );
            add_filter( 'manage_page_posts_columns', 			array($this, 'addColumns'));
            add_filter( 'manage_edit-page_sortable_columns',	array($this, 'sortableColumns'));
            add_action( 'manage_page_posts_custom_column', 		array($this, 'contentColumns'), 11, 2 );
            add_filter( 'manage_edit-product_sortable_columns',	array($this, 'sortableColumns'));
        }
    }

    function addColumns($columns) {
        // check for selected external ranking system and display only that column connected to that system
        $external_ranking_system = $this->settings['dpc_external_ranking_system'];

        if (isset($external_ranking_system)) {
            if ($external_ranking_system === 'dpc_sistrix_system') {
                $columns['dpc-ranking'] = esc_html__('Ranking', 'delucks-seo');
            } elseif ($external_ranking_system === 'dpc_google_search_console_system') {
                $columns['dpc-gsc-rankings'] = esc_html__('GSC Rankings', 'delucks-seo');
            }
        }

        return $columns;
    }

    function contentColumns($column, $postID){
        $rankingDisplay = '';
        if ($column == 'dpc-ranking'){
            $lastWeek              = 0;
            $currentWeek           = 0;
            $sistrixLastWeek       = get_post_meta($postID, '_dpc-sistrix-last-week', true);
            $sistrixCurrentWeek    = get_post_meta($postID, '_dpc-sistrix-current-week', true);
            $activeKeywords        = $this->getPostKeywords($postID);

            $lw_count = 0;
            $cw_count = 0;
            foreach($activeKeywords as $keyword){
                $keyword = trim($keyword);
                if(is_array($sistrixLastWeek) && isset($sistrixLastWeek[$keyword]) && $sistrixLastWeek[$keyword] > 0){
                    $lastWeek += $sistrixLastWeek[$keyword];
                    $lw_count++;
                }

                if(is_array($sistrixCurrentWeek) && isset($sistrixCurrentWeek[$keyword]) && $sistrixCurrentWeek[$keyword] > 0){
                    $currentWeek += $sistrixCurrentWeek[$keyword];
                    $cw_count++;
                }
            }

            $lastWeek       = $lw_count > 0 ? round($lastWeek / $lw_count) : 0;
            $currentWeek    = $cw_count > 0 ? round($currentWeek / $cw_count) : 0;
            $rankingData    = $this->getRankingTrend(false, false, $lastWeek, $currentWeek);
            $rankingDisplay = '<span class="ranking dpc-ranking" title="'.esc_attr(__('Last Week', 'delucks-seo')).': '.esc_html($rankingData['last_rank']).'">
            						<span class="dots-loader">
	            						<span class="dot dot-one"></span>
	            						<span class="dot dot-two"></span>
	            						<span class="dot dot-three"></span>
	            						<span class="dot dot-four"></span>
            						</span>
            						<span class="ranking_count">'. esc_html($rankingData['current_rank']) .'</span>
            						<span class="ranking_trend '. esc_attr($rankingData['trend']) .'"><i class="dpc-ranking-icon dashicons '. $rankingData['icon'] .'"></i></span>
            					</span>';
            echo ($lastWeek ==false && $currentWeek == false) ? '&mdash;' : $rankingDisplay;
            
        } elseif ($column == 'dpc-gsc-rankings') {
            // Add Google Search Rankings in the Rankings column of the all pages admin page
            // Retrieve the JSON data from the wp_options table
            $dpc_gsc_data = json_decode(get_option('dpc_gsc_data'));
            $rows = $dpc_gsc_data->rows;

            // Counter variable to track the number of displayed keywords
            $counter = 0;
            $found = false; // Flag variable to track if $url and $row->keys were found

            // Run the foreach loop only if $rows has data
            if (!empty($rows)) {
                // Loop through the rows and display the data for the first five keywords
                foreach ($rows as $row) {
                    $url = get_permalink($post->ID);
                    if (in_array($url, $row->keys)) {
                        $rankingDisplay = '<span class="ranking dpc-ranking gsc-ranking" title="'.esc_attr(__('Position', 'delucks-seo')).': '.round($row->position, 1).'"><span class="dots-loader"><span class="dot dot-one"></span><span class="dot dot-two"></span><span class="dot dot-three"></span><span class="dot dot-four"></span></span><span class="ranking_count border-right">'. round($row->position, 1) .'</span></span>';

                        echo nl2br($rankingDisplay);

                        $found = true; // Set the flag to true

                        // Increment the counter
                        $counter++;
                        break; // Break the loop after finding one result
                    }
                }
            }

            // If $url and $row->keys were not found, echo &mdash; only if no match was found
            if (!$found && $counter === 0) {
                echo "&mdash;";
            }


        }
    }

    function sortableColumns( $columns ) {
        $columns['dpc-ranking'] = 'dpc-ranking';
        $columns['dpc-gsc-rankings'] = 'dpc-gsc-rankings';
        return $columns;
    }

    function executeFrontendActions(){
        wp_enqueue_script('dpc-basic-statistics-frontend', plugins_url( 'assets/js/frontend.js', __FILE__ ), array('jquery'), DPC_VERSION, $this->dpc->load_scripts_in_footer);
        add_action('wp_head', array($this, 'outputTrackingCodes'));
        add_action('wp_footer', array($this, 'outputTrackingCodesBody'));
    }

    function ajaxSaveStatisticsInterval(){
        if(isset($_REQUEST['dpc_statistics_interval'])){
            update_option('dpc_basic_statistics_interval', intval($_REQUEST['dpc_statistics_interval']));
        }
        require_once( 'widgets/dashboard.php' );
        $obj = new DelucksStatistic($this->dpc);
        $obj->ajax_dpc_basic_statistics_diagramm();
        die();
    }

    function outputTrackingCodes(){
        if(isset($this->settings['trackingCodes']) && strlen($this->settings['trackingCodes']) > 0){
            //analytics local hosting
            if(isset($this->settings['analytics']) && isset($this->settings['analytics']['hostAnalyticsLocal']) && $this->settings['analytics']['hostAnalyticsLocal'] == 'on' && is_readable($this->getAnalyticsDir())){
                $this->settings['trackingCodes'] = str_replace(array('http://www.google-analytics.com/analytics.js', 'https://www.google-analytics.com/analytics.js', 'http://google-analytics.com/analytics.js', 'https://google-analytics.com/analytics.js', '//google-analytics.com/analytics.js', '//www.google-analytics.com/analytics.js'), $this->getAnalyticsUrl(), $this->settings['trackingCodes']);
            }

            if(strlen($this->settings['trackingCodes']) && $this->dpc->oFunctions->pluginActive('contact-form-7')){
                $this->settings['trackingCodes'] .= "
					<script>
					document.addEventListener( 'wpcf7mailsent', function( event ) {
					    ga('send', 'event', 'Contact Form', 'submit');
					}, false );
					</script>";
            }

            //analytics opt-out
            if(isset($this->settings['analytics']['gaOptOut']) && $this->settings['analytics']['gaOptOut'] == 'on'){
                $matches = array();
                preg_match("/ga\('create', '(.*)', 'auto'\)/i", $this->settings['trackingCodes'], $matches);
                if(count($matches) && isset($matches[1])){
                    $gaUaCode = $matches[1];
                    $optOutCode = "\n".'<script type="text/javascript">;// <![CDATA['."\n";
                    $optOutCode .= "var gaProperty = '".$gaUaCode."';"."\n";

                    $optOutCode .= "var disableStr = 'ga-disable-' + gaProperty;"."\n";
                    $optOutCode .= "if (document.cookie.indexOf(disableStr + '=true') > -1) {"."\n";
                    $optOutCode .= "	window[disableStr] = true;"."\n";
                    $optOutCode .= "}"."\n";

                    $optOutCode .= "function gaOptout() {"."\n";
                    $optOutCode .= "	document.cookie = disableStr + '=true; expires=Thu, 31 Dec 2099 23:59:59 UTC; path=/';"."\n";
                    $optOutCode .= "	window[disableStr] = true;"."\n";
                    $optOutCode .= "}"."\n";

                    $optOutCode .= "</script>"."\n";
                    $this->settings['trackingCodes'] = $optOutCode . $this->settings['trackingCodes'];
                }
            }

            echo "\n".'<script type="text/javascript">'."\n";
            echo '	var dpc_basic_statistics_settings = ' . wp_json_encode($this->settings) . ";\n";
            echo '	var dpc_basic_statistics_hostname = \'' . esc_js($_SERVER['SERVER_NAME']) . "';\n";
            echo '</script>'."\n";

            if((!isset($this->settings['analytics']['trackingOptIn']) || $this->settings['analytics']['trackingOptIn'] !== 'on') || (isset($_COOKIE['dpcTrackingCodesOptIn']) && $_COOKIE['dpcTrackingCodesOptIn'] == '1')){
                echo  stripslashes($this->settings['trackingCodes']);
            } else {
                echo "\n".'<script type="text/javascript">'."\n";
                echo "function dpcTrackingCodesOptIn(){";
                echo "	jQuery('head').append(dpc_basic_statistics_settings.trackingCodes);";
                echo "	document.cookie = 'dpcTrackingCodesOptIn=1';";
                echo "}";
                echo '</script>'."\n";
            }

            wp_enqueue_script('dpc-basic-statistics-google-analytics-option', plugins_url( 'assets/js/ganalyticsoption.js', __FILE__ ), array('jquery'), $this->dpc->load_scripts_in_footer);
        }
    }

    function outputTrackingCodesBody(){
        if(isset($this->settings['trackingCodesBody']) && strlen($this->settings['trackingCodesBody']) > 0){
            echo stripslashes($this->settings['trackingCodesBody']) . "\r\n";
        }
    }

    function registerWidgets(){
        add_action( 'widgets_init', array(&$this, 'hook_widgets_init'));
    }

    function hook_widgets_init(){
        require_once( 'widgets/ga_opt_out.php' );
        register_widget( 'DelucksGaOptOut' );
    }

    function modalEnqueueStyles(){
        wp_enqueue_style('dpc-basic-statistics-modalstyles', plugins_url( 'assets/css/modal.css', __FILE__ ));
    }

    function modalEnqueueScripts(){
        wp_enqueue_script('dpc-basic-statistics-modal', plugins_url( 'assets/js/modal.js', __FILE__ ), array('jquery'), DPC_VERSION, true);
    }

    function initXMLText(){
        esc_html__('Statistics', 'delucks-seo');
        esc_html__('Track hits on your site and let them be displayed in your dashboard, if you want to use our internal statistics. Else, implement here your Google Analytics tracking code instead and disable internal tracking.', 'delucks-seo');
        esc_html__('This module doesn�t need any data privacy note, because neither the IP address nor other user specific information is being stored to your website. You can exclude user groups from being tracked and add an additional considerable statistic codes.', 'delucks-seo');
        esc_html__('Activate statistics to see the current website access.', 'delucks-seo');
        esc_html__('Activated', 'delucks-seo');
        esc_html__('Deactivated', 'delucks-seo');
        esc_html__('Exclude tracking for', 'delucks-seo');
        esc_html__('We recommend you to except all the user groups from the tracking, who collaborate in your website contents, to avoid forged access numbers.', 'delucks-seo');
        esc_html__('Tracking-Codes', 'delucks-seo');
        esc_html__('Visit Google Analytics', 'delucks-seo');
        esc_html__('Google Analytics How-To', 'delucks-seo');
        esc_html__('Anonymize IP address', 'delucks-seo');
        esc_html__('Usergroups', 'delucks-seo');
        esc_html__('Post statistics are not affected.', 'delucks-seo');
        esc_html__('Activate Internal Statistics', 'delucks-seo');
        esc_html__('Host analytics local', 'delucks-seo');
        esc_html__('Activate Google Analytics Opt-out code', 'delucks-seo');
        esc_html__('Please setup cronjob', 'delucks-seo');
        esc_html__('Use the PHP code if you want to embed the link directly in your theme.', 'delucks-seo');
        esc_html__('Load Tracking-Codes after Opt-in', 'delucks-seo');
    }

    function dpc_analytics_opt_out($atts, $content = null){
        $a = shortcode_atts( array(
            'link_text' => '',
        ), $atts );

        return '<a onclick="alert(\''.esc_attr(__('Google Analytics has been deactivated', 'delucks-seo')).'\');" href="javascript:gaOptout();">'.(isset($a['link_text']) && strlen($a['link_text']) ? $a['link_text'] : esc_html__('Deactivate Google Analytics', 'delucks-seo')).'</a>';
    }

    function getAnalyticsDir(){
        return dirname(__FILE__) . '/assets/js/analytics.js';
    }

    function getAnalyticsUrl(){
        return plugins_url( 'assets/js/analytics.js', __FILE__ );
    }

    function getAnalyticsUpdateScriptUrl(){
        return plugins_url( 'analytics-update.php', __FILE__ );
    }

    /**
     * ======================================================================================
     * 
     * Google Search Console Integration
     * 
     * ======================================================================================
     */

    /**
     * Initialize Google Search Console integration.
     * Enqueues scripts, localizes data for the Google API client, registers cron jobs, and AJAX actions.
     *
     * @action admin_enqueue_scripts Enqueue scripts and localize data for the Google API client integration.
     * @action wp Schedule the cron job to run every day at midnight.
     * @action gsc_cron_job Define the function to run during the cron job.
     * @action wp_ajax_openai_api_request Register the AJAX action for OpenAI API requests (authenticated users).
     * @action wp_ajax_nopriv_openai_api_request Register the AJAX action for OpenAI API requests (non-authenticated users).
     * @action wp_ajax_save_selected_open_ai_prompt_option Register the AJAX action for saving OpenAI prompt options (authenticated users).
     * @action wp_ajax_nopriv_save_selected_open_ai_prompt_option Register the AJAX action for saving OpenAI prompt options (non-authenticated users).
     */
    private function init_gsc_integration() {
        // Enqueue scripts and localize data for the Google API client integration.
        add_action( 'admin_enqueue_scripts', array( $this, 'dpc_google_api_client' ) );

        // Register the cron job to run every day at midnight.
        add_action( 'init', array( $this, 'dpc_gsc_cron_job' ) );

        // Define the function to run.
        add_action( 'gsc_cron_job', array( $this, 'dpc_gsc_update_data' ) );

        // Register the AJAX action.
        add_action( 'wp_ajax_openai_api_request', array( $this, 'openai_api_request_callback' ) );
        add_action( 'wp_ajax_nopriv_openai_api_request', array( $this, 'openai_api_request_callback' ) );
        
        add_action( 'wp_ajax_save_selected_open_ai_prompt_option', array( $this, 'save_selected_open_ai_prompt_option' ) );
        add_action( 'wp_ajax_nopriv_save_selected_open_ai_prompt_option', array( $this, 'save_selected_open_ai_prompt_option' ) ); // Allows non-logged-in users to access the AJAX endpoint.

        // Run GSC code when the class is instantiated.
        $this->dpc_exchange_code_for_tokens();
    }

        
    /**
     * Enqueue scripts and localize data for the Google API client integration.
     *
     * Enqueues necessary scripts for the Google API client integration,
     * including the Google Sign-In API script and custom scripts.
     * Localizes client_id, redirect_uris, and authentication state to be passed to JavaScript.
     */
    public function dpc_google_api_client() {

        wp_enqueue_script( 'google-api-api', 'https://apis.google.com/js/api.js', array(), '1.0', false );
        wp_enqueue_script( 'google-api-client', 'https://accounts.google.com/gsi/client', array(), '1.0', false );
        wp_script_add_data( 'google-api-client', 'onload', 'initClient' );
        wp_script_add_data( 'google-api-client', 'async', true );
        wp_script_add_data( 'google-api-client', 'defer', true );
        wp_enqueue_script( 'jquery' );
        wp_enqueue_script( 'dpc-gsc-script', plugin_dir_url( dirname( dirname( __DIR__ ) ) ) . 'assets/dpc/js/google-auth-button.js', array( 'jquery' ), '1.0', true );

        $settings           = get_option( 'dpc_basic_statistics' );
        $client_secret_json = $settings['dpc_gsc_json_client_secret'] ?? '';
        $client_secret_json = stripslashes( $client_secret_json );
        $client_secret_data = json_decode( $client_secret_json, true );

        if ( json_last_error() === JSON_ERROR_NONE ) {
            $client_id      = $client_secret_data['web']['client_id'];
            $client_id_data = array(
                'client_id' => $client_id,
            );
            $json_client_id = wp_json_encode( $client_id_data );
            // Pass client_id to JavaScript file using wp_localize_script.
            wp_localize_script( 'dpc-gsc-script', 'my_client_id', $json_client_id );

            $redirect_uris      = $client_secret_data['web']['redirect_uris'];
            $redirect_uris_data = array(
                'redirect_uris' => $redirect_uris,
            );
            $json_redirect_uris = wp_json_encode( $redirect_uris_data );
            // Pass redirect_uri to JavaScript file using wp_localize_script.
            wp_localize_script( 'dpc-gsc-script', 'my_redirect_uris', $json_redirect_uris );
        }

        // get Google OAuth 2.0 State from the database.
        $dpc_google_console_connection_state = get_option( 'dpc_google_console_connection_state' );
        if ( $dpc_google_console_connection_state ) {
            $dpc_google_console_connection_state_data = array(
                'auth_state' => $dpc_google_console_connection_state,
            );
            $json_dpc_google_console_connection_state = wp_json_encode( $dpc_google_console_connection_state_data );
            wp_localize_script( 'dpc-gsc-script', 'my_dpc_google_console_connection_state', $json_dpc_google_console_connection_state );
        }
    }
    


    /**
     * Load webmasters data from the Google Search Console API.
     *
     * Retrieves and processes data from the Google Search Console API based on stored settings,
     * including client credentials, date range, and site URL. The retrieved data is then stored in the database.
     */
    public function load_webmasters_data() {
        // get client ids, client secret, redirect_uri, client credentials from the database.
        $settings           = get_option( 'dpc_basic_statistics' );
        $client_secret_json = $settings['dpc_gsc_json_client_secret'];
        $client_secret_json = stripslashes( $client_secret_json );
        $client_secret_data = json_decode( $client_secret_json, true );
        if ( json_last_error() === JSON_ERROR_NONE ) {
            $client_id           = $client_secret_data['web']['client_id'];
            $client_secret       = $client_secret_data['web']['client_secret'];
            $redirect_uris_array = $client_secret_data['web']['redirect_uris'];
            // iterate the redirect uri array and find the right uri associated with this domain.
            $redirect_uri = $this->get_valid_redirect_uri( $redirect_uris_array );
            $site_url     = '';

            if ( isset( $_SERVER['HTTP_HOST'] ) ) {
                $protocol = ( isset( $_SERVER['HTTPS'] ) && 'on' === $_SERVER['HTTPS'] ) ? 'https' : 'http';
                $host     = isset( $_SERVER['HTTP_HOST'] ) ? sanitize_text_field( wp_unslash( $_SERVER['HTTP_HOST'] ) ) : '';

                $site_url = esc_url( $protocol . '://' . $host );

            }
        }
        $grant_type    = 'authorization_code';
        $one_week_ago  = gmdate( 'Y-m-d', strtotime( '-1 week' ) );
        $one_month_ago = gmdate( 'Y-m-d', strtotime( '-1 month' ) );
        $days_ago_90   = gmdate( 'Y-m-d', strtotime( '-90 days' ) );
        $start_date    = ( '7' === $settings['date_range'] ) ? $one_week_ago : ( ( '30' === $settings['date_range'] ) ? $one_month_ago : $days_ago_90 );
        $end_date      = gmdate( 'Y-m-d' );
        $dimensions    = array( 'query', 'page' );
        $search_type   = 'web';
        $access_token  = get_option( 'dpc_google_access_token' );
        $refresh_token = get_option( 'dpc_google_refresh_token' );

        $url = 'https://www.googleapis.com/webmasters/v3/sites/' . rawurlencode( $site_url ) . '/searchAnalytics/query';

        $headers = array(
            'Authorization' => 'Bearer ' . $access_token,
            'Content-Type'  => 'application/json',
        );

        $body = array(
            'startDate'  => $start_date,
            'endDate'    => $end_date,
            'dimensions' => $dimensions,
            'searchType' => $search_type,
        );

        $args = array(
            'headers' => $headers,
            'body'    => wp_json_encode( $body ),
        );

        $response = wp_remote_post( $url, $args );

        // Check for WP_Error.
        if ( is_wp_error( $response ) ) {
            $error_message = $response->get_error_message();
            // Handle the error message as needed.
            $this->add_admin_message( 'error', wp_kses( __( 'Error: ' . $error_message, 'delucks-seo' ), '' ) ); // phpcs:ignore -- Ignore this rule because it's a plugin function that I didn't create and can't modify.
            return;
        }

        // Decode the response body.
        $response_body    = wp_remote_retrieve_body( $response );
        $decoded_response = json_decode( $response_body, true );
        $error_displayed  = false;

        if ( isset( $decoded_response['error'] ) && ! $error_displayed ) {
            $error_code    = $decoded_response['error']['code'];
            $error_message = $decoded_response['error']['message'];
            // Handle the error code and message as needed.
            $this->add_admin_message( 'error', wp_kses( __( 'Error: ' . $error_code . '-' . $error_message . ' Make sure that this domain is indexed in the Google Search Console', 'delucks-seo' ), '' ) ); // phpcs:ignore -- Ignore this rule because it's a plugin function that I didn't create and can't modify.
            $error_displayed = true;
            return;
        }

        // No errors, process the response data.
        update_option( 'dpc_gsc_data', $response_body ); // Store the response in the database.
    }

    /**
     * Exchange the authorization code for access and refresh tokens.
     *
     * If the 'code' parameter is set in the query string, this function exchanges it for access and refresh tokens
     * using the Google OAuth2 token endpoint. The tokens are then stored in the database.
     */
    public function dpc_exchange_code_for_tokens() {
        // Generic.Commenting.Todo.Found -- Security is primarily handled by Google Tokens, no need for nonces.
        if ( isset( $_GET['code'] ) ) { // phpcs:ignore 
            // Unslash the code parameter.
            $auth_code = isset( $_GET['code'] ) ? sanitize_text_field( wp_unslash( $_GET['code'] ) ) : ''; // phpcs:ignore 
            // get client ids, client secret, redirect_uri, client credentials from the database.
            $settings           = get_option( 'dpc_basic_statistics' );
            $client_secret_json = $settings['dpc_gsc_json_client_secret'];
            $client_secret_json = stripslashes( $client_secret_json );
            $client_secret_data = json_decode( $client_secret_json, true );
            if ( json_last_error() === JSON_ERROR_NONE ) {
                $client_id           = $client_secret_data['web']['client_id'];
                $client_secret       = $client_secret_data['web']['client_secret'];
                $redirect_uris_array = $client_secret_data['web']['redirect_uris'];
                // iterate the redirect uri array and find the right uri associated with this domain.
                $redirect_uri = $this->get_valid_redirect_uri( $redirect_uris_array );
                $site_url     = '';

                if ( isset( $_SERVER['HTTP_HOST'] ) ) {
                    $protocol = ( isset( $_SERVER['HTTPS'] ) && 'on' === $_SERVER['HTTPS'] ) ? 'https' : 'http';
                    $host     = isset( $_SERVER['HTTP_HOST'] ) ? sanitize_text_field( wp_unslash( $_SERVER['HTTP_HOST'] ) ) : '';

                    $site_url = esc_url( $protocol . '://' . $host );

                }
            }

            $grant_type    = 'authorization_code';
            $one_week_ago  = gmdate( 'Y-m-d', strtotime( '-1 week' ) );
            $one_month_ago = gmdate( 'Y-m-d', strtotime( '-1 month' ) );
            $days_ago_90   = gmdate( 'Y-m-d', strtotime( '-90 days' ) );
            $start_date    = ( '7' === $settings['date_range'] ) ? $one_week_ago : ( ( '30' === $settings['date_range'] ) ? $one_month_ago : $days_ago_90 );
            $end_date      = gmdate( 'Y-m-d' );
            $dimensions    = array( 'query', 'page' );
            $search_type   = 'web';
            // Set the endpoint URL.
            $url = 'https://oauth2.googleapis.com/token';

            // Set the request parameters.
            $params = array(
                'grant_type'    => 'authorization_code',
                'code'          => $auth_code,
                'client_id'     => $client_id,
                'client_secret' => $client_secret,
                'redirect_uri'  => $redirect_uri,
            );

            // Send the POST request to the endpoint.
            $response = wp_remote_post(
                $url,
                array(
                    'body' => $params,
                )
            );

            // Check if there was an error with the request.
            if ( is_wp_error( $response ) ) {
                return false;
            }

            // Parse the response body as JSON.
            $response_body = wp_remote_retrieve_body( $response );
            $json_data     = json_decode( $response_body, true );

            // Return the access token and refresh token.
            // Check if access_token and refresh_token are not null, empty, or undefined.
            if ( ! empty( $json_data['access_token'] ) && ! empty( $json_data['refresh_token'] ) ) {
                // Access token and refresh token are valid, store them in the database.
                $this->store_tokens_in_database( $json_data['access_token'], $json_data['refresh_token'] );

                // Get GSC data and store it in the database.
                $this->load_webmasters_data();

                // Store the state in the database.
                $this->store_state_in_database( true );
            }
        }
    }   


    /**
     * Store Google API access and refresh tokens in the database.
     *
     * This function updates the WordPress options for Google API access and refresh tokens with the provided values.
     *
     * @param string $access_token  The Google API access token to be stored.
     * @param string $refresh_token The Google API refresh token to be stored.
     */
    public function store_tokens_in_database( $access_token, $refresh_token ) {
        // Update the options.
        update_option( 'dpc_google_access_token', $access_token );
        update_option( 'dpc_google_refresh_token', $refresh_token );
    }


    /**
     * Regenerate the access token using the refresh token.
     *
     * Retrieves client credentials and other necessary information from the database,
     * then sends a request to the Google OAuth2 token endpoint to regenerate the access token
     * using the stored refresh token. If successful, the new access token is stored in the database.
     *
     * @return string|false The new access token if regeneration is successful, false otherwise.
     */
    public function regenerate_access_token() {
        // get client ids, client secret, redirect_uri, client credentials from the database.
        $settings           = get_option( 'dpc_basic_statistics' );
        $client_secret_json = $settings['dpc_gsc_json_client_secret'];
        $client_secret_json = stripslashes( $client_secret_json );
        $client_secret_data = json_decode( $client_secret_json, true );
        if ( json_last_error() === JSON_ERROR_NONE ) {
            $client_id           = $client_secret_data['web']['client_id'];
            $client_secret       = $client_secret_data['web']['client_secret'];
            $redirect_uris_array = $client_secret_data['web']['redirect_uris'];
            // iterate the redirect uri array and find the right uri associated with this domain.
            $redirect_uri = $this->get_valid_redirect_uri( $redirect_uris_array );
            $site_url     = '';

            if ( isset( $_SERVER['HTTP_HOST'] ) ) {
                $protocol = ( isset( $_SERVER['HTTPS'] ) && 'on' === $_SERVER['HTTPS'] ) ? 'https' : 'http';
                $host     = isset( $_SERVER['HTTP_HOST'] ) ? sanitize_text_field( wp_unslash( $_SERVER['HTTP_HOST'] ) ) : '';

                $site_url = esc_url( $protocol . '://' . $host );

            }
        }
        $grant_type    = 'refresh_token';
        $one_week_ago  = gmdate( 'Y-m-d', strtotime( '-1 week' ) );
        $one_month_ago = gmdate( 'Y-m-d', strtotime( '-1 month' ) );
        $days_ago_90   = gmdate( 'Y-m-d', strtotime( '-90 days' ) );
        $start_date    = ( '7' === $settings['date_range'] ) ? $one_week_ago : ( ( '30' === $settings['date_range'] ) ? $one_month_ago : $days_ago_90 );
        $end_date      = gmdate( 'Y-m-d' );
        $dimensions    = array( 'query', 'page' );
        $search_type   = 'web';
        $refresh_token = get_option( 'dpc_google_refresh_token' );

        // Make the token request to the API.
        $body = array(
            'client_id'     => $client_id,
            'client_secret' => $client_secret,
            'grant_type'    => $grant_type,
        );

        if ( $refresh_token ) {
            $body['refresh_token'] = $refresh_token;
        }

        $response = wp_remote_post(
            'https://oauth2.googleapis.com/token',
            array(
                'body' => $body,
            )
        );

        // If the token request is successful, return the new access token.
        if ( ! is_wp_error( $response ) && in_array( $response['response']['code'], array( 200, 201 ), true ) ) {
            $body = json_decode( $response['body'] );
            update_option( 'dpc_google_access_token', $body->access_token );
            // Run load_webmasters_data function if token generation is successful.
            $this->load_webmasters_data();
            return $body->access_token;
        } else {
            // If the token request fails, log the error and return false.
            return false;
        }
    }

    /**
     * Check if a Google access token is valid by querying Google's token information endpoint.
     *
     * This function sends a request to Google's token information endpoint with the provided access token.
     * It checks for errors in the response and the expiration status of the token.
     *
     * @param string $access_token The Google access token to be validated.
     *
     * @return bool True if the token is valid; otherwise, false.
     */
    public function is_google_token_valid( $access_token ) {
        $url      = 'https://www.googleapis.com/oauth2/v3/tokeninfo?access_token=' . $access_token;
        $response = wp_remote_get( $url );

        if ( is_wp_error( $response ) ) {
            // Error handling.
            return false;
        }

        $response_body = wp_remote_retrieve_body( $response );
        $data          = json_decode( $response_body );

        if ( isset( $data->error ) ) {
            // Error handling.
            return false;
        }

        if ( $data->expires_in <= 0 ) {
            // Token has expired.
            return false;
        }

        return true;
    }

    /**
     * Schedule and manage the Google Search Console (GSC) cron job.
     *
     * This function clears any existing scheduled GSC cron job hooks, then schedules
     * a new daily event named 'gsc_cron_job' starting from midnight today.
     * The purpose of this cron job is to trigger the update of GSC data on a daily basis.
     */
    public function dpc_gsc_cron_job() {
        wp_clear_scheduled_hook( 'gsc_cron_job' );
        if ( ! wp_next_scheduled( 'gsc_cron_job' ) ) {
            wp_schedule_event( strtotime( 'today midnight' ), 'daily', 'gsc_cron_job' );
        }
    }

    /**
     * Updates Google Search Console (GSC) data.
     *
     * This function checks the validity of the Google access token. If the token is valid,
     * it loads the latest GSC data using the `load_webmasters_data` function. If the token is invalid or expired,
     * it regenerates a new access token using the `regenerate_access_token` function.
     */
    public function dpc_gsc_update_data() {
        $access_token = get_option( 'dpc_google_access_token' );

        if ( $this->is_google_token_valid( $access_token ) ) {
            // Token is valid, get latest GSC data..
            $this->load_webmasters_data();
        } else {
            // Regenerate new access_token if it expired or is invalid.
            $this->regenerate_access_token();
        }
    }

    /**
     * Determines a valid redirect URI from a given array based on the current server environment.
     *
     * @param array $redirect_uris_array An array of redirect URIs to check against.
     *
     * @return string|null The valid redirect URI if found; otherwise, null.
     */
    public function get_valid_redirect_uri( $redirect_uris_array ) {
        // Ensure that the code below assumes the presence of valid values for $_SERVER['HTTP_HOST']
        // and $_SERVER['REQUEST_URI']. The following logic relies on these server variables being set.
        // If they are not set, adjustments may be needed to handle such cases appropriately.
        $current_url = isset( $_SERVER['HTTP_HOST'] ) ? sanitize_text_field( wp_unslash( $_SERVER['HTTP_HOST'] ) ) : '';
        $current_uri = isset( $_SERVER['REQUEST_URI'] ) ? sanitize_text_field( wp_unslash( $_SERVER['REQUEST_URI'] ) ) : '';

        $current_url = $current_url . $current_uri;

        // Iterate over the redirect URIs and check if the current URL matches any of them.
        foreach ( $redirect_uris_array as $redirect_uri ) {
            if ( strpos( $current_url, $redirect_uri ) === 0 ) {
                return $redirect_uri;
            }
        }

        // If no direct match is found, check if the current URL matches the host/domain of any redirect URI.
        foreach ( $redirect_uris_array as $redirect_uri ) {
            $url           = wp_wp_parse_url( $redirect_uri );
            $redirect_host = isset( $url['host'] ) ? sanitize_text_field( $url['host'] ) : '';

            if ( strpos( $current_url, $redirect_host ) !== false ) {
                return $redirect_uri;
            }
        }

        return null;
    }


    /**
     * Adds an admin message to the options table.
     *
     * This function adds or updates an admin message with the provided type and text to the WordPress options table.
     * If a message with the same type and text already exists, it updates the existing message; otherwise, it adds a new message.
     *
     * @param string $type - The type of the admin message.
     * @param string $text - The text content of the admin message.
     */
    public function add_admin_message( $type, $text ) {
        $messages = get_option( 'dpc_admin_messages', array() );

        // Check if the same error message already exists.
        $existing_message_index = false;
        foreach ( $messages as $index => $message ) {
            if ( $message['type'] === $type && $message['text'] === $text ) {
                $existing_message_index = $index;
                break;
            }
        }

        if ( false !== $existing_message_index ) {
            // Update the existing error message.
            $messages[ $existing_message_index ]['type'] = $type;
            $messages[ $existing_message_index ]['text'] = $text;
        } else {
            // Add a new error message.
            $messages[] = array(
                'type' => $type,
                'text' => $text,
                'hash' => md5( $type . $text . time() ),
            );
        }

        update_option( 'dpc_admin_messages', $messages );
    }


    /**
     * Handle storing the state variable in the options table.
     *
     * This function updates the 'dpc_google_console_connection_state' option in the WordPress options table
     * with the provided authentication code status.
     *
     * @param string $auth_code_status - The authentication code status to be stored.
     */
    public function store_state_in_database( $auth_code_status ) {
        update_option( 'dpc_google_console_connection_state', $auth_code_status );
    }


    /*
    ============================================================================
        Add OpenAI functions
    =============================================================================
    */  

    /**
     * Callback function for the AJAX request.
     *
     * This function handles AJAX requests related to OpenAI API.
     * It retrieves the messages content from the POST request, unslashes and sanitizes it,
     * then calls the openai_api_request function with the sanitized content.
     * Finally, it echoes the JSON-encoded response and exits to prevent WordPress from returning extra data.
     */
    public function openai_api_request_callback() {
        if ( isset( $_POST['messages_content'] ) ) {
            // Unslash and sanitize the messages content in one line.
            $messages_content = sanitize_text_field( wp_unslash( $_POST['messages_content'] ) );

            // Call the openai_api_request function.
            $response = $this->openai_api_request( $messages_content );

            // Return a response (if needed).
            echo wp_json_encode( $response ); // You can encode the response as JSON or return it in any desired format.

            // Always exit to prevent WordPress from returning extra data.
            wp_die();
        }
    }


    /**
     * Make a request to the OpenAI API for chat completions.
     *
     * Sends a message content to the OpenAI API using the specified model and API key.
     *
     * @param string $messages_content The content of the user's message.
     *
     * @return mixed|array|string The decoded response from the OpenAI API, or an error message if an issue occurs.
     */
    public function openai_api_request( $messages_content ) {
        $settings = get_option( 'dpc_basic_statistics' );
        $api_key  = $settings['dpc_open_ai_key']; // Replace with your OpenAI API key.
        $model    = $settings['dpc_gpt_modal']; // Replace with your OpenAI Model (e.g., gpt-3.5-turbo or gpt-4).
        $messages = array(
            array(
                'role'    => 'user',
                'content' => $messages_content,
            ),
        );

        $data = array(
            'model'    => $model, // Fixed the variable name.
            'messages' => $messages,
        );

        $response = wp_remote_post(
            'https://api.openai.com/v1/chat/completions',
            array(
                'timeout' => 15000,
                'headers' => array(
                    'Content-Type'  => 'application/json',
                    'Authorization' => 'Bearer ' . $api_key,
                ),
                'body'    => wp_json_encode( $data ),
            )
        );

        if ( is_wp_error( $response ) ) {
            // Handle WP error.
            return 'WP Error: ' . $response->get_error_message();
        }

        $body = wp_remote_retrieve_body( $response );

        if ( empty( $body ) ) {
            // Handle empty response.
            return 'Empty Response';
            // Comment: Logging the response for debugging purposes in production should be avoided.
        }

        // Decode the JSON response.
        $decoded_response = json_decode( $body );

        if ( null === $decoded_response ) {
            // Handle JSON decoding error.
            return 'JSON Decoding Error';
        }

        // Return the decoded response.
        return $decoded_response;
    }

    /**
     * Save selected OpenAI prompt option.
     *
     * If the 'saveOption' and 'updatedText' parameters are set in the POST request, sanitize and save the data
     * in the database table 'dpc_open_ai_saved_prompts'. It uses WordPress global $wpdb for database operations.
     * It returns a response indicating the success of the operation.
     */
    public function save_selected_open_ai_prompt_option() {
        if ( isset( $_POST['saveOption'] ) && isset( $_POST['updatedText'] ) ) {
            $save_option  = sanitize_text_field( wp_unslash( $_POST['saveOption'] ) );
            $updated_text = sanitize_text_field( wp_unslash( $_POST['updatedText'] ) );

            global $wpdb;
            $table_name = $wpdb->prefix . 'dpc_open_ai_saved_prompts';

            // Prepare data for insertion.
            $data = array(
                'option' => $save_option,
                'text'   => $updated_text,
            );

            // Format data types.
            $format = array(
                '%s', // option.
                '%s', // text.
            );

            // Insert data into the database.
            $wpdb->insert( $table_name, $data, $format ); // phpcs:ignore WordPress.DB.DirectDatabaseQuery.DirectQuery, WordPress.DB.DirectDatabaseQuery.NoCaching

            // You can return a response if needed.
            wp_send_json_success( 'Data saved successfully' );
        }

        // Terminate the script.
        wp_die();
    }

}

if(!function_exists('dpc_analytics_opt_out')){
    function dpc_analytics_opt_out(){
        if(shortcode_exists('[dpc_analytics_opt_out]')){
            return do_shortcode('[[dpc_analytics_opt_out]]');
        }
    }
}