<?php

class DPC_Module_Basic_Urls implements DPC_Module_Interface{

	var $dpc				= null;
	var $isSubmodule		= true;
	var $moduleDirectory 	= 'basic/';
	var $moduleTitle 		= '';
	var $optionsHook		= 'dpc-basic-urls';
	var $settingsFile		= false;
	var $settings			= array();
	var $lang				= '';

	function registerWidgets(){ }
	function adminHead(){ }


	function __construct(&$dpc){
		$this->dpc 			= $dpc;
		$this->moduleTitle	= esc_html__('URL optimization', 'delucks-seo');
		$this->settingsFile = dirname(__FILE__) . '/settings.xml';
		$this->settings 	= $this->dpc->getModuleSettings('basic_urls');
	}

	function adminSettings(){
		$this->dpc->parseSettingsByXml($this->settingsFile, 'general');
	}

	function adminSettingsHead(){
		wp_enqueue_style('dpc-basic-urls-settings', plugins_url( 'assets/css/settings.css', __FILE__ ));
	}


	function executeBackendActions(){
		if (isset($this->settings['stopwords']) && isset($this->settings['stopwords']) == 'on'){
			#add_filter('wp_insert_post_data', array($this, 'applyStopWords'), 1000, 2);
			add_filter( 'wp_unique_post_slug', array($this, 'customUniquePostSlug'), 1000, 4 );
		}
	}

	function executeFrontendActions(){
	    if(!is_admin() && isset($this->settings['forceSSL'])){
	        if(!isset($_SERVER['HTTPS']) || $_SERVER['HTTPS'] != 'on'){
	            if(isset($_SERVER['HTTP_HOST'])){
	                $permanent = true;
	                $url = 'https://'.$_SERVER['HTTP_HOST'].$_SERVER['REQUEST_URI'];
	                header('Location: ' . $url, true, $permanent ? 301 : 302);
	                exit;
	            }
	        }
	    }
	    
	    add_action('wp', array($this, 'set404Header'));
	}

	function set404Header(){
		$userLang = $this->dpc->oFunctions->getUserLang();
		if(isset($this->settings) && isset($this->settings[404]) && isset($this->settings[404][$userLang])){
			$redirectData = $this->settings[404][$userLang];
			if($redirectData['type'] !== 'category' && $redirectData['type'] !== 'nothing' && $redirectData['type'] !== 'url'){
				if($redirectData[$redirectData['type']]['value'] == get_the_ID()){
					status_header(404);
					nocache_headers();
				}
			}
		} elseif((is_array($this->settings) && isset($this->settings['404page']) && isset($this->settings[$this->settings['404page']])) && is_int(get_the_ID())){
	 		$id = $this->settings[$this->settings['404page']];
	 		if($id == get_the_ID()){
	 			status_header(404);
	 			nocache_headers();
	 		}
	 	}
	}

	function applyStopWords($data, $postarr){
		$language = isset($postarr['icl_post_language']) ? $postarr['icl_post_language'] : '';
		$postname = $data['post_name'];
		$post_ID  = isset($_POST['post_ID']) ? intval($_POST['post_ID']) : '0';

		$names = explode('-', $postname);
		$names = preg_replace($this->getStopWordsPattern($language), '', $names);
		$names = array_filter($names);

		$postname = implode('-', $names);
		$postname = wp_unique_post_slug($postname, $post_ID, $data['post_status'], $data['post_type'], $data['post_parent']);

		$data['post_name'] = $postname;

		return $data;
	}

	function customUniquePostSlug($slug, $post_ID, $post_status, $post_type){
		if(in_array($post_type, array('attachment', 'image'))){
			return $slug;
		}

		$language = $this->dpc->oFunctions->getPostLang($post_ID, false);
		$language = substr($language, 0, strpos($language, '_'));

		$names = explode('-', $slug);
		$names = preg_replace($this->getStopWordsPattern($language), '', $names);
		$names = array_filter($names);

		$slug = implode('-', $names);
		return $slug;
	}

	function getStopWordsPattern($language = ''){
		if ($language == ''){
			$language = $this->getLang();
		}
		$stopWords 	= $this->getStopWords($language);
		foreach($stopWords as &$item){
			$item = '|^'.remove_accents($item).'$|i';
		}
		return $stopWords;
	}

	function getLang(){
		if (defined('ICL_LANGUAGE_CODE')){
			$lang = ICL_LANGUAGE_CODE;
		}else {
			$lang = substr(get_locale(), 0, strpos(get_locale(), '_'));
		}

		return $lang;
	}

	function getStopWords($lang){
		if (isset($this->settings[$lang]['stopwordlist'])){
			return explode(',', $this->settings[$lang]['stopwordlist']);
		}

		return array();
	}

	function filterSanitizeTitle($title, $raw_title, $context){
		if ($context != 'save') return $title;

		$titles	= explode('-', $title);
		$titles = preg_replace($this->stopWordsPattern, '', $titles);
		for($i = 0; $i < count($titles); $i++){
			if (empty($titles[$i])){
				unset($titles[$i]);
			}
		}

		return implode('-', $titles);
	}

	function showForm($objectType = null){
		if($objectType == 'url'){
			$this->dpc->parseSettingsByXml($this->settingsFile, 'manager');
		}
	}

	function managerUpdate(){
		$formdata		= $_REQUEST['dpc']['basic_urls'];
		$targetPostId	= $formdata[$formdata['404page']];

		if($formdata['404page'] == 'url'){
			update_option('dpc-suite-updated-'.md5($_REQUEST['object_id']), true);
			$this->updateRedirectData(intval($_REQUEST['object_id']), array('post_id' => '', 'status' => 1, 'type' => $formdata['404page'], 'target_url' => $formdata['url']));
		} elseif($formdata['404page'] == 'nothing'){
			update_option('dpc-suite-updated-'.md5($_REQUEST['object_id']), true);
			$this->updateRedirectData(intval($_REQUEST['object_id']), array('post_id' => '', 'status' => 1, 'type' => $formdata['404page'], 'target_url' => ''));
		} else {
			if(get_post($targetPostId)){
				update_option('dpc-suite-updated-'.md5($_REQUEST['object_id']), true);
				$this->updateRedirectData(intval($_REQUEST['object_id']), array('post_id' => $targetPostId, 'status' => 1, 'type' => $formdata['404page'], 'target_url' => ''));
			}
		}
		die();
	}

	function initXMLText(){
	    esc_html__('URL optimization', 'delucks-seo');
	    esc_html__('This module improves the your website urls. The url includes everything, which appears in the browsers address bar (e.g. https://www.delucks.com/seo).', 'delucks-seo');
	    esc_html__('The keywords in a vital part to almost every url of your website as these are very important for SEO. The earlier a keyword appears in searches and the shorter your url is, the better it is for your website. So, you should exclude irrelevant words like mine, yours, the, he, she, it from your urls. We call those words: stopwords.', 'delucks-seo');
	    esc_html__('Activate the url optimization to remove stopwords from your urls. Redirect visitors from dead links and ensure a secure connection.', 'delucks-seo');
	    esc_html__('Activated', 'delucks-seo');
	    esc_html__('Deactivated', 'delucks-seo');
	    esc_html__('Remove stopwords from urls', 'delucks-seo');
	    esc_html__('Stopwords', 'delucks-seo');
	    esc_html__('Default error page', 'delucks-seo');
		esc_html__('Category', 'delucks-seo');
		esc_html__('Custom URL', 'delucks-seo');
		esc_html__('Do nothing', 'delucks-seo');
		esc_html__('Target', 'delucks-seo');
		esc_html__('Set up a standard error page, where all visitors will be redirected, who click on dead links on your website.', 'delucks-seo');
		esc_html__('Learn how to structure the perfect 404 page', 'delucks-seo');
		esc_html__('Force encrypted connection', 'delucks-seo');
		esc_html__('Please only activate after having set up a SSL certificate for your domain and haveing set up the https urls in your general WordPress settings.', 'delucks-seo');
		esc_html__('Go to settings page', 'delucks-seo');
		esc_html__('Redirect to', 'delucks-seo');
		esc_html__('Domain for ranking check', 'delucks-seo');
		esc_html__('If WordPress is used headless on a different domain, we need the domain of the live system to compare Google rankings.', 'delucks-seo');
	}

}