<?php

class DPC_Module_Dpc_Importexport {

	var $dpc				= null;
	var $isSubmodule		= true;
	var $moduleTitle 		= '';
	var $moduleDirectory 	= 'importexport/';
	var $optionsHook		= 'dpc-dpc-importexport';
	var $defaultSettingsDir	= '';
	var $defaultSettingsUrl	= '';
	var $settingsFile		= false;
	var $settings			= false;
	var $doAction		 	= false;
	var $saveComment 		= '';
	var $files				= array();
	var $migrationAdapters	= array();

	function __construct(&$dpc){
		$this->settingsFile 		= dirname(__FILE__) . '/settings.xml';
		$this->dpc 					= $dpc;
		$this->moduleTitle			= esc_html__('Backup and migration', 'delucks-seo');
		$this->settings 			= $this->dpc->getModuleSettings('dpc_importexport');
		$this->defaultSettingsDir	= plugin_dir_path( __FILE__ ) . 'default_settings/';
		$this->defaultSettingsUrl	= plugin_dir_url( __FILE__ ) . 'default_settings/';
		
		$this->dpc->registerAjaxAction('dpc_dashboard_upload_settings_file', array($this, 'uploadSettingsFile'));
		$this->getMigrationAdapters();

		if(!$this->dpc->show_expert_settings){
			add_filter('admin_head', array($this, 'hideExpertSettings'));
		}

		if(isset($_GET['masterSettingsImported'])){
		    array_push($this->dpc->messages, array('type' => 'success', 'text' => wp_kses(__('The recommended settings have been installed', 'delucks-seo'), $this->dpc->text_allowed_html) . '.'));
		}
	}
	

	function uploadSettingsFile(){
	    $done = false;
	    if(function_exists('current_user_can') && (current_user_can('administrator') || current_user_can('editor'))){
	        if(isset($_POST['file']) && isset($_POST['file']['fileData']) && strlen($_POST['file']['fileData'])){
	            #list($dataType, $dataContent) = explode(',', $_POST['file']['fileData']);
	            #$dataContent = base64_decode(base64_decode($dataContent)); // decode data uri AND base64 encoded content
	            
	            if(strlen($dataContent)){
	                $dpcFunctions = new DPC_Functions($this->dpc);
	                $fileData = json_decode($dataContent);
	                $fileData = $dpcFunctions->object2Array($fileData);

	                if(is_array($fileData) && isset($fileData['settings'])){
	                    foreach($fileData['settings'] as $moduleName => $settings){
	                        $done = true;
	                        #update_option('dpc_'.$moduleName, $settings);
	                        if(isset($fileData['status']) && isset($fileData['status'][$moduleName])){
	                            #update_option('dpc_status_' . $moduleName, $fileData['status'][$moduleName]);
	                        }
	                    }
	                }
	            }
	        }
	    }
	    
	    if($done){
	        $this->migrationInstalled = true;
	        $response = array('succeed' => 1, 'messages' => array(array('type' => 'success', 'text' => esc_html__('Settings have been restored', 'delucks-seo'))));
	        $this->dpc->addAdminMessage('success', wp_kses(__('Settings have been restored.', 'delucks-seo'), $this->dpc->text_allowed_html));
	    } else {
	        $this->migrationInstalled = false;
	        $response = array('succeed' => 0, 'messages' => array(array('type' => 'danger', 'text' => esc_html__('Cant restore settings. Settings file seems to be broken.', 'delucks-seo'))));
	        $this->dpc->addAdminMessage('error', wp_kses(__('Cant restore settings. Settings file seems to be broken.', 'delucks-seo'), $this->dpc->text_allowed_html));
	    }
	    
	    echo wp_json_encode($response);
	    die();
	}
	

	function hideExpertSettings(){
		echo '<style type="text/css">';
		echo '	[data-expert_mode="true"]{ display: none !important; }';
		echo '</style>';
	}

	function getTranslationJs(){
		return array(
		    'save' 											=> esc_html__('Save', 'delucks-seo'),
		    'cancel' 										=> esc_html__('Cancel', 'delucks-seo'),
		    'close' 										=> esc_html__('Close', 'delucks-seo'),
		    'Migration wizard'								=> esc_html__('Migration wizard', 'delucks-seo'),
		    'Please select at least one option.'			=> esc_html__('Please select at least one option.', 'delucks-seo'),
		    'Yes, continue'									=> esc_html__('Yes, continue', 'delucks-seo'),
		    'Attention'										=> esc_html__('Attention', 'delucks-seo'),
		    'Override all settings?'   						=> esc_html__('Override all settings?', 'delucks-seo'),
		    'With the completion of the migration wizard, all data previously selected will be overwritten. Are you sure you want to perform this step?'	=>	esc_html__('With the completion of the migration wizard, all data previously selected will be overwritten. Are you sure you want to perform this step?', 'delucks-seo'),
		);
	}

	function activatePlugin(){ }

	function getMigrationAdapter($name){
		if(isset($this->migrationAdapters[$name])){
			return $this->migrationAdapters[$name];
		}
		return null;
	}

	function getMigrationAdapters(){
		if(!count($this->migrationAdapters)){
			foreach(scandir(dirname(__FILE__) . '/migration/') as $filename){
				if(substr( $filename, 0, 1 ) === '_'){
					continue;
				}

				if(!preg_match('/\.php$/', $filename)) continue;

				$adapterClassName = get_class() . '_Migration_' . ucfirst(preg_replace('/\.php$/', '', $filename));
				$adapterClassFile = dirname(__FILE__) . '/migration/' . $filename;
				if(is_dir($adapterClassFile) || !is_readable($adapterClassFile)) continue;

				require_once($adapterClassFile);
				$adapter = new $adapterClassName($this->dpc);
				$this->migrationAdapters[$adapter->adapterKey] = $adapter;
			}
		}

		$adapters = array();
		if(count($this->migrationAdapters)){
		    foreach($this->migrationAdapters as $adapterName => $adapter){
		        $adapters[$adapterName] = array('adapter' => $adapter->adapterKey, 'label' => esc_html__($adapter->adapterName, 'delucks-seo'), 'version' => $adapter->adapterVersion, 'directory' => $adapter->adapterDirectory, 'installed' => (int)$this->dpc->oFunctions->pluginExists($adapter->adapterDirectory));
			}

			$this->dpc->oFunctions->aksort($adapters, true);
		}

		return $adapters;
	}

	function migrationStart(){
		return $this->executeMigrationAdapterMethod($_REQUEST['adapter'], 'migrate');
	}

	function executeMigrationAdapterMethod($adapter, $method, $param = ''){
		$adapter = $this->getMigrationAdapter($adapter);
		if(is_object($adapter) && method_exists($adapter, $method)){
			if(strpos($param, '#') !== false){
				return call_user_func_array(array($adapter, $method), explode('#', $param));
			} else {
				return $adapter->$method($param);
			}
		}
		return false;
	}

	function migrationWizard(){
		if(is_object($adapter = $this->getMigrationAdapter($_REQUEST['adapter']))){
			echo '<form method="POST" id="migrationWizard">';
			echo '	<input type="hidden" name="adapter" value="'.esc_attr($_REQUEST['adapter']).'" />';
			echo '	<input type="hidden" name="current_step" value="'.(isset($_REQUEST['step']) ? esc_attr($_REQUEST['step']) : '').'" />';
			$adapter->showWizard();
			echo '</form>';
		}
		die();
	}

	function adminSettings(){
		$this->dpc->parseSettingsByXml($this->settingsFile, 'general');
	}

	function adminSettingsHead(){
		wp_enqueue_style('dpc-dpc-importexport-settings', plugins_url( 'assets/css/settings.css', __FILE__ ));
		wp_enqueue_script('dpc-dpc-importexport-settings', plugins_url( 'assets/js/settings.js', __FILE__ ), array('jquery'));
		wp_localize_script('dpc-dpc-importexport-settings', 'dpc_dpc_importexport_translator', $this->getTranslationJs());
		wp_enqueue_script('dpc-dpc-importexport-wizard', plugins_url( 'assets/bootstrap-wizard/jquery.bootstrap.wizard.min.js', __FILE__ ), array('jquery'));
	}

	function adminHead(){ }
	function executeFrontendActions(){ }

	function executeBackendActions(){
		if(isset($_GET['action']) && $_GET['action'] == 'restore'){ $this->restore(); }
		$this->dpc->registerAjaxAction('dpc_importexport_download', array($this, 'download'));
		$this->dpc->registerAjaxAction('dpc_importexport_migration_wizard', array($this, 'migrationWizard'));
		$this->dpc->registerAjaxAction('dpc_importexport_migration_execute', array($this, 'migrationStart'));
	}

	function download(){
		if(isset($_GET['system'])){
			if(!isset($_GET['file']) || !is_readable($file = $this->defaultSettingsDir . str_replace('../', '', $_GET['file']))){
				return false;
			}
		} else {
			if(!isset($_GET['file']) || !is_readable($file = $this->dpc->uploadDir . 'settings/' . str_replace('../', '', $_GET['file']))){
				return false;
			}
		}

		header('Content-Description: File Transfer');
		header('Content-Type: text/dpc');
		header('Content-Disposition: attachment; filename='.str_replace('../', '', $_GET['file']));
		header('Content-Transfer-Encoding: binary');
		header('Expires: 0');
		header('Cache-Control: must-revalidate, post-check=0, pre-check=0');
		header('Pragma: public');
		header('Content-Length: ' . filesize($file));
		readfile($file);
		exit;
	}

	function setupDefault(){
		if(!is_readable($file = $this->defaultSettingsDir . 'settings-master.dpc')){
			return false;
		}

		require_once(DPC_PLUGIN_HELPER_DIR . 'functions.php');
		$dpcFunctions = new DPC_Functions($this->dpc);
		$fileData = $dpcFunctions->object2Array($this->readSettingsByFile('settings-master.dpc', true));

		if(is_array($fileData) && isset($fileData['settings'])){
			foreach($fileData['settings'] as $moduleName => $settings){
				$done = true;
				delete_option('dpc_'.$moduleName);
				update_option('dpc_'.$moduleName, $settings);
				if(isset($fileData['status']) && isset($fileData['status'][$moduleName])){
					update_option('dpc_status_' . $moduleName, $fileData['status'][$moduleName]);

				}
			}
			return true;
		}
	}

	function restore(){
		$done = false;
		if(isset($_GET['system'])){
			if(!isset($_GET['file']) || !is_readable($file = $this->defaultSettingsDir . str_replace('../', '', $_GET['file']))){
				return false;
			}
		} else {
			if(!isset($_GET['file']) || !is_readable($file = $this->dpc->uploadDir . 'settings/' . str_replace('../', '', $_GET['file']))){
				return false;
			}
		}

		require_once(DPC_PLUGIN_HELPER_DIR . 'functions.php');
		$dpcFunctions = new DPC_Functions($this->dpc);
		$fileData = $dpcFunctions->object2Array($this->readSettingsByFile(str_replace('../', '', $_GET['file']), (isset($_GET['system']) ? true : false)));

		if(is_array($fileData) && isset($fileData['settings'])){
			foreach($fileData['settings'] as $moduleName => $settings){
				$done = true;
				update_option('dpc_'.$moduleName, $settings);
				if(isset($fileData['status']) && isset($fileData['status'][$moduleName])){
					update_option('dpc_status_' . $moduleName, $fileData['status'][$moduleName]);
				}
			}
		}

		if($done){
		    array_push($this->dpc->messages, array('type' => 'success', 'text' => esc_html__('Settings have been restored.', 'delucks-seo') . '!'));
		} else {
		    array_push($this->dpc->messages, array('type' => 'error', 'text' => esc_html__('Cant restore settings. Settings file seems to be broken.', 'delucks-seo') ));
		}
	}

	function delete(){
		if(isset($this->files) && is_array($this->files) && count($this->files)){
			foreach($this->files as $file => $v){
				if(!is_readable($file = $this->dpc->uploadDir . 'settings/' . $file)){
					continue;
				}
				wp_delete_file($file);
			}
		}
	}

	function upload(){
		if(isset($_FILES['settingsfile']) && !$_FILES['settingsfile']['error']){
			rename($_FILES['settingsfile']['tmp_name'], $file = $this->dpc->uploadDir . 'settings/' . $_FILES['settingsfile']['name']);
			wp_delete_file($_FILES['settingsfile']['tmp_name']);
		}
	}


	function onSaveActionBefore(){
		if(isset($_POST['dpc']['dpc_dashboard']['action'])){
			$this->doAction 	= $_POST['dpc']['dpc_dashboard']['action'];
			$this->saveComment	= isset($_POST['dpc']['dpc_dashboard']['saveComment']) ? $_POST['dpc']['dpc_dashboard']['saveComment']: '';
			$this->files		= isset($_POST['dpc']['dpc_dashboard']['file']) ? $_POST['dpc']['dpc_dashboard']['file']: '';
			unset($_POST['dpc']['dpc_dashboard']['action']);
			unset($_POST['dpc']['dpc_dashboard']['saveComment']);
			unset($_POST['dpc']['dpc_dashboard']['file']);
		}
	}

	function onSaveActionAfter(){
		switch($this->doAction){
			case 'save':
				$this->saveSettings();
				break;
			case 'delete':
				$this->delete();
				break;
			case 'upload':
				$this->upload();
				break;
		}
	}

	function saveSettings(){
		$postData = isset($_POST['dpc']['dpc_dashboard']['save']) ? $_POST['dpc']['dpc_dashboard']['save'] : array();
		if(!count($postData)){ return; }

		$settings = array();
		foreach($postData as $moduleKey => $value){
			//if(in_array($moduleKey, array('dpc_dashboard'))){ continue; } //exclude from export
			if(count($moduleSettings = $this->dpc->getModuleSettings($moduleKey))){
				$settings['settings'][$moduleKey] = $moduleSettings;
			}
			$settings['status'][$moduleKey] = ($this->dpc->moduleIsActive($moduleKey) ? 1 : 0);
		}

		$settings['fileInfo']['comment']	= $this->saveComment;
		$settings['fileInfo']['timestamp']	= time();
		$settings_encoded 					= base64_encode(wp_json_encode($settings));
		$filename 							= $this->dpc->uploadDir . 'settings/settings-' . gmdate('Y-m-d', $settings['fileInfo']['timestamp']) . '-' . strtoupper(substr(md5(time()), 0, 5)) .  '.dpc';

		$this->dpc->setupPluginDirectory($this->dpc->uploadDir . 'settings');
		$fh = fopen($filename, 'w');
		fwrite($fh,$settings_encoded);
		fclose($fh);
	}


	function restoreSettings(){
		#$settingsFile = json_decode(base64_decode($settings_encoded));
		#$upload_dir = wp_upload_dir();
		//'dpc_status_' . $moduleName;
	}

	function getFiles(){
		$data = array();

		if(is_readable($this->defaultSettingsDir)){
			foreach(scandir($this->defaultSettingsDir) as $file){
				if($file == '.' || $file == '..'){ continue; }
				$fileInfo		= $this->getFileInfo($file, true);
				$urlRestore 	= '?page=' . $_GET['page'] . '&action=restore&file='.$file.'&system=true';
				$urlDownload 	= 'admin-ajax.php?&action=dpc_importexport_download&file='.$file.'&system=true';
				$data[]			= array('id' => md5($file), 'system' => 1, 'filename' => $file, 'restore_url' => $urlRestore, 'download_url' => $urlDownload, 'comment' => $fileInfo['comment'], 'timestamp' => $fileInfo['timestamp'], 'date' => 'System', 'time' => 'Default');
			}
		}

		if(is_readable($this->dpc->uploadDir . 'settings')){
			foreach(scandir($this->dpc->uploadDir . 'settings') as $file){
				if($file == '.' || $file == '..'){ continue; }
				$fileInfo		= $this->getFileInfo($file);
				$urlRestore 	= '?page=' . $_GET['page'] . '&action=restore&file='.$file;
				$urlDownload 	= 'admin-ajax.php?&action=dpc_importexport_download&file='.$file;
				$data[]			= array('id' => md5($file), 'system' => 0, 'filename' => $file, 'restore_url' => $urlRestore, 'download_url' => $urlDownload, 'comment' => $fileInfo['comment'], 'timestamp' => $fileInfo['timestamp'], 'date' => gmdate('d.m.Y', $fileInfo['timestamp']), 'time' => gmdate('H:i:s', $fileInfo['timestamp']));
			}
		}

		return $data;
	}

	function getFileInfo($filename, $default = false){
		$settings = $this->readSettingsByFile($filename, $default);
		if(is_object($settings) && property_exists($settings, 'fileInfo')){
			return (array)$settings->fileInfo;
		}
		return array();
	}

	function readSettingsByFile($filename, $default = false){
		if($default !== false && is_readable($file = $this->defaultSettingsDir . $filename)){
			return json_decode(base64_decode(file_get_contents($file)));
		}
		if(is_readable($file = $this->dpc->uploadDir . 'settings/' . $filename)){
			return json_decode(base64_decode(file_get_contents($file)));
		}
		return array();
	}

	function initXMLText(){
	    esc_html__('Backup and migration', 'delucks-seo');
	    esc_html__('Activated', 'delucks-seo');
	    esc_html__('Deactivated', 'delucks-seo');
	    esc_html__('Setup', 'delucks-seo');
		esc_html__('Backup', 'delucks-seo');
		esc_html__('Comment', 'delucks-seo');
		esc_html__('Backup', 'delucks-seo');
		esc_html__('Install our Master-Settings by click on the Setup button, to have an optimal preparation of our plugin. You can also import your settings and metadata from another SEO plugin below. You can also safe here your settings and import them in other projects of you.', 'delucks-seo');
		esc_html__('Date / Time', 'delucks-seo');
		esc_html__('Download', 'delucks-seo');
		esc_html__('Upload', 'delucks-seo');
		esc_html__('Performance', 'delucks-seo');
		esc_html__('Get more possibilities for fine tuning if you display the expert settings. By loading the Javascript files in the footer area, you can improve the the loading behavior of your website.', 'delucks-seo');
		esc_html__('No', 'delucks-seo');
		esc_html__('Yes', 'delucks-seo');
		esc_html__('Load Scriptfiles in footer', 'delucks-seo');
		esc_html__('Show expert settings', 'delucks-seo');
		esc_html__('Save', 'delucks-seo');
		esc_html__('Migration', 'delucks-seo');
		esc_html__('Settings', 'delucks-seo');
		esc_html__('Metadata', 'delucks-seo');
		esc_html__('Overview', 'delucks-seo');
		esc_html__('What would you like to migrate?', 'delucks-seo');
		esc_html__('Select all', 'delucks-seo');
		esc_html__('Unselect all', 'delucks-seo');
		esc_html__('Sitemaps', 'delucks-seo');
		esc_html__('URL optimization', 'delucks-seo');
		esc_html__('Microdata', 'delucks-seo');
		esc_html__('Breadcrumbs', 'delucks-seo');
		esc_html__('Which metadatas should be migrated?', 'delucks-seo');
		esc_html__('Focus keyword', 'delucks-seo');
		esc_html__('SEO Title', 'delucks-seo');
		esc_html__('Meta description', 'delucks-seo');
		esc_html__('Migrate data only if source data is not empty', 'delucks-seo');
		esc_html__('The following settings will be migrated from Yoast:', 'delucks-seo');
		esc_html__('Attention', 'delucks-seo');
		esc_html__('Please make sure that the specified URL have a valid SSL certificate! Make sure to set your domain to https:// within the Wordpress adress and the page URL.', 'delucks-seo');
		esc_html__('Go to settings page', 'delucks-seo');
		esc_html__('The following metadatas will be migrated from Yoast', 'delucks-seo');
		esc_html__('Migration process completed.', 'delucks-seo');
		esc_html__('Migration process successfully completed. Please deactivate the Yoast SEO plugin and review your settings within in DELUCKS SEO Plugin.', 'delucks-seo');
		esc_html__('Are you sure you want to migrate? Existing metadata will be overridden.', 'delucks-seo');
		esc_html__('Migration process successfully completed. Please deactivate the All in one SEO plugin and review your metadata.', 'delucks-seo');
		esc_html__('Migration process successfully completed. Please deactivate the SEO Ultimate plugin and review your metadata.', 'delucks-seo');
		esc_html__('Migration process successfully completed. Please deactivate the wpSEO plugin and review your metadata.', 'delucks-seo');
		esc_html__('PageSpeed Test', 'delucks-seo');
		esc_html__('Performane Optimization', 'delucks-seo');
		esc_html__('Are you sure?', 'delucks-seo');
		esc_html__('Reminder for text optimization', 'delucks-seo');
	}

}