<?php

class DPC_Module_Dpc_Wizard {

    var $dpc				= null;
    var $isSubmodule		= true;
    var $moduleDirectory 	= 'wizard/';
    var $moduleTitle 		= '';
    var $optionsHook		= 'dpc-dpc-wizard';
    var $settingsFile		= false;
    var $settings			= false;
    var $translatorJs		= array();
    var $defaultsInstalled  = false;
    var $migrationInstalled = false;

    function __construct(&$dpc){
        #delete_option('dpc_wizard_done'); // reset wizard

        $this->settingsFile 		       = dirname(__FILE__) . '/settings.xml';
        $this->dpc 					       = $dpc;
        $this->moduleTitle			       = esc_html__('Backup and migration', 'delucks-seo');
        $this->settings                    = $this->dpc->getModuleSettings('dpc_wizard');
        $this->translatorJs = array(
            'unlock' 	=> esc_html__('Unlock', 'delucks-seo')
        );

        $this->dpc->registerAjaxAction('dpc_dashboard_import_metadata_csv', array($this, 'importMetadataCsv'));
        $this->dpc->registerAjaxAction('dpc_dashboard_wizard_restore', array($this, 'wizardRestore'));
        $this->dpc->registerAjaxAction('dpc_dashboard_wizard_install_master', array($this, 'wizardInstallMaster'));

        if(isset($_GET['skipWizard'])){
            update_option('dpc_wizard_done', true);
        }
        #$this->wizardCheck();
    }

    function adminSettings(){
        $this->dpc->parseSettingsByXml($this->settingsFile, 'general');
    }

    function adminSettingsHead(){
        wp_enqueue_script('dpc-wizard-settings', plugins_url( 'assets/js/settings.js', __FILE__ ), array('jquery'));
        wp_localize_script('dpc-wizard-settings', 'dpc_wizard_translator', $this->translatorJs);
        wp_enqueue_style('dpc-wizard-settings', plugins_url( 'assets/css/settings.css', __FILE__ ));
    }

    function onSaveActionAfter(){
        if(is_admin()){
            require_once(ABSPATH . 'wp-includes/pluggable.php');
            if(is_user_logged_in()){
                if(isset($_POST['dpc']) && isset($_POST['dpc']['save_wizard'])){
                    $metadataSettings                      = $this->dpc->getModuleSettings('basic_metadata');
                    $metadataSettings['verify']['google']  = sanitize_text_field($_POST['dpc']['dpc_dashboard']['verify']['google']);
                    $metadataSettings['verify']['bing']    = sanitize_text_field($_POST['dpc']['dpc_dashboard']['verify']['bing']);
                    $licenseKey                            = sanitize_text_field($_POST['dpc']['dpc_dashboard']['active_license']);

                    /* set default titles: start */
                    $title		= get_option('blogname', '');
                    $subtitle	= get_option('blogdescription', '');

                    foreach($this->dpc->oFunctions->getLanguages() as $lang){
                        if(!isset($metadataSettings[$lang['lang_key']]) || !isset($metadataSettings[$lang['lang_key']]['title']) || !isset($metadataSettings[$lang['lang_key']]['title']['frontpage']) || empty($metadataSettings[$lang['lang_key']]['title']['frontpage'])){
                            $metadataSettings[$lang['lang_key']]['title']['frontpage'] = $subtitle;
                        }

                        if(!isset($metadataSettings[$lang['lang_key']]) || !isset($metadataSettings[$lang['lang_key']]['title']) || !isset($metadataSettings[$lang['lang_key']]['title']['website']) || empty($metadataSettings[$lang['lang_key']]['title']['website'])){
                            $metadataSettings[$lang['lang_key']]['title']['website'] = $title;
                        }

                        if(!isset($metadataSettings[$lang['lang_key']]) || !isset($metadataSettings[$lang['lang_key']]['title']) || !isset($metadataSettings[$lang['lang_key']]['title']['delimiter']) || empty($metadataSettings[$lang['lang_key']]['title']['delimiter'])){
                            $metadataSettings[$lang['lang_key']]['title']['delimiter'] = ' - ';
                        }
                    }
                    /* set default titles: end */

                    //TODO:

                    #echo "<pre>";
                    #var_dump($this->defaultsInstalled);
                    #var_dump($this->migrationInstalled);
                    #echo "</pre>";

                    #die();
                    update_option('dpc_basic_metadata', $metadataSettings);
                    $this->dpc->activateAllModules();

                    update_option('dpc_wizard_done', true);
                    array_push($this->dpc->messages, array('type' => 'success', 'text' => wp_kses(__('The wizard is now complete', 'delucks-seo'), $this->dpc->text_allowed_html)));
                }
            }
        }
    }

    function getVerifySettings(){
        $values = array('google' => '', 'bing' => '');
        $metadataSettings = $this->dpc->getModuleSettings('basic_metadata');
        if(isset($metadataSettings['verify']) && isset($metadataSettings['verify']['google'])) { $values['google'] = $metadataSettings['verify']['google']; }
        if(isset($metadataSettings['verify']) && isset($metadataSettings['verify']['bing'])) { $values['bing'] = $metadataSettings['verify']['bing']; }
        return $values;
    }

    function registerWidgets(){ }
    function adminHead(){ }
    function executeFrontendActions(){ }

    function wizardRestore(){
        $done = false;
        if(isset($_POST['file']) && isset($_POST['file']['fileData']) && strlen($_POST['file']['fileData'])){
            list($dataType, $dataContent) = explode(',', $_POST['file']['fileData']);
            $dataContent = base64_decode(base64_decode($dataContent)); // decode data uri AND base64 encoded content

            if(strlen($dataContent)){
                $dpcFunctions = new DPC_Functions($this->dpc);
                $fileData = json_decode($dataContent);
                $fileData = $dpcFunctions->object2Array($fileData);

                if(is_array($fileData) && isset($fileData['settings'])){
                    foreach($fileData['settings'] as $moduleName => $settings){
                        $done = true;
                        update_option('dpc_'.$moduleName, $settings);
                        if(isset($fileData['status']) && isset($fileData['status'][$moduleName])){
                            update_option('dpc_status_' . $moduleName, $fileData['status'][$moduleName]);
                        }
                    }
                }
            }
        }

        if($done){
            $this->migrationInstalled = true;
            $response = array('succeed' => 1, 'messages' => array(array('type' => 'success', 'text' => esc_html__('Settings have been restored', 'delucks-seo') . '!')));
            $this->dpc->addAdminMessage('success', wp_kses(__('Settings have been restored.', 'delucks-seo'), $this->dpc->text_allowed_html));
        } else {
            $this->migrationInstalled = false;
            $response = array('succeed' => 0, 'messages' => array(array('type' => 'danger', 'text' => esc_html__('Cant restore settings. Settings file seems to be broken.', 'delucks-seo'))));
            $this->dpc->addAdminMessage('error', wp_kses(__('Cant restore settings. Settings file seems to be broken.', 'delucks-seo'), $this->dpc->text_allowed_html));
        }

        echo wp_json_encode($response);
        die();
    }


    function wizardInstallMaster(){
        $done = false;
        if(is_object($importexport = $this->dpc->getModuleInstance('dpc_importexport'))){
            if($importexport->setupDefault()){
                $done = true;
            }
        }

        if($done){
            $this->defaultsInstalled = true;
            $response = array('succeed' => 1, 'messages' => array(array('type' => 'success', 'text' => esc_html__('Settings have been restored', 'delucks-seo') . '!')));
        } else {
            $this->defaultsInstalled = false;
            $response = array('succeed' => 0, 'messages' => array(array('type' => 'danger', 'text' => esc_html__('Cant restore settings. Settings file seems to be broken.', 'delucks-seo'))));
        }

        echo wp_json_encode($response);
        die();
    }
    
    
    function importMetadataCsv(){
        
        if(!isset($_REQUEST['file']) || !isset($_REQUEST['file']['fileData'])){
            $response = array('succeed' => 0, 'messages' => array(array('type' => 'danger', 'text' => esc_html__('Cant read file.', 'delucks-seo'))));
            echo wp_json_encode($response);
            die();
        }
        
        // make csv readable 
        if($_REQUEST['file']['fileData']){
            $b64 = $_REQUEST['file']['fileData'];
            $pos  = strpos($b64, ';');
            if($pos !== false){
                $b64 = base64_decode(preg_replace('#^data:text/\w+;base64,#i', '', $b64));
            }
            
            $lines = explode("\n", $b64);
        }

        // proceed
        if($lines && count($lines)){
            // _delucks-seo-status
            // _dpc-keyword
            $fields['url']                       = array('index' => 0, 'skip' => true);
            $fields['_dpc-keyword-list']         = array('index' => 1, 'type' => 'custom', 'delimiter' => '|');
            $fields['_dpc-meta-title']           = array('index' => 2, 'type' => 'text');
            $fields['_dpc-meta-description']     = array('index' => 3, 'type' => 'text');
            $fields['_dpc-no-index']             = array('index' => 4, 'type' => 'on_off');
            $fields['_dpc-no-follow']            = array('index' => 5, 'type' => 'on_off');
            $fields['_dpc-meta-og-title']        = array('index' => 6, 'type' => 'text');
            $fields['_dpc-meta-og-description']  = array('index' => 7, 'type' => 'text');
            $fields['_dpc-meta-og-image']        = array('index' => 8, 'type' => 'text');
            $fields['_dpc-hide-sharing-buttons'] = array('index' => 9, 'type' => 'on_off');
            $fields['_dpc-competitor-sites']     = array('index' => 10,'type' => 'custom', 'delimiter' => '|');
            
            $header = str_getcsv($lines[0], ';');   // get header
            unset($lines[0]);                       // remove header from dataset
            
            if(count($lines)){
                foreach($lines as $row){
                    $row        = str_getcsv($row, ';');
                    $postUrl    = $row[$fields['url']['index']];
                    
                    if(function_exists('icl_get_languages')){ // WPML FIX
                        $languagesKeys = array_keys(icl_get_languages());
                        $postUrl = preg_replace('#^/('.implode('|', array_keys(icl_get_languages())).')/#', '/', $postUrl);
                    }
                    
                    $postId     = url_to_postid($postUrl);
                    
                    if($postId){
                        foreach($fields as $fieldname => $config){
                            if((isset($config['skip']) && $config['skip'] == true) || !strlen(trim($row[$config['index']]))){ continue; }
                            $value = trim($row[$config['index']]);
                            
                            switch($value){
                                case '{empty}':
                                    update_post_meta($postId, $fieldname, '');
                                    $imported = true;
                                    continue 2;
                                case '{delete}':
                                    delete_post_meta($postId, $fieldname);
                                    $imported = true;
                                    continue 2;
                                default:
                            }

                            switch($config['type']){
                                case 'custom':
                                    switch($fieldname){
                                        case '_dpc-keyword-list':
                                            $value = explode($config['delimiter'], trim($value));
                                            if(count($value)){
                                                update_post_meta($postId, '_dpc-keyword', array_shift($value));
                                                update_post_meta($postId, '_dpc-keyword-list', $value);
                                                $imported = true;
                                            }
                                            break;
                                        case '_dpc-competitor-sites':
                                            $value = explode($config['delimiter'], trim($value));
                                            
                                            if(count($value)){
                                                $newValue = array();
                                                foreach($value as $url){
                                                    $newValue[] = array('url' => $url);
                                                }
                                                if(count($newValue)){
                                                    update_post_meta($postId, '_dpc-competitor-sites', $newValue);
                                                    $imported = true;
                                                }
                                            }
                                            break;
                                    }
                                    break;
                                case 'text':
                                    update_post_meta($postId, $fieldname, trim($value));
                                    $imported = true;
                                    break;
                                case 'array':
                                    update_post_meta($postId, $fieldname, explode($config['delimiter'], trim($value)));
                                    $imported = true;
                                    break;
                                case 'on_off':
                                    $value = strtolower($value);
                                    if($value == 'on' || $value == '1'){ $value = 'on'; }
                                    if($value == 'off' || $value == '0'){ $value = ''; }
                                    update_post_meta($postId, $fieldname, trim($value));
                                    $imported = true;
                                    break;
                            }
                        }
                    } 
                }
            }
        }
    
        
        if($imported == true){
            $response = array('succeed' => 1, 'messages' => array(array('type' => 'success', 'text' => esc_html__('Metadata have been imported.', 'delucks-seo'))));
        } else {
            $response = array('succeed' => 0, 'messages' => array(array('type' => 'danger', 'text' => esc_html__('Cant import Metadata.', 'delucks-seo'))));
        }
     
        echo wp_json_encode($response);
        die();
    }
    

    function wizardCheck(){
        if(!get_option('dpc_wizard_done') && !isset($_POST['dpc']['save_wizard'])){
            array_push($this->dpc->messages, array('type' => 'warning', 'text' => wp_kses(__('Please run our installation wizard once. As an expert or with an existing setup you can', 'delucks-seo'), $this->dpc->text_allowed_html) . ' <a href="?page=dpc-dpc&skipWizard">'. esc_html__('skip it here', 'delucks-seo') .'</a>.'));
        }
    }


}