<?php

class DPC_Module_Professional_Author implements DPC_Module_Interface{

	var $dpc						= null;
	var $isSubmodule				= true;
	var $moduleDirectory 		= 'professional/';
	var $moduleTitle 			= '';
	var $optionsHook				= 'dpc-professional-author';
	var $settingsFile			= false;
	var $settings				= array();
	var $defaultSeperator		= ' &raquo; ';
	var $defaultHomePrefix		= '';
	var $defaultArchivePrefix	= '';
	var $defaultSearchPrefix		= '';
	var $default404Prefix		= '';
	var $breacrumbChildIndex		= '';
	var $items                  = array();

	function __construct(&$dpc){
		$this->settingsFile 		= dirname(__FILE__) . '/settings.xml';
		$this->dpc 					= $dpc;
		$this->moduleTitle			= esc_html__('Author settings', 'delucks-seo');
		$this->settings 			= $this->dpc->getModuleSettings('professional_author');
	}

	function adminSettings(){
		$this->dpc->parseSettingsByXml($this->settingsFile, 'general');
	}

	public function adminSettingsHead(){
		wp_register_script('dpc-useredit', plugin_dir_url( __FILE__ ) . '/assets/js/module.js', array( 'jquery' ), null, true);
		wp_enqueue_script('dpc-useredit');
	}
	

	public function executeBackendActions(){
		add_filter('user_contactmethods', array($this, 'extend_author_profile'), 10, 1);
		add_action('webseide_author_social', array($this, 'author_social'));
		
		// Add the WP_Editor
		add_action( 'show_user_profile', array($this, 'visual_editor') );
		add_action( 'edit_user_profile', array($this, 'visual_editor') );
		
		// Don't sanitize the data for display in a textarea
		add_action( 'admin_init', array($this, 'save_filters') );
		
		// Add content filters to the output of the description
		add_filter( 'get_the_author_description', 'wptexturize' );
		add_filter( 'get_the_author_description', 'convert_chars' );
		add_filter( 'get_the_author_description', 'wpautop' );
		
		add_shortcode( 'author_social', array($this, 'author_social_shortcode') );
	    add_shortcode( 'dpc_author', array($this, 'author_social_shortcode') );

		add_action('personal_options_update', array($this, 'update_user_meta') );  
		
	}
	
	public function executeFrontendActions(){
		add_action('wp', array($this, 'init'));
		
	}

	function init(){
		add_shortcode( 'author_social', array($this, 'author_social_shortcode') );
	    add_shortcode( 'dpc_author_social', array($this, 'author_social_shortcode') );
	}

	public function registerWidgets(){
		
	}
	
	public function adminHead(){
		
	}
	
	function extend_author_profile( $fields ){
		$fields['facebook'] 			= 'Facebook';
		$fields['twitter'] 			= 'Twitter';
		$fields['instagram'] 		= 'Instagram';
		$fields['youtube'] 			= 'Youtube';
		$fields['linkedin'] 			= 'Linkedin';
		$fields['xing'] 				= 'Xing';
		$fields['profile_page'] 		= __('Profile Page', 'delucks-seo');
		$fields['profile_image'] 	= __('Profile image', 'delucks-seo');
		
		return $fields;
	}
	
	static function author_social($author){		
		$facebook 		= get_the_author_meta( 'facebook', $author );
		$twitter 		= get_the_author_meta( 'twitter', $author );
		$instagram 		= get_the_author_meta( 'instagram', $author );
		$youtube 		= get_the_author_meta( 'youtube', $author );
		$linkedin 		= get_the_author_meta( 'linkedin', $author );
		$xing 			= get_the_author_meta( 'xing', $author );			
		
		if($facebook == '' && $instagram == '' && $youtube == '' && $xing == '' && $linkedin == ''){ return; }
		
		$output  = '	<div class="webseide-social-widget">';
		
		if($facebook !== ''){
			$output .= '		<a href="' . $facebook . '" target="_blank" rel="nofollow" class="single-scial social-facebook">';
			$output .= '			<span class="single-social-icon social-icon-facebook">
									<svg version="1.1" id="Ebene_1" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" x="0px" y="0px" viewBox="0 0 32 32" style="enable-background:new 0 0 32 32;" xml:space="preserve"> <g> <path d="M20.2,0c0.9,0,1.7,0,2.6,0.1c0.6,0,1.1,0.1,1.6,0.2l0,5.2c-0.2,0-0.3,0-0.4,0c-0.6,0-1.1,0-1.7,0c-0.4,0-0.8,0-1.3,0 c-1.4,0-2.4,0.6-2.4,2.3c0,1.2-0.1,3.9-0.1,4.1c0.1,0,1.4,0,2.8,0c0.7,0,1.3,0,1.8,0c0.4,0,0.7,0,1.1,0c-0.3,2-0.5,3.9-0.8,5.8 h-4.9L18.4,32h-6l0.1-14.5H7.6l0-5.8h4.9l0-0.4c0-1.2,0-2.3,0-3.5c0-0.7,0.1-1.5,0.2-2.2c0.3-1.8,1.2-3.2,2.6-4.3 C16.7,0.4,18.2,0,19.7,0C19.9,0,20,0,20.2,0z"/> </g> </svg>
								</span>';	
			$output .= '		</a>';	
		} 
		if($twitter !== ''){
			$output .= '		<a href="' . $twitter . '" target="_blank" rel="nofollow" class="single-scial social-twitter">';
			$output .= '			<span class="single-social-icon social-icon-twitter">
									<svg version="1.1" id="Ebene_1" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" x="0px" y="0px" viewBox="0 0 32 32" style="enable-background:new 0 0 32 32;" xml:space="preserve"> <g> <path d="M32,6.1c-0.2,0.4-0.5,0.7-0.7,1c-0.7,0.9-1.5,1.6-2.4,2.3l-0.1,0.2c0,1,0,1.9-0.1,2.9c-0.3,2-0.9,4-1.7,5.8 c-0.9,1.9-2.1,3.6-3.6,5.2c-2.5,2.6-5.5,4.3-9,5c-1.2,0.3-2.4,0.4-3.7,0.4c-0.2,0-0.4,0-0.6,0c-3.5,0-6.8-0.9-9.9-2.8L0,26 c0.5,0.1,1,0.1,1.5,0.1c1.7,0,3.4-0.3,5-1c1.1-0.5,2.2-1.1,3.1-1.8c-3.4-0.1-5.5-2.6-6.1-4.5c0.4,0.1,0.8,0.1,1.2,0.1 c0.5,0,1.1-0.1,1.6-0.2l0.1,0c-1.8-0.4-3.3-1.4-4.2-3c-0.7-1.1-1-2.2-0.9-3.5c0.9,0.5,1.9,0.8,2.9,0.8C2.9,12,2,10.7,1.6,9 C1.2,7.3,1.5,5.7,2.4,4.3c3.5,4.2,8,6.5,13.4,6.8c0-0.3-0.1-0.5-0.1-0.7c-0.2-1.5,0.2-3,1-4.3c1.1-1.6,2.5-2.7,4.5-3 C21.6,3.1,22,3,22.3,3c1.7,0,3.2,0.7,4.5,1.9C27,5,27,5.1,27.1,5.1c0,0,0.1,0,0.1,0c1.3-0.3,2.6-0.8,3.8-1.5l0.1,0 c-0.5,1.5-1.4,2.7-2.8,3.6C29.6,7,30.8,6.6,32,6.1z"/> </g> </svg>
								</span>';	
			$output .= '		</a>';	
		}
		if($instagram !== ''){
			$output .= '		<a href="' . $instagram . '" target="_blank" rel="nofollow" class="single-scial social-instagram">';
			$output .= '			<span class="single-social-icon social-icon-instagram">
									<svg version="1.1" id="Ebene_1" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" x="0px" y="0px" viewBox="0 0 32 32" style="enable-background:new 0 0 32 32;" xml:space="preserve"> <g> <g> <path d="M24.6,5.6C24.6,5.6,24.6,5.6,24.6,5.6c1.1,0,1.9,0.9,1.9,1.9c0,1.1-0.9,1.9-1.9,1.9c-1.1,0-1.9-0.9-1.9-1.9 C22.7,6.4,23.5,5.6,24.6,5.6z"/> </g> <g> <path d="M31.8,8c0.2,2.5,0.4,14.4-0.3,17.2c-1,3.6-3.4,5.8-7.1,6.4C23,31.9,19.3,32,15.6,32c-3.8,0-7.6-0.1-8.9-0.5 c-3.6-0.9-5.8-3.3-6.4-7c-0.4-2.7-0.5-15.2,0.3-18C1.6,3,4,1,7.6,0.4c1.6-0.3,4.8-0.4,8-0.4c3,0,6,0.1,7.8,0.2 c2.2,0.1,4.2,0.8,5.8,2.4C30.8,4.1,31.6,5.9,31.8,8z M28.7,24c0.5-2.8,0.5-13.1,0.2-15.7c-0.3-2.8-2-4.7-4.8-5.1 c-1.4-0.2-4.7-0.3-8-0.3c-3.2,0-6.4,0.1-7.8,0.2C5.6,3.5,3.7,5.2,3.3,8c-0.5,2.9-0.5,13.9-0.1,16.1c0.4,2.3,1.7,3.9,3.9,4.5 c1.2,0.3,5,0.5,8.6,0.5c3.4,0,6.7-0.1,7.9-0.2C26.4,28.6,28.3,26.8,28.7,24"/> </g> <g> <path d="M16.1,7.8c4.5,0,8.2,3.7,8.1,8.2c0,4.5-3.7,8.2-8.3,8.2c-4.5,0-8.2-3.7-8.1-8.2C7.8,11.5,11.5,7.8,16.1,7.8z M15.9,21.3 c2.9,0,5.3-2.3,5.3-5.3c0-2.9-2.3-5.3-5.2-5.4c0,0,0,0,0,0c-2.9,0-5.3,2.3-5.3,5.3C10.6,18.9,13,21.3,15.9,21.3 C15.9,21.3,15.9,21.3,15.9,21.3"/> </g> </g> </svg>
								</span>';	
			$output .= '		</a>';	
		}
		if($youtube !== ''){
			$output .= '		<a href="' . $youtube . '" target="_blank" rel="nofollow" class="single-scial social-youtube">';
			$output .= '			<span class="single-social-icon social-icon-youtube">
									<svg version="1.1" id="Ebene_1" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" x="0px" y="0px" 	 viewBox="0 0 32 32" style="enable-background:new 0 0 32 32;" xml:space="preserve"> <g> 	<path d="M31.2,7.7c1.1,3,1.1,14-0.3,16.9c-0.4,0.8-1,1.3-1.7,1.8c-1.1,0.6-7.3,0.8-13.5,0.8C10,27.2,4.5,27,3.2,26.6 		c-1.2-0.4-2-1.2-2.4-2.4C-0.3,21.3-0.5,9,1.6,6.6c0.7-0.8,1.6-1.3,2.7-1.4c2.5-0.3,7.2-0.4,11.9-0.4C22,4.8,27.7,5,28.8,5.4 		C29.9,5.8,30.8,6.6,31.2,7.7z M12,20.8c3.3-1.7,6.6-3.4,10-5.2c-3.3-1.7-6.6-3.5-9.9-5.2C12.1,13.9,12.1,17.4,12,20.8"/> </g> </svg>
								</span>';	
			$output .= '		</a>';	
		}
		if($linkedin !== ''){
			$output .= '		<a href="' . $linkedin . '" target="_blank" rel="nofollow" class="single-scial social-linkedin">';
			$output .= '			<span class="single-social-icon social-icon-linkedin">
									<svg version="1.1" id="Ebene_1" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" x="0px" y="0px" viewBox="0 0 32 32" style="enable-background:new 0 0 32 32;" xml:space="preserve"> <g> <g> <path d="M3.9,0C6,0,7.7,1.7,7.7,3.9C7.7,6,6,7.7,3.8,7.7C1.7,7.7,0,6,0,3.9S1.8,0,3.9,0z"/> </g> <g> <polygon points="7.2,10.6 7,32 0.3,32 0.5,10.6 		"/> </g> <g> <path d="M24.1,10.1c6.7,0,7.9,4.4,7.9,10.2L31.9,32h-6.6l0.1-10.4c0-2.5,0-5.7-3.4-5.7c-3.5,0-4,2.7-4,5.5L17.8,32h-6.7l0.2-21.4 h6.4l0,2.9h0.1C18.7,11.9,20.8,10.1,24.1,10.1z"/> </g> </g> </svg>
								</span>';	
			$output .= '		</a>';	
		}
		if($xing !== ''){
			$output .= '		<a href="' . $xing . '" target="_blank" rel="nofollow" class="single-scial social-xing">';
			$output .= '			<span class="single-social-icon social-icon-xing">
									<svg version="1.1" id="Ebene_1" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" x="0px" y="0px" viewBox="0 0 32 32" style="enable-background:new 0 0 32 32;" xml:space="preserve"> <g> <path d="M31,0h-7.5L12.4,19.5L19.6,32h7.5l-7.2-12.5L31,0z"/> <path d="M9.3,6h-7l4.1,7.2L1,22h7l5.4-8.8L9.3,6z"/> </g> </svg>
								</span>';	
			$output .= '		</a>';	
		}
		
		$output .= '	</div>';
		
		return $output;
	}
	
	function author_social_shortcode($atts = []){
		
		$attributes = shortcode_atts(
			array(
				'id' => get_queried_object()->post_author,
			), $atts, $tag
		);
		
		echo '<pre>';
		print_r( $attributes );
		echo '</pre>';

		return (isset($attributes['id']) ? $this->author_social($attributes['id']) : '' );
	}
	/**
	 *	Create Visual Editor
		*
		*	Add TinyMCE editor to replace the "Biographical Info" field in a user profile
		*
		* @uses wp_editor() http://codex.wordpress.org/Function_Reference/wp_editor
		* @param $user An object with details about the current logged in user
		*/
	public function visual_editor( $user ) {
		
		/**
		 * Removed Jan 2024
		 */ 
		return;



		// Contributor level user or higher required
		if ( !current_user_can('edit_posts') )
			return;
		?>

		<table class="form-table">
			<tr>
				<th><label for="short_description"><?php echo esc_html(__('Short description', 'delucks-seo')); ?></label></th>
				<td>
					<?php 
					$short_description = get_user_meta( $user->ID, 'short_description', true);
					wp_editor( $short_description, 'webseide_usermeta[short_description]' ); 
					?>
					<p class="description"><?php echo esc_html(__('Share a little short biographical information.', 'delucks-seo')); ?></p>
				</td>
			</tr>
			<tr>
				<th><label for="description"><?php echo esc_html(__('Biographical Info', 'delucks-seo')); ?></label></th>
				<td>
					<?php 
					$description = get_user_meta( $user->ID, 'description', true);
					wp_editor( $description, 'description' ); 
					?>
					<p class="description"><?php echo esc_html(__('Share a biographical information to fill out your profile. This may be shown publicly.', 'delucks-seo')); ?></p>
				</td>
			</tr> 
		</table>
		<?php
	}

	/**
	 * Remove textarea filters from description field
	 */
	public function save_filters() {
		
		// Contributor level user or higher required
		if ( !current_user_can('edit_posts') )
			return;
			
		remove_all_filters('pre_user_description');
	}

	function update_user_meta(){
		
		if(is_array($_POST['webseide_usermeta']) && !empty($_POST['webseide_usermeta'])){
			foreach($_POST['webseide_usermeta'] as $k => $v){
				update_user_meta( (int)$_POST['user_id'], $k, $v ); 
			}
		}
		
	}
	
}


if(!function_exists('dpc_author_social')){
	function dpc_author(){
		if(shortcode_exists('dpc_author_social')){
			return do_shortcode('[dpc_author_social]');
		}
	}
}