<?php


class DPC_Module_Professional_Externallinks implements DPC_Module_Interface {

	var $dpc					= null;
	var $isSubmodule			= true;
	var $moduleDirectory 	= 'professional/';
	var $moduleTitle 		= '';
	var $optionsHook			= 'dpc-professional-externallinks';
	var $settingsFile		= false;
	var $settings			= false;

	function registerWidgets(){ }

    /**
     * The constructor simply saves the reference to the Master-DPC object into an attribute.
     *
     * @param $dpc The DPC object that instantiated this module
     */
    function __construct(&$dpc){
		$this->settingsFile = dirname(__FILE__) . '/settings.xml';
		$this->dpc 			= $dpc;
		$this->moduleTitle	= esc_html__('Links', 'delucks-seo');
		$this->settings 	= $this->dpc->getModuleSettings('professional_externallinks');

		global $wpdb;
		$wpdb->show_errors();
		$wpdb->query($wpdb->prepare("CREATE TABLE  IF NOT EXISTS  `%s`(
			`ID` int(11) NOT NULL AUTO_INCREMENT,
			`timestamp` bigint(20) DEFAULT NULL,
			`link` text,
			`host` varchar(255) DEFAULT NULL,
			PRIMARY KEY (`ID`),
			KEY `ix_dpc_externallinks_statistics_host` (`host`),
			KEY `ix_dpc_externallinks_statistics_timestamp` (`timestamp`)
		  ) DEFAULT CHARSET=utf8;", array(
			$wpdb->prefix . 'dpc_externallinks_statistics'
		  )));

		$wpdb->query($wpdb->prepare("CREATE TABLE  IF NOT EXISTS  `%s` (
			`id` int(11) NOT NULL AUTO_INCREMENT,
			`post_id` bigint(20) DEFAULT NULL,
			`host` varchar(255) DEFAULT NULL,
			`rating` tinyint(1) DEFAULT NULL,
			PRIMARY KEY (`id`),
			KEY `ix_dpc_externallinks_post_links_post_id` (`post_id`)
		  ) DEFAULT CHARSET=utf8;", array(
			$wpdb->prefix . 'dpc_externallinks_post_links'
		  )));
		$wpdb->hide_errors();

		$this->dpc->registerAjaxAction('dpc-externallink-click', array($this, 'ajaxDpcExternalLinkClick'));
		$this->dpc->registerAjaxAction('dpc-externallink-click', array($this, 'ajaxDpcExternalLinkClick'), true);
	}

    /**
     * adminSettings() gets called when the admin page of the parent module is loaded and starts the population of
     * the data described in the settingsFile. Based on this data a backend form is created and displayed.
     */
    function adminSettings(){
		echo '<a name="anchor-dpc-professional-externallinks"></a>';
		$this->dpc->parseSettingsByXml($this->settingsFile, 'general');
	}

    function adminSettingsHead(){
    	wp_enqueue_style('dpc-professional-externallinks-settings', plugins_url( 'assets/css/settings.css', __FILE__ ));
    }

    function adminHead(){ }

    function executeBackendActions(){ }

    function getPostLinks($postId){
    	global $wpdb;
    	return $wpdb->get_results($wpdb->prepare("SELECT * FROM `%s` WHERE `post_id`=%d", array(
			$wpdb->prefix . 'dpc_externallinks_post_links',
			$postId
		)), ARRAY_A);
    }

    function getPostLinkRatingByHost($postId, $host){
    	global $wpdb;
    	return $wpdb->get_var($wpdb->prepare("SELECT `rating` FROM `%s` WHERE `post_id`=%d AND `host`=%s", array(
			$wpdb->prefix . 'dpc_externallinks_post_links',
			$postId,
			$host
		)));
    }

    function clearPostLinks($postId){
    	global $wpdb;
    	return $wpdb->query($wpdb->prepare("DELETE FROM `%s` WHERE `post_id`=%d", array(
			$wpdb->prefix . 'dpc_externallinks_post_links',
			$postId
		)));
    }

    function addPostLink($postId, $host, $rating){
    	global $wpdb;
    	return $wpdb->query($wpdb->prepare("INSERT INTO `%s` (`post_id`, `host`, `rating`) VALUES (%d, %s, %d)", array(
			$wpdb->prefix . 'dpc_externallinks_post_links',
			$postId,
			$host,
			$rating
		)));
    }

    /**
     * getCustomPostTypes() gets all custom post types except for the ones specified in the $skip array and returns them.
     *
     * @return array Returns an array with all custom post types that aren't specified in $skip
     */
    function getCustomPostTypes(){
		$skip = array('post', 'page', 'attachment', 'revision', 'nav_menu_item');
		$data = array();
		foreach(get_post_types()  as $k => $v){
			if(in_array($k, $skip)){ continue; }
			$data[] = array('label' => $k, 'value' => $k);
		}
		return $data;
	}

    /**
     * executeFrontendActions() is an optional hook that gets triggered every time a frontend page is loaded. Depending
     * on the settings this registers actions on a global level (to alter all the content displayed) or just while the
     * main content or the excerpt are shown.
     */
    function executeFrontendActions(){
    	add_filter('the_content', array($this, 'modifyOutput'), 1000);
    	add_filter('the_excerpt', array($this, 'modifyOutput'), 1000);
	}

    /**
     * modifyOutput() registers regEx callbacks to look for links and modify every found link.
     *
     * @param $buffer The content that should be modified
     * @return mixed Modified content to display
     */
    function modifyOutput($buffer){
    	if(!isset($this->settings['include']) || !is_array($this->settings['include']) || !in_array(get_post_type(), $this->settings['include'])){
			return $buffer;
		}

		$pattern 	= '/<a (.*?)href=[\"\'](.*?)\/\/(.*?)[\"\'](.*?)>(.*?)<\/a>/i';
		$buffer 	= preg_replace_callback($pattern, array($this,'proceedMatches'), $buffer);
		return $buffer;
	}

    /**
     * proceedMatches() decides for every link if it should be nofollow or dofollow.
     *
     * @param $matches The parts of the link as matched by the regEx
     * @return string The new link with nofollow, dofollow or original - depending on exceptions and information in the link
     */
	function proceedMatches($matches){
		$linkAttr	= ' dpc-external="true" ';
		$postRules	= array();
		$hostname 	= wp_parse_url($matches[2] . '//' . $matches[3]);
		$hostname	= isset($hostname['host']) ? $hostname['host'] : '';
		$exceptions = explode(',', $this->settings['exceptions']);
		$exceptions = array_map(array($this, 'getHostByUrl'), $exceptions);

		if($postRulesArray = $this->getContentLinksById(get_the_ID())){
			foreach($postRulesArray as $rule){
				if(isset($rule['host'])){
					$postRules[$rule['host']] = $rule;
				}
			}
		}

		if($hostname == $_SERVER['HTTP_HOST'] || empty($hostname) || in_array($hostname, $exceptions)){
			return $matches[0];
		}

		if (isset($this->settings['external']) && in_array('blank', $this->settings['external'])){
		    if(!in_array($hostname, $exceptions) && !in_array(str_replace('www.', '', $hostname), $exceptions)){
		        $linkAttr .= ' target="_blank" ';
		    }
		}

		if(key_exists($hostname, $postRules) && strlen($postRules[$hostname]['rating'])){
		    $linkAttr .= ' rel="'.($postRules[$hostname]['rating'] == 1 ? 'follow' : 'nofollow' ).'" ';
		} elseif (isset($this->settings['external']) && in_array('noFollow', $this->settings['external'])){
		    if(!in_array($hostname, $exceptions) && !in_array(str_replace('www.', '', $hostname), $exceptions)){
		        $linkAttr .= ' rel="nofollow" ';
		    }
		}

		$newLink = '<a ' . $matches[1] . ' href="' . $matches[2] . '//' . $matches[3] . '" ' . $matches[4] . $linkAttr . '>'. $matches[5] .'</a>';

		return $newLink;
	}

	function getHostByUrl($val){
		if(strlen($val)){
			$val = 'http://' . str_replace(array('https://', 'http://', 'www.'), '', $val);
			$val = wp_parse_url($val);
			return $val['host'];
		}
	}

	function ajaxDpcExternalLinkClick(){
		global $wpdb;
		if (isset($_REQUEST['link']) && isset($_REQUEST['host'])){
			$wpdb->query($wpdb->prepare("INSERT INTO `%s` (timestamp, link, host) VALUES (%d, %s, %s)", array(
				$wpdb->prefix.'dpc_externallinks_statistics',
				time(),
				sanitize_text_field($_REQUEST['link']),
				sanitize_text_field($_REQUEST['host'])
			)));
		}
	}

	function getContentLinksById($postId, $type = null){
		if(!is_object($post = get_post($postId))){
			return false;
		}

		$links		= array();
		$data		= array();
		if(isset($post->post_content) && strlen($post->post_content)){
			$pattern 	= '/<a (.*?)href=[\"\'](.*?)\/\/(.*?)[\"\'](.*?)>(.*?)<\/a>/i';
			if(preg_match_all($pattern, $post->post_content, $matches)){
				foreach($matches[3] as $link){
					$link = wp_parse_url('http://'.$link);
					if(isset($link['host'])){
						if(strpos($link['host'], 'www.') !== false){
							$link = str_replace('www.', '', $link['host']);
						} else {
							$link = $link['host'];
						}
						if(!in_array($link, $links)){
							$links[] = $link;
						}
					}
				}
			}
		}

		if(count($links)){
			foreach($links as $link){
				$data[] = array('empty' => 'false', 'host' => $link, 'rating' => $this->getPostLinkRatingByHost($postId, $link));
			}
		}else{
			$data[] = array('empty' => 'true');
		}
		return $data;
	}

	function initXMLText(){
	    esc_html__('Links', 'delucks-seo');
	    esc_html__('With this module you can control the behavior of the website concerning outgoing links, because meaningful build in links prove your contents’ quality.', 'delucks-seo');
	    esc_html__('To prove the reliability of your contents you should insert continuative links. But, to avoid losing visitors after clicking on your outgoing links you should enable opening up a new browser window with the reffered website. Additionally, your contents’ link value will be inherited to the external websites. In this module you can control your entire website on what should happen with external links. For example you can set that your collection of (a huge amount of) external links should not depreciate its link value to (all) of the linked sites.', 'delucks-seo');
		esc_html__('Activate the link management module, to set up standard rules about how to treat external links in connection to your whole website.', 'delucks-seo');
		esc_html__('Activated', 'delucks-seo');
		esc_html__('Deactivated', 'delucks-seo');
		esc_html__('Links of external domains', 'delucks-seo');
		esc_html__('Open in new window', 'delucks-seo');
		esc_html__('Devalue, no follow', 'delucks-seo');
		esc_html__('Limit to', 'delucks-seo');
		esc_html__('Exceptions', 'delucks-seo');
		esc_html__('ignore', 'delucks-seo');
		esc_html__('follow', 'delucks-seo');
		esc_html__('no follow', 'delucks-seo');
		esc_html__('External links', 'delucks-seo');
		esc_html__('Follow', 'delucks-seo');
		esc_html__('No links were found', 'delucks-seo');
		esc_html__('Yes', 'delucks-seo');
		esc_html__('No', 'delucks-seo');
	}

}