<?php
define( 'DPC_JOBS_PATH', plugin_dir_path( __FILE__ ) );
define( 'DPC_JOBS_URL', plugin_dir_url( __FILE__ ) );

require_once(DPC_PLUGIN_HELPER_DIR . 'admin_settings.class.php');
require_once(DPC_PLUGIN_HELPER_DIR . 'admin_metaboxes.class.php');
require_once(DPC_PLUGIN_HELPER_DIR . 'admin_tax_metaboxes.class.php');

class DPC_Module_Professional_Jobs implements DPC_Module_Interface {

	var $dpc				= null;
	var $isSubmodule		= true;
	var $moduleDirectory 	= 'professional/';
	var $moduleTitle 		= '';
	var $optionsHook		= 'dpc-professional-jobs';
	var $settingsFile		= false;
	var $settings			= array();
	var $wdfIdf				= null;

	function __construct(&$dpc){

		$this->dpc 				= $dpc;
		$this->moduleTitle		= esc_html__('Jobs', 'delucks-seo');
		$this->settingsFile 		= dirname(__FILE__) . '/settings.xml';
		$this->settings 			= $this->dpc->getModuleSettings('professional_jobs');
		
		/** add posttype & taxonomies */
		add_action( 'init', array($this, 'add_job_posttype'), 11 );
		add_action( 'init', array($this, 'add_job_taxonomies') );	 

		/** add meta boxes */
		add_action( 'init', array($this, 'init_metaboxes'), 11 );	 
		
		require_once('admin_settings.php');
		$this->jobs_admin_settings = new jobs_admin_settings();
	}

	function adminHead(){}

	function executeFrontendActions(){
		/** Add shortcode */
		add_shortcode( 'jobs', array($this, 'shortcode_callback_jobs_archive') );
		add_shortcode( 'dpc_jobs', array($this, 'shortcode_callback_jobs_archive') );

		add_shortcode( 'jobs_data', array($this, 'shortcode_callback_jobs_data') );
		add_shortcode( 'dpc_jobs_data', array($this, 'shortcode_callback_jobs_data') );

		add_shortcode( 'jobs_archive', array($this, 'shortcode_callback_jobs_archive') );
		add_shortcode( 'dpc_jobs_archive', array($this, 'shortcode_callback_jobs_archive') );

		/** Add templates for display */
		add_filter('single_template', array($this, 'job_template_single'));
		add_filter('archive_template', array($this, 'job_template_archive'));			

		/** Ajax function for filter */
		//$this->dpc->registerAjaxAction('dpc_professional_jobs_wdf_idf_analysis', array($this, 'wdf_idf_analysis_ajax'));
		add_action('wp_ajax_get_jobs_filtered', array($this, 'get_jobs_filtered_callback'));
		add_action('wp_ajax_nopriv_get_jobs_filtered', array($this, 'get_jobs_filtered_callback'));
	}

	function registerWidgets(){}
	
	function adminSettings(){
		$this->dpc->parseSettingsByXml($this->settingsFile, 'general');
	}
	
	function adminSettingsHead(){}
	
	function executeBackendActions(){
		// echo '<pre>';
		// print_r($this->settings);
		// echo '</pre>';
		// die();
		
	}

	function enqueue_frontend_assets(){
		wp_register_style( 'dpc-jobs-styles', DPC_JOBS_URL . 'assets/style.css' );
		wp_register_script( 'dpc-jobs-script', DPC_JOBS_URL . 'assets/functions.js', array('jquery'), '', true );
		
		wp_localize_script( 'dpc-jobs-script', 'script_data', array(
	        'ajaxurl' => admin_url( 'admin-ajax.php' ),
	    	));
			
		wp_enqueue_style( 'dpc-jobs-styles' );
		wp_enqueue_script( 'dpc-jobs-script' );
	}

	function printScripts(){
		// wp_register_style( 'dpc-jobs-styles', DPC_JOBS_URL . 'assets/style.css' );
		// wp_register_script( 'dpc-jobs-script', DPC_JOBS_URL . 'assets/functions.js', array('jquery'), '', true );
// 		
		// wp_localize_script( 'dpc-jobs-script', 'script_data', array(
	        // 'ajaxurl' => admin_url( 'admin-ajax.php' ),
	    	// ));
// 				
	}

	function add_job_posttype() {
		// var_dump( empty(get_option('dpc_jobs_permalink')) );
		// die();
		$labels = array(
			'name'                  	=> _x( 'Jobs', 'Post Type General Name', 'delucks-seo' ),
			'singular_name'         	=> _x( 'Job', 'Post Type Singular Name', 'delucks-seo' ),
			'menu_name'             	=> __( 'Jobs', 'delucks-seo' ),
		);
		$args = array(
			'label'                 	=> __( 'Job', 'delucks-seo' ),
			'description'           	=> __( 'Jobs', 'delucks-seo' ),
			'labels'                	=> $labels,
			'supports'              	=> array( 'title', 'editor', 'excerpt', 'custom-fields' ),
			'taxonomies'            	=> array( 'dpc_job_location', 'dpc_job_section' ),
			'hierarchical'          	=> false,
			'public'                	=> true,
			'show_ui'               	=> true,
			'show_in_menu'          	=> true,
			'menu_position'         	=> 5,
			'show_in_nav_menus'     	=> true,
			'show_in_rest' 			=> true,
			'can_export'            	=> true, 
			'exclude_from_search'   	=> false,
			'publicly_queryable'    	=> true,
			'capability_type'       	=> 'post',
			'has_archive'           	=> ( !empty(get_option('dpc_jobs_permalink')) ? get_option('dpc_jobs_permalink') : 'jobs'),
		    'rewrite' 				=> array(
		        'slug' => ( !empty(get_option('dpc_jobs_permalink')) ? get_option('dpc_jobs_permalink') : 'jobs'),
		        'with_front' => false 
		    ),  
			'menu_icon'   			=> 'dashicons-welcome-learn-more',
		);
		
		register_post_type( 'dpc_jobs', $args );
	}
	
	function add_job_taxonomies(){		
		if((isset($this->settings) && isset($this->settings['jobLocation'])) && $this->settings['jobLocation'] === 'on'){
			register_taxonomy('dpc_job_location', array('dpc_jobs'), array(
				'hierarchical' 		=> true,
				'labels' 			=> array(
					'name' 				=> __( 'Location', 'delucks-seo' ),
					'singular_name' 	=> __( 'Location', 'delucks-seo' )
				),
				'show_ui' 			=> true,
				'show_admin_column' => true,
				'query_var' 			=> true,
				'rewrite' 			=> false,
				'show_in_rest'		=> true,
			));
		}
		
		if((isset($this->settings) && isset($this->settings['jobSection'])) && $this->settings['jobSection'] === 'on'){
			register_taxonomy('dpc_job_section', array('dpc_jobs'), array(
				'hierarchical' 		=> true,
				'labels' 			=> array(
					'name' 				=> __( 'Section', 'delucks-seo' ),
					'singular_name' 	=> __( 'Section', 'delucks-seo' )
				),
				'show_ui' 			=> true,
				'show_admin_column' => true,
				'query_var' 			=> true,
				'rewrite' 			=> false,
				'show_in_rest'		=> true,
			));
		}
		
	}

	function init_metaboxes(){
		if(is_admin() && current_user_can('edit_posts')){
			$this->add_metaboxes();
		}
	}
	
	function add_metaboxes(){
		$this->add_posttype_metaboxes();
		$this->add_taxonomy_metaboxes();
	}
	
	function add_taxonomy_metaboxes(){
		
		if(class_exists('DPC_Admin_Tax_Metaboxes')){
			$tax_settings = new DPC_Admin_Tax_Metaboxes( __( 'Job offer settings', 'delucks-seo' ), 'job_category_settings', 'dpc_job_location' );
		
			$tax_settings->add_field( array(
				'type' 	=> 'text',
				'name' 	=> 'street',
				'title' => __( 'Street address & number', 'delucks-seo' ),
			) );
			
			$tax_settings->add_field( array(
				'type' 	=> 'text',
				'name' 	=> 'postalcode',
				'title' => __( 'Postalcode', 'delucks-seo' ),
			) );
			
			$tax_settings->add_field( array(
				'type' 	=> 'text',
				'name' 	=> 'city',
				'title' => __( 'City', 'delucks-seo' ),
			) );
			
			$tax_settings->add_field( array(
				'type' 	=> 'text',
				'name' 	=> 'state',
				'title' => __( 'State', 'delucks-seo' ),
			) );
			
			$tax_settings->add_field( array(
				'type' 	=> 'text',
				'name' 	=> 'country',
				'title' => __( 'Country', 'delucks-seo' ),
			) );
		}
	}
	
	function add_posttype_metaboxes(){
		if(class_exists('DPC_Admin_Metaboxes')){
			$jobsettings = new DPC_Admin_Metaboxes( __( 'Job offer settings', 'delucks-seo' ), 'jobsettings', array( 'dpc_jobs') );
		
			$jobsettings->add_field( array(
				'type' 		=> 'date',
				'name' 		=> 'hiring_date',
				'title' 		=> __( 'Hiring date', 'delucks-seo' ),
				'desc' 		=> sprintf(__( 'You can use the shortcode %s to display the value within the content.', 'delucks-seo' ), '[jobs_data data="hiring_date"]'),
				'default'	=> time()
			) );
			$jobsettings->add_field( array(
				'type' 	=> 'date',
				'name' 	=> 'deadline',
				'title' => __( 'Application deadline', 'delucks-seo' ),
				'desc' 	=> sprintf(__( 'You can use the shortcode %s to display the value within the content.', 'delucks-seo' ), '[jobs_data data="deadline"]'),
			) );
			$jobsettings->add_field( array(
				'type' 	=> 'text',
				'name' 	=> 'working_time',
				'title' => __( 'Working time (h/week)', 'delucks-seo' ),
				'desc' 	=> sprintf(__( 'You can use the shortcode %s to display the value within the content.', 'delucks-seo' ), '[jobs_data data="working_time"]'),
			) );

			$jobsettings->add_field( array(
				'type' 	=> 'divider',
				'name' 	=> 'divider_1',
			) );
			
			$jobsettings->add_field( array(
				'type' 	=> 'text',
				'name' 	=> 'base_salary',
				'title' => __( 'Base salary', 'delucks-seo' ),
				'desc' 	=> sprintf(__( 'You can use the shortcode %s to display the value within the content.', 'delucks-seo' ), '[jobs_data data="base_salary"]'),
			) );
			
			$jobsettings->add_field( array(
				'type' 	=> 'select',
				'name' 	=> 'base_salary_unit',
				'options' => array(
					'HOUR'	=> 'per hour',
					'MONTH'	=> 'per month',
					'YEAR'	=> 'per year'
				),
				'title' => __( 'Base salary unit', 'delucks-seo' ),
				'desc' 	=> sprintf(__( 'You can use the shortcode %s to display the value within the content.', 'delucks-seo' ), '[jobs_data data="base_salary_unit"]'),
			) );
			
			$jobsettings->add_field( array(
				'type' 	=> 'text',
				'name' 	=> 'currency',
				'title' => __( 'Currency', 'delucks-seo' ),
				'desc' 	=> sprintf(__( 'You can use the shortcode %s to display the value within the content.', 'delucks-seo' ), '[jobs_data data="currency"]'),
			) );
			
			$jobsettings->add_field( array(
				'type' 	=> 'divider',
				'name' 	=> 'divider_2',
			) );
			
			$jobsettings->add_field( array(
				'type' 	=> 'multi_checkbox',
				'name' 	=> 'employment_type',
				'options' => array(
					'FULL_TIME'	=> __( 'full time', 'delucks-seo' ),
					'PART_TIME'	=> __( 'part time', 'delucks-seo' ),
					'CONTRACTOR'	=> __( 'contractor', 'delucks-seo' ),
					'TEMPORARY'	=> __( 'temporary', 'delucks-seo' ),
					'INTERN'		=> __( 'intern', 'delucks-seo' ),
					'VOLUNTEER'	=> __( 'volunteer', 'delucks-seo' ),
					'PER_DIEM'	=> __( 'per diem', 'delucks-seo' ),
					'OTHER'		=> __( 'other', 'delucks-seo' ),
				),
				'title' => __( 'Employment type', 'delucks-seo' ),
				'desc' 	=> sprintf(__( 'You can use the shortcode %s to display the value within the content.', 'delucks-seo' ), '[jobs_data data="employment_type"]'),
			) );
		}
	}
	
	function shortcode_callback_jobs_data($atts) {
		global $post;
		
		$attrs = shortcode_atts( array(
			'data' => '',
		), $atts );
		
		if( !isset($attrs['data']) ){ return; }
		
		$data		= $attrs['data'];	
		$settings 	= get_post_meta( $post->ID, 'jobsettings', true);
		
		// If setting is empty
		if($settings[$data] == ''){ return; }

		if($attrs['data'] == 'hiring_date' || $attrs['data'] == 'deadline'){
			return gmdate('d.m.Y', $settings[$data]);
		}elseif($attrs['data'] == 'employment_type'){
			$types = json_decode($settings[$data]);
			$return = "";
			
			// cannot use implode, else the translation would not work
			foreach( $types as $k => $type){
				$type = str_replace('_', ' ', strtolower($type)); 
				
				$return != "" && $return .= ", ";
			    $return .= __($type, 'delucks-seo');
			}
						
			return $return;

			return implode(', ', $types);
		}else{
			return $settings[$data];
		}
		
	}

	function shortcode_callback_jobs_archive($atts){
		global $wp_query;
		$current_section 	= '';
		$current_location 	= '';

		$attrs = shortcode_atts( array(
			'section' 	=> '',
			'location' 	=> '',
		), $atts );
		
		$args 				= array(
			'post_type' 	=> 'dpc_jobs',
			'post_status'	=> 'publish'
		);
		
		// if( isset($_POST['dpc_job_section']) && $_POST['dpc_job_section'] != '' || ){
		// 	$current_section = $_POST['dpc_job_section'];
		// 	$args['tax_query'] = array(
		// 			'taxonomy' 		=> 'dpc_job_section',
		// 			'terms' 		=> $_POST['dpc_job_section'],
		// 			'field' 		=> 'slug',
		// 			'operator' 		=> 'IN'
		// 	);
		// }
		$args['tax_query'] = array(
			'relation' => 'AND'
		);

		if( isset($attrs['section']) && $attrs['section'] != '' ){
			$args['tax_query'][] 	= array(
				'taxonomy' 		=> 'dpc_job_section',
				'terms' 		=> $attrs['section'],
				'field' 		=> 'slug',
				'operator' 		=> 'IN'
			);
		}

		if( isset($attrs['location']) && $attrs['location'] != '' ){
			$args['tax_query'][] 	= array(
				'taxonomy' 		=> 'dpc_job_location',
				'terms' 		=> $attrs['location'],
				'field' 		=> 'slug',
				'operator' 		=> 'IN'
			);
		}
		
		$query = new WP_Query( $args );
		
		$sections 	= get_terms( array( 'taxonomy' => 'dpc_job_section', 'hide_empty' => true ) );
		$locations 	= get_terms( array( 'taxonomy' => 'dpc_job_location', 'hide_empty' => true ) );
		
		$this->enqueue_frontend_assets();
		
		ob_start();
		
		?>
		<div class="jobs-table">
			<div class="jobs-table-filters">
				<form method="POST" id="dpc_job_filters">
					<?php
					if(!isset($attrs['section']) || $attrs['section'] == ''){
						$sections = get_terms( array( 'taxonomy' => 'dpc_job_section', 'hide_empty' => true ) );
						if ( !empty($sections) && isset($this->settings['jobSectionFilter']) && $this->settings['jobSectionFilter'] === 'on' ) {
						?>
							<select name="dpc_job_section" id="job_filter_section">
								<option value=""><?php echo esc_html(__( 'All sections', 'delucks-seo' )); ?></option>
								<?php
								foreach ( $sections as $section ) {
									echo '<option value="' . esc_attr($section->slug) . '" ' . selected(sanitize_text_field($_POST['dpc_job_section']), $section->slug) . '>' . esc_html($section->name) . '</option>';
								}
								?>
								
							</select>
						<?php
						}
					}else{
						echo '<input type="hidden" name="dpc_job_section" value="'.esc_attr($attrs['section']).'" />';
					}

					if(!isset($attrs['location']) || $attrs['location'] == ''){
						$locations = get_terms( array( 'taxonomy' => 'dpc_job_location', 'hide_empty' => true ) );
						if ( !empty($locations) && isset($this->settings['jobSectionFilter']) && $this->settings['jobLocationFilter'] === 'on' ) {
						?>
							<select name="dpc_job_location" id="job_filter_location">
								<option value=""><?php echo esc_html(__( 'All locations', 'delucks-seo' )); ?></option>
								<?php
								foreach ( $locations as $location ) {
									echo '<option value="' . esc_attr($location->slug) . '" ' . selected(sanitize_text_field($_POST['dpc_job_location']), $location->slug) . '>' . esc_html($location->name) . '</option>';
								}
								?>
								
							</select>
						<?php
						}
					}else{ 
						echo '<input type="hidden" name="dpc_job_section" value="'.esc_attr($attrs['location']).'" />';
					}					
					?>
					<?php
					if($attrs['section'] == '' || $attrs['location'] == ''){
						echo '<button id="submit_jobs_form" type="submit">' . esc_html(_x( 'Filter', 'verb', 'delucks-seo' )) . '</button>';
					} 
					?>
				</form>
				
			</div>
			
			<?php
				$posts = $query->posts;
				if( !empty($posts) ){
					if ( file_exists( DPC_JOBS_PATH . '/templates/archive-jobs-content-table.php' ) && isset($this->settings['jobsView']) && $this->settings['jobsView'] != 'list' ) {
			            include(DPC_JOBS_PATH . '/templates/archive-jobs-content-table.php');
			        }elseif( file_exists( DPC_JOBS_PATH . '/templates/archive-jobs-content-list.php' ) && isset($this->settings['jobsView']) &&  $this->settings['jobsView'] != 'table' ){
			            include(DPC_JOBS_PATH . '/templates/archive-jobs-content-list.php');
			        }
				}else{
					include(DPC_JOBS_PATH . '/templates/archive-jobs-empty.php');
				}
			?>
					
		</div>
		<?php
		
		$output = ob_get_clean();
		
		ob_end_flush();
		
		return $output;
		
	}

	function shortcode_callback_bereich() {
		global $post;

		$terms 			= get_the_terms( $post->ID , array('dpc_job_section') );
		$entry_terms 	= '';
		
		if ( !empty($terms) ) {
			foreach ( $terms as $term ) {
				$entry_terms .= $term->name . ', ';
			}
			$entry_terms = rtrim( $entry_terms, ', ' );
		}
		
		return $entry_terms;
	}
	
	function job_template_single($single) {
		global $post;
	
		if ( $post->post_type == 'dpc_jobs' ) {
			if ( file_exists( DPC_JOBS_PATH . '/templates/single-jobs.php' ) ) {
				include DPC_JOBS_PATH . '/templates/single-jobs.php';
			}
		}
	
		return $single;
	
	}
	
	function job_template_archive($single) {
		global $post,$posts;
				
		$this->enqueue_frontend_assets(); 
				
		if ( $post->post_type == 'dpc_jobs' ) {				
			if ( file_exists( DPC_JOBS_PATH . '/templates/archive-jobs.php' ) ) {
				include DPC_JOBS_PATH . '/templates/archive-jobs.php';
			}
		}
	
		return $single;
	
	}
	
	function get_jobs_filtered_callback(){
		$data 		= $_GET;
		
		$args 				= array(
			'post_type' 		=> 'dpc_jobs',
			'post_status'	=> 'publish'
		);
		
		$args['tax_query'] 	= array();
		
		$dpc_job_section 	= isset($data['dpc_job_section']) && $data['dpc_job_section'] != '' ? true : false;
		$dpc_job_location 	= isset($data['dpc_job_location']) && $data['dpc_job_location'] != '' ? true : false;
		
		if($dpc_job_section){
			$args['tax_query'][]		= array(
					'taxonomy' 			=> 'dpc_job_section',
					'terms' 				=> $data['dpc_job_section'],
					'field' 				=> 'slug',
					'operator' 			=> 'IN'
			);
		}
		
		if($dpc_job_location){
			$args['tax_query'][]		= array(
					'taxonomy' 			=> 'dpc_job_location',
					'terms' 				=> $data['dpc_job_location'],
					'field' 				=> 'slug',
					'operator' 			=> 'IN'
			);
		}
		
		if($dpc_job_section && $dpc_job_location){
			$args['tax_query']['relation']	= 'AND';
		}
		
		$query 	= new WP_Query( $args );
		$return	= array();
		$output	= '';
		
		if( $query->have_posts() ){
			$return['success'] = true;
			foreach($query->posts as $post){
				$settings		= get_post_meta( $post->ID, 'jobsettings', true);
				$job_sections 	= get_the_terms( $post->ID , 'dpc_job_section' );
				$job_locations 	= get_the_terms( $post->ID , 'dpc_job_location' );
				$location_list 	= '';
				
				if ( !empty($job_locations) ) {
					foreach ( $job_locations as $location ) {
						$location_list .= $location->name . ', ';
					}
					$location_list = rtrim( $location_list, ', ' );
				}
				
				$section_lis = '';
				
				if ( !empty($job_sections) ) {
					foreach ( $job_sections as $section ) {
						$section_list .= $section->name . ', ';
					}
					$section_list = rtrim( $section_list, ', ' );
				}
			
				ob_start();
			
				if ( isset($this->settings['jobsView']) && $this->settings['jobsView'] == 'list'  ) {
					$return['type'] = 'list';
					include(DPC_JOBS_PATH . '/templates/partials/list-content.php');
				}else{
					$return['type'] = 'table';
					include(DPC_JOBS_PATH . '/templates/partials/table-content.php');
				}
					
				$output .= ob_get_clean();
					
				ob_end_flush();
			}
		}else{
			$return['success'] = false;
			$return['type'] = 'none';
			ob_start();
			
			include(DPC_JOBS_PATH . '/templates/archive-jobs-empty.php');
			
			$output .= ob_get_clean();

			ob_end_flush();
		}
		
		$return['output'] = $output;
		
		wp_send_json($return);
	}
	
	function initXMLText(){
	    esc_html__('Activated', 'delucks-seo');
	    esc_html__('Deactivated', 'delucks-seo');
	    esc_html__('Active taxonomies', 'delucks-seo');
	    esc_html__('Job section', 'delucks-seo');
	    esc_html__('Job location', 'delucks-seo');
	    esc_html__('Job section filter', 'delucks-seo');
	    esc_html__('Job location filter', 'delucks-seo');
	    esc_html__('Active filters', 'delucks-seo');
	    esc_html__('Styling of jobs archive', 'delucks-seo');
	    esc_html__('Table view', 'delucks-seo');
	    esc_html__('List view', 'delucks-seo');
	}

}