<?php
/**
 * Template part for displaying single posts.
 *
 * @link https://codex.wordpress.org/Template_Hierarchy
 *
 * @package webseide
 */
global $post, $plugin_dpc;

$post_ID = get_the_ID();

// do a redirect if configured in post
if(get_post_meta( $post_ID, 'redirect', true ) != ''){
	header('HTTP/1.1 301 Moved Permanently');
	header('Location:'.get_post_meta( get_the_ID(), 'redirect', true ));
	exit;
}

$settings		= get_post_meta( $post_ID, 'jobsettings', true);
$hide_title 	= is_singular('webseide_jobs') && get_theme_mod( 'webseide_h1-pages' ) == '1' ? true : false;
$caption 		= (!is_singular('post') ? false : get_post( get_post_thumbnail_id() ));


// prepare shema.org data - START  ------------------------------------------------------------------
$image = get_the_post_thumbnail_url($post->ID, 'webseide_large');
if(!$image){ $image = get_theme_mod('webseide_logo'); } // fallback to website logo

$authorName = get_the_author_meta('user_nicename');
if(is_object($plugin_dpc) && is_object($metadata = $plugin_dpc->getModuleInstance('basic', 'metadata')) && method_exists($metadata, 'getWebsiteTitle')){
    $publisherName = $metadata->getWebsiteTitle();
} else {
    $publisherName = get_option('blogname');
}
// prepare shema.org data - END   ------------------------------------------------------------------

get_header();
?>

<div id="primary" class="content-area">
		<main id="main" class="site-main" role="main">
			
			<div class="blogdetailcontent">
			    <article id="post-<?php the_ID(); ?>" <?php post_class(); ?>>
					<?php if (has_post_thumbnail( $post->ID ) ): ?>
						<div class="fullthumbnail" style="background-image: url('<?php echo esc_url((get_theme_mod('webseide_layout') == 'edge-dropping' ?  get_the_post_thumbnail_url( $post->ID, 'full' ) : get_the_post_thumbnail_url( $post->ID, 'webseide_large' )));?>')"></div>
						<?php if (is_object($caption) && $caption->post_excerpt) : ?>
							<p class="caption"><?php echo esc_html($caption->post_excerpt); ?></p>
						<?php endif; ?>
					<?php endif; ?>
			
			
			        <header class="entry-header">
			            <?php
			            if(!$hide_title ){
			        			the_title( '<h1 class="entry-title">', '</h1>' );
			        		}
			        		?>
			            <strong class="post-intro"><?php echo (function_exists('webseide_get_post_intro') ? webseide_get_post_intro() : ''); ?></strong>
			            <div class="entry-meta">
			                <?php
				                	if(function_exists('webseide_posted_on')){
				                		webseide_posted_on(array(
				                        'delimiter' => ' · ',
				                        'show_author' => (is_author() ? false : true),
				                        'show_date' => true,
				                        'show_category' => (is_category() ? false : true),
				                        'show_reading_time' => true
				                    ));
				                	}
			                ?>
			            </div><!-- .entry-meta -->
			        </header><!-- .entry-header -->
					
			        <div class="entry-content">
			            <?php the_content(); ?>
			        </div><!-- .entry-content -->
			
			
			        <footer class="entry-footer">
			            <?php
				            	if(function_exists('webseide_entry_footer')){
			                		webseide_entry_footer();
			                	}
			            ?>
			        </footer><!-- .entry-footer -->
			
			    </article><!-- #post-## -->
			</div> <!-- .blogdetailcontent -->
			
		</main><!-- #main -->
	</div><!-- #primary -->
	
	<!-- Schema data -->
	<script type="application/ld+json">
    {
		"@context" : "https://schema.org/",
      	"@type" : "JobPosting",
      	"title" : "<?php echo esc_attr(get_the_title()); ?>",
      	"description" : "<?php echo esc_attr(get_the_excerpt()); ?>",
      	"datePosted" : "<?php echo esc_attr(isset($settings['hiring_date']) ? gmdate('Y-m-d', $settings['hiring_date']) : ''); ?>",
      	"validThrough" : "<?php echo esc_attr(isset($settings['deadline']) ? gmdate('Y-m-d h:i:s', $settings['deadline']) : ''); ?>",
      	"employmentType" : <?php echo esc_attr(isset($settings['employment_type']) ? $settings['employment_type'] : ''); ?>,
      	"hiringOrganization" : {
        		"@type" : "Organization",
        		"name" : "<?php echo esc_attr(get_bloginfo('name')); ?>",
        		"sameAs" : "<?php echo esc_url(site_url()); ?>",
        		"logo" : "<?php echo esc_url(get_site_icon_url()); ?>"
      	},
      	"jobLocation": {
      		
      		<?php 
      		$terms 			= get_the_terms( $post->ID, 'webseide_job_location' );
			$term_settings 	= get_term_meta( $terms[0]->term_id, 'job_category_settings', true );				
      		if(!empty($terms[0])){	
      		?>
      			"@type": "Place",
	        		"address": {
	        			"@type": "PostalAddress",
	        			"streetAddress": "<?php echo esc_attr(isset($term_settings['street']) ? $term_settings['street'] : ''); ?>",
	        			"addressLocality": "<?php echo esc_attr(isset($term_settings['city']) ? $term_settings['city'] : ''); ?>",
	        			"postalCode": "<?php echo esc_attr(isset($term_settings['postalcode']) ? $term_settings['postalcode'] : ''); ?>"
	        			<?php echo esc_attr(isset($term_settings['country']) ? ',"addressCountry" : "'. $term_settings['country'] .'"' : ''); ?>
	        			<?php echo esc_attr(isset($term_settings['state']) ? ',"addressRegion" : "'. $term_settings['state'] .'"' : ''); ?>
	        		}
      		<?php 
			}
			?>
      		
      		
      	}
      	<?php 
      	if(isset($settings['base_salary'])){
      	?>
	      	, "baseSalary": {
	        		"@type": "MonetaryAmount",
	        		"currency": "<?php echo esc_attr(isset($settings['currency']) ? $settings['currency'] : 'EUR'); ?>",
	        		"value": {
	          		"@type": "QuantitativeValue",
	          		"value": <?php echo esc_attr(isset($settings['base_salary']) ? $settings['base_salary'] : ''); ?>,
	          		"unitText": "<?php echo esc_attr(isset($settings['base_salary_unit']) ? $settings['base_salary_unit'] : 'YEAR'); ?>"
	        		}
	      	}
		<?php
		}
		?>
    }
    </script>
    
<?php get_footer(); ?>