<?php

class DPC_Module_Professional_Microdata implements DPC_Module_Interface {

	var $dpc						= null;
	var $isSubmodule				= true;
	var $moduleDirectory 		= 'professional/';
	var $moduleTitle 			= '';
	var $optionsHook				= 'dpc-professional-microdata';
	var $settingsFile			= false;
	var $settings				= array();
	var $addressDone				= false;
	var $microdataTypes			= array('video' => 'dashicons-video-alt3', 'location' => 'dashicons-location-alt', 'cwork' => 'dashicons-art', 'recipe' => 'dashicons-editor-ul', 'event' => 'dashicons-calendar', 'job' => 'dashicons-businessman', 'person' => 'dashicons-admin-users', 'faq' => 'dashicons-info');
	var $microdataLabels			= array('video' => 'name', 'location' => 'name', 'cwork' => 'name', 'recipe' => 'name', 'event' => 'name', 'job' => 'name', 'person' => 'name', 'faq' => 'question');
	var $microdataLang			= array();
	var $fallbackYoutubeApiKey	= 'AIzaSyAvD6MaYK1tZFgnOw8rQhNEFJwgaLSZw7U';

	function adminHead(){ }
	function registerWidgets(){ }

	function __construct(&$dpc){
		global $plugin_dpc;
		$this->dpc 				= $dpc;
		$this->moduleTitle		= esc_html__('Microdata', 'delucks-seo');
		$this->settingsFile 	= dirname(__FILE__) . '/settings.xml';
		$this->settings 		= $this->dpc->getModuleSettings('professional_microdata');
		$this->microdataLang	= array('video' => esc_html__('Video', 'delucks-seo'), 'location' => esc_html__('Company', 'delucks-seo'), 'cwork' => esc_html__('Creative Work', 'delucks-seo'), 'recipe' => esc_html__('Recipe', 'delucks-seo'), 'event' => esc_html__('Event', 'delucks-seo'), 'person' => esc_html__('Person', 'delucks-seo'), 'job' => esc_html__('Job', 'delucks-seo'), 'faq' => esc_html__('Faq', 'delucks-seo'));
		add_action('admin_enqueue_scripts', array($this, 'hook_admin_enqueue_scripts'));
		 
	}

	function onEditPage(){
		if(!is_array($this->settings) || !isset($this->settings['youtube_api_key']) || empty($this->settings['youtube_api_key'])){
			$this->settings['youtube_api_key'] = $this->fallbackYoutubeApiKey;
		}
	}

	function hook_admin_enqueue_scripts(){
		echo '<script type="text/javascript">'."\n";
		echo '	var dpc_professional_microdata_settings = ' . wp_json_encode($this->settings) . ";\n";
		echo '	var dpc_professional_microdata_type_to_label = ' . wp_json_encode($this->microdataLang) . ";\n";
		echo '</script>'."\n";
	}


	function modalEnqueueScripts(){
	    wp_enqueue_script('dpc-professional-microdata-modal', plugins_url( 'assets/js/modal.js', __FILE__ ), array('jquery'), DPC_VERSION, true);
		
		echo '<script type="text/javascript">'."\n";
		echo '	var dpc_professional_microdata_settings = ' . wp_json_encode($this->settings) . ";\n";
		echo '	var dpc_professional_microdata_type_to_label = ' . wp_json_encode($this->microdataLang) . ";\n";
		echo '</script>'."\n";
		
	}
	
	function adminSettings(){
		$this->dpc->parseSettingsByXml($this->settingsFile, 'general');
	}

	function adminSettingsHead(){
		wp_enqueue_style('dpc-professional-microdata-settings', plugins_url( 'assets/css/settings.css', __FILE__ ));
		//wp_enqueue_script('dpc-professional-microdata-settings', plugins_url( 'assets/js/settings.js', __FILE__ ), array('jquery'));
	}

	function executeBackendActions(){
		if($this->dpc->isEditPage()){
			$this->onEditPage();
		}
	}

	function executeFrontendActions(){
		add_action('wp',		array($this, 'proceedMicrodata'),	1000);
		add_action('wp_head', 	array($this, 'injectArticleMetadata'));
	}
	
	function injectArticleMetadata(){
	    $posttype = get_post_type();
	    if((is_page() && isset($this->settings['article_microdata_page']) && !empty($this->settings['article_microdata_page'])) || (is_single() && isset($this->settings['article_microdata_'. $posttype]) && !empty($this->settings['article_microdata_'. $posttype]))){
	        $id        = get_queried_object_id();
	        $post      = get_post($id);
	        $permalink = get_permalink($id);
	        $metadata = $this->dpc->getModuleInstance('basic_metadata');
	        $websiteTitle = $metadata->getWebsiteTitle();
	        $postTitle = (is_null($metadata->title) ? get_the_title() : $metadata->title);
	        $articleType = 'NewsArticle';
	        $organizationLogo = isset($this->settings['logo']) && strlen($this->settings['logo']) ? $this->settings['logo'] : '';
	        $postImage = get_the_post_thumbnail_url($id, 'large');
	        $authorUrl = get_the_author_meta('user_url', $post->post_author);
	        
	        if(!strlen($authorUrl)){
	            $authorUrl = site_url();
	        }

    	    echo '<script type="application/ld+json">';
    	    echo '{';
    	    echo '    "@context": "https://schema.org",';
    	    echo '    "@type": "'. esc_attr($articleType) .'",';
    	    echo '    "mainEntityOfPage": {';
    	    echo '        "@type": "WebPage",';
    	    echo '        "@id": "'. esc_url($permalink) .'"';
    	    echo '    },';
    	    echo '    "headline": "'. esc_attr(htmlspecialchars($postTitle)) .'",';
    	    echo '    "image": [';
    	    echo '        "'. esc_attr($postImage) .'"';
    	    echo '    ],';
    	    echo '    "datePublished": "'. esc_attr($post->post_date) .'",';
    	    echo '    "dateModified": "'. esc_attr($post->post_modified) .'",';
    	    echo '    "author": {';
    	    echo '        "@type": "Person",';
    	    echo '        "name": "'. esc_attr(get_the_author_meta('display_name', $post->post_author)) .'",';
    	    echo '        "url": "'. esc_url($authorUrl) .'"';
    	    echo '    },';
    	    echo '    "publisher": {';
    	    echo '        "@type": "Organization",';
    	    echo '        "name": "'. esc_attr(htmlspecialchars($websiteTitle)) .'",';
    	    echo '        "logo": {';
    	    echo '            "@type": "ImageObject",';
    	    echo '            "url": "'. esc_url($organizationLogo) .'"';
    	    echo '        }';
    	    echo '    }';
    	    echo '}';
    	    echo '</script>';
	    }
	}

	function modalGetEntries(){
	    $this->microdataLang	= array('video' => esc_html__('Video', 'delucks-seo'), 'location' => esc_html__('Company', 'delucks-seo'), 'cwork' => esc_html__('Creative Work', 'delucks-seo'), 'recipe' => esc_html__('Recipe', 'delucks-seo'), 'event' => esc_html__('Event', 'delucks-seo'), 'person' => esc_html__('Person', 'delucks-seo'));
		$content 				= false;
		$hasContent 			= false;

		foreach($this->microdataTypes as $type => $value){
			$data 		= get_post_meta( $this->dpc->modal->currentId, '_dpc-microdata-' . $type, true );
			$content 	= ((is_array($data) && count($data)) ? true : false);
			$icon 		= $value;

			add_thickbox();

			echo '<ul class="dpc-microdata-list '. esc_attr($type) .'" '.($content ? '' : 'style="display:none;"').'>';

			if($content){
				$hasContent = true;
				foreach($data as $k => $v){
				    $label      = '';
				    $labelKey   = $this->microdataLabels[$type];

				    if(isset($v[$labelKey]) && strlen($v[$labelKey])){
				        $label = $v[$labelKey];
				    } elseif(isset($v['name']) && strlen($v['name'])){
				        $label = $v['name'];
				    }

					echo '<li class="dpc-microdata-entry '. esc_attr($type) .'" data-type="'. esc_attr($type) .'" data-index="'. esc_attr($k) .'" ';
					foreach($v as $key => $value){
						echo 'data-'.esc_attr($key).'="'.esc_attr(htmlentities($value)).'" ';
					}
					echo '>
							<span class="entry-icon"><i class="dashicons '.esc_attr($icon).'"  data-title="'.esc_attr(isset($this->microdataLang[$type]) ? $this->microdataLang[$type] : ucfirst($type)).' " ></i></span>
							<span class="entry-type">'.esc_html(isset($this->microdataLang[$type]) ? $this->microdataLang[$type] : ucfirst($type)).'</span>
                            <span class="entry-label">'. esc_html($label) .'</span>
							<span class="dpc-microdata-action ">
								<a class="dpc-microdata-edit thickbox" data-type="'. esc_attr($type) .'" data-index="'. esc_attr($k) .'" href="#TB_inline?width=600&height=auto&inlineId=microdata-'.esc_attr($type).'-form"><i class="dashicons dashicons-edit"></i></a>
								<a class="dpc-microdata-delete" data-type="'. esc_attr($type) .'" data-index="'. esc_attr($k) .'"><i class="dashicons dashicons-trash"></i></a>
							</span>
						</li>';
				}
			}
			echo '</ul>';
		}

		echo '<div role="alert" class="alert alert-warning"'.($hasContent ? ' style="display: none;"' : '').'><strong>Oh!</strong> '.esc_html__("You have not added microdatas yet. It is time to do so!", 'delucks-seo').'</div>';

		echo '<div class="fixed-action-btn horizontal click-to-toggle active">';
		echo '<h4 class="addMicrodataText">' . esc_html(__('Add new microdata', 'delucks-seo')) . '</h4>';
		echo '	<ul class="addMicrodataList">';
		foreach($this->microdataTypes as $type => $class){
		    echo '		<li><a type="'.esc_attr($type).'" href="#TB_inline?width=600&height=auto&inlineId=microdata-'.esc_attr($type).'-form" class="thickbox addMicrodata btn btn-white" data-type="'. esc_attr($type) .'"><i class="dashicons '.esc_attr($class).'"></i><span class="addMicrodataLabel">'.esc_html(isset($this->microdataLang[$type]) ? $this->microdataLang[$type] : ucfirst($type)).'</span></a></li>';
		}
		echo '	</ul>';
		echo '</div> ';
		echo '<div class="clear"></div><br/>';
	}

	function getJsonLdFields($data, $fields){
		$op = '';
		foreach($fields as $field){
			if(is_array($field)){
				if(isset($field['itemValue'])){
					$field['itemValue'] = htmlentities($field['itemValue']);
				}
				$fieldName = (isset($field['forceName']) ? $field['forceName'] : $field['name']);

				if(isset($field['type'])){
					$op .= ', "'.$fieldName.'": { "@type": "'.$field['type'].'"';
				}

				if(isset($field['fields'])){
					$op .= $this->getJsonLdFields($data, $field['fields']);
				} else {
					$fieldName	= (isset($field['forceName']) ? $field['forceName'] : $field['name']);
					$fieldValue	= (isset($field['itemValue']) && isset($data[$field['itemValue']]) ? $data[$field['itemValue']] : (isset($data[$field['name']]) ? $data[$field['name']] : $field['itemValue']));

					if(isset($field['list']) && $field['list'] == true && isset($field['seperator'])){
						$entries = explode($field['seperator'], $fieldValue);
						if(is_array($entries) && count($entries)){
							$op .= ', "'. $field['name'] .'": [';
							foreach($entries as $k => $v){
							    $op .= ($k !== 0 ? ', ' : '') . '"'. htmlentities(trim($v)) .'"';
							}
							$op .= ']';
						}
					} elseif((isset($field['class']) && isset($field['method'])) || isset($field['function'])){
						if((class_exists((is_object($field['class']) ? get_class($field['class']) : $field['class'])) || is_object($field['class'])) && method_exists($field['class'], $field['method']) || function_exists($field['function'])){
						    $ret = call_user_func_array((isset($field['function']) ? $field['function'] : array($field['class'], $field['method'])), (isset($field['params']) ? $field['params'] : array($data[$field['name']])));
							$op .= ($ret ? ', "'. $fieldName .'": "'. $ret .'"' : '');
						}
					} else {
					    $op .= ', "'. $fieldName .'": "'. htmlentities(trim($fieldValue)) .'"';
					}
				}

				if(isset($field['type'])){
					$op .= ' }';
				}
			} else {
			    $op .= (isset($data[$field]) ? ', "'. $field .'": "'. htmlentities(trim($data[$field])) .'"' : '');
			}
		}
		return $op;
	}

	function getJsonLdOrganizationAdditions(){
		global $plugin_dpc;
		$id 		= get_the_ID();
		$instance 	= $this->dpc->getModuleInstance('professional_rating');
		$skip		= array('name');

		return ', ' . $instance->getRatingSchema($id, $skip);
	}


	function getJsonLdString($data, $fields, $type, $additions = true){
		$op = '';
		$op .= "\n".'<script type="application/ld+json">{"@context": "http://schema.org", "@type": "'.$type.'"';
		$op .= $this->getJsonLdFields($data, $fields);
		$op .= (($type == 'Organization' && $additions == true) ? $this->getJsonLdOrganizationAdditions() : '');
		$op .= '}</script>';
		$op = str_replace(', }', '}', $op);

		return $op;
	}

	function proceedMicrodata($wp){
		global $post;

		foreach($this->microdataTypes as $type => $value){
			add_action('wp_footer', array($this, 'proceedMicrodata_'. $type), 1000);
		}

		if (is_object($post) && $post->post_type == 'product'){
			add_action('woocommerce_before_single_product', array($this, 'proceedMicrodataProduct'));
		} elseif(is_author()){
			//add_filter('wp_footer', array($this, 'proceedMicrodataPerson'), 1000);
			add_filter('wp_footer', array($this, 'proceedMicrodata_person'), 1000);
		} else {
			add_filter('wp_footer',	array($this, 'proceedMicrodataArticle'), 1000);
		}

		if(isset($this->settings['logo']) && !empty($this->settings['logo'])){
		    add_filter('wp_footer',	array($this, 'proceedLogo'), 1000);
		}
	}

	function proceedLogo(){
	    $fields	   = array('url', 'logo');
	    $data      = array('url' => get_site_url(), 'logo' => $this->settings['logo']);
	    echo $this->getJsonLdString($data, $fields, 'Organization', false);
	}

	function proceedMicrodataProduct(){
		global $post, $product;

		$image          	= '';
		$product_id 		= $product->get_id();
		$dimensions 		= @trim($product->get_dimensions());
		$description 	= trim(wp_strip_all_tags(apply_filters('woocommerce_short_description', $post->post_excerpt)));
		$tags 			= wc_get_product_tag_list($product_id);
		$categories 		= wc_get_product_category_list($product_id);

		if(!empty($dimensions)){
			$dimensions = explode("x", $dimensions);
			if (count($dimensions) == 3){
				$depth	= trim($dimensions[2]);
				$unit	= explode(" ", $depth);
				$unit	= (count($unit) == 2 ? $unit[1] : '');
				$width 	= $dimensions[0] . ' '. $unit;
				$height	= $dimensions[1] . ' ' .$unit;
			}
		}
		
		
		
		if(has_post_thumbnail($product_id)){
			$attachment_ids[0]	= get_post_thumbnail_id($product_id);
			$attachment			= wp_get_attachment_image_src($attachment_ids[0], 'full');
			$image				= $attachment[0];
		}
		
		
		
		$data	= array('name' => htmlspecialchars(get_the_title()), 'url' => get_the_permalink(), 'description' => $description, 'image' => $image, 'sku' => $product->get_sku(), 'weight' => $product->get_weight(), 'width' => (isset($width) ? $width : ''), 'height' => (isset($height) ? $height : ''), 'depth' => (isset($depth) ? $depth : ''), 'brand' => wp_strip_all_tags($tags), 'price' => (!strlen($product->get_price()) ? 0 : $product->get_price()), 'priceCurrency' => get_woocommerce_currency(), 'availability' => ($product->is_in_stock() ? 'InStock' : 'OutOfStock'), 'category' => wp_strip_all_tags($categories), 'ratingValue' => ($product->get_average_rating() > 0 ? $product->get_average_rating() : '0.9'), 'reviewCount' => $product->get_rating_count());
		$fields	= array('name', 'url', 'weight', 'width', 'height', 'depth', 'description', 'image', array('name' => 'brand', 'type' => 'Organization', 'fields' => array(array('forceName' => 'name', 'itemValue' => 'brand'))), array('name' => 'offers', 'type' => 'Offer', 'fields' => array('price', 'priceCurrency', 'availability', 'category')), 'sku');

		if($data['ratingValue'] >= 1){
			array_push($fields,  array('name' => 'aggregateRating', 'type' => 'AggregateRating', 'fields' => array('ratingValue', 'reviewCount')));
		}

		echo $this->getJsonLdString($data, $fields, 'Product');
	}

	function proceedMicrodata_person(){
		$data	= get_post_meta( get_the_id(), '_dpc-microdata-person', true, array());
		$fields	= array('name', 'address' => array('type' => 'PostalAddress', 'name' => 'address', 'fields' => array('name', 'addresscountry', 'addresslocality', 'addressregion', 'postofficeboxnumber', 'postalcode', 'streetaddress')), 'birthdate', 'worksfor', 'jobtitle', 'email', 'description', 'telephone', 'faxNumber', 'image');

		if(is_array($data) && count($data)){
			foreach($data as $row){
				echo $this->getJsonLdString($row, $fields, 'Person');
			}
		}
	}

	function proceedMicrodata_job(){
		$data	= get_post_meta( get_the_id(), '_dpc-microdata-job', true, array());
		$fields	= 	array(
						'name',
		                'title' => array('name' => 'title', 'itemValue' => 'name'),
						'description',
						'validthrough',
						'employmenttype',
						'hiringorganization' =>  array(
							'type' => 'Organization',
							'name' => 'hiringOrganization',
							'fields' => array('name' => array('name' => 'name', 'itemValue' => 'companyname'),
								'sameAs' => array('name' => 'sameAs', 'itemValue' => 'companywebsite')
							)
						),
						'joblocation' => array(
							'type' => 'Place',
							'name' => 'jobLocation',
							'fields' => array('address' => array('type' => 'PostalAddress', 'name' => 'address', 'fields' => array('name', 'addresscountry', 'addressregion', 'addresslocality', 'postalcode', 'streetaddress')))
						),
						'baseSalary' =>  array(
							'type' 		=> 'MonetaryAmount',
							'name' 		=> 'baseSalary',
							'fields'	=> 	array('value','currency')
						),
						'educationrequirements',
						'employmenttype',
						'experiencerequirements',
						'industry',
						'qualifications',
						'responsibilities',
						'skills',
						'workhours',
                        'datePosted' => array(
                            'name' => 'datePosted',
                            'class' => $this->dpc->oFunctions,
                            'method' => 'getPostDate',
                            'params' => array(get_the_ID())
                        )
					);

		if(is_array($data) && count($data)){
			foreach($data as $row){
				echo $this->getJsonLdString($row, $fields, 'JobPosting');
			}
		}
	}

	function proceedMicrodata_video(){
		$data	= get_post_meta( get_the_id(), '_dpc-microdata-video', true, array());
		$fields	= array('name', 'caption', 'description', 'thumbnailurl', 'uploaddate', 'duration', 'contenturl', 'embedurl', 'videoquality');
		if(is_array($data) && count($data)){
			foreach($data as $row){
				echo $this->getJsonLdString($row, $fields, 'VideoObject');
			}
		}
	}

	function proceedMicrodata_location(){
		$data	= get_post_meta( get_the_id(), '_dpc-microdata-location', true, array());
		$fields	= array('name', 'location' => array('type' => 'Place', 'name' => 'location', 'fields' => array('address' => array('type' => 'PostalAddress', 'name' => 'address', 'fields' => array('addresscountry', 'addresslocality', 'addressregion', 'postofficeboxnumber', 'postalcode', 'streetaddress')))), 'faxnumber', 'telephone', 'email');
		if(is_array($data) && count($data)){
			foreach($data as $row){
				echo $this->getJsonLdString($row, $fields, 'Organization');
			}
		}
	}

	function proceedMicrodata_cwork(){
		$data	= get_post_meta( get_the_id(), '_dpc-microdata-cwork', true, array());
		$fields	= array('name', 'description', 'creator', 'copyrightholder');
		if(is_array($data) && count($data)){
			foreach($data as $row){
				echo $this->getJsonLdString($row, $fields, 'CreativeWork');
			}
		}
	}

	function convertMinutesToISO8601($value){
	    if(!is_int(($value))){ $value = 0; }

		$res = '';
		if(strpos($value, ':') !== false){
			$split 		= explode(':', $value);
			$minutes	= $split[1];
			$hours		= $split[0];
		} else {
			$minutes	= intval($value) % 60;
			$hours		= intval($value / 60);
		}

		if($minutes > 0 || $hours > 0){
			$res = 'PT' . ($hours > 0 ? $hours.'H' : '') . ($minutes > 0 ? $minutes.'M' : '');
		}
		return $res;
	}

	function proceedMicrodata_recipe(){
		$data	= get_post_meta( get_the_id(), '_dpc-microdata-recipe', true, array());
		$fields	= array('name', 'author', 'description', 'image', 'preptime' => array('name' => 'preptime', 'class' => $this, 'method' => 'convertMinutesToISO8601'), 'totaltime' => array('name' => 'totaltime', 'class' => $this, 'method' => 'convertMinutesToISO8601'), 'recipeyield', 'recipeingredient' => array('name' => 'recipeingredient', 'list' => true, 'seperator' => ','), 'recipeinstructions' => array('name' => 'recipeinstructions', 'list' => true, 'seperator' => ','), 'nutrition' => array('name' => 'nutrition', 'type' => 'NutritionInformation', 'fields' => array('calories', 'fatcontent', 'carbohydratecontent', 'cholesterolcontent', 'fibercontent', 'proteincontent', 'saturatedfatcontent', 'servingsize', 'sodiumcontent', 'sugarcontent')));
		if(is_array($data) && count($data)){
			foreach($data as $row){
				echo $this->getJsonLdString($row, $fields, 'Recipe');
			}
		}
	}

	function proceedMicrodata_event(){
		$data	= get_post_meta( get_the_id(), '_dpc-microdata-event', true, array());
		$fields	= array('name', 'description', 'location' => array('type' => 'Place', 'name' => 'location', 'fields' => array('name', 'addresscountry', 'addresslocality', 'addressregion', 'postofficeboxnumber', 'postalcode', 'streetaddress')), 'startdate', 'enddate', 'image', 'offers', 'performer', 'url');
		if(is_array($data) && count($data)){
			foreach($data as $row){
				echo $this->getJsonLdString($row, $fields, 'Event');
			}
		}
	}

	function proceedMicrodata_faq(){
	    $data	= get_post_meta( get_the_id(), '_dpc-microdata-faq', true, array());

	    if(is_array($data) && count($data)){
	        $i = 0;
            $str = '<script type="application/ld+json">';
            $str .= '{';
            $str .= '    "@context": "https://schema.org",';
            $str .= '    "@type": "FAQPage",';
            $str .= '    "mainEntity": [';
            foreach($data as $row){
                $str .= (++$i != 1 ? ', ' : '');
                $str .= '{ "@type": "Question", "name": "'.htmlentities($row['question']).'", "acceptedAnswer": { "@type": "Answer", "text": "'.htmlentities($row['answer']).'" } }';

            }
            $str .= '    ]';
            $str .= '}';
            $str .= '</script>';
	        echo $str;
	    }
	}

	function proceedMicrodataArticle($content = false){
	    if($content === false) { return ''; }
		global $post;
		if(is_null($post)) { return ''; }

		$newContent = $content;
		$section 	= array();
		$keywords 	= array();
		$image 		= '';
		$thumbnail 	= get_the_post_thumbnail($post->ID, 'full');

		foreach(get_the_category() as $category){
			$section[] = $category->name;
		}
		$section = !empty($section) ? implode(', ', $section) : '';


		$posttags = get_the_tags();
		if(!empty($posttags)){
			foreach($posttags as $tag){
				$keywords[] = $tag->name;
			}
		}
		$keywords = !empty($keywords) ? implode(', ', $keywords) : '';


		if (preg_match('|\ssrc="(.*?)"|i', $thumbnail, $matches)){
			$image = $matches[1];
		}

		$microData  = '<div itemscope itemtype="http://schema.org/Article">
							<div style="display:none">
								<span itemprop="name">'.htmlspecialchars(get_the_title()).'</span>
								<link itemprop="url" href="'.get_the_permalink().'" />
								<meta itemprop="author" content="'.esc_attr(get_the_author()).'" />';
		$microData .= strlen($section) ?
		'<span itemprop="ArticleSection">'.$section.'</span>' : '';
		$microData .= strlen($keywords)?
		'<meta itemprop="keywords" content="'.esc_attr($keywords).'" />' : '';
		$microData .= strlen($post->post_excerpt) ?
		'<div itemprop="description">'. $post->post_excerpt .'</div>' : '';
		$microData .= strlen($image) ?
		'<a itemprop="image" href="'.$image.'">'.get_the_post_thumbnail($post->ID, 'full').'</a>' : '';
		$microData .= '			</div>
							<div itemprop="ArticleBody">'.$newContent .'</div>
						</div>';

		$newContent = $microData;

		return $newContent;
	}

	function add_opengraph_doctype( $output ) {
		return $output . ' xmlns:og="http://opengraphprotocol.org/schema/" xmlns:fb="http://www.facebook.com/2008/fbml"';
	}

	function modalEnqueueStyles(){
	    wp_enqueue_style('dpc-basic-metadata-jquery-context', plugins_url( 'assets/contextMenu/jquery.contextMenu.min.css', __FILE__ ));
	    wp_enqueue_style('dpc-professional-microdata-modalstyles', plugins_url( 'assets/css/modal.css', __FILE__ ));
	}

	function modalTab(){
		echo '<span class="dpc-modal-tab dpc-modal-tab-trigger '.(isset($_GET['tab']) && $_GET['tab'] == 'micro' ? 'active' : '').'" id="tab-'. esc_attr($this->optionsHook) .'">'. esc_html($this->moduleTitle) .'</span>';
	}

	function modalBody(){
		echo '<div class="dpc-modal-content '.(isset($_GET['tab']) && $_GET['tab'] == 'micro' ? 'active' : '').'" id="content-'. esc_attr($this->optionsHook) .'">';
		$this->dpc->new_parseSettingsByXml($this->settingsFile, 'modal', false, true, false, $this->dpc->modal->meta);
		echo '</div>';
	}

	function modalSave(){
		foreach($this->microdataTypes as $type => $value){
		    update_post_meta(intval($_POST['dpc_current_id']), '_dpc-microdata-'.$type, isset($_POST['dpc-microdata'][$type]) ? $_POST['dpc-microdata'][$type] : array());
		}

		//update status
		$status = get_post_meta(intval($_POST['dpc_current_id']), '_delucks-seo-status', true);
		$status = is_array($status) ? $status : array();
		if((isset($_POST['dpc-microdata']) && count(['dpc-microdata']))) {
		    $status['micro'] = 2;
		} else {
		    $status['micro'] = 1;
		}
		update_post_meta(intval($_POST['dpc_current_id']), '_delucks-seo-status', $status);
	}

	function initXMLText(){
	    esc_html__('Microdata', 'delucks-seo');
	    esc_html__('Microdata submit invisble information to search engines which they can display as additional infos to your meta data in its search results pages.', 'delucks-seo');
	    esc_html__('Microdata are for example release dates, (WooCommerce) shop, author, business and social media information.', 'delucks-seo');
	    esc_html__('With activated microdata you support Google, Facebook etc. to match your websites information easier.', 'delucks-seo');
	    esc_html__('Activated', 'delucks-seo');
	    esc_html__('Deactivated', 'delucks-seo');
	    esc_html__('Frontpage', 'delucks-seo');
	    esc_html__('Activate structurized data for videos', 'delucks-seo');
	    esc_html__('Activate structurized data for locations', 'delucks-seo');
	    esc_html__('User', 'delucks-seo');
	    esc_html__('Activate structurized data for videos', 'delucks-seo');
	    esc_html__('Activate structurized data for locations', 'delucks-seo');
	    esc_html__('Automatic structuring', 'delucks-seo');
	    esc_html__('Choose the post types to automatically embed its relevant information. Also supports WooCommerce products.', 'delucks-seo');
	    esc_html__('Activate structuring of data on', 'delucks-seo');
	    esc_html__('We recommend you to insert location data and publishing information on posts and pages. Only if you know that you won’t update your posts and pages in the next year, you should switch this of and structure your Microdata manually (below).', 'delucks-seo');
	    esc_html__('Users', 'delucks-seo');
	    esc_html__('Insert structured data of locations', 'delucks-seo');
	    esc_html__('YouTube API Key', 'delucks-seo');
	    esc_html__('Claim your API key', 'delucks-seo');
	    esc_html__('Add location data', 'delucks-seo');
	    esc_html__('Logo', 'delucks-seo');
	    esc_html__('Select an image', 'delucks-seo');
	    esc_html__('Address', 'delucks-seo');
	    esc_html__('Please fill out all relevant contact information about your company for better ranking results on regional searches. If you have more than one location, you can add more.', 'delucks-seo');
	    esc_html__('Company name', 'delucks-seo');
	    esc_html__('Street, house number', 'delucks-seo');
	    esc_html__('Zipcode', 'delucks-seo');
	    esc_html__('City', 'delucks-seo');
	    esc_html__('Country', 'delucks-seo');
	    esc_html__('Contact', 'delucks-seo');
	    esc_html__('E-Mail address', 'delucks-seo');
	    esc_html__('Phone number', 'delucks-seo');
	    esc_html__('Fax number', 'delucks-seo');
	    esc_html__('Opening times', 'delucks-seo');
	    esc_html__('Opened', 'delucks-seo');
	    esc_html__('Mon', 'delucks-seo');
	    esc_html__('Tue', 'delucks-seo');
	    esc_html__('Wed', 'delucks-seo');
	    esc_html__('Thu', 'delucks-seo');
	    esc_html__('Fri', 'delucks-seo');
	    esc_html__('Sat', 'delucks-seo');
	    esc_html__('Sun', 'delucks-seo');
	    esc_html__('from', 'delucks-seo');
	    esc_html__('until', 'delucks-seo');
	    esc_html__('Price Range', 'delucks-seo');
	    esc_html__('Time format settings', 'delucks-seo');
	    esc_html__('Cancel', 'delucks-seo');
	    esc_html__('Save', 'delucks-seo');
	    esc_html__('Name', 'delucks-seo');
	    esc_html__('Job Title', 'delucks-seo');
	    esc_html__('Works For', 'delucks-seo');
	    esc_html__('Email Address', 'delucks-seo');
	    esc_html__('Description', 'delucks-seo');
	    esc_html__('Birthday', 'delucks-seo');
	    esc_html__('Address', 'delucks-seo');
	    esc_html__('P.O. Box', 'delucks-seo');
	    esc_html__('City', 'delucks-seo');
	    esc_html__('State / Region', 'delucks-seo');
	    esc_html__('Postal Code', 'delucks-seo');
	    esc_html__('Telephone', 'delucks-seo');
	    esc_html__('URL to video', 'delucks-seo');
	    esc_html__('Get YouTube Data', 'delucks-seo');
	    esc_html__('Name', 'delucks-seo');
	    esc_html__('Length of video', 'delucks-seo');
	    esc_html__('Quality', 'delucks-seo');
	    esc_html__('Thumbnail URL', 'delucks-seo');
	    esc_html__('Upload date', 'delucks-seo');
	    esc_html__('Embed URL', 'delucks-seo');
	    esc_html__('Street, house number', 'delucks-seo');
	    esc_html__('Creative Work', 'delucks-seo');
	    esc_html__('Creator', 'delucks-seo');
	    esc_html__('Copyright', 'delucks-seo');
	    esc_html__('Recipe', 'delucks-seo');
	    esc_html__('Or insert the image url', 'delucks-seo');
	    esc_html__('Image URL', 'delucks-seo');
	    esc_html__('Recipe category', 'delucks-seo');
	    esc_html__('Author', 'delucks-seo');
	    esc_html__('Ingredients (comma separated)', 'delucks-seo');
	    esc_html__('Instructions', 'delucks-seo');
	    esc_html__('Preparation time', 'delucks-seo');
	    esc_html__('Total time', 'delucks-seo');
	    esc_html__('Cooking method', 'delucks-seo');
	    esc_html__('Yield', 'delucks-seo');
	    esc_html__('Serving size', 'delucks-seo');
	    esc_html__('Calories', 'delucks-seo');
	    esc_html__('Fat content', 'delucks-seo');
	    esc_html__('Carbohydrate content', 'delucks-seo');
	    esc_html__('Cholesterol content', 'delucks-seo');
	    esc_html__('Fiber content', 'delucks-seo');
	    esc_html__('Protein content', 'delucks-seo');
	    esc_html__('Saturated fat content', 'delucks-seo');
	    esc_html__('Sodium content', 'delucks-seo');
	    esc_html__('Sugar content', 'delucks-seo');
	    esc_html__('Event', 'delucks-seo');
	    esc_html__('Start date', 'delucks-seo');
	    esc_html__('End date', 'delucks-seo');
	    esc_html__('Event image', 'delucks-seo');
	    esc_html__('Event address', 'delucks-seo');
	    esc_html__('Answer', 'delucks-seo');
	    esc_html__('Question', 'delucks-seo');
	    esc_html__('Add', 'delucks-seo');
	    esc_html__('Save changes', 'delucks-seo');
	    esc_html__('Video microdata', 'delucks-seo');
	    esc_html__('Company microdata', 'delucks-seo');
	    esc_html__('Project microdata', 'delucks-seo');
	    esc_html__('Recipe microdata', 'delucks-seo');
	    esc_html__('Event microdata', 'delucks-seo');
	    esc_html__('Job microdata', 'delucks-seo');
	    esc_html__('Person microdata', 'delucks-seo');
	    esc_html__('FAQ microdata', 'delucks-seo');
	    esc_html__('Job details', 'delucks-seo');
	    esc_html__('Job location', 'delucks-seo');
	    esc_html__('Ingredients', 'delucks-seo');
	}

}