(function ($, undefined) {
    'use strict';
  
    var OFFSET = 5;
  
    $.fn.rating = function (options) {
      return this.each(function () {
        var $input = $(this);
        // Prevent against multiple instantiations.
        if ($input.data('rating'))
          return;
        $input.data('rating', true);
        // Merge data and parameter options.
        // Those provided as parameter prevail over the data ones.
        var opts = $.extend({}, $input.data(), options);
        // Sanitize start, stop, step, and fractions.
        // All of them start, stop, and step must be integers.
        opts.start = parseInt(opts.start, 10);
        opts.start = isNaN(opts.start) ? undefined : opts.start;
        // In case we don't have a valid stop rate try to get a reasonable
        // one based on the existence of a valid start rate.
        opts.stop = parseInt(opts.stop, 10);
        opts.stop = isNaN(opts.stop) ?
          opts.start + OFFSET || undefined : opts.stop;
        // 0 step is ignored.
        opts.step = parseInt(opts.step, 10) || undefined;
        // Symbol fractions and scale (number of significant digits).
        // 0 is ignored and negative numbers are turned to positive.
        opts.fractions = Math.abs(parseInt(opts.fractions, 10)) || undefined;
        opts.scale = Math.abs(parseInt(opts.scale, 10)) || undefined;
  
        // Extend/Override the default options with those provided either as
        // data attributes or function parameters.
        opts = $.extend({}, $.fn.rating.defaults, opts);
  
        // Fill rating symbols until index.
        var fillUntil = function (index) {
          // Get the index of the last whole symbol.
          var i = Math.floor(index);
          // Hide completely hidden symbols background.
          $rating.find('.rating-symbol-background')
            .css('visibility', 'visible')
            .slice(0, i).css('visibility', 'hidden');
          var $rates = $rating.find('.rating-symbol-foreground');
          // Reset foreground
          $rates.width(0);
          // Fill all the foreground symbols up to the selected one.
          $rates.slice(0, i).width('auto');
          // Partially fill the fractional one.
          $rates.eq(i).width(index % 1 * 100 + '%');
        };
  
        // Calculate the rate of an index according the the start and step.
        var indexToRate = function (index) {
          return opts.start + Math.floor(index) * opts.step +
            opts.step * roundToFraction(index % 1);
        };
  
        // Calculate the corresponding index for a rate.
        var rateToIndex = function (rate) {
          return (rate - opts.start) / opts.step;
        };
  
        // Round index to the configured opts.fractions.
        var roundToFraction = function (index) {
          // Get the closest top fraction.
          var fraction = Math.ceil(index % 1 * opts.fractions) / opts.fractions;
          // Truncate decimal trying to avoid float precission issues.
          var p = Math.pow(10, opts.scale);
          return Math.floor(index) + Math.floor(fraction * p) / p;
        };
  
        // Check the rate is in the proper range [start..stop].
        var contains = function (rate) {
          var start = opts.step > 0 ? opts.start : opts.stop;
          var stop = opts.step > 0 ? opts.stop : opts.start;
          return start <= rate && rate <= stop;
        };
  
        // Update empty and filled rating symbols according to a rate.
        var updateRate = function (rate) {
          var value = parseFloat(rate);
          if (contains(value)) {
              if(!$($input).parents('.dpc-rating').hasClass('thumbs')){
                  fillUntil(rateToIndex(value));
              }
          }
        };
  
        // Call f only if the input is enabled.
        var ifEnabled = function (f) {
          return function (e) {
            if (!$input.prop('disabled') && !$input.prop('readonly')) {
              f.call(this, e);
            }
          }
        };
  
        // Build the rating control.
        var $rating = $('<div></div>').insertBefore($input);
        for (var i = 1; i <= rateToIndex(opts.stop); i++) {
          // Create the rating symbol container.
          var $symbol = $('<div class="rating-symbol"></div>').css({
              display: 'inline-block',
              position: 'relative'
          });
          // Add background symbol to the symbol container.
          $('<div class="rating-symbol-background ' + opts.empty + '"></div>')
            .appendTo($symbol);
          // Add foreground symbol to the symbol container.
          // The filled icon is wrapped with a div to allow fractional selection.
          $('<div class="rating-symbol-foreground"></div>')
            .append('<div class="dpc-rating-icon ' + opts.filled + '"></div>')
            .css({
              display: 'inline-block',
              position: 'absolute',
              overflow: 'hidden',
              left: 0,
              width: 0
            }).appendTo($symbol);
          $rating.append($symbol);
          opts.extendSymbol.call($symbol, indexToRate(i));
        }
        // Initialize the rating control with the associated input value rate.
        updateRate($input.val());
  
        // Keep rating control and its associated input in sync.
        $input.on('change', function () {
            updateRate($(this).val());
        });
  
        var fractionalIndex = function (e) {
          var $symbol = $(e.currentTarget);
          var x = (e.pageX || e.originalEvent.touches[0].pageX) - $symbol.offset().left;
          return $symbol.index() + x / $symbol.width();
        };
        $rating
          .on('mousedown touchstart', '.rating-symbol', ifEnabled(function (e) {
            // Set input 'trigger' the change event.
            $input.val(indexToRate(fractionalIndex(e))).change();
          }))
          .on('mousemove touchmove', '.rating-symbol', ifEnabled(function (e) {
            // Fill the symbols as fractions chunks.
              if(!$($input).parents('.dpc-rating').hasClass('thumbs')){
                  fillUntil(roundToFraction(fractionalIndex(e)));
              }
          }))
          .on('mouseleave touchend', '.rating-symbol', ifEnabled(function () {
            // Restore on hover out.
              if(!$($input).parents('.dpc-rating').hasClass('thumbs')){
                  fillUntil(rateToIndex(parseFloat($input.val())));
              }
          }));
      });
    };
  
    // Plugin defaults.
    $.fn.rating.defaults = {
      filled: 'glyphicon glyphicon-star',
      empty: 'glyphicon glyphicon-star-empty',
      start: 0,
      stop: OFFSET,
      step: 1,
      fractions: 1,
      scale: 3,
      extendSymbol: function (rate) {},
    };
  
        $(function () {
            $('input.dpc-rating-input').rating();
            
        console.log(dpc_rating_settings)
            /* patch for thumb up/down view*/
        if(jQuery('.dpc-rating').hasClass('thumbs')){
          jQuery('.dpc-rating.thumbs .rating-symbol:not(:first):not(:last)').hide();
                $(".rating-symbol-background", ".dpc-rating.thumbs .rating-symbol:first").attr('class', 'rating-symbol-background dpc-rating-icon thumb-rating ' + dpc_rating_settings.rating_symbol_thumbsdown);
                $(".rating-symbol-foreground > .dpc-rating-icon", ".dpc-rating.thumbs .rating-symbol:first").attr('class', 'rating-symbol-background dpc-rating-icon  thumb-rating ' + dpc_rating_settings.rating_symbol_thumbsdown);
                $(".rating-symbol-background", ".dpc-rating.thumbs .rating-symbol:last").attr('class', 'rating-symbol-background dpc-rating-icon  thumb-rating ' + dpc_rating_settings.rating_symbol_thumbsup);
                $(".rating-symbol-foreground > .dpc-rating-icon", ".dpc-rating.thumbs .rating-symbol:last").attr('class', 'rating-symbol-background dpc-rating-icon  thumb-rating ' + dpc_rating_settings.rating_symbol_thumbsup);
            }
      
            $(".rating-symbol-background").css("color", (dpc_rating_settings.starEmpty == '' ? '#b3b3b3' : dpc_rating_settings.starEmpty));
            $(".rating-symbol-foreground > .dpc-rating-icon").css("color", (dpc_rating_settings.starRated == '' ? '#f8d300' : dpc_rating_settings.starRated));
      });
  
  }(jQuery));
  
  jQuery(document).ready(function($){
      var rating = 0;
  
      $('.dpc-rating.thumbs .rating-symbol').on('click', function(){
          jQuery('.rating-symbol-background', this).css('color', (dpc_rating_settings.starRated == '' ? '#f8d300' : dpc_rating_settings.starRated));
          jQuery('.rating-symbol-background', $(this).siblings()).css('color', (dpc_rating_settings.starEmpty == '' ? '#b3b3b3' : dpc_rating_settings.starEmpty));
      });
      
      $(".dpc-rating-input.comment-do-rate").on('change', function () {
          afterRating();
      });
  
      $(".dpc-rating-input.dpc-do-rate").on('change', function () {
  
          rating = $(this).val();
          $.post("/wp-admin/admin-ajax.php",{
                  idBox : dpc_rating_settings.idBox,
                  rate : $(this).val(),
                  action :'dpc_professional_rating_rate'
              },
              function(data) {
  
                  if(!data.error){
                      if(dpc_rating_settings.commentRating == 'on'){
                          if ($("#dpc-rating-comment").length > 0){
                              $("#dpc-rating-comment").val(rating);
                              $("html, body").animate({ scrollTop: ($(".comment-form-dpc-rating-comment").offset().top)}, "slow");
                          }
                      }
                      $('form .dpc-rating-message').html('<div class="dpc-alert dpc-alert-success"><div class="dpc-entry">'+dpc_rating_settings.textRated+'</div></div>');
  
                      $(".dpc-rating-input.dpc-do-rate").attr("readonly", "readonly");
                  }else{
                      $('form .dpc-rating-message').html('<div class="dpc-alert dpc-alert-danger"><div class="dpc-entry">'+dpc_rating_settings.textError+'</div></div>');
                  }
                  afterRating();
              },
              'json'
          );
      });
  
      $(".dpc-rating-input.comment-do-rate.dpc-undo-rating-comments").on('change', function () {
          rating = $(this).val();
          if(dpc_rating_settings.commentRating == 'on'){
              if ($("#dpc-rating-comment").length > 0){
                  $("#dpc-rating-comment").val(rating);
                  $("html, body").animate({ scrollTop: ($(".comment-form-dpc-rating-comment").offset().top)}, "slow");
              }
          }
          $('form .dpc-rating-message').html('<div class="dpc-alert dpc-alert-success"><div class="dpc-entry">'+dpc_rating_settings.textRated+'</div></div>');
      });
  
  
      if(window.location.hash == '#respond') {
          setTimeout(function(){
              $('.btn-delucks-show-comment-form').trigger('click');
              if(typeof($("#respond")) !== 'undefinded'){
                  $(document).scrollTop( $(".comment-respond").last().offset().top );
              }
          }, 500);
      }
  
      //Remove all other comment listings if comment block is used
      if($('.dpc-comments-area.delucks-comments-block.dpc-comments-area-inline').length){
          $('.comments-area-background').each(function(){
              if(!$(this).parents('.dpc-comments-area-inline').length){
                  $('.comments-title, .delucks_comment_loadmore, .comments-area.delucks-comments-block.comments-area', this).remove();
              }
          });
      }
  
      if(typeof(dpc_rating_settings.gutenberg_toggle !== 'undefined') && dpc_rating_settings.gutenberg_toggle !== '' && $('.dpc-comments-area-inline').length){
          if(dpc_rating_settings.gutenberg_toggle == 'form'){
              //Remove all other comment forms if comment block is used
              $('.comment-respond').each(function(){
                  if(!$(this).parent('.dpc-comments-area-inline').length){
                      if(!$(this).parents('.dpc-comments-area-inline').length){
                          $(this).parent().remove();
                      }
                      $(this).remove();
                  }
              });
          }
      }
  
  
  });
  
  function afterRating(elem){
      var comment_rating = jQuery('.dpc-rating-input.comment-do-rate').val();
      jQuery('input#the_comment_rating').val(comment_rating);
  }