<?php

class DPC_Module_Professional_Rating implements DPC_Module_Interface {

    var $dpc						= null;
    var $isSubmodule				= true;
    var $moduleDirectory 			= 'professional/';
    var $moduleTitle 				= '';
    var $optionsHook				= 'dpc-professional-rating';
    var $settingsFile				= false;
    var $settings 					= array();
    var $isRating					= false;
    var $objectId					= 0;
    var $maxRating					= 5;
    var $worstRating				= 1;
    var $defaultRatingStartValue 	= 4;
    var $defaultRatingIcons			= array('starfilled','staroutline','heartoutline','heart','heartalt','smiley','smileyalt');

    /**
     * @param object $dpc
     */
    function __construct(&$dpc){
        $this->settingsFile = dirname(__FILE__) . '/settings.xml';
        $this->dpc 			= $dpc;
        $this->moduleTitle	= esc_html__('Rating', 'delucks-seo');
        $this->settings 	= $this->dpc->getModuleSettings('professional_rating');
		
		
        if(!isset($this->settings) || !is_array($this->settings) || !isset($this->settings['rating_symbol']) || empty($this->settings['rating_symbol']) || !in_array($this->settings['rating_symbol'], $this->defaultRatingIcons)){
            $this->settings['rating_symbol'] = 'starfilled'; //set default rating icon
        }

        add_filter('comment_notification_text', array( $this, 'dpc_rating_notification'), 10, 2);
        add_filter('comment_moderation_text', array( $this, 'dpc_rating_notification'), 10, 2);

        require_once( 'gutenberg/gutenberg.php' );
    }

    function set_mail_content_type() {
        return 'text/html';
    }

    function getTranslationJs(){
        return array(
            'out_of'					=> esc_html__('out of', 'delucks-seo'),
            'based_on'					=> esc_html__('based on', 'delucks-seo'),
            'ratings'					=> esc_html__('ratings', 'delucks-seo'),
            'total_ratings'				=> esc_html__('Total ratings', 'delucks-seo'),
            'rating'					=> esc_html__('rating', 'delucks-seo'),
            'comments'					=> esc_html__('Comments', 'delucks-seo'),
            'commentsDesc'				=> esc_html__('Displays comments within content.', 'delucks-seo'),
        );
    }

    function addCommentRatingColumn($columns){
        return array_merge( $columns, array(
            'rating' => esc_html__('Valuation', 'delucks-seo')
        ));
    }

    function showCommentRatingColumn($column, $comment_ID){
        switch($column){
            case 'rating':
                if(($rating = get_comment_meta($comment_ID, 'dpc-rating-comment', true)) &&  $rating > 0){
                    echo esc_html(str_repeat('&#9733', (int)$rating));
                } else {
                    echo '-';
                }
                break;
        }
    }

    function addRatingMetaBoxCallback(){
        $screens = array();
        array_push($screens, 'comment');
        foreach($screens as $screen){
            add_meta_box('edit_rating_metabox', esc_html__('DELUCKS comment information', 'delucks-seo'), array($this, 'editCommentRatingMetabox'), $screen, 'normal', 'high');
        }
    }

    function editCommentRatingMetabox($comment){
        $comment_rating = get_comment_meta($comment->comment_ID, 'dpc-rating-comment', true);
        $privacy		= get_comment_meta($comment->comment_ID, 'dpc_privacy', true);
        $company		= get_comment_meta($comment->comment_ID, 'dpc_company_name', true);

        if(isset($comment_rating)){
            ?>
		    <div id="namediv">
		        <table class="form-table editcomment comment_xtra">
			        <tbody>
				        <tr>
							<td class="first"><label for="newcomment_author_url"><?php echo esc_html__('Comment rating', 'delucks-seo'); ?></label></td>
							<td>
								<input type="number" min="1" max="5" id="dpc_rating_comment" name="dpc-rating-comment" class="code" value="<?php echo esc_attr($comment_rating); ?>" />
							</td>
						</tr>
			       </tbody>
		       </table>
			</div>
	    <?php
		}else{
		    echo esc_html__('This comment does not have a rating.', 'delucks-seo');
		}

		if (isset($this->settings['gdprCommentCheckbox']) && isset($privacy)){
		?>
			<div id="namediv">
		        <table class="form-table editcomment comment_xtra">
			        <tbody>
				        <tr>
							<td class="first"><label for="newcomment_author_url"><?php echo esc_html__('Accepted privacy agreement', 'delucks-seo'); ?></label></td>
							<td>
								<strong><?php echo esc_html__('Yes', 'delucks-seo'); ?></strong>
							</td>
						</tr>
			       </tbody>
		       </table>
			</div>
		<?php
		}

		if (isset($this->settings['companyCommentField']) && isset($company)){
			?>
			<div id="namediv">
		        <table class="form-table editcomment comment_xtra">
			        <tbody>
				        <tr>
							<td class="first"><label for="newcomment_author_url"><?php echo esc_html__('Company name', 'delucks-seo'); ?></label></td>
							<td>
								<input type="text" id="dpc_comment_company" name="dpc_company_name" class="code" value="<?php echo esc_attr($company); ?>" />
							</td>
						</tr>
			       </tbody>
		       </table>
			</div>
		<?php
		}
	}

	function commentMetaSave($comment_ID, $comment_approved) {

		if(isset($_POST['dpc-rating-comment'])){
		    update_comment_meta( $comment_ID, 'dpc-rating-comment', sanitize_textarea_field($_POST['dpc-rating-comment']));
		}
		if(isset($_POST['dpc_company_name'])){
		    update_comment_meta( $comment_ID, 'dpc_company_name', sanitize_text_field($_POST['dpc_company_name']));
		}
	}

	function getLang($objectID = 0){
		$lang = $this->dpc->oFunctions->getPostLang($objectID, false);
		$lang = substr($lang, 0, (strpos($lang, '_') ? strpos($lang, '_') : 2));
		return $lang;
	}

	function adminSettingsHead(){
		wp_enqueue_style('dpc-professional-rating-settings', plugins_url( 'assets/css/settings.css', __FILE__ ), array(), DPC_VERSION);
		wp_enqueue_style('dpc-professional-rating-backend', plugins_url( 'assets/css/backend.css', __FILE__ ), array(), DPC_VERSION);
	}

	function adminSettings(){
		$this->dpc->parseSettingsByXml($this->settingsFile, 'general');
	}

	function adminHead(){
		wp_localize_script('dpc-professional-rating-backend', 'dpc_professional_rating_translator', $this->getTranslationJs());
	}

	function executeBackendActions(){
		add_filter('manage_comments_custom_column', array($this, 'showCommentRatingColumn'), 10, 2);
		add_filter('manage_edit-comments_columns', array($this, 'addCommentRatingColumn'));
		add_action('admin_enqueue_scripts', array(&$this, 'hook_admin_enqueue_scripts'));
		add_action('add_meta_boxes', array($this, 'addRatingMetaBoxCallback'), 100);
		add_action('edit_comment', array($this, 'commentMetaSave'), 10, 2 );
	}

	function hook_admin_enqueue_scripts(){
		echo '<script type="text/javascript">'."\n";
		echo '	var dpc_professional_rating_translator = ' . wp_json_encode($this->getTranslationJs()) . ";\n";
		echo '</script>'."\n";
	}

	function append_form_fields( $fields ) {
		$addition = '';
		$additionCheck = '';
		$lang = (defined('ICL_LANGUAGE_CODE') && ICL_LANGUAGE_CODE != 'all' ? ICL_LANGUAGE_CODE : $this->getLang(0));

		if (isset($this->settings['companyCommentField'])){
		    $comapanyLabel 	= (isset($this->settings[$lang]['companyCommentFieldLabel']) && strlen($this->settings[$lang]['companyCommentFieldLabel'])) ? $this->settings[$lang]['companyCommentFieldLabel'] : esc_html__( 'Company name', 'delucks-seo' );

			$addition .= '<p class="dpc-comment-form-company">';
			$addition .= '	<label>' . $comapanyLabel . '</label>';
			$addition .= '	<input id="dpc_company_name" name="dpc_company_name" value="" size="30" maxlength="245" type="text" />';
			$addition .= '</p>';
		}

		if (isset($this->settings['gdprCommentCheckbox'])){
		    $gdprLabel 	= (isset($this->settings[$lang]['gdprCommentText']) && strlen($this->settings[$lang]['gdprCommentText'])) ? stripslashes($this->settings[$lang]['gdprCommentText']) : esc_html__('I accept the privacy policy.', 'delucks-seo');

			$additionCheck .= '<p class="dpc-comment-form-dataprivacy">';
			$additionCheck .= '	<label><input type="checkbox" name="dpc_privacy" value="privacy-key" class="dpc-privacy-checkbox" aria-req="true">';
			$additionCheck .= '	' . $gdprLabel . '</label>';
			$additionCheck .= '</p>';
		}

		$comment_field = $fields['comment'];
		unset( $fields['comment'] );
		$fields['comment'] = $addition.$comment_field.$additionCheck;
		#$fields['comment'] .= $addition;
	    return $fields;
	}

	function verify_comment_privacy( $commentdata ) {
	    if ( isset($this->settings['gdprCommentCheckbox']) && !isset( $_POST['dpc_privacy'] ) && !is_admin() ){
	        wp_die( esc_html__( 'Error: You must agree to our privacy terms.', 'delucks-seo' ) );
		}

	    return $commentdata;
	}

	function save_comment_meta( $comment_id ) {
		if (isset( $_POST['dpc_privacy'])){
		    add_comment_meta( $comment_id, 'dpc_privacy', sanitize_text_field($_POST[ 'dpc_privacy' ] ));
		}
		if (isset( $_POST['dpc_company_name'])){
		    add_comment_meta( $comment_id, 'dpc_company_name', sanitize_text_field($_POST[ 'dpc_company_name' ] ));
		}
	}

	function executeFrontendActions(){
        if(!in_array('postman-smtp/postman-smtp.php', (array)get_option('active_plugins', array()), true) && (!function_exists('is_plugin_active_for_network') || !is_plugin_active_for_network('postman-smtp/postman-smtp.php'))){
	        include_once( ABSPATH . "wp-includes/pluggable.php" );	        
	    }

		if (isset($this->settings['gdprCommentCheckbox']) || isset($this->settings['companyCommentField'])){
			add_filter( 'comment_form_fields', array($this, 'append_form_fields') );
			add_filter( 'preprocess_comment', array($this, 'verify_comment_privacy'));
			add_action( 'comment_post', array($this, 'save_comment_meta'));

			if (isset($this->settings['companyCommentField'])){
				add_filter( 'get_comment_author', array($this, 'filter_comment_data'), 10, 1);
			}
		}

		add_action('wp',                          array($this, 'init'));
		add_action('wp_head',                     array($this, 'printScript'));
		add_filter('comment_text',                array($this, 'showCommentRating'), 10, 2);
		add_action('comment_post',                array($this, 'saveCommentRating'));
		add_filter( 'body_class',                 array($this, 'append_body_comment_classes') );
		remove_filter('wp_mail_content_type',     array($this, 'set_mail_content_type'));
		add_filter('wp_mail_content_type',        array($this, 'set_mail_content_type'));

		add_shortcode( 'delucks_show_comments',   array($this, 'dpc_show_comments_inline'));
	}

	function filter_comment_data($author){
		global $comment;

		$company 	= get_comment_meta($comment->comment_ID, 'dpc_company_name', true);
		$addition 	= ($company != '' ? '<span class="comment-company-name"><span>,</span>&nbsp;' . $company . '</span>' : '');
		return $author . $addition;
	}

	function registerWidgets(){
		add_action( 'widgets_init', array(&$this, 'hook_widgets_init'));
	}

	function hook_widgets_init(){
		require_once( 'widgets/rating.php' );
		register_widget( 'DelucksRating' );
	}

	function init(){
	    $this->objectId = get_queried_object_id();
	    if (isset($this->settings['showSearchFieldIn'])){
	        if (in_array('home', $this->settings['showSearchFieldIn']) && is_home()){
	            $this->isRating = true;
	            $this->objectId = -1;
	        }elseif(in_array('archives', $this->settings['showSearchFieldIn']) && is_archive()){
	            $this->isRating = true;
	            $this->objectId = -2;
	        }elseif($this->objectId != 0) {
	            $queriedObject = get_queried_object();
	            if(is_object($queriedObject) && property_exists($queriedObject, 'post_type')){
	                $postType = $queriedObject->post_type;
	                if(in_array($postType, $this->settings['showSearchFieldIn'])){
	                    $this->isRating = true;
	                }
	            }
	        }
	    }

	    if($this->isRating){
	        add_filter('preprocess_comment',      array($this, 'verifyRatingComment'));
	        add_filter('comment_form_fields',     array($this, 'the_rating'), 10, 1);
	    }

	    add_shortcode( 'dpc_rating', array($this, 'dpc_rating') );
	}

	function printScript(){
		$lang = (defined('ICL_LANGUAGE_CODE') && ICL_LANGUAGE_CODE != 'all' ? ICL_LANGUAGE_CODE : $this->getLang(0));

		$this->settings['idBox']		= $this->objectId;
		$this->settings['textRated'] 	= (isset($this->settings[$lang]) && isset($this->settings[$lang]['textRated']) && strlen($this->settings[$lang]['textRated'])) ? $this->settings[$lang]['textRated'] : esc_html__('Thanks for your review', 'delucks-seo');
		$this->settings['textIntial'] 	= (isset($this->settings[$lang]) && isset($this->settings[$lang]['textIntial']) && strlen($this->settings[$lang]['textIntial'])) ? $this->settings[$lang]['textIntial'] : esc_html__('Rate', 'delucks-seo');
		$this->settings['textError'] 	= (isset($this->settings[$lang]) && isset($this->settings[$lang]['textError']) && strlen($this->settings[$lang]['textError'])) ? $this->settings[$lang]['textError'] : esc_html__('An error has occurred', 'delucks-seo');
		$this->settings['starsNumber'] 	= (isset($this->settings[$lang]) && isset($this->settings[$lang]['starsNumber']) ? $this->settings['starsNumber'] : '5');

		echo '<script type="text/javascript">'."\n";
		echo '	var dpc_rating_settings = ' . wp_json_encode($this->settings) . ";\n";
		echo '</script>'."\n";

		if(get_post_type(get_the_ID())){ // skip on tags, 404
		    echo $this->getSchema(get_the_ID());
		}
	}

	/**
	 * Adds rating stars to each comment text
	 */
	function showCommentRating($text, $comment = null){
		$text = '<div class="comment_text">'.$text.'</div>';
		if($comment === null){
			return $text;
		}

		if(($rating = get_comment_meta( $comment->comment_ID, 'dpc-rating-comment', true )) &&  $rating > 0){
		    wp_enqueue_style('frontend', plugins_url( 'assets/css/frontend.css', __FILE__ ), array(), DPC_VERSION);
		    wp_enqueue_script('rating', plugins_url( 'assets/js/rating.min.js', __FILE__ ), array('jquery'), DPC_VERSION, $this->dpc->load_scripts_in_footer);

		    if($rating > $this->maxRating){ $rating = $this->maxRating; }
		    $thumbView    = (isset($this->settings['view_' . get_post_type()]) && $this->settings['view_' . get_post_type()] == 'thumbs') ? true : false;
		    $thumbSymbol  = ($rating <= 2.5 ? 'thumbdown' : 'thumbup');

			$commentRating = '<div class="dpc-rating result">';
			if($thumbView){
			    $commentRating .= '<span class="dpc-rating-icon '. $thumbSymbol .' resultIcon"></span>';
			} else {
			    $commentRating .= '		<input type="hidden" readonly="readonly" class="dpc-rating-input" data-filled="dpc-rating-icon '.$this->settings["rating_symbol"].'" data-empty="dpc-rating-icon '.$this->settings["rating_symbol"].' fa-3x" value="'.$rating.'"/>';
			}

			$commentRating .= '</div>';
			$commentRating .= '<div class="dpc-rating-message"></div>';
			$text =  $commentRating . $text;
		}
		return $text;
	}

	function the_rating($fields){
	    $this->defaultRatingStartValue = apply_filters('dpc_rating_default_rating_start_value', $this->defaultRatingStartValue);
	    wp_enqueue_style('frontend', plugins_url( 'assets/css/frontend.css', __FILE__ ), array(), DPC_VERSION);
	    #wp_enqueue_script('rating', plugins_url( 'assets/js/rating.js', __FILE__ ), array('jquery'), DPC_VERSION);
	    wp_enqueue_script('rating', plugins_url( 'assets/js/rating.min.js', __FILE__ ), array('jquery'), DPC_VERSION, $this->dpc->load_scripts_in_footer);

		$lang = (defined('ICL_LANGUAGE_CODE') && ICL_LANGUAGE_CODE != 'all' ? ICL_LANGUAGE_CODE : $this->getLang(0));
		if(!isset($this->settings["rating_symbol"]) || empty($this->settings["rating_symbol"])){
			$this->settings["rating_symbol"] = 'star';
		}
		
		$data = $this->getPostRatingData($this->objectId);
		$str  = '<div class="comment-form-rating"><label for="author">'.(isset($this->settings[$lang]) && isset($this->settings[$lang]['form_field_label']) && strlen($this->settings[$lang]['form_field_label']) ? $this->settings[$lang]['form_field_label'] : esc_html__('Your rating', 'delucks-seo')).'</label>';
		$str .= '	<div class="dpc-rating '.($this->settings['view_' . get_post_type()] ?? '').'" data-average="' . round($data['comment_rating_score'], 2) . '" data-id="' . $this->objectId . '">';
		$str .= '		<input type="hidden" data-id="'.$this->objectId.'" class="dpc-rating-input comment-do-rate '.(isset($this->settings['forceComments']) ? 'dpc-undo-rating-comments' : 'dpc-do-rate').'" data-filled="dpc-rating-icon '.$this->settings["rating_symbol"].'" data-empty="dpc-rating-icon '.$this->settings["rating_symbol"].' fa-3x" value="'.(is_int($this->defaultRatingStartValue) ? $this->defaultRatingStartValue : round($data['comment_rating_score'], 2)).'" '.($this->allreadyCounted($this->objectId) ? ' readonly="readonly"' : '').'/>';
		$str .= '		<input id="the_comment_rating" name="dpc-rating-comment" type="hidden" value="false" />';
		$str .= '	</div>';
		$str .= '	<div class="dpc-rating-message"></div>';
		$str .= '</div>';

		$fields['comment'] .= $str;
		return $fields;
	}

	function dpc_rating_notification($notify_message, $comment_id){
	    $comment 				= get_comment( $comment_id );
		if(empty($comment)){return $notify_message;}

	    $post 					= get_post( $comment->comment_post_ID );
		$comment_content 		= wp_specialchars_decode( $comment->comment_content );
		$comment_author_domain  = (strlen($comment->comment_author_IP) && gethostbyaddr($comment->comment_author_IP)) ? gethostbyaddr($comment->comment_author_IP) : 'Unknown';
		$comment_rating			= get_comment_meta($comment_id, 'dpc-rating-comment', true);

		if(!is_int($comment_rating) && isset($_POST) && isset($_POST['dpc-rating-comment'])){
		    $comment_rating = sanitize_text_field($_POST['dpc-rating-comment']);
		}

    	switch ( $comment->comment_type ) {
			case 'trackback':
			    $notify_message  = '<h2>'.sprintf( esc_html__( 'New trackback on your post "%s"' ), $post->post_title ) . '</h2><br/>';
			    $notify_message .= '<p>'.sprintf( esc_html__('Website: %1$s (IP: %2$s, %3$s)'), $comment->comment_author, $comment->comment_author_IP, $comment_author_domain ) . '</p><br/>';
			    $notify_message .= '<p>'.sprintf( esc_html__( 'URL: %s' ), $comment->comment_author_url ) . '</p>';
			    $notify_message .= '<p>'.sprintf( esc_html__( 'Comment: %s' ), "\r\n" . $comment_content ) . '</p>';
			    $notify_message .= '<p><strong>'.esc_html__( 'You can see all trackbacks on this post here:' ) . '</strong> ' . get_permalink($comment->comment_post_ID) . '#comments</p>';
			    $notify_message .= '</br> <p>'.sprintf( esc_html__('Permalink: %s'), get_comment_link( $comment ) ) . '</p>';
				break;
			case 'pingback':
			    $notify_message  = '<h2>'.sprintf( esc_html__( 'New pingback on your post "%s"' ), $post->post_title ) . '</h2><br/>';
			    $notify_message .= '<p>'.sprintf( esc_html__('Website: %1$s (IP: %2$s, %3$s)'), $comment->comment_author, $comment->comment_author_IP, $comment_author_domain ) . '</p>';
			    $notify_message .= '<p>'.sprintf( esc_html__( 'URL: %s' ), $comment->comment_author_url ) . '</p>';
				$notify_message .= '<p>'.sprintf( __( 'Comment: %s' ), "\r\n" . $comment_content ) . '</p>';
				$notify_message .= '<p><strong>'.esc_html__( 'You can see all pingbacks on this post here:' ) . '</strong> ' . get_permalink($comment->comment_post_ID) . '#comments</p>';
				$notify_message .= '</br> <p>'.sprintf( esc_html__('Permalink: %s'), get_comment_link( $comment ) ) . '</p>';
				break;
			default: // Comments
			    $notify_message  = '<h2>'.sprintf( esc_html__( 'New comment on your post "%s"' ), $post->post_title ) . '</h2><br/>';
			    $notify_message .= '<p>'.sprintf( esc_html__( 'Author: %1$s (IP: %2$s, %3$s)' ), $comment->comment_author, $comment->comment_author_IP, $comment_author_domain ) . '</p>';
			    $notify_message .= '<p>'.sprintf( esc_html__( 'Rating: %s' ), $comment_rating ) . '</p>';
			    $notify_message .= '<p>'.sprintf( esc_html__( 'Email: %s' ), $comment->comment_author_email ) . '</p>';
			    $notify_message .= '<p>'.sprintf( esc_html__( 'URL: %s' ), $comment->comment_author_url ) . '</p>';
			    $notify_message .= '<p>'.sprintf( esc_html__('Comment: %s' ), "\r\n" . $comment_content ) . '</p>';
			    $notify_message .= '<p><strong>'.esc_html__( 'You can see all comments on this post here:' ) . '</strong> ' . get_permalink($comment->comment_post_ID) . '#comments</p>';
			    $notify_message .= '</br> <p>'.sprintf( esc_html__('Permalink: %s'), get_comment_link( $comment ) ) . '</p>';
				break;
		}

		if ( user_can( $post->post_author, 'edit_comment', $comment->comment_ID ) ) {
			if ( EMPTY_TRASH_DAYS ) {
			    $notify_message .= '<p>'.sprintf( esc_html__( 'Trash it: %s' ), admin_url( "comment.php?action=trash&c={$comment->comment_ID}#wpbody-content" ) ) . '</p>';
			} else {
			    $notify_message .= '<p>'.sprintf( esc_html__( 'Delete it: %s' ), admin_url( "comment.php?action=delete&c={$comment->comment_ID}#wpbody-content" ) ) . '</p>';
			}
			$notify_message .= '<p>'.sprintf( esc_html__( 'Spam it: %s' ), admin_url( "comment.php?action=spam&c={$comment->comment_ID}#wpbody-content" ) ) . '</p>';
		}

	    return $notify_message;
	}

	function getSchema($id){
		$rating			= $this->getPostRatingData($id);
		$thePost		= get_post($id);
		$comments 		= get_comments(array('post_id' => $id, 'status' => 'approve', 'meta_key' => 'dpc-rating-comment'));

		//Check wether location microdata has been added. If so skip adding rating structured data, it will be added from microdata module.
		$locationMicro 	= get_post_meta((int)$id, '_dpc-microdata-location', true);
		$content 		= (!empty($locationMicro) ? true : false);

		if($content){
			return false;
		}

		//Add rating structured data
		$schema  = '<script type="application/ld+json">';
		$schema .= '	{
							"@context": "http://schema.org",
						  	"@type": "'.( is_front_page() ? 'CreativeWorkSeries' : $this->getSchemaType($thePost) ).'",';
		$schema .= 		$this->getRatingSchema($thePost);
		$schema .= '}';
		$schema .= '</script>';

		return $schema;

	}

	function getRatingSchema($id, $skip = array()){
		if(!is_object($id)){ return false; }
		
	    $rating			= $this->getPostRatingData($id->ID);
	    $thePost			= get_post($id);
	    $comments 		= get_comments(array('post_id' => $id->ID, 'status' => 'approve', 'meta_key' => 'dpc-rating-comment'));
	    $rating 			= $this->getPostRatingData($thePost->ID);

		$schema  = '';
		$schema .= '		"description": "'.htmlentities(dpc_metadescription()).'" '. (in_array('name', $skip) ? '' : ', "name": "'.htmlspecialchars((is_object($thePost) && property_exists($thePost, 'post_title') ? $thePost->post_title : '')).'"');

		if($rating['comment_count'] > '0' && round($rating['comment_rating_score'], 2) > '0'){
		    $schema = trim($schema, ',');
			$schema .= ',	  	"aggregateRating": {
							    	"@type": "AggregateRating",
							    	"ratingValue": "'.round($rating['comment_rating_score'], 2).'",
							    	"reviewCount": "'.$rating['comment_count'].'"
								},';
		}

		if(has_post_thumbnail($id)){
		    $schema = trim($schema, ',');
			$image_data = wp_get_attachment_image_src( get_post_thumbnail_id( $id ), "full" );
			$schema .= ',	"image": {
								"@type": "ImageObject",
							    "url": "'.$image_data[0].'",
							    "height": '.$image_data[1].',
							    "width": '.$image_data[2].'
							}';
		}
		if($this->getSchemaType($thePost) == 'Article'){
		    $schema = trim($schema, ',');
		    $schema .= ', "author": {
			        		"@type": "Person",
			        		"name": "'.(strlen($comment->comment_author) ? $comment->comment_author : 'Anonymous').'"
			      		  },';
			$schema .= '"headline": "'.$thePost->post_title.'",';
			$schema .= '"datePublished": "'.get_the_date( 'Y-m-d', $id ).'",';
			$schema .= '"dateModified": "'.get_the_modified_date( 'Y-m-d', $id ).'"';
			$schema .= '}';

			$schema .= '", mainEntityOfPage": {
					         "@type": "WebPage",
					         "@id": "'.get_site_url().'"
					      }';
		}
		if($rating['comment_count'] > '0' && round($rating['comment_rating_score'], 2) > '0'){
		    $schema = trim($schema, ',');
			$schema .= ',	"review": [';
								foreach($comments as $comment){
									$comment_meta = (int)get_comment_meta($comment->comment_ID, 'dpc-rating-comment', true);
									if($comment_meta > $this->maxRating){ $comment_meta = $this->maxRating; }

									$schema .= '	{
											      		"@type": "Review",
                							      		"author": {
                							        		"@type": "Person",
                							        		"name": "'.(strlen($comment->comment_author) ? $comment->comment_author : 'Anonymous').'"
                							      		},
											      		"datePublished": "'.$comment->comment_date.'",
											      		"description": "'.htmlspecialchars($comment->comment_content).'"';

									if($comment_meta && $comment_meta > 0){
							      		$schema .=',
							      		"reviewRating": {
							        		"@type": "Rating",
							        		"bestRating": "'.$this->maxRating.'",
							        		"ratingValue": "'.$comment_meta.'",
							        		"worstRating": "'.$this->worstRating.'"
							      		}';
									}
									$schema .= '},';
								}
			$schema = trim($schema,',');
			$schema .= '	]';
		}

		return $schema;
	}

	function getSchemaType($post){
		$type = get_post_type($post);
		$schema = (isset($this->settings['schema_'.$type]) && $this->settings['schema_'.$type] != '') ? $this->settings['schema_'.$type] : 'BlogPost';

		if($schema == 'CreativeWork'){
		    $schema = 'CreativeWorkSeries';
		}

		return $schema;
	}

	function dpc_rating($atts = [], $tag = ''){
		//$showRating = (isset($this->settings["commentRating"]) && $this->settings["commentRating"] == 'on' ? false : true);
		$atts = array_change_key_case((array)$atts, CASE_LOWER);
		$rating_atts = shortcode_atts(['alt_link' => '#respond'], $atts, $tag);
		return $this->getLegende(get_the_ID(), $rating_atts['alt_link']);
	}

	function verifyRatingComment($commentData){
		if(isset($_REQUEST['dpc-rating-comment']) && $commentData['comment_parent'] == 0){
			if ((isset($this->settings['forceComments']) && $this->settings['forceComments'] == 'on') && (!isset($_REQUEST['dpc-rating-comment']) || $_REQUEST['dpc-rating-comment'] <= 0)){
				$lang = (defined('ICL_LANGUAGE_CODE') && ICL_LANGUAGE_CODE != 'all' ? ICL_LANGUAGE_CODE : $this->getLang(0));
				wp_die((isset($this->settings[$lang]) && isset($this->settings[$lang]['textNotRated']) && strlen($this->settings[$lang]['textNotRated'])) ? esc_html($this->settings[$lang]['textNotRated']) : esc_html__('Please add a rating', 'delucks-seo').' <a href="#" onclick="window.history.back();return false;">'.esc_html__('Go back', 'delucks-seo').'</a>');
			}
		}
		return $commentData;
	}

	function saveCommentRating($comment_id){
		if(isset($_REQUEST['dpc-rating-comment']) && $_REQUEST['dpc-rating-comment'] > 0){
		    add_comment_meta($comment_id, 'dpc-rating-comment', sanitize_textarea_field(($_POST['dpc-rating-comment'])));
		}
	}

	function getLegende($postId, $altlink = false){
		$lang = (defined('ICL_LANGUAGE_CODE') && ICL_LANGUAGE_CODE != 'all' ? ICL_LANGUAGE_CODE : $this->getLang(0));
		if(isset($this->settings[$lang]) && isset($this->settings[$lang]['legend']) && strlen($this->settings[$lang]['legend'])){
			$s 				= $this->settings[$lang]['legend'];
			$rating 		= $this->getPostRatingData($postId);
			$ratingScore	= round($rating['comment_rating_score'], 2);
			$ratingValue	= round($rating['comment_rating_score'], 2);
			$ratingTotal	= $rating['rating_count'];
			$bestRating		= round($rating['comment_max_rating'], 2);
			$worstRating	= round($rating['comment_min_rating'], 2);
			$commentCount	= round($rating['comment_count'], 2);
			$ratingLinkText	= $this->settings[$lang]['textIntial'];

			if(!$ratingScore > 0 && isset($this->settings['show_legend_no_rating_text'])){
				$output  = '<div class="dpc-rating-wrap">';
				if(isset($this->settings['link_legend'])){ $output .= '<a href="'.($altlink ? $altlink : '#respond').'">'; }
				$output .= (isset($this->settings[$lang]) && isset($this->settings[$lang]['legend_no_rating_text']) && strlen($this->settings[$lang]['legend_no_rating_text']) ? $this->settings[$lang]['legend_no_rating_text'] : esc_html__('No ratings have been added yet', 'delucks-seo'));
				if(isset($this->settings['link_legend'])){ $output .= '</a>'; }
				$output .= '</div>';
				return $output;
			} else {
			    $rating = str_replace('['.esc_html__('average', 'delucks-seo').']', '<span class="ratingscore">'.$ratingScore.'</span>', $s);
			    $rating = str_replace('[average]', '<span class="ratingscore">'.$ratingScore.'</span>', $rating);
			    $rating = str_replace('[durchschnitt]', '<span class="ratingscore">'.$ratingScore.'</span>', $rating);
			    $rating = str_replace('['.esc_html__('number', 'delucks-seo').']', '<span class="ratingtotal">'.$ratingTotal.'</span>', $rating);
			    $rating = str_replace('[number]', '<span class="ratingtotal">'.$ratingTotal.'</span>', $rating);
			    $rating = str_replace('[anzahl]', '<span class="ratingtotal">'.$ratingTotal.'</span>', $rating);
			    $rating = str_replace('[maxrating]', '<span class="maxrating">'.$this->maxRating.'</span>', $rating);
			    $rating = str_replace('[bestrating]', '<span class="bestrating">'.$bestRating.'</span>', $rating);
			    $rating = str_replace('[worstrating]', '<span class="worstrating">'.$worstRating.'</span>', $rating);
			    $rating = str_replace('[commentcount]', '<span class="commentcount">'.$commentCount.'</span>', $rating);
			    $rating = str_replace('[ratinglink]', '<span class="ratinglink">'. (!isset($this->settings['link_legend']) ? '<a href="'.($altlink ? $altlink : '#respond').'">'.$ratingLinkText.'</a>' : $ratingLinkText) .'</span>', $rating);
			    $rating = str_replace('[br]', '<br/>', $rating);
			    $rating = str_replace('[stars]', '<span class="stars"><input type="hidden" readonly="readonly" class="dpc-rating-input" data-filled="dpc-rating-icon '.$this->settings["rating_symbol"].'" data-empty="dpc-rating-icon '.$this->settings["rating_symbol"].' fa-3x" value="'.$ratingValue.'"/></span>', $rating);
			    $rating = str_replace('[sterne]', '<span class="stars"><input type="hidden" readonly="readonly" class="dpc-rating-input" data-filled="dpc-rating-icon '.$this->settings["rating_symbol"].'" data-empty="dpc-rating-icon '.$this->settings["rating_symbol"].' fa-3x" value="'.$ratingValue.'"/></span>', $rating);

				$output  = '<div class="dpc-rating-wrap">';
				if(isset($this->settings['link_legend'])){ $output .= '<a href="'.($altlink ? $altlink : '#respond').'">'; }
				$output .= implode(' ', explode(',', $rating));
				if(isset($this->settings['link_legend'])){ $output .= '</a>'; }
				$output .= '</div>';
				return $output;
			}
		}
	}

	function getStoredCount(){
		if(is_user_logged_in()){
			$user_rating = get_user_meta(get_current_user_id(), 'dpc_rating', true); //NOTICE: User meta working only with posts and not pages! Using session instead
		}else{
			$user_rating = isset($_SESSION['dpc_rating']) ? $_SESSION['dpc_rating'] : '';
		}
		return is_array($user_rating) ? $user_rating : array();
	}

	function allreadyCounted($postId){
		$counts = $this->getStoredCount();
		if(is_array($counts) && isset($counts[$postId])){
			return true;
		}
		return false;
	}

	function getPostRatingData($postId){
		$postId 			= intval($postId);
		$comments 			= get_comments(array('post_id' => $postId, 'status' => 'approve', 'meta_key' => 'dpc-rating-comment'));
		$comment_count		= 0;
		$comment_rating		= 0;
		$rating_count		= 0;
		$comment_min_rating = null;
		$comment_max_rating = null;

		foreach($comments as $comment){
			$comment_count += 1;
			$comment_meta = get_comment_meta($comment->comment_ID, 'dpc-rating-comment', true);
			if($comment_meta > $this->maxRating){ $comment_meta = $this->maxRating; }
			if(strlen($comment_meta) && floatval($comment_meta) > 0){
				$rating_count += 1;
				$comment_rating += $comment_meta;

				if($comment_max_rating == null || $comment_meta > $comment_max_rating){
					$comment_max_rating = $comment_meta;
				}

				if($comment_min_rating == null || $comment_meta < $comment_min_rating){
					$comment_min_rating = $comment_meta;
				}
			}
		}

		if($comment_count > 0){
			return array('post_id' => $postId, 'comment_count' => $comment_count, 'comment_rating_score' => ($rating_count == 0 ? 0 : $comment_rating/$rating_count), 'rating_count' => $rating_count, 'comment_min_rating' => $comment_min_rating, 'comment_max_rating' => $comment_max_rating);
		}

		return array('post_id' => $postId, 'comment_count' => 0, 'comment_rating_score' => 0, 'rating_count' => $rating_count, 'comment_min_rating' => 0, 'comment_max_rating' => 0);
	}

	/**
	 * Comments Shortcode
	 *
	 */
	function dpc_show_comments_inline($atts, $content){
	    if( is_singular() && post_type_supports( get_post_type(), 'comments' ) ){
	        ob_start();
	        $lang                      = (defined('ICL_LANGUAGE_CODE') && ICL_LANGUAGE_CODE != 'all' ? ICL_LANGUAGE_CODE : $this->getLang(0));
	        $textCommentButton 	       = (isset($this->settings[$lang]['textCommentButton']) && strlen($this->settings[$lang]['textCommentButton'])) ? $this->settings[$lang]['textCommentButton'] : esc_html__( 'Commentate', 'delucks-seo' );
	        $textHideCommentFormButton = (isset($this->settings[$lang]['textHideCommentFormButton']) && strlen($this->settings[$lang]['textHideCommentFormButton'])) ? $this->settings[$lang]['textHideCommentFormButton'] : esc_html__( 'Hide comment form', 'delucks-seo' );
	        $rand                      = wp_rand(0, 99999);

			echo '<div id="rand_'. esc_attr($rand) .'" class="dpc-comments-area delucks-comments-block dpc-comments-area-inline" style="display:block;">';
				
				do_action('delucks_before_comments_template');
				
				comments_template();
				
				do_action('delucks_after_comments_template');
				
				echo '<div class="wp-block-buttons">';
					do_action('delucks_before_comments_buttons', '');

	                if ( comments_open() && post_type_supports( get_post_type(), 'comments' ) ){
	                    if(isset($this->settings['gutenberg_toggle']) && $this->settings['gutenberg_toggle'] == 'form'){
	                        echo '<div class="wp-block-button">';
	                        echo '	<a id="reply-title" href="#" class="dpc-btn btn-delucks-show-comment-form show-comment-form" data-toggle="rand_'. esc_attr($rand) .'" data-show-text="'.esc_attr($textCommentButton).'" data-hide-text="'. esc_attr($textHideCommentFormButton) .'">'.esc_html($textCommentButton).'</a>';
	                        echo '</div>';
						} elseif(isset($this->settings['gutenberg_toggle']) && $this->settings['gutenberg_toggle'] == 'anchor'){
	                        echo '<div class="wp-block-button">';
	                        echo '	<a id="reply-title" href="#respond" class="dpc-btn btn-delucks-show-comment-form goto-comment-form" data-toggle="rand_'. esc_attr($rand) .'" data-show-text="'.esc_attr($textCommentButton).'" data-hide-text="'. esc_attr($textHideCommentFormButton) .'">'.esc_html($textCommentButton).'</a>';
	                        echo '</div>';
						}
	                
}
					do_action('delucks_after_comments_buttons');
	           	echo '</div>';
				
				do_action('delucks_before_comments_form');

                if(isset($this->settings['gutenberg_toggle']) && $this->settings['gutenberg_toggle'] == 'form'){
                    comment_form();
                }
				
				do_action('delucks-after-comments-form');
			echo '</div>';

			return ob_get_clean();
	    }
	    return;
	}

	function append_body_comment_classes( $classes ) {
		global $post;

	    if( isset($post->post_content) && has_shortcode( $post->post_content, 'delucks_show_comments' ) ) {
	        $classes[] = 'has-dpc-comments-shortcode';
	    }

		return $classes;
	}

	function initXMLText(){
	    esc_html__('Extend the comment function with ratings, which then can be displayed with stars in search engine results pages and increase the click rate on your site. Use the shortcode, php code or widget to display the summary of your ratings as a preview in the above the fold area of your site, to prove keen customers or visitors.', 'delucks-seo');
	    esc_html__('You can decide if your visitors should only rate your content or if they should also be able to write a review in your comments. Maybe you also want to improve through the reviews, too. This shows Google that you count on your visitors opinion and their opinion helps the search engine to improve its results. So that’s why ratings and reviews are that important.', 'delucks-seo');
	    esc_html__('Activated', 'delucks-seo');
	    esc_html__('Deactivated', 'delucks-seo');
	    esc_html__('Shortcode', 'delucks-seo');
	    esc_html__('PHP', 'delucks-seo');
	    esc_html__('Use it as a widget', 'delucks-seo');
	    esc_html__('Rateable is', 'delucks-seo');
	    esc_html__('Let your visitors rate your blog posts quality.', 'delucks-seo');
		esc_html__('Colors', 'delucks-seo');
		esc_html__('Empty', 'delucks-seo');
		esc_html__('Rated', 'delucks-seo');
		esc_html__('Rating symbol', 'delucks-seo');
		esc_html__('Posts', 'delucks-seo');
		esc_html__('Show legend', 'delucks-seo');
		esc_html__('Rating in new row', 'delucks-seo');
		esc_html__('Link legend to comment form', 'delucks-seo');
		esc_html__('Note text if not rated', 'delucks-seo');
		esc_html__('Text', 'delucks-seo');
		esc_html__('Rate', 'delucks-seo');
		esc_html__('Review received', 'delucks-seo');
		esc_html__('Error', 'delucks-seo');
		esc_html__('Thanks for your review', 'delucks-seo');
		esc_html__('An error has occurred', 'delucks-seo');
		esc_html__('Not rated', 'delucks-seo');
		esc_html__('You did not add a rating', 'delucks-seo');
		esc_html__('Note text', 'delucks-seo');
		esc_html__('No ratings have been added yet', 'delucks-seo');
		esc_html__('Form field label', 'delucks-seo');
		esc_html__('Your rating', 'delucks-seo');
		esc_html__('Legend', 'delucks-seo');
		esc_html__('Kind of', 'delucks-seo');
		esc_html__('Please select', 'delucks-seo');
		esc_html__('CreativeWork', 'delucks-seo');
		esc_html__('LocalBusiness', 'delucks-seo');
		esc_html__('Product', 'delucks-seo');
		esc_html__('Connect rating with comments', 'delucks-seo');
		esc_html__('Force rating in comments', 'delucks-seo');
		esc_html__('Activate testimonial ratings', 'delucks-seo');
		esc_html__("Rating Stars as mandatory field", 'delucks-seo');
		esc_html__("Add company input field", 'delucks-seo');
		esc_html__("Add privacy notice checkbox", 'delucks-seo');
		esc_html__("Checkbox text", 'delucks-seo');
		esc_html__("Company input label", 'delucks-seo');
		esc_html__("Gutenberg Block Toggle", 'delucks-seo');
		esc_html__("No display", 'delucks-seo');
		esc_html__("Show comment form", 'delucks-seo');
		esc_html__("Link to comment form", 'delucks-seo');
		esc_html__("Button/Link", 'delucks-seo');
		esc_html__("Commentate button", 'delucks-seo');
		esc_html__("Hide comment form button", 'delucks-seo');
		esc_html__("Company input label", 'delucks-seo');
		esc_html__("Privacy notice text", 'delucks-seo');
		esc_html__("Hide comment form button", 'delucks-seo');
		esc_html__('Placeholder', 'delucks-seo');
		esc_html__('5 Stars', 'delucks-seo');
		esc_html__('Thumbs', 'delucks-seo');
		esc_html__('View of', 'delucks-seo');
	}
}

if(!function_exists('dpc_rating')){
	function dpc_rating(){
		if(shortcode_exists('dpc_rating')){
			return do_shortcode('[dpc_rating]');
		}
	}
}