/*!
 jQuery inlineEdit
 (c) 2015 Phuong Vu,
 updated: 2016-09-06
 license: http://www.opensource.org/licenses/mit-license.php
 http://phuong.github.io/jqueryInlineEdit/
 */
// Next version, escape white space
(function($) {

    var pluginName = "inlineEdit";

    //Default options
    var defaults = {
        //Function on update hidden field
        onUpdate: function() {
        },
        //Function on focus to ediable element
        onFocus: function() {
        },
        //Function call on key down on input field
        onKeyDown: function() {
        },
        //Css class when hover on editable item
        hoverClass: 'hover',
        //Show or hide counter in case set max length
        showCounter: false,
        
        showPixelCounter: false,
        
        hasAddition: false,
        //Input type, textarea or input type text
        inputType: 'textarea',
        //Maxth length of this input
        maxLength: false,
        minLength: false,
        //Connect with an input
        connectWith: '#createOne',
        //Default text
        defaultText: true
    };

    var properties = [
        'font-size', 'font-family',
        'font-weight', 'line-height',
        'margin', 'letter-spacing',
        'text-transform', 'display'
    ];

    function escapeSpaces(str) {
        return str.replace(/^ +/mg, function(match) {
            return match.replace(/ /g, "&nbsp;");
        });
    }

    function prepareOptions($element, options) {
        var elementOptions = {};
        
        //Todo, foreach for shorter code
        if ($element.data('connect-with')) {
            elementOptions.connectWith = $element.data('connect-with');
        }
        if ($element.attr('data-max-length')) {
            elementOptions.maxLength = $element.attr('data-max-length');
        }
        
        if ($element.attr('data-min-length')) {
            elementOptions.minLength = $element.attr('data-min-length');
        }
        
        if ($element.attr('data-extra-text')){
            elementOptions.extraText = $element.attr('data-extra-text');
        }else{
            elementOptions.extraText = '';
        }

        if ($element.attr('data-pixel-length')) {
            showPixelCounter = true;
        }
        
        if ($element.attr('data-has-addition')) {
            hasAddition = true;
        }
        
        if ($element.data('input-type')) {
            elementOptions.inputType = $element.data('input-type');
        }
        if ($element.data('show-counter')) {
            elementOptions.connectWith = $element.data('show-counter');
        }
        options = $.extend(options, elementOptions);
        return options;
    }
	
	function InlineEditDefault(element,$element, options) {
		//Hover
        $element.hover(function() {
            $element.addClass(options.hoverClass);
        }, function() {
            $element.removeClass(options.hoverClass);
        });
        $element.click(function() {
            var $this = $(this);
            var $connectWith = $(options.connectWith);
            if (!$connectWith.length) {
                $connectWith = $('<input type="hidden" />').attr('id', options.connectWith.replace(/#/g, ''));
                $("body").append($connectWith);
            }
            var defaultText = $this.data('text');

            if (options.inputType == 'textarea') {
                var $replaceWith = $('<textarea></textarea>');
            } else {
                var $replaceWith = $('<input type=text />');
            }
            
            var $parent 		= $this.parents().closest('.dpc-editable');
            var $pixelCounter 	= $('<span class="pixel-counter"></span>');
            var $counter 		= $('<span class="counter"></span>');
            var $extraText 		= $('<span class="extra-text"></span>');
            var elemWidth		= parseInt($element.innerWidth());
            var width			= 0;
            var extraWidth		= 0;
            var extraChar		= 0;
            
            if (options.showCounter == false) {
                $counter.hide();
            }
            
			//if element has additional text to count
            if($this.attr('data-has-addition') == 'true'){
            	extraWidth 	= $parent.find('.dpc-editable-addition').innerWidth();
            	width		= elemWidth + extraWidth;
            }else{
            	width		= elemWidth;
            }
                       		
            var css = {
                'width': $this.width()
            };
            //Clone all css properties
            for (var i in properties) {
                css[properties[i]] = $this.css(properties[i]);
            }
            if(options.defaultText) {
                //options.defaultText = false;
                $replaceWith.val($element.text().trim());
            }else{
                $replaceWith.val($connectWith.val().trim());
            }
            $replaceWith.css(css);
            
            if($element.attr('data-has-addition') == 'true'){
            	extraChar = $parent.find('.dpc-editable-addition').text().length;
            }
           	var currentLength 	= $replaceWith.val().length + extraChar;
                        
            //Check if autosize is loaded
            if (options.inputType == 'textarea' && $().autosize) {
                $replaceWith.autosize();
            }
			         
            $this.hide();           
            $this.after($extraText.text(options.extraText));        
            
            if($this.attr('data-pixel-length') == 'true'){
            	$this.after($pixelCounter.text(' ' + dpc_basic_metadata_translator.or + ' ' + width + "px"));
            }
            $this.after($counter.text( currentLength + " / max. " + options.maxLength + ' ' + dpc_basic_metadata_translator.chars) );
            
            $this.after($replaceWith);
            $replaceWith.focus();
            var strLength = $replaceWith.val().length * 2;
            $replaceWith[0].setSelectionRange(strLength, strLength);
            //Call on focus method
            options.onFocus.call(element, $replaceWith.val());            

            $counter.text(currentLength + " / " + options.maxLength + ' ' + dpc_basic_metadata_translator.chars);
            
            if(currentLength >= $this.attr('data-pixel-length-alert')){
            	$parent.addClass('dpc-pixel-alert');
            }else if(currentLength >= $this.attr('data-pixel-length-danger')){
            	$parent.addClass('dpc-pixel-danger');
            }
            
            
            
            $replaceWith.blur(function(e) {
                var val = $(this).val().trim();
                if (options.maxLength) {
                    //val = val.substring(0, options.maxLength);
                }
                $connectWith.val(val).change();
                if (val !== '') {
                    $this.html(escapeSpaces(val.replace(/\n/g, "<br />")));
                } else {
                    $this.html(defaultText);
                }
                $(this).remove();
                $counter.remove();
                $pixelCounter.remove(); 
                $extraText.remove();
                $this.show();
                options.onUpdate.call(element, val);
            }).keydown(function(e) {
                options.onKeyDown.call(element, e);
                
            }).keyup(function(e) {            	
            	$parent 		= $element.parents().closest('.dpc-editable');
	            elemWidth		= parseInt($element.innerWidth());
            	
            	if($element.attr('data-has-addition') == 'true'){
	            	extraChar = $parent.find('.dpc-editable-addition').text().length;
	            }

            	currentLength = $(this).val().length + extraChar;
            	
                if (options.maxLength > 0) {
                    $counter.text(currentLength + " / max. " + options.maxLength + ' ' + dpc_basic_metadata_translator.chars);                    
                }
                $element.text($(this).val());
                					
				//if element has additional text to count
	            if($element.attr('data-has-addition') == 'true'){
	            	extraWidth 	= $parent.find('.dpc-editable-addition').innerWidth();
	            	width		= elemWidth + extraWidth;
	            }else{
	            	width		= elemWidth;
	            }
	            
                if($element.attr('data-pixel-length') == 'true'){
	            	$pixelCounter.text(' ' + dpc_basic_metadata_translator.or + ' ' + width + "px");
	            }


                if(currentLength >= $this.attr('data-pixel-length-alert')){
                	$parent.addClass('dpc-pixel-alert');
                }else if(currentLength < $this.attr('data-pixel-length-alert')){
              		$parent.removeClass('dpc-pixel-alert');
                }
                
                if(currentLength >= $this.attr('data-pixel-length-danger')){
                	$parent.addClass('dpc-pixel-danger');
                }else if(currentLength < $this.attr('data-pixel-length-danger')){
              		$parent.removeClass('dpc-pixel-danger');
                }
            });
        });
	}
	function InlineEditReverse(element, $element, options) {
		//Hover
        $element.hover(function() {
            $element.addClass(options.hoverClass);
        }, function() {
            $element.removeClass(options.hoverClass);
        });
        $element.click(function() {
            var $this = $(this);
            var $connectWith = $(options.connectWith);
            if (!$connectWith.length) {
                $connectWith = $('<input type="hidden" />').attr('id', options.connectWith.replace(/#/g, ''));
                $("body").append($connectWith);
            }
            var defaultText = $this.data('text');

            if (options.inputType == 'textarea') {
                var $replaceWith = $('<textarea></textarea>');
            } else {
                var $replaceWith = $('<input type=text />');
            }
            
            var $parent 		= $this.parents().closest('.dpc-editable');
            var $pixelCounter 	= $('<span class="pixel-counter"></span>');
            var $counter 		= $('<span class="counter"></span>');
            var $extraText 		= $('<span class="extra-text"></span>');
            var elemWidth		= parseInt($element.innerWidth());
            var width			= 0;
            var extraWidth		= 0;
            var extraChar		= 0;
            
            if (options.showCounter == false) {
                $counter.hide();
            }
            
			//if element has additional text to count
            if($this.attr('data-has-addition') == 'true'){
            	extraWidth 	= $parent.find('.dpc-editable-addition').innerWidth();
            	width		= elemWidth + extraWidth;
            }else{
            	width		= elemWidth;
            }
                       		
            var css = {
                'width': $this.width()
            };
            //Clone all css properties
            for (var i in properties) {
                css[properties[i]] = $this.css(properties[i]);
            }
            if(options.defaultText) {
                //options.defaultText = false;
                $replaceWith.val($element.text().trim());
            }else{
                $replaceWith.val($connectWith.val().trim());
            }
            $replaceWith.css(css);
            
            if($element.attr('data-has-addition') == 'true'){
            	extraChar = $parent.find('.dpc-editable-addition').text().length;
            }
           	var currentLength 	= $replaceWith.val().length + extraChar;
                        
            //Check if autosize is loaded
            if (options.inputType == 'textarea' && $().autosize) {
                $replaceWith.autosize();
            }
			         
            $this.hide();           
            $this.after($extraText.text(options.extraText));        
            
            if($this.attr('data-pixel-length') == 'true'){
            	$this.after($pixelCounter.text(' ' + dpc_basic_metadata_translator.or + ' ' + width + "px"));
            }
            $this.after($counter.text( currentLength + " / " + options.maxLength + ' ' + dpc_basic_metadata_translator.chars) );
            
            $this.after($replaceWith);
            $replaceWith.focus();
            var strLength = $replaceWith.val().length * 2;
            $replaceWith[0].setSelectionRange(strLength, strLength);
            //Call on focus method
            options.onFocus.call(element, $replaceWith.val());            

            $counter.text(currentLength + " / min. " + options.minLength + ' ' + dpc_basic_metadata_translator.chars);
            
            if(currentLength <= $this.attr('data-pixel-length-alert')){
            	$parent.addClass('dpc-pixel-alert');
            }else if(currentLength <= $this.attr('data-pixel-length-danger')){
            	$parent.addClass('dpc-pixel-danger');
            }else if(currentLength >= $this.attr('data-pixel-length-okay')){
            	$parent.removeClass('dpc-pixel-alert','dpc-pixel-danger');
            }
            
            
            
            $replaceWith.blur(function(e) {
                var val = $(this).val().trim();
                if (options.minLength) {
                    //val = val.substring(0, options.minLength);
                }
                $connectWith.val(val).change();
                if (val !== '') {
                    $this.html(escapeSpaces(val.replace(/\n/g, "<br />")));
                } else {
                    $this.html(defaultText);
                }
                $(this).remove();
                $counter.remove();
                $pixelCounter.remove(); 
                $extraText.remove();
                $this.show();
                options.onUpdate.call(element, val);
            }).keydown(function(e) {
                options.onKeyDown.call(element, e);
                
            }).keyup(function(e) {            	
            	$parent 		= $element.parents().closest('.dpc-editable');
	            elemWidth		= parseInt($element.innerWidth());
            	
            	if($element.attr('data-has-addition') == 'true'){
	            	extraChar = $parent.find('.dpc-editable-addition').text().length;
	            }

            	currentLength = $(this).val().length + extraChar;
            	
                if (options.minLength > 0) {
                    $counter.text(currentLength + " / min. " + options.minLength + ' ' + dpc_basic_metadata_translator.chars);                    
                }
                $element.text($(this).val());
                					
				//if element has additional text to count
	            if($element.attr('data-has-addition') == 'true'){
	            	extraWidth 	= $parent.find('.dpc-editable-addition').innerWidth();
	            	width		= elemWidth + extraWidth;
	            }else{
	            	width		= elemWidth;
	            }
	            
                if($element.attr('data-pixel-length') == 'true'){
	            	$pixelCounter.text(' ' + dpc_basic_metadata_translator.or + ' ' + width + "px");
	            }
	            
                if(currentLength <= $this.attr('data-pixel-length-alert')){
                	$parent.addClass('dpc-pixel-alert');
                }else if(currentLength > $this.attr('data-pixel-length-alert')){
              		$parent.removeClass('dpc-pixel-alert');
                }
                
                if(currentLength <= $this.attr('data-pixel-length-danger')){
                	$parent.addClass('dpc-pixel-danger');
                }else if(currentLength > $this.attr('data-pixel-length-danger')){
              		$parent.removeClass('dpc-pixel-danger');
                }
            });
        });
	}
	
    function InlineEdit(element, opts) {
        var options = $.extend({}, defaults, opts);
        var $element = $(element);
        //Prepare in-tag options
        options = prepareOptions($element, options);
        
        if(!options.reverseCounter){
        	InlineEditDefault(element,$element,options);
        }else{
        	InlineEditReverse(element,$element,options);
        }
        
        
    }
    
    $.fn[pluginName] = function(options) {
        return this.each(function() {
            $.data(this, 'plugin_' + pluginName, new InlineEdit(this, options));
        });
    };
})(jQuery);
