<?php

class DPC_Module_Professional_Sharing_Button {

	var $dpc				= null;
	var $network			= null;
	var $count				= 0;
	var $postId				= 0;
	var $title				= '';
	var $description		= '';
 	var $url				= '';
	var $forceDefault		= false;
	var $config				= array();
	var $settingsButton		= array();
	var $settingsGeneral	= array();

	function __construct($dpc, $network, $settingsButton, $settingsGeneral, $config = array()){
		$this->dpc				= $dpc;
		$this->network			= $network;
		$this->config			= $config;
		$this->settingsButton	= $settingsButton;
		$this->settingsGeneral	= $settingsGeneral;
		$this->postId 			= (is_int(get_the_ID()) && is_object(get_post(get_the_ID())) ? get_the_ID() : 0);

		if(!$this->postId){
		    $queried_object = get_queried_object();
		    if(is_object($queried_object) && property_exists($queried_object, 'ID')){
		        $this->postId = $queried_object->ID;
		    }
		}

		$this->title			= ($this->postId > 0 ? get_post($this->postId)->post_title : get_bloginfo('name'));
		$this->url				= ($this->postId > 0 ? get_permalink($this->postId) : get_bloginfo('url'));
		$this->description		= ($this->postId > 0 ? get_post_meta($this->postId, '_dpc-meta-description', true) : get_bloginfo('description'));
		$this->count			= $this->getCount();
		$this->forceDefault		= (!count($this->settingsButton) || isset($this->settingsButton['default']) ? true : false);

		$ogTitle		= get_post_meta($this->postId, '_dpc-meta-og-title', true);
		$ogDescription	= get_post_meta($this->postId, '_dpc-meta-og-description', true);
		if(strlen($ogTitle)){		$this->title = esc_attr(trim($ogTitle)); }
		if(strlen($ogDescription)){	$this->description = esc_attr(trim($ogDescription));	}
		$this->forceDefault();
	}

	function getCount(){
		if($count = get_post_meta($this->postId, '_dpc-sharing-count-'.$this->network, true)){
			return $count;
		}
		return 0;
	}

	function show(){
		echo $this->get();
	}

	function get(){
		return	$this->wrapButton(array($this->getNetworkIcon(), $this->getNetworkLabel()));
	}

	function wrapButton($content){
		if(!is_admin() && ((!isset($this->settingsButton['mobile']) && wp_is_mobile()) || (!isset($this->settingsButton['desktop']) && !wp_is_mobile()))){
			return '';
		}

		$styleArray = array();
		#if(isset($this->settingsGeneral['button']['marginLeft'])) { $styleArray['margin-left'] = $this->settingsGeneral['button']['marginLeft']; }
		#if(isset($this->settingsGeneral['button']['marginRight'])) { $styleArray['margin-right'] = $this->settingsGeneral['button']['marginRight']; }
		#if(isset($this->settingsGeneral['button']['borderRadius'])) { $styleArray['border-radius'] = $this->settingsGeneral['button']['borderRadius']; }
		if(isset($this->settingsButton['label']['background'])) { $styleArray['background-color'] = $this->settingsButton['label']['background']; }

		if(isset($this->settingsGeneral['button']['consistentSize']) && strlen($this->settingsGeneral['button']['maxWidth'])){
			$styleArray['width'] = $this->settingsGeneral['button']['maxWidth'];
		}

		$style = $this->createStyleString($styleArray);
		return '<span href="#" class="dpc-btn dpc-btn-share dpc-'.$this->network.' '.$this->network.($this->forceDefault ? ' default' : '').'" data-lang="'.$this->dpc->oFunctions->getUserLang().'" data-config=\''.wp_json_encode($this->config).'\' data-wa_text=\''.(isset($this->config['sharing_text']) && isset($this->config['sharing_text'][$this->dpc->oFunctions->getUserLang()]) ? $this->config['sharing_text'][$this->dpc->oFunctions->getUserLang()] : '') .'\' data-network="'.$this->network.'" data-count="'.$this->count.'" data-post_id="'.$this->postId.'" data-post_url="'.urlencode($this->url).'" data-post_title="'.urlencode($this->title).'" data-post_description="'.urlencode($this->description).'"> <div class="dpc-btn-wrap"'.(strlen($style) ? ' style="'.$style.'"' : '').'>'.join($content).'</div>'.$this->getNetworkCounter().'</span>';
	}

	function getNetworkIcon(){
		if(isset($this->settingsButton['icon']) && isset($this->settingsButton['icon']['display'])){
			$styleArray = array();
			if(isset($this->settingsButton['icon']['color'])) { $styleArray['color'] = $this->settingsButton['icon']['color']; }
			$style = $this->createStyleString($styleArray);
			if(isset($this->settingsButton['icon']['background'])){
				$style['background-color'] = $this->settingsButton['icon']['background'];
			}
			return '<i class="dpc-icon dpc-icon-'.strtolower($this->network).'"'.(strlen($style) ? ' style="'.$style.'"' : '').'></i>';
		}
	}

	function getNetworkLabel(){
		if(isset($this->settingsButton['label']) && isset($this->settingsButton['label']['display'])){
			$styleArray = array();
			if(isset($this->settingsButton['label']['color'])) { $styleArray['color'] = $this->settingsButton['label']['color']; }
			$style = $this->createStyleString($styleArray);
			$label = ( isset($this->settingsButton['label']['text']) && isset($this->settingsButton['label']['text'][$this->dpc->oFunctions->getUserLang()]) && strlen($this->settingsButton['label']['text'][$this->dpc->oFunctions->getUserLang()]) ? $this->settingsButton['label']['text'][$this->dpc->oFunctions->getUserLang()] : $this->network);
			return '<span class="dpc-label"'.(strlen($style) ? ' style="'.$style.'"' : '').'>'.ucfirst($label).'</span>';
		}
	}

	function getNetworkCounter(){
		if(isset($this->settingsButton['counter']) && isset($this->settingsButton['counter']['display'])){
			if(isset($this->settingsButton['counter']['background']) && strlen($this->settingsButton['counter']['background'])){
				echo "<style>";
				echo '.dpc-btn.dpc-btn-share.dpc-'.esc_attr($this->network).'.'.esc_attr($this->network).' .dpc-btn-wrap .dpc-counter::before{ border-right-color: '.esc_attr($this->settingsButton['counter']['background']).' !important; }';
				echo "</style>";
			}

			$styleArray = array();
			if(isset($this->settingsButton['counter']['color'])) { $styleArray['color'] = $this->settingsButton['counter']['color']; }
			if(isset($this->settingsButton['counter']['background'])) { $styleArray['background-color'] = $this->settingsButton['counter']['background']; }
			$style = $this->createStyleString($styleArray);
			return '<div class="dpc-btn-wrap"'.(strlen($style) ? ' style="'.$style.'"' : '').'><span class="dpc-counter">'.$this->count.'</span></div>';
	}
	}


	function createStyleString($properties = array()){
		$append = array('border-radius' => 'px', 'margin-left' => 'px', 'margin-right' => 'px', 'max-width' => 'px', 'width' => 'px');
		$string = '';

		foreach($properties as $prop => $val){
			if(strlen($val)){
				$string .= $prop.':'.$val.(key_exists($prop, $append) ? $append[$prop] : '').';';
			}
		}

		return $string;
	}

	function forceDefault(){
		if(!$this->forceDefault){
			return false;
		}

		$this->settingsButton['icon']['display']	= true;
		$this->settingsButton['label']['display']	= true;
		$this->settingsButton['counter']['display']	= true;
		return true;
	}

}

