<?php

class DPC_Module_Professional_Sharing implements DPC_Module_Interface {

	var $dpc				= null;
	var $isSubmodule		= true;
	var $moduleDirectory 	= 'professional/';
	var $moduleTitle 		= '';
	var $optionsHook		= 'dpc-professional-sharing';
	var $settingsFile		= false;
	var $settings			= array();

	/**
	 * @param object $dpc
	 */
	function __construct(&$dpc){
		$this->dpc 			= $dpc;
		$this->moduleTitle	= esc_html__('Social sharing', 'delucks-seo');
		$this->settingsFile = dirname(__FILE__) . '/settings.xml';
		$this->settings 	= $this->dpc->getModuleSettings('professional_sharing');
		$this->setup();
		$this->dpc->registerAjaxAction('dpc_professional_sharing_count', array($this, 'count'), true);
		$this->dpc->registerAjaxAction('dpc_professional_sharing_count', array($this, 'count'));
		$this->dpc->registerAjaxAction('dpc_professional_sharing_get_print_content', array($this, 'getPrintContent'), true);
		add_action('admin_init', array($this, 'hook_new_post_columns'));
		add_shortcode('dpc_sharing', array(&$this, 'shortcodeShowButtons'));

		if(!is_admin()){
			add_filter('delucks_seo_metadata_og_title',          array($this, 'filter_setMetaOgTitle'), 20);
			add_filter('delucks_seo_metadata_og_description',    array($this, 'filter_setMetaOgDescription'), 20);
			add_filter('delucks_seo_metadata_og_image',          array($this, 'filter_setMetaOgImage'), 20);
			add_filter('delucks_seo_metadata_og_image',          array($this, 'filter_setMetaOgImageDefault'), 30);
		}
	}

	function filter_setMetaOgTitle($value){
	    $newValue = get_post_meta(get_the_ID(), '_dpc-meta-og-title', true);
	    if(strlen($newValue)){
			$value = trim($newValue);
		}
		return $value;
	}

	function filter_setMetaOgDescription($value){
	    $newValue = get_post_meta(get_the_ID(), '_dpc-meta-og-description', true);
	    if(strlen($newValue)){
			$value = trim($newValue);
		}
		return $value;
	}

	function filter_setMetaOgImage($value){
	    $newValue = get_post_meta(get_the_ID(), '_dpc-meta-og-image', true);
	    if(strlen($newValue)){
			$value = trim($newValue);
		}
		return $value;
	}

	function filter_setMetaOgImageDefault($value){
		if(!get_post_meta(get_the_ID(), '_dpc-meta-og-image', true) && isset($this->settings['defaultMetaOgImage']) && strlen($this->settings['defaultMetaOgImage']) && !wp_get_attachment_url(get_post_thumbnail_id(get_the_ID()))){
			$value = $this->settings['defaultMetaOgImage'];
		}
		return $value;
	}

	function adminHead(){  }

	function getLang($objectID = 0){
		$lang = $this->dpc->oFunctions->getPostLang($objectID, false);
		$lang = substr($lang, 0, (strpos($lang, '_') ? strpos($lang, '_') : 2));
		return $lang;
	}

	function setup(){
		global $wpdb;
		$wpdb->query($wpdb->prepare("CREATE TABLE  IF NOT EXISTS  `%s`(
			`id` bigint(20) NOT NULL AUTO_INCREMENT,
			`post_id` bigint(20),
			`network` varchar(255),
			`timestamp` bigint(20) DEFAULT NULL,
		  PRIMARY KEY (`id`),
		  KEY `ix_dpc_sharing_network` (`network`),
		  KEY `ix_dpc_sharing_post_id` (`post_id`)
		) DEFAULT CHARSET=utf8;", array(
			$wpdb->prefix . 'dpc_sharing_count'
		)));
	}

	function adminSettings(){
		$this->dpc->parseSettingsByXml($this->settingsFile, 'general');
	}

	function adminSettingsHead(){
		wp_enqueue_style('dpc-professional-sharing-settings', plugins_url( 'assets/css/settings.css', __FILE__ ), array(), DPC_VERSION);
		wp_enqueue_style('dpc-professional-sharing-icons', plugins_url( 'assets/css/loveicons.css', __FILE__ ), array(), DPC_VERSION);
		wp_enqueue_style('dpc-professional-sharing-defaults', plugins_url( 'assets/css/defaults.css', __FILE__ ), array(), DPC_VERSION);
		wp_enqueue_script('dpc-professional-sharing-settings', plugins_url( 'assets/js/settings.js', __FILE__ ), array('jquery'), DPC_VERSION);
		if(isset($this->settings['options']['button']['showCounterArrows'])){
			wp_enqueue_style('dpc-professional-sharing-arrows', plugins_url( 'assets/css/arrows.css', __FILE__ ), array(), DPC_VERSION);
		}
		add_action('admin_enqueue_scripts', array(&$this, 'hook_admin_enqueue_scripts'));
	}

	function hook_admin_enqueue_scripts(){
		echo '<script type="text/javascript">'."\n";
		echo '	var dpc_professional_sharing_settings = ' . wp_json_encode($this->dpc->getModuleSettings('professional_sharing')) . ";\n";
		echo '</script>'."\n";
	}

	function getNetworkList($list){
		$unsortedNetworks	= array();
		$data				= array();

		if(isset($this->settings['sort_order'])){
			foreach(explode(',',$list) as $v){
				if(!in_array($v, $this->settings['sort_order'])){
					$unsortedNetworks[] = trim($v);
				}
			}
			$list = join(',', array_values(array_merge($this->settings['sort_order'], $unsortedNetworks)));
		}

		foreach(explode(',',$list) as $v){
			$data[] = array('label' => trim($v), 'name' => trim(strtolower($v)), 'new' => (in_array(trim($v), $unsortedNetworks) ? true : false));
		}

		return $data;
	}

	function shortcodeShowButtons($atts){
		return $this->showButtons(true);
	}

	function networkColor($network){
		switch($network){
			case 'twitter':		$color = '#55acee';	break;
			case 'facebook':	$color = '#3b5998';	break;
			case 'google':		$color = '#e74b3b';	break;
			case 'pinterest':	$color = '#cf3136';	break;
			case 'xing':		$color = '#026466';	break;
			case 'whatsapp':	$color = '#268132';	break;
			case 'tumblr':		$color = '#34506b';	break;
			case 'linkedin':	$color = '#3399cc';	break;
			case 'flattr':		$color = '#f8b234';	break;
			case 'buffer':		$color = '#51962e';	break;
			case 'stumbleupon':	$color = '#ea4a24';	break;
			case 'print':		$color = '#333333';	break;
			case 'mail':		$color = '#1375c4';
		}
		return $color;
	}

	function getNetworkCountByPost($postId, $networkName){
	    $count = get_post_meta($postId, '_dpc-sharing-count-_'.strtolower($networkName), true);
	    if(strlen($count)){
			return $count;
		}
		return 0;
	}

	function getActiveNetworkCountByPost($postId, $skip = ''){
		$data = array();
		if(count($this->settings['activeNetworks'])){
			foreach($this->settings['activeNetworks'] as $networkName){
				$networkName = strtolower($networkName);
				if(in_array($networkName, explode(',', $skip))){
					continue;
				}
				$count = $this->getNetworkCountByPost($postId, $networkName);
				$color = $this->networkColor($networkName);
				$data[] = array('label' => $networkName, 'name' => $networkName, 'count' => $count, 'color' => $color);
			}
		}
		return $data;
	}

	function getActiveNetworkTotalByPost($postId, $skip = ''){
		$total = 0;
		$networkCounters = $this->getActiveNetworkCountByPost($postId);
		if(count($networkCounters)){
			foreach($networkCounters as $network){
				if(in_array($network['name'], explode(',', $skip))){
					continue;
				}
				$total += $network['count'];
			}
		}
		return $total;
	}

	function showButton($networkName, $settingsButton = array(), $settingsGeneral = array(), $config = array(), $return = false){
		$networkName = strtolower($networkName);
		require_once('button.php');
		if(isset($this->settings['networks'][$networkName]) || is_admin()){
			$config				= isset($this->settings['networks'][$networkName]['settings']) ? $this->settings['networks'][$networkName]['settings'] : array();
			$settingsButton 	= (count($settingsButton) ? $settingsButton : (isset($this->settings['networks']) && isset($this->settings['networks'][$networkName]) ? $this->settings['networks'][$networkName] : array()));
			$settingsGeneral	= (count($settingsGeneral) ? $settingsGeneral : (isset($this->settings['options']) ? $this->settings['options'] : array()));
			if(is_admin()){
				$settingsButton = array_merge($settingsButton, array('default' => true));
				$network = new DPC_Module_Professional_Sharing_Button($this->dpc, $networkName, $settingsButton, $settingsGeneral, $config);
			} else {
				$network = new DPC_Module_Professional_Sharing_Button($this->dpc, $networkName, $settingsButton, $settingsGeneral, $config);
			}
			if($return == true){
				return $network->get();
			} else {
				$network->show();
			}
		}
	}

	function showButtons($return = false){
	    $queried_object = get_queried_object();
	    if(is_object($queried_object)){
	        $post_id       = $queried_object->ID;
	        $hideOnPost    = get_post_meta($post_id, '_dpc-hide-sharing-buttons', true) ? true : false;

	        if(is_array($this->settings) && isset($this->settings['showButtonsOn']) && in_array($queried_object->post_type, $this->settings['showButtonsOn']) && !$hideOnPost){
	            /* all checks done: show buttons */
	            add_action('wp_footer', array(&$this, 'hook_wp_enqueue_scripts'));
	            if(isset($this->settings['networks']) && count($this->settings['networks'])){
	                add_action('wp_footer', array($this, 'printScripts'));
	            }

	            $output = '';
	            if(count($this->settings['activeNetworks'])){
	                $i = 0;

	                $countPerRow = 999;
	                if(isset($this->settings['options']) && isset($this->settings['options']['button']) && isset($this->settings['options']['button']['countPerRow']) && $this->settings['options']['button']['countPerRow'] > 0){
	                    $countPerRow = $this->settings['options']['button']['countPerRow'];
	                }

	                if(isset($this->settings['sort_order'])){
	                    $output .= '<div class="dpc-sharing">';
	                    foreach($this->settings['sort_order'] as $networkName){
	                        if(in_array($networkName, $this->settings['activeNetworks'])){
	                            $networkName = strtolower($networkName);
	                            if(isset($this->settings['networks'][$networkName])){
	                                $output .= $this->showButton($networkName, $this->settings['networks'][$networkName], $this->settings['options'], array(), true);
	                                if(++$i % $countPerRow == 0){
	                                    $output .= "<br/>";
	                                }
	                            }
	                        }
	                    }
	                    $output .= '</div>';
	                }
	            }

	            if($return == true){ return $output; }
	            echo $output;
	        }
	    }

	    if($return == true){ return ''; }
	    echo '';
	}

	function count(){
		if(isset($_REQUEST['network'])){
			global $wpdb;
			$postId = (isset($_REQUEST['post_id']) && is_object(get_post(intval($_REQUEST['post_id']))) ? $_REQUEST['post_id'] : 0);
			$wpdb->query($wpdb->prepare("INSERT INTO `%s` (`post_id`, `network`, `timestamp`) VALUES (%s, %s, %d)", array(
				$wpdb->prefix . 'dpc_sharing_count',
				$postId,
				sanitize_text_field($_REQUEST['network']),
				time()
			)));
			if($postId > 0){
			    update_post_meta($postId, '_dpc-sharing-count-'.sanitize_text_field($_REQUEST['network']), get_post_meta($postId, '_dpc-sharing-count-'.sanitize_text_field($_REQUEST['network']), true)+1);
				update_post_meta($postId, '_dpc-sharing-count-total', get_post_meta($postId, '_dpc-sharing-count-total', true)+1);
			}
		}
		die();
	}

	function executeBackendActions(){ }
	function registerWidgets(){
		add_action( 'widgets_init', array(&$this, 'hook_widgets_init'));
	}

	function hook_widgets_init(){
		require_once( 'widgets/sharing.php' );
		register_widget('DPC_Widget_Sharing_Sharing');
	}

	function executeFrontendActions(){
		add_action('wp_head', array($this, 'action_printOpenGraphMeta'), 10);
		add_action('wp_footer', array($this, 'action_printSocialMetaMicrodata'), 10);

		/*
		if(isset($this->settings['openGraphTags'])){
		    add_action('delucks_seo_metadata_add_meta', array($this, 'action_printOpenGraphTags'));
		}
        */
	}

	function printScripts(){
		$loadIconFontCss	= false;
		$loadDefaultCss		= false;

		wp_enqueue_script('dpc-professional-sharing-frontend', plugins_url( 'assets/js/frontend.js', __FILE__ ), array('jquery'), DPC_VERSION, $this->dpc->load_scripts_in_footer);
		foreach($this->settings['sort_order'] as $networkName){
			if(in_array($networkName, $this->settings['activeNetworks'])){
				$networkName = strtolower($networkName);
				if(isset($this->settings['networks'][$networkName]['default']) && isset($this->settings['networks'][$networkName]['icon']['display'])){
					$loadIconFontCss = true;
				}
				if(isset($this->settings['networks'][$networkName]['default'])){
					$loadDefaultCss = true;
				}
				if(isset($this->settings['networks'][$networkName]) && is_readable(dirname(__FILE__) . '/assets/js/'.$networkName.'.js')){
					wp_enqueue_script('dpc-professional-sharing-'.$networkName, plugins_url( 'assets/js/'.$networkName.'.js', __FILE__ ), array('jquery'), DPC_VERSION, $this->dpc->load_scripts_in_footer);
				}
			}
		}

		if($loadIconFontCss){
			wp_enqueue_style('dpc-professional-sharing-icons', plugins_url( 'assets/css/loveicons.css', __FILE__ ), array(), DPC_VERSION);
		}
		if($loadDefaultCss){
			wp_enqueue_style('dpc-professional-sharing-defaults', plugins_url( 'assets/css/defaults.css', __FILE__ ), array(), DPC_VERSION);
		}
		wp_enqueue_style('dpc-professional-sharing-frontend', plugins_url( 'assets/css/frontend.css', __FILE__ ), array(), DPC_VERSION);
		if(isset($this->settings['options']['button']['showCounterArrows'])){
			wp_enqueue_style('dpc-professional-sharing-arrows', plugins_url( 'assets/css/arrows.css', __FILE__ ), array(), DPC_VERSION);
		}
	}

	function hook_wp_enqueue_scripts(){
		echo '<script type="text/javascript">'."\n";
		echo '	var dpc_professional_sharing_settings = ' . wp_json_encode($this->dpc->getModuleSettings('professional_sharing')) . ";\n";
		
		/* proceed utm tracking */
		if(isset($this->settings['utm_tracking_status']) && $this->settings['utm_tracking_status'] == 'on'){
		    $utmTrackingData = array();
		    $utmData = $this->settings['utm_campaign'];
		    
		    $utmId = get_post_meta(get_the_ID(), '_dpc-utm-tracking-id', true);
		    if($utmId){ $utmData['id'] = $utmId; }
		    $utmTrackingData[] = array('empty' => true, 'key' => 'utm_id', 'value' => urlencode($utmData['id']));
		    
		    $utmMedium = get_post_meta(get_the_ID(), '_dpc-utm-tracking-medium', true);
		    if($utmMedium){ $utmData['medium'] = $utmMedium; }
		    $utmTrackingData[] = array('empty' => true, 'key' => 'utm_medium', 'value' => urlencode($utmData['medium']));
		    
		    $utmTerm = get_post_meta(get_the_ID(), '_dpc-utm-tracking-term', true);
		    if($utmTerm){ $utmData['term'] = $utmTerm; }
		    $utmTrackingData[] = array('empty' => true, 'key' => 'utm_term', 'value' => urlencode($utmData['term']));
		    
		    $utmSource = get_post_meta(get_the_ID(), '_dpc-utm-tracking-source', true);
		    if($utmSource){ $utmData['source'] = $utmSource; }
		    $utmTrackingData[] = array('empty' => true, 'key' => 'utm_source', 'value' => urlencode($utmData['source']));
		    
		    $utmName = get_post_meta(get_the_ID(), '_dpc-utm-tracking-name', true);
		    if($utmName){ $utmData['name'] = $utmName; }
		    $utmTrackingData[] = array('empty' => true, 'key' => 'utm_campaign', 'value' => urlencode($utmData['name']));
		    
		    $utmContent = get_post_meta(get_the_ID(), '_dpc-utm-tracking-content', true);
		    if($utmContent){ $utmData['content'] = $utmContent; }
		    $utmTrackingData[] = array('empty' => true, 'key' => 'utm_content', 'value' => urlencode($utmData['content']));
		    echo '	var dpc_professional_sharing_utm_params = ' . wp_json_encode($utmTrackingData) . ";\n";
		}
		echo '</script>'."\n";
	}

	function getPrintContent(){
	    $postId = (isset($_REQUEST['post_id']) && is_object(get_post(intval($_REQUEST['post_id']))) ? $_REQUEST['post_id'] : 0);
		if($postId > 0){
			$description	= trim(get_post_meta($postId, '_dpc-meta-description', true));
			$title			= trim(get_post($postId)->post_title);
			$content		= trim(get_post($postId)->post_content);
			echo '<div class="dpc-print-wrapper">';
			echo '	<h1 class="dpc-print-title">'.esc_html($title).'</h1>';
			if(strlen($description)){
				echo '	<p class="dpc-print-description">'.esc_html($description).'</p>';
			}
			echo '	<p class="dpc-print-content">'.esc_html($content).'</p>';
			echo '</div>';
		}
		die();
	}

	function action_printOpenGraphMeta(){
		global $post;
		if(!is_object($post)){ return; }

		$post = get_post($post->ID);

		if(is_singular()){
			if(isset($this->settings['facebookUrl']) && strlen($this->settings['facebookUrl'])){
				$facebook = '';
				if(strlen(trim($post->post_excerpt))){
					$facebook .= '<meta property="og:description" content="'.esc_attr(substr(trim(wp_strip_all_tags($post->post_excerpt)),0,110).'...').' "/>'."\n";
				}
				$facebook .= '<meta property="article:author" content="'.(esc_attr($this->settings['facebookUrl'])).' "/>'."\n";
				$facebook .= '<meta property="article:publisher" content="'.(esc_attr($this->settings['facebookUrl'])).' "/>'."\n";
				echo $facebook;
			}

			if(isset($this->settings['twittername']) && strlen($this->settings['twittername'])){
				//use description, title, image from opengraph-tags
				$twitter  = '<meta property="twitter:card" content="summary_large_image" />'."\n";
				$twitter .= '<meta property="twitter:site" content="'.esc_attr('@'.str_replace('@', '', ($this->settings['twittername'] ?? ''))).'" />'."\n";
				$twitter .= '<meta property="twitter:creator" content="'.esc_attr('@'.str_replace('@', '', ($this->settings['twittername'] ?? ''))).'" />'."\n";

				$metadataSettings = $this->dpc->getModuleSettings('basic_metadata');
				$lang = $this->dpc->oFunctions->getPostLang($post->ID, false);
				$lang = substr($lang, 0, strpos($lang, '_'));
				if(!isset($metadataSettings[$lang])){
					$lang = $this->dpc->oFunctions->getDefaultLanguage('slug');
				}

				if(isset($metadataSettings[$lang]) && isset($metadataSettings[$lang]['title']) && isset($metadataSettings[$lang]['title']['website'])){
					$domain = $metadataSettings[$lang]['title']['website'];
				} else {
					$domain = get_bloginfo('name');
				}

				$twitter .= '<meta property="twitter:domain" content="'.esc_url($domain).'" />'."\n";
				echo $twitter;
			}
		}
	}

	function action_printSocialMetaMicrodata(){
		global $post;
		if(!is_object($post)){ return; }

		$post 		= get_post($post->ID);
		$socialUrls = array();

		$socialUrls['Twitter'] = ((isset($this->settings['twittername']) && strlen($this->settings['twittername'])) ? 'https://twitter.com/'.str_replace('@', '', $this->settings['twittername']) : '');
		$socialUrls['Facebook'] = $this->settings['facebookUrl'] ?? '';
		echo '<span itemscope itemtype="http://schema.org/Organization" style="display: none !important;">';
		echo '	<link itemprop="url" href="/">';
		foreach($socialUrls as $label => $url){
			if(strlen($url)){
				echo '	<a itemprop="sameAs" href="'.esc_url($url).'">'.esc_html($label).'</a>';
			}
		}
		echo '</span>';
	}

	function dpc_post_column( $cols ) {
	    $cols["dpc-shares"] = esc_html__('Shares');
		return $cols;
	}

	function dpc_post_value( $column_name, $id ) {
		switch($column_name){
			case 'dpc-shares':
				$val = get_post_meta($id, '_dpc-sharing-count-total', true);
				echo esc_html($val);
				break;
		}
	}

	function dpc_post_column_sortable( $cols ) {
		$cols["dpc-shares"] = "dpc_shares";
		return $cols;
	}

	function hook_new_post_columns() {
		#add_filter( 'manage_posts_columns', array($this, 'dpc_post_column') );
		#add_action( 'manage_posts_custom_column', array($this, 'dpc_post_value'), 10, 2 );
		#add_filter( 'manage_edit-post_sortable_columns', array($this, 'dpc_post_column_sortable') );
		#add_filter( 'manage_pages_columns', array($this, 'dpc_post_column') );
		#add_action( 'manage_pages_custom_column', array($this, 'dpc_post_value'), 10, 2 );
		#add_filter( 'manage_edit-page_sortable_columns', array($this, 'dpc_post_column_sortable') );
		add_filter( 'request', array($this, 'dpc_shares_column_orderby') );
	}

	function dpc_shares_column_orderby( $vars ) {
		if ( isset( $vars['orderby'] ) && 'dpc_shares' == $vars['orderby'] ) {
			$vars = array_merge( $vars, array(
					'meta_key' => '_dpc-sharing-count-total',
					'orderby' => 'meta_value'
			) );
		}
		return $vars;
	}

	function modalEnqueueStyles(){
	    wp_enqueue_style('dpc-professional-sharing-modalstyles', plugins_url( 'assets/css/modal.css', __FILE__ ));
	}

	function modalEnqueueScripts(){
	    wp_enqueue_script('dpc-inlineedit', plugins_url( 'assets/js/jquery.inline-edit.js', __FILE__ ), array('jquery'), DPC_VERSION, true);
	    wp_enqueue_script('dpc-professional-sharing-modaljs', plugins_url( 'assets/js/modal.js', __FILE__ ), array('jquery'), DPC_VERSION, true);

	    $args = array('post' => $this->dpc->modal->currentId);
	    wp_enqueue_media($args);
	}

	function modalTab(){
		echo '<span class="dpc-modal-tab dpc-modal-tab-trigger '.esc_attr(isset($_GET['tab']) && $_GET['tab'] == 'social' ? 'active' : '').'" id="tab-'. esc_attr($this->optionsHook) .'">'. esc_html($this->moduleTitle) .'</span>';
	}

	function modalBody(){
		echo '<div class="dpc-modal-content '.esc_attr(isset($_GET['tab']) && $_GET['tab'] == 'social' ? 'active' : '').'" id="content-'. esc_attr($this->optionsHook) .'">';
		$this->dpc->new_parseSettingsByXml($this->settingsFile, 'modal', false, true, false, $this->dpc->modal->meta);
		echo '</div>';
	}

	function modalInputs(){
		$post 				= get_post($this->dpc->modal->currentId);
		$title				= get_post_meta($post->ID, '_dpc-meta-og-title', true);
		$description		= get_post_meta($post->ID, '_dpc-meta-og-description', true);

		?>
		<div class="haswrap dpc-col dpc-col-9-12 no-margin-top">
			<div class="dpc-googlesearch-preview-column">
				<div id="dpc_googlesearch_preview" class="">

                    <div class="serp-item">
                        <div class="headline dpc-editable" id="headline_serp">
                        	<span id="dpc_meta_og_title" style="display:none;" data-applicable="false"></span>
                        	<span id="og_title_serp" class="dpc-edit" data-default="" data-input-type="input" data-has-addition="true" data-save="dpc-meta-og-title" data-max-length="70" data-pixel-length="true" data-pixel-length-alert="64" data-pixel-length-danger="70" data-extra-text="<?php echo esc_html__('. Integrate focus keyword!', 'delucks-seo');?>"  data-prevtext="<?php echo esc_attr($title); ?>">
	                            <?php echo esc_html(strlen($title) ? $title : esc_html__('Automatically', 'delucks-seo')); ?>
                        	</span>
                        </div>
                        <div style="clear:both;">
	                        <span class="description dpc-editable">
	                        	<span id="dpc_meta_og_description" style="display:none;" data-applicable="false"></span>
	                            <span id="og_description_serp" class="dpc-edit" data-default="" data-input-type="textarea" data-save="dpc-meta-og-description"  data-min-length="110"  data-pixel-length-okay="141" data-pixel-length-alert="140" data-pixel-length-danger="110" data-extra-text="<?php echo esc_html__('. If possible, include keywords, if necessary 1 special character.');?>">
	                            	<?php echo esc_html(strlen($description) ? $description : esc_html__('Automatically', 'delucks-seo')); ?>
	                            </span>
	                        </span>
                        </div>
                    </div>
                </div>
        	</div>
	 	</div>
	 	<?php
	}

	function modalSave(){
	    update_post_meta(intval($_POST['dpc_current_id']), '_dpc-meta-og-description', (isset($_POST['dpc-meta-og-description']) === true ? sanitize_textarea_field($_POST['dpc-meta-og-description']) : ''));
	    update_post_meta(intval($_POST['dpc_current_id']), '_dpc-meta-og-title', (isset($_POST['dpc-meta-og-title']) === true ? sanitize_text_field($_POST['dpc-meta-og-title']) : ''));
	    update_post_meta(intval($_POST['dpc_current_id']), '_dpc-meta-og-image', (isset($_POST['dpc-meta-og-image']) === true ? sanitize_text_field($_POST['dpc-meta-og-image']) : ''));
	    update_post_meta(intval($_POST['dpc_current_id']), '_dpc-hide-sharing-buttons', (isset($_POST['dpc-hide-sharing-buttons']) === true ? sanitize_text_field($_POST['dpc-hide-sharing-buttons']) : ''));
	    update_post_meta(intval($_POST['dpc_current_id']), '_dpc-utm-tracking-id', (isset($_POST['dpc-utm-tracking-id']) === true ? sanitize_text_field($_POST['dpc-utm-tracking-id']) : ''));
	    update_post_meta(intval($_POST['dpc_current_id']), '_dpc-utm-tracking-medium', (isset($_POST['dpc-utm-tracking-medium']) === true ? sanitize_text_field($_POST['dpc-utm-tracking-medium']) : ''));
	    update_post_meta(intval($_POST['dpc_current_id']), '_dpc-utm-tracking-source', (isset($_POST['dpc-utm-tracking-source']) === true ? sanitize_text_field($_POST['dpc-utm-tracking-source']) : ''));
	    update_post_meta(intval($_POST['dpc_current_id']), '_dpc-utm-tracking-name', (isset($_POST['dpc-utm-tracking-name']) === true ? sanitize_text_field($_POST['dpc-utm-tracking-name']) : ''));
	    update_post_meta(intval($_POST['dpc_current_id']), '_dpc-utm-tracking-term', (isset($_POST['dpc-utm-tracking-term']) === true ? sanitize_text_field($_POST['dpc-utm-tracking-term']) : ''));
	    update_post_meta(intval($_POST['dpc_current_id']), '_dpc-utm-tracking-content', (isset($_POST['dpc-utm-tracking-content']) === true ? sanitize_text_field($_POST['dpc-utm-tracking-content']) : ''));

		//update status
	    $status = get_post_meta(intval($_POST['dpc_current_id']), '_delucks-seo-status', true);
		$status = is_array($status) ? $status : array();
		if((isset($_POST['dpc-meta-og-description']) && strlen($_POST['dpc-meta-og-description'])) ||
		    (isset($_POST['dpc-meta-og-title']) && strlen($_POST['dpc-meta-og-title'])) ||
		    (isset($_POST['dpc-meta-og-image']) && strlen($_POST['dpc-meta-og-image'])) ||
		    (isset($_POST['dpc-hide-sharing-buttons']) && strlen($_POST['dpc-hide-sharing-buttons'])) ||
		    (isset($_POST['dpc-utm-tracking-id']) && strlen($_POST['dpc-utm-tracking-id'])) ||
		    (isset($_POST['dpc-utm-tracking-medium']) && strlen($_POST['dpc-utm-tracking-medium'])) ||
		    (isset($_POST['dpc-utm-tracking-source']) && strlen($_POST['dpc-utm-tracking-source'])) ||
		    (isset($_POST['dpc-utm-tracking-name']) && strlen($_POST['dpc-utm-tracking-name'])) ||
		    (isset($_POST['dpc-utm-tracking-term']) && strlen($_POST['dpc-utm-tracking-term'])) ||
		    (isset($_POST['dpc-utm-tracking-content']) && strlen($_POST['dpc-utm-tracking-content']))
		    ) { $status['social'] = 2; } else { $status['social'] = 1; }
		    update_post_meta(intval($_POST['dpc_current_id']), '_delucks-seo-status', $status);
	}

	function initXMLText(){
	    esc_html__('Facebook etc. are important sources for qualitative traffic, if your contents are worth sharing. Therefor offer your visitors sharing buttons.', 'delucks-seo');
	    esc_html__('Use the social sharing buttons to offer your visitors an easy way to share your contents. You can design the buttons the way you want and furthermore: you get sharing statistics and this solution honors data privacy of your visitors. That data will only be submitted to social media, if users click on the designated buttons (with the original plugins social networks also collect data without being clicked on).', 'delucks-seo');
		esc_html__('Make social sharing easier for your visitors. First you feed search engines and second you see how relevant your contents are.', 'delucks-seo');
		esc_html__('Activated', 'delucks-seo');
		esc_html__('Deactivated', 'delucks-seo');
		esc_html__('Shortcode', 'delucks-seo');
		esc_html__('PHP', 'delucks-seo');
		esc_html__('Use it as a widget', 'delucks-seo');
		esc_html__('Select an image', 'delucks-seo');
		esc_html__('Enable Social metatags', 'delucks-seo');
		esc_html__('Facebook site url', 'delucks-seo');
		esc_html__('Visit site', 'delucks-seo');
		esc_html__('Facebook App ID', 'delucks-seo');
		esc_html__('Google+ author url', 'delucks-seo');
		esc_html__('X name', 'delucks-seo');
		esc_html__('If your domain isnt whitelisted for Twitter Cards yet, you can request it via the validator', 'delucks-seo');
		esc_html__('Enable social microdata (knowledge graph)', 'delucks-seo');
		esc_html__('Enable Open Graph metatags', 'delucks-seo');
		esc_html__('Active networks', 'delucks-seo');
		esc_html__('Use defaults', 'delucks-seo');
		esc_html__('Mobile', 'delucks-seo');
		esc_html__('Desktop', 'delucks-seo');
		esc_html__('Label', 'delucks-seo');
		esc_html__('Font color', 'delucks-seo');
		esc_html__('Background color', 'delucks-seo');
		esc_html__('Icon', 'delucks-seo');
		esc_html__('Icon color', 'delucks-seo');
		esc_html__('Counter', 'delucks-seo');
		esc_html__('Counter color', 'delucks-seo');
		esc_html__('Counter background', 'delucks-seo');
		esc_html__('Image minimum dimensions', 'delucks-seo');
		esc_html__('Height', 'delucks-seo');
		esc_html__('Width', 'delucks-seo');
		esc_html__('Default preview image', 'delucks-seo');
		esc_html__('Sharing message', 'delucks-seo');
		esc_html__('This text will be shown to the reciever of the whatsapp message', 'delucks-seo');
		esc_html__('User-ID', 'delucks-seo');
		esc_html__('Label when unfolded', 'delucks-seo');
		esc_html__('Number of buttons per row', 'delucks-seo');
		esc_html__('Button margin', 'delucks-seo');
		esc_html__('Left (px)', 'delucks-seo');
		esc_html__('Right (px)', 'delucks-seo');
		esc_html__('Font size (px)', 'delucks-seo');
		esc_html__('Icon size (px)', 'delucks-seo');
		esc_html__('Representation', 'delucks-seo');
		esc_html__('Border radius (px)', 'delucks-seo');
		esc_html__('Uniform buttons', 'delucks-seo');
		esc_html__('Show Counter Arrows', 'delucks-seo');
		esc_html__('Maximum width', 'delucks-seo');
		esc_html__('Counter margin', 'delucks-seo');
		esc_html__('Counter font size (px)', 'delucks-seo');
		esc_html__('Sharing Widget', 'delucks-seo');
		esc_html__('Show your social sharing buttons', 'delucks-seo');
		esc_html__('Recommended to link your social media channels when contents are shared.', 'delucks-seo');
		esc_html__('Social Media Title', 'delucks-seo');
		esc_html__('Social Media Description', 'delucks-seo');
		esc_html__('Preview Image', 'delucks-seo');
		esc_html__('Hide sharing buttons', 'delucks-seo');
		esc_html__('Show buttons on', 'delucks-seo');
		esc_html__('Enable UTM Tracking', 'delucks-seo');
		esc_html__('Campaign ID', 'delucks-seo');
		esc_html__('Campaign Source', 'delucks-seo');
		esc_html__('Campaign Medium', 'delucks-seo');
		esc_html__('Campaign Name', 'delucks-seo');
		esc_html__('Campaign Term', 'delucks-seo');
		esc_html__('Campaign Content', 'delucks-seo');
		esc_html__('The ads campaign id', 'delucks-seo');
		esc_html__('The referrer (e.g. google, newsletter)', 'delucks-seo');
		esc_html__('Marketing medium (e.g. cpc, banner, email)', 'delucks-seo');
		esc_html__('One of campaign name or campaign id are required', 'delucks-seo');
		esc_html__('Identify the paid keywords', 'delucks-seo');
		esc_html__('Use to differentiate ads', 'delucks-seo');
	}
}

if(!function_exists('dpc_sharing')){
	function dpc_sharing(){
		if(shortcode_exists('dpc_sharing')){
			return do_shortcode('[dpc_sharing]');
		}
	}
}