<?php

class DPC_Module_Professional_Textopt implements DPC_Module_Interface {

	var $dpc				= null;
	var $isSubmodule		= true;
	var $moduleDirectory 	= 'professional/';
	var $moduleTitle 		= '';
	var $optionsHook		= 'dpc-professional-textopt';
	var $settingsFile		= false;
	var $settings			= array();
	var $wdfIdf				= null;

	function __construct(&$dpc){

		$this->dpc 				= $dpc;
		$this->moduleTitle		= esc_html__('Text optimization', 'delucks-seo');
		$this->settingsFile 	= dirname(__FILE__) . '/settings.xml';
		$this->settings 		= $this->dpc->getModuleSettings('professional_textopt');
	}

	function adminHead(){
		wp_enqueue_script('dpc-professional-textopt-settings', plugins_url( 'assets/js/settings.js', __FILE__ ));

		$this->wdf_idf_init();
		$this->dpc->registerAjaxAction('dpc_professional_textopt_wdf_idf_analysis', array($this, 'wdf_idf_analysis_ajax'));
		$this->dpc->registerAjaxAction('dpc_professional_textopt_wdf_idf_add_stopword', array($this, 'wdf_idf_add_stopword_ajax'));
		$this->dpc->registerAjaxAction('dpc_professional_textopt_wdf_idf_clear_cache', array($this, 'wdf_idf_clear_cache_ajax'));
	}

	function hook_admin_enqueue_scripts(){
		echo '<script type="text/javascript">'."\n";
		echo '	var dpc_professional_textopt_translator = ' . wp_json_encode($this->getTranslationJs()) . ";\n";
		echo '	var dpc_professional_textopt_settings 	= ' . wp_json_encode($this->settings) . ";\n";
		echo '	var dpc_textopt_av_countries 			= ' . wp_json_encode($this->dpc->oFunctions->getLanguages()) . ";\n";
		echo '</script>'."\n";
	}

	function getTranslationJs(){
		return array(
		    'title' 				=> esc_html__('Title', 'delucks-seo'),
		    'keyword'				=> esc_html__('Focus-Keyword', 'delucks-seo'),
		    'pleaseEnterKeyword'	=> esc_html__('Please enter a focus keyword first', 'delucks-seo'),
		);
	}

	/*
	 * WDF*IDF ajax methods */
	function wdf_idf_add_stopword_ajax(){		$this->wfdIdf->wdf_idf_add_stopword();			}
	function wdf_idf_analysis_ajax(){			$this->wfdIdf->wdf_idf_analysis();				}
	function wdf_idf_clear_cache_ajax(){		$this->wfdIdf->clearCache();					}
	function wdf_idf_init(){
		if($this->wdfIdf == null){
			include_once(dirname(__FILE__) . '/include/wdf_idf.php');
			$this->wfdIdf = new DPC_Module_Professional_Textopt_Wdf_Idf($this->dpc);
		}
	}

	/*
	 * Triggered from index file */
	function executeFrontendActions(){
		if(isset($this->settings['feedUseDescriptionAsContent']) && $this->settings['feedUseDescriptionAsContent'] == 'on'){
			add_action('the_content', array($this, 'feedSetDescritpionAsContent'));
		}

		/* Pages in RSS Feed */
		if(isset($this->settings['feedIncludePages']) && $this->settings['feedIncludePages'] == 'on'){
			add_filter('posts_where', array($this, 'rss_page_posts_where'));
			add_filter('get_lastpostmodified', array($this, 'rss_page_get_lastpostmodified'),10,2);
			add_action('rss2_ns', array($this, 'rss_page_feed_true'));
			add_action('atom_ns', array($this, 'rss_page_feed_true'));
			add_action('rdf_ns', array($this, 'rss_page_feed_true'));
			add_action ('rss2_comments_ns', array($this, 'rss_page_feed_false'));
			add_action ('atom_comments_ns', array($this, 'rss_page_feed_false'));
		}

		add_filter('the_excerpt_rss', array($this, 'featuredtoRSS'));
		add_filter('the_content_feed', array($this, 'featuredtoRSS'));
	}

	/*
	 * Triggered from index file */
	function registerWidgets(){}

	/*
	 * Add featured image to rss entries */
	function featuredtoRSS($content) {
		if(has_post_thumbnail(get_the_ID())){
			$image = wp_get_attachment_image_src(get_post_thumbnail_id(get_the_ID()), 'medium');
			$image_url = $image[0];
			$content = '<div><img style="width: 100px" src="'.$image_url.'" alt="" /></div>' . $content;
		}
		return $content;
	}

	/*
	 * Replace post content with description text within rss feeds */
	function feedSetDescritpionAsContent($content){
		if(is_feed()){
			$desc = get_post_meta(get_the_ID(), '_dpc-meta-description', true);
			if(strlen($desc)){
				return strip_shortcodes($desc);
			}
			return strip_shortcodes(wp_trim_words( $content, $num_words = 55, $more = null ));
		}
		return $content;
	}

	/*
	 * Triggered from index file */
	function initModule(){
		add_action('init', array($this, 'adminHead'));
	}

	/*
	 * Triggered from index file */
	function adminSettings(){
		$this->dpc->parseSettingsByXml($this->settingsFile, 'general');
	}

	/*
	 * Triggered from index file */
	function adminSettingsHead(){
		wp_enqueue_style('dpc-professional-textopt-settings', plugins_url( 'assets/css/settings.css', __FILE__ ));
	}

	/*
	 * Triggered from index file */
	function executeBackendActions(){
		add_action('admin_enqueue_scripts', array(&$this, 'hook_admin_enqueue_scripts'));
	}

	function printScripts(){
	    wp_enqueue_style('dpc-textopt-frontend', plugins_url( 'assets/css/frontend.css', __FILE__ ), array(), DPC_VERSION);
	}

	/*
	 * Add pages to RSS Feeds */
	function rss_page_posts_where($var){
		if (!is_feed()){
			return $var;
		}
		global $table_prefix;
		$find 		= $table_prefix . 'posts.post_type = \'post\''; // find where the query filters by post_type
		$replace	= '(' . $find . ' OR ' . $table_prefix . 'posts.post_type = \'page\')';
		return str_replace($find, $replace, $var);
	}

	function rss_page_get_lastpostmodified($lastpostmodified, $timezone){
		global $ma_feed, $wpdb;;
		if (!($ma_feed)){
			return $lastpostmodified;
		}

		$lastpostmodified = $wpdb->get_var("SELECT post_modified_gmt FROM $wpdb->posts WHERE post_status = 'publish' AND (post_type = 'post' OR post_type = 'page') ORDER BY post_modified_gmt DESC LIMIT 1");
		$lastpostdate = $wpdb->get_var("SELECT post_date_gmt FROM $wpdb->posts WHERE post_status = 'publish' AND (post_type = 'post' OR post_type = 'page') ORDER BY post_date_gmt DESC LIMIT 1");
		if ( $lastpostdate > $lastpostmodified ) {
			$lastpostmodified = $lastpostdate;
		}
		return $lastpostmodified;
	}

	function rss_page_feed_true(){
		global $ma_feed;
		$ma_feed = true;
	}
	function rss_page_feed_false(){
		global $ma_feed;
		$ma_feed = false;
	}

	function getLang($objectID = 0){
		$lang = $this->dpc->oFunctions->getPostLang($objectID, false);
		$lang = substr($lang, 0, (strpos($lang, '_') ? strpos($lang, '_') : 2));
		return $lang;
	}

	function edit_page_metabox_context($post){
		if(isset($this->settings['activateWdfIdf']) && $this->settings['activateWdfIdf'] == 'on'){
			include_once(dirname(__FILE__) . '/include/wdf_idf.php');
			$wfdIdf = new DPC_Module_Professional_Textopt_Wdf_Idf($this->dpc);
			$wfdIdf->wdf_idf_metabox_callback($post);
		}
	}


	function modalEnqueueStyles(){
	    wp_enqueue_style('dpc-professional-textopt-modalstyles', plugins_url( 'assets/css/modal.css', __FILE__ ));
	}
	function modalEnqueueScripts(){
	    wp_enqueue_script('dpc-basic-textopt-modal', plugins_url( 'assets/js/modal.js', __FILE__ ), array('jquery'), DPC_VERSION, true);
	    wp_localize_script('dpc-basic-textopt-modal', 'dpc_professional_textopt_wdf_idf_translator', array(
	        'Keyword' 					=> esc_html__('Keyword', 'delucks-seo'),
	        'Density' 					=> esc_html__('Density', 'delucks-seo'),
	        'Frequency' 				=> esc_html__('Frequency', 'delucks-seo'),
	        'Copy results to clipboard' => esc_html__('Copy results to clipboard', 'delucks-seo'),
	        'matches' 					=> esc_html__('matches', 'delucks-seo'),
	        'on' 						=> esc_html__('on', 'delucks-seo'),
	        'sites' 					=> esc_html__('sites', 'delucks-seo'),
	        'wdfIdfScore'				=> esc_html__('wdf*idf score', 'delucks-seo'),
	        'AddToStopwords' 			=> esc_html__('Add to stopwords', 'delucks-seo'),
	        'short'						=> esc_html__('short', 'delucks-seo'),
	        'ideal'						=> esc_html__('ideal', 'delucks-seo'),
	        'keywordMissing' 			=> esc_html__('Keyword not yet used in metadata, please mention it.', 'delucks-seo'),
	        'keywordFound' 				=> esc_html__('Keyword used in metadata.', 'delucks-seo'),
	        'alertFocusKey' 			=> esc_html__('Please define a focus keyword.', 'delucks-seo'),
	    ));
	}


	function getMdoalTranslationJs(){
        return array(
            'Keyword' 					=> esc_html__('Keyword', 'delucks-seo'),
            'Density' 					=> esc_html__('Density', 'delucks-seo'),
            'Frequency' 				=> esc_html__('Frequency', 'delucks-seo'),
            'Copy results to clipboard' => esc_html__('Copy results to clipboard', 'delucks-seo'),
            'matches' 					=> esc_html__('matches', 'delucks-seo'),
            'on' 						=> esc_html__('on', 'delucks-seo'),
            'sites' 					=> esc_html__('sites', 'delucks-seo'),
            'wdfIdfScore'				=> esc_html__('wdf*idf score', 'delucks-seo'),
            'AddToStopwords' 			=> esc_html__('Add to stopwords', 'delucks-seo'),
            'short'						=> esc_html__('short', 'delucks-seo'),
            'ideal'						=> esc_html__('ideal', 'delucks-seo'),
            'keywordMissing' 			=> esc_html__('Keyword not yet used in metadata, please mention it.', 'delucks-seo'),
            'keywordFound' 				=> esc_html__('Keyword used in metadata.', 'delucks-seo'),
            'alertFocusKey' 			=> esc_html__('Please define a focus keyword.', 'delucks-seo'),
		);
    }

	function modalTab(){
		echo '<span class="dpc-modal-tab dpc-modal-tab-trigger '.esc_attr(isset($_GET['tab']) && $_GET['tab'] == 'text' ? 'active' : '').'" id="tab-'. esc_attr($this->optionsHook) .'">'. esc_html($this->moduleTitle) .'</span>';
	}

	function modalBody(){
		echo '<div class="dpc-modal-content '.(isset($_GET['tab']) && $_GET['tab'] == 'text' ? 'active' : '').'" id="content-'. esc_attr($this->optionsHook) .'">';
		$this->dpc->new_parseSettingsByXml($this->settingsFile, 'modal', false, true, false, $this->dpc->modal->meta);
		echo '</div>';
	}

	function modalSave(){
		if(isset($_POST['dpc-competitor-sites']) && strlen($_POST['dpc-competitor-sites'])){
			$sites = array();
			foreach(explode(',', $_POST['dpc-competitor-sites']) as $k => $v){
			    $sites[] = array('url' => esc_url_raw($v));
			}
			update_post_meta(intval($_POST['dpc_current_id']), '_dpc-competitor-sites', $sites);
		} else {
		    update_post_meta(intval($_POST['dpc_current_id']), '_dpc-competitor-sites', array());
		}

		//update status
		$status = get_post_meta(intval($_POST['dpc_current_id']), '_delucks-seo-status', true);
		$status = is_array($status) ? $status : array();
		if((isset($_POST['dpc-competitor-sites']) && strlen($_POST['dpc-competitor-sites']))) { $status['text'] = 2; } else { $status['text'] = 1; }
		update_post_meta(intval($_POST['dpc_current_id']), '_delucks-seo-status', $status);
	}

	function initXMLText(){
	    esc_html__('Text optimization', 'delucks-seo');
	    esc_html__('Create more relevant contents. The text analysis tells you when editing contens below the editor which keywords you should mention, which Google may expect as relevant for your keyword.', 'delucks-seo');
	    esc_html__('Activate the Text Optimization to see a keyword field in your editor. There, you get recommendations while filling in your focus keyword.', 'delucks-seo');
	    esc_html__('Activated', 'delucks-seo');
		esc_html__('Deactivated', 'delucks-seo');
		esc_html__('Upgrade to the Suite-license to activate the text optimization (semantic and wdf*idf analysis) here', 'delucks-seo');
		esc_html__('More about the text optimization', 'delucks-seo');
		esc_html__('Include pages in RSS feeds', 'delucks-seo');
		esc_html__('Use description as content within RSS feeds', 'delucks-seo');
		esc_html__('Restore Link-Title field in editor', 'delucks-seo');
		esc_html__('Activate content optimization (semantic and wdf*idf analysis)', 'delucks-seo');
		esc_html__('Search Engine', 'delucks-seo');
		esc_html__('Stopwords for content optimization', 'delucks-seo');
		esc_html__('Also consider stop words of URL optimization', 'delucks-seo');
		esc_html__('Stopwords are expletives like „a, one, the“ which are not interesting for SEO. You can set up that the stopword lists from the URLs module can be used here (we prepared lists in English and German in the Standard Settings for you). You can also add your own stopwords above this field. In the content editor you can add unnecessary words to the stopwords list by clicking on the red „remove“-icon.', 'delucks-seo');
		esc_html__('Clear search result cache', 'delucks-seo');
		esc_html__('Maximum keywords in list', 'delucks-seo');
		esc_html__('Caching time for results in minutes', 'delucks-seo');
		esc_html__('Minimum keyword length', 'delucks-seo');
		esc_html__('Minimum keyword count', 'delucks-seo');
		esc_html__('Activate shortcode and filter menu for persona/ topic based content', 'delucks-seo');
		esc_html__('Personas', 'delucks-seo');
		esc_html__('Here you can add multiple Schema.org data for any location within your page/post', 'delucks-seo');
		esc_html__('Add location data', 'delucks-seo');
		esc_html__('Company name', 'delucks-seo');
		esc_html__('Street, house number', 'delucks-seo');
		esc_html__('Zipcode', 'delucks-seo');
		esc_html__('City', 'delucks-seo');
		esc_html__('Country', 'delucks-seo');
		esc_html__('E-Mail address', 'delucks-seo');
		esc_html__('Phone number', 'delucks-seo');
		esc_html__('Fax number', 'delucks-seo');
		esc_html__('Opened', 'delucks-seo');
		esc_html__('Mon', 'delucks-seo');
		esc_html__('Tue', 'delucks-seo');
		esc_html__('Wed', 'delucks-seo');
		esc_html__('Thu', 'delucks-seo');
		esc_html__('Fri', 'delucks-seo');
		esc_html__('Sat', 'delucks-seo');
		esc_html__('Sun', 'delucks-seo');
		esc_html__('from', 'delucks-seo');
		esc_html__('until', 'delucks-seo');
		esc_html__('Time format settings', 'delucks-seo');
		esc_html__('Add site', 'delucks-seo');
		esc_html__('Competitor URL', 'delucks-seo');
		esc_html__('Text %s (Target: %s words +/- %s, Is: %s words)', 'delucks-seo');
		esc_html__("Paste on the left URLs of your competitors for the same topic. Therefor click on the search results button and have a look of the competitors in Google. Check if their content or offer is similar to yours, copy their URLs then and paste them by clicking on the add site button. After you pasted all links to your competitors (at least 2 recommended), click on the compare sites button to see, which words Google expects in your content. The words which are highlighted with a red cross are not in your context yet and should be mentioned, if they fit in your text. If a word does not fit at all, add it to the stopword (ignore) list by clicking on the red icon on the right. All words which are orange, could be mentioned more often. All green words are already good.", 'delucks-seo');
		esc_html__('Competitors', 'delucks-seo'); 	// @todo 
		esc_html__('Words', 'delucks-seo');			// @todo 
		esc_html__('Structure', 'delucks-seo');		// @todo 
	}

}